(* mcirc.m*)
 
(*Created by Rolf Sulanke, Sep 8, 1998, last revision February 29, 2000.*)   
  
(*The package contains definitions and procedures for the euclidean and
the Moebius geometry of circles in the 3-sphere. It continues the package \
euvec.m which 
contains concepts of the euclidean 
geometry of spheres. Operations and functions of pseudo-euclidean vector \
algebra (see 
the package pseuvec.m and the notebook pseuklid.m) are used.The package is
tested, explained, and  applied in the notebooks mcircles.nb, pairs.nb.*)

(* Free software: Feel free to apply, distribute or change the packages and
notebooks of the item "Spheres". Please, cite the source and notice the
changes, or new concepts,  which you introduced.*)

(* Hint: Before importing this package be sure that the numbers
dim (= dimension) and ind (=index) of the pseudo-euclidean vector space are \
\
defined as constants within the Global context !!!*)

(*Changes July 6,2000: circle2spv, plotcircle2spv new definitions! *)

(* WARNING: The procedure pscomplement, and the depending  of this \
procedures,
e.g. circle2spv,plotcircle2spv, do not always depend continuously of their
vector aeguments. *)

BeginPackage["mcirc`", {"mspher`","euvec`","pseuvec`"}] 
  
posvec3pts::usage= "posvec3pts[p1,p2,p3] is the position vector of the circle 
through the 3 points p1,p2,p3: it is a unit vector orthogonal to the plane of 
the circle uniquely defined up to sign. If the points belong to a 
line posvec3pts returnes the null vector nul[3]."

center3pts::usage = "center3pts[p1,p2,p3] returns the center of the circle 
through the three points p1,p2,p3, or the vector \
{Infinity,Infinity,Infinity},
if the points belong to a line."

radius3pts::usage = "radius3pts[p1,p2,p3] returns the radius of the circle 
through the three points p1,p2,p3, or Infinity, if the points belong 
to a line."

circle3pts::usage = "circle3pts[p1,p2,p3][t] is a parameter representation of \
the circle (or line) through the three points p1,p2,p3."

plotcircle3pts::usage = "plotcircle3pts[p1,p2,p3] plots the circle 
(or line) through the three points p1,p2,p3."

(* WARNING: Always, if there exist a line through the three points, e.g. if 
the points coincide, center3pts and radius3pts are infinite!*)

control::usage = "control[v,w, prod] measures the orthonormality of the 
spacelike vectors v,w with respect  to the  scalar product prod; 
the default prod is the pseudo-euclidean scalar product pssp."
  
pscomplement::usage = "pscomplement[v,w] yields a 
pseudo-orthonormal basis of the orthogonal complement of the euclidean 2-space spanned by v,w in the 
pseudo-euclidean 5-space of index 1, or an error message if span[v,w] is not euclidean."

circle2spv::usage = "circle2spv[v,w][t]is a parameter  representation of
the circle in the euclidean 3-space corresponding to the euclidean  
2-subspace, spanned by the spacelike vectors v, w of the pseudo-euclidean 
5-space, ind = 1."

threepoints::usage="threepoints[v,w] yields three points of \
circle2spv[v,w][t]."

adaptsplframe::usage="adaptsplframe[v,w] transforms the arbitrary basis v,w 
of an euclidean 2-space into a frame adapted to the circle defined by the 2-space; 
the adapted frame is an orthonormal basis such that the first vector defines the plane 
of the circle and the second the sphere with the same radius and center as the circle. 
For a line any  orthonormal basis is considered to be adapted."

posvec::usage="posvec[v,w] yields the position vector of the circle \
circle2spv[v,w][t]."  

center::usage="center[v,w] yields the center of the circle \
circle2spv[v,w][t]."  

radius::usage="radius[v,w] yields the radius of the circle \
circle2spv[v,w][t]."  

plotcircle2spv::usage = "plotcircle2spv[v,w, opts] plots the circle with 
parameter representation circle2spv[v, w][t] with the options opts of 
ParametricPlot3D."
  
circlefamily::usage = "circlefamily[v,w,t,a,b, d:20, opt] plots d elements of \
the family of circles defined by the family ({v[t], w[t]}, a<=t<=b, 
of orthonormal spacelike vectors in the pseudo-euclidean 5-dimenensional 
vector space of index 1. d = 20 is the default value; options opt are 
transferred to ParametricPlot3D."

circleplane::usage = "circleplane[x,y,z,u,v] yields the spacelike unit vector \
corresponding to the plane in the euclidean 3-space containing the circle \
with center {x,y,z} and position vector unit[2][{u,v}]; it does not depend on the 
radius of the circle."

circlespacevectors::usage = "circlespacevectors[x,y,z,u,v,r] yields the \
adapted 
orthonormal 2-frame of the euclidean subspace of the pseudo-euclidean 5-space \
defining the circle with center {x,y,z}, position vector unit[2][{u,v}], and 
radius r; the first vector corresponds to the plane of the circle and the \
second 
to the sphere with the same center and radius as the circle."
 
vspace::usage = "vspace[p1,p2,p3] yields an orthonormal base of the spacelike
2-space in the 5-space corresponding  	to the circle (or line!) through the 3
points p1,p2,p3 of the 3-dimensional euclidean space."
  
unitvec::usage = "unitvec[u,v][t] is for fixed u,v and variable t, 
0 < t < 2 Pi, a parameter representation of the  unit circle with center o in \
the plane orthogonal to the radius vector 
unit[2][{u,v}]={Cos[u] Cos[v], Sin[u] Cos[v], Sin[v]} 
of the unit sphere in the euclidean 3-space." 	 
  
circle3D::usage = "circle3D[x,y,z,u,v,r][t] is the parameter
representation of the circle in the euclidean 3-space with  radius r and 
center {x,y,z} in the plane orthogonal to the position vector 
unit[2][{u,v]]={Cos[u] Cos[v], Sin[u] Cos[v], Sin[v]}."
  
gencircle::usage = "gencircle[p_List, v_List][t] is the parameter
representation of the circle in the euclidean 3-space with radius r = norm[v] \
and center p in the plane orthogonal to the vector v."
  
plotcircle3D::usage = "plotcircle3D[x,y,z,u,v,r,opts] plots  the circle in 
the euclidean 3-space with radius r and center {x,y,z} in the  plane 
orthogonal to the vector unit[2][u,v]={Cos[u] Cos[v], Sin[u] Cos[v], Sin[v]}, \
with options opts of ParametricPlot3D."
  
plotgencircle::usage = "plotgencircle[p_List,v_List, opts] plots  the circle 
in the euclidean 3-space with radius norm[v]  and center p in the 
plane orthogonal to  the vector v with options opts of ParametricPlot3D."
  
pp::usage = "pp: If V and W are the spacelike vector subspaces spanned by the 
orthonormal bases {v1, v2}, {w1, w2}  respectively, then pp[v1, v2, w1,w2] 
is the matrix of the composition p = p2 p1 of the orthogonal projections 
p1: V -> W, and p2: W -> V, with respect to the base {v1, v2}."
  
tube::usage = "tube[x,a,b,r,opt] plots the tube of the curve x (Enter only 
the name, not the parameter!)  with tube radius r for the parameter s, 
a<s<b, with options opt of ParametricPlot3D."
  
ppdet::usage = "ppdet is the determinant of the matrix pp."
  
pptr::usage = "pptr is the trace of the matrix pp."
  
  
Begin["`Private`"]
  
dim = Global`dim;

ind = Global`ind;
  
(*Control of orthonormality*)
  
  
control[v_List, w_List, prod_:pssp] := 
    Simplify[(prod[v, v] - 1)^2 + prod[v, w]^2 + (prod[w, w] - 1)^2]

  
(*Parametrisation of the circle space. The general circle *)
  
  
unitvec[u_,v_][t_] := {-Cos[t] Sin[u] - Sin[t] Cos[u]Sin[v],
	Cos[t]  Cos[u] - Sin[t] Sin[u] Sin[v], Sin[t]Cos[v]};
  
  
circle3D[x_,y_,z_,u_,v_,r_][t_]:={x, y,z} + unitvec[u,v][t]*r;
  
  
gencircle[p_List,v_List][t_]:=Module[{a = v.v,b1, b2},
			If[Length[p]!=3||Length[v]!=3, Return[Print["Incorrect arguments!!!"]]];
	  	If[a== 0, Return[p]];
			If[v[[1]] == 0 && v[[2]] ==0, Return[p + {Cos[t], Sin[t], 0}*Sqrt[a]]];
			b1={-v[[2]], v[[1]], 0}/Sqrt[a - v[[3]]^2];
			b2 = Cross[v, b1];
			Return[p+b1*Sqrt[a]*Cos[t] +b2 Sin[t]]]
  
  
(*The corresponding Plot commands *)
  
  
plotcircle3D[x_,y_,z_,u_,v_,r_, opts___]:= 
ParametricPlot3D[Evaluate[circle3D[x,y,z,u,v,r][t]], {t, 0, 2 Pi}, opts]
  
  
plotgencircle[p_List,v_List, opts___]:= ParametricPlot3D[
Evaluate[gencircle[p,v][t]], {t, 0, 2 Pi}, opts]
  
  
(* Circle through 3 points *)

posvec3pts[p1_List,p2_List,p3_List] :=
	Module[{ a},
		a = Cross[p2-p1, p3-p1];
		If[a !=nul[3],Return[smoothing[a/Sqrt[a.a]]], Return[nul[3]],
      Return[smoothing[a/Sqrt[a.a]]]]]

center3pts[p1_List,p2_List,p3_List]:= 
  Module[{q1= p1 - p3, q2 =p2 - p3,dn,dn1,dn2},
    dn = Det[{{q1.q1, q1.q2},{q1.q2,q2.q2}}];
    If[Chop[dn] == 0, Return[{Infinity, Infinity,Infinity}]];
    dn1 = Det[{{q1.q1/2, q1.q2},{q2.q2/2,q2.q2}}];
    dn2 = Det[{{q1.q1, q1.q1/2},{q1.q2,q2.q2/2}}];
    Return[p3+(dn1*q1 + dn2*q2)/dn]]

radius3pts[p1_List,p2_List,p3_List] :=
	Module[{ a},
		a = center3pts[p1,p2,p3];
		If[a =={Infinity,Infinity,Infinity}, Return[Infinity]];
		Return[ Sqrt[(p1-a).(p1 - a)]]]
		
(* Now we  give a definition of a 3D-circle through three points. The circle 
may eventually be a line (circle of infinite radius): *)

circle3pts[p1_List,p2_List,p3_List][t_] :=
	Module[{q2 , q3 , a, m,r},
    If[p1 == {Infinity,Infinity, Infinity},Return[p2 + t*(p3-p2)]];
    If[p2 == {Infinity,Infinity, Infinity},Return[p1 + t*(p3-p1)]];
    If[p3 == {Infinity,Infinity, Infinity},Return[p1 + t*(p2-p1)]];
    m = center3pts[p1,p2,p3];
    If[m == {Infinity,Infinity,Infinity},
      If[smoothing[p2 -p1] ==nul[3], Return[p1 + t*(p3 - p1)], 
        Return[p1 + t*(p2 - p1)]]];
    r = Sqrt[(p1-m).(p1 - m)];	
    	q2 = p2-p1; q3 = p3-p1;a = Cross[q2,q3];
    		q3 = Cross[a,q2];
    		q2  = q2/Sqrt[q2.q2];
    		q3 =  q3/Sqrt[q3.q3];
    		m +(q2 *Cos[t] + q3*Sin[t])*r]

(* Here is the corresponding plot command: *)

plotcircle3pts[p1_List,p2_List,p3_List,opt___]:= 
  ParametricPlot3D[Evaluate[circle3pts[p1,p2,p3][t]],{t, -Pi, Pi},opt] 

(*Tubes*)
  
  
tube[x_, a_,b_,r_, opts___] := Module[{t,s, w,v, u},
			v[s_] := D[x[u],u]/.u -> s;
			w[s_]:=If[v[s].v[s]=== 0.,Return[v[s]], Return[ v[s]/Sqrt[v[s].v[s]]]];
			ParametricPlot3D[gencircle[x[s],r*w[s]][t], {s,a,b},{t,0,2 Pi},opts]]


(*Now we introduce a parameter representation of the circle  corresponding to \
an euclidean 2-subspace of the pseudo-euclidean 5-space, ind = 1. 
First we define the submodule pscomplement which may be useful at other \
opportunities, too.*)

pscomplement[v_List,w_List]:= Module[{j=0,w1,w2,w3,chj},
    Label[chj];
    If[j==dim, 
      Print["The complement not found! 
      Vectors do not span an euclidean 2-space?"];
      Return[];Break[],j=j+1];              
     w1 = Simplify[pscross[{v,w,stb[j],stb[5]}]];
    If[w1 == nul[5], Goto[chj], w2 = Simplify[pscross[{v,w,w1,stb[5]}]], 
      w2 = Simplify[pscross[{v,w,w1,stb[5]}]]];
    w3 = Simplify[pscross[{v,w,w1,w2}]];
    Return[{w1/Sqrt[pssp[w1,w1]],w2/Sqrt[pssp[w2,w2]],
        w3/Sqrt[-pssp[w3,w3]]}]]
        

(* Three points *)

threepoints[v_,w_]:= Module[{b=pscomplement[v,w]},
If[b == Null, Return[];Break[]];
		{stepro[b[[3]]+  b[[1]]], stepro[b[[3]] - b[[1]]], stepro[b[[3]]+ b[[2]]]}]

(* Adapting frames. *)
adaptsplframe[vx_List,vy_List] := Module[{aob,d1,d2,q = 0,v,nor},
    aob = orthonorm[{vx,vy}, normed \[Rule] False];
    If[Length[aob]<2,
      Print["The span is not 2-dimensional and spacelike!"];Return[]];
    If[Simplify[pssp[aob[[1]], aob[[1]]]]\[LessEqual] 0 ||
        Simplify[pssp[aob[[2]], aob[[2]]]]\[LessEqual] 0 ,
      Print["The span is not 2-dimensional and spacelike!"];Return[]];
    d1=Simplify[aob[[1]][[5]] - aob[[1]][[4]]];
    d2= Simplify[aob[[2]][[5]] - aob[[2]][[4]]];
    If[d1\[Equal] 0, q=1;Goto[nor]];
    If[d2\[Equal]0,v=aob[[1]]; aob[[1]] = aob[[2]];aob[[2]]= v;q=1];
    Label[nor];
    aob[[1]] = aob[[1]]/Sqrt[pssp[aob[[1]], aob[[1]]]];
    aob[[2]] = aob[[2]]/Sqrt[pssp[aob[[2]], aob[[2]]]];
    If[q\[Equal]1,Return[Simplify[aob]]];
    d1=Simplify[aob[[1]][[5]] - aob[[1]][[4]]];
    d2= Simplify[aob[[2]][[5]] - aob[[2]][[4]]];
    q = Sqrt[d1^2 + d2^2];
    v = aob[[1]]; 
    aob[[1]] = Simplify[ -v*d2/q + aob[[2]]*d1/q];
    aob[[2]] =Simplify[ v*d1/q + aob[[2]]*d2/q];
    Return[aob]];

(*Position vector, center, and radius of the circle defined by an euclidean \
2-space spanned by the vectors v,w. *)

posvec[vx_List,vy_List] := Module[{ab},
ab=adaptsplframe[vx,vy];
    If[ab[[2]][[4]] \[Equal] ab[[2]][[5]], Return[{0,0,0}]];
    Return[Take[ab[[1]],3]]];

center[vx_List,vy_List] := Module[{ab},
ab=adaptsplframe[vx,vy];
    If[ab[[2]][[4]] \[Equal] ab[[2]][[5]], 
      Return[{Infinity,Infinity,Infinity}]];
    Return[Take[ab[[2]],3]/(ab[[2]][[5]] - ab[[2]][[4]])]];
    
radius[vx_List,vy_List] := Module[{ab},
ab=adaptsplframe[vx,vy];
    If[ab[[2]][[4]] \[Equal] ab[[2]][[5]], Return[Infinity]];
    Return[1/(ab[[2]][[5]] - ab[[2]][[4]])]];

(*Plotting circles.*)

(* OLD: circle2spv[v_List,w_List][t_]:= Module[{p1,p2,p3,b},
		b =threepoints[v,w];
		If[b == Null, Return[];Break[]];
		p1=  b[[1]];
		p2 =b[[2]];
		p3 =  b[[3]];
		circle3pts[p1,p2,p3][t]]  *)
  
(* Old : plotcircle2spv[ v_,w_, opt___]:= Module[{p1,p2,p3,b},
		b =threepoints[v,w];
		If[b == Null, Return[];Break[]];
		p1=  b[[1]];
		p2 =b[[2]];
		p3 =  b[[3]];
		plotcircle3pts[p1,p2,p3,opt]] *)
		
circle2spv[v_List,w_List][t_]:= Module[{psc},
    psc = pscomplement[v,w];
    If[psc == Null, Return[]];
      stepro[-psc[[1]]*Cos[t] -psc[[2]]*Sin[t] + psc[[3]]]];
      
plotcircle2spv[v_List,w_List,opt___]:= 
  ParametricPlot3D[circle2spv[v,w][t], {t,0,2*Pi}, opt];


      circlefamily[v_,w_,t_,a_,b_, d_:20, opt___] := ParametricPlot3D[
		Evaluate[
		Table[circle2spv[v[t], w[t]][u], {t, a, b, (b-a)/d}]],{u,0,2 Pi}, opt] 

(*Finding the euclidean 2-space of the circle through three points.*)

(* WARNING: Replacing pscross[{...}] by psCross[...] may lead to very strange
errors if applied to floating point vectors; try it with *)

(* zz[u_]:= vspace[{1,0,u},{0,1,u},{-1,0,u}] *)

(* and compare the results for u = 3, u = 3.0, or u = 5, u = 5.0 ! REASON? *)

circleplane[x_,y_,z_,u_,v_] :={Cos[u] Cos[v],Cos[v] Sin[u],Sin[v],
    x Cos[u] Cos[v]+y Cos[v] Sin[u]+z Sin[v],
    x Cos[u] Cos[v]+y Cos[v] Sin[u]+z Sin[v]}
    
circlespacevectors[x_,y_,z_,u_,v_,r_] :={circleplane[x,y,z,u,v],
    spacevec[x,y,z,r]}

vspace[p1_,p2_,p3_]:=Module[{q1,q2,q3,j=1,w1,w2,chj},
    q1=Simplify[invstepro[p1]];
    q2=Simplify[invstepro[p2]];
    q3=Simplify[invstepro[p3]];
    Label[chj];
    If[j==dim,
      Print["vspace not found! Points not in general position?"];Return[],
      j=j+1];
        w1=Simplify[pscross[{q1,q2,q3,stb[j]}]];
    If[smoothing[w1]==nul[dim],Goto[chj],
      w1=Simplify[w1/Sqrt[pssp[w1,w1]]],
      w1=Simplify[w1/Sqrt[pssp[w1,w1]]]];      
    w2=Simplify[pscross[{q1,q2,q3,w1}]];
    If[w2==nul[dim],Goto[chj]];
    w2=Simplify[w2/Sqrt[pssp[w2,w2]]];		
    Return[smoothing[{w1,w2}]]]  
    
  
(*Invariants of pairs of circles.*)
  
  
 pp[v1_, v2_, w1_, w2_] := 
    {{pssp[v1, w1]^2 + pssp[v1, w2]^2, 
      pssp[v1, w1]*pssp[v2, w1] + pssp[v1, w2]*pssp[v2, w2]}, 
     {pssp[v1, w1]*pssp[v2, w1] + pssp[v1, w2]*pssp[v2, w2], 
      pssp[v2, w1]^2 + pssp[v2, w2]^2}}
  
  
ppdet[v1_, v2_, w1_, w2_] := Det[pp[v1, v2, w1, w2]]
  
  
pptr[v1_, v2_, w1_, w2_] := Sum[pp[v1, v2, w1, w2][[i,i]], {i, 1, 2}]

End[]
  
  
Protect[adaptsplframe, center, center3pts, circle2spv, circle3D, circle3pts, circlefamily, \
circleplane, 
circlespacevectors, control, 
gencircle, pp, ppdet, pptr, plotcircle2spv, plotcircle3D,  plotgencircle, 
plotcircle3pts, posvec, posvec3pts, pscomplement, radius, radius3pts, \
threepoints, tube, unitvec, vspace]

EndPackage[]
