(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    116600,       4395]*)
(*NotebookOutlinePosition[    117501,       4426]*)
(*  CellTagsIndexPosition[    117457,       4422]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Spheres in Euclidean Spaces", "Title",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontFamily->"Times New Roman"],

Cell["by Rolf Sulanke", "Subtitle",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontFamily->"Times New Roman"],

Cell["February 24, 1999, last revision November 1, 2000", "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontFamily->"Times New Roman"],

Cell[CellGroupData[{

Cell["Summary", "Subsubsection",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Left,
  FontFamily->"Times New Roman"],

Cell["\<\
This notebook contains  basic definitions of spheres as objects of euclidean \
and  Riemannian spherical geometry. It needs the packages euvec.m, \
spiralsf.m. As an application the construction and plotting of a sphere \
through four points in the euclidean 3-space is given. Furthermore, it \
contains a recursive definition of the generalized geographical parameter \
representations of n-spheres in the (n+1)-dimensional euclidean space. Some \
concepts needed in M\[ODoubleDot]bius geometry,  the conformal geometry of \
the n-sphere, are introduced in euclidean terms. These concepts are: \
Stereographic projection and its inversion, reflections at hyperspheres (also \
called inversions),  spiral transformations, spiral surfaces, in particular \
the spiral cylinder, which is of interest in M\[ODoubleDot]bius differential \
geometry: it is a homogeneous surface with constant negative curvature, see \
[3]. Spiral surfaces have been considered already by S. Lie  (1891), and E. \
Vessiot (1926), these and other references see  [3]. 
Changes Feb 2000:  A version of Erhard Schmidt's orthogonalization: \
esorthonorm, is introduced in section 4.1. The procedure normalized is \
renamed in dotnorme, normed in normalize, and the new concept normed is now \
an option for the orthogonalization esorthonorm. The procedures called \
sphere1, sphere3, spherrefl are renamed: sphere1 -> subsphere, sphere3 -> \
subspheremf, spherrefl -> sphericalreflection. The procedures sphere, \
sphereplot3D are introduced in euvec.m.\
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"]
}, Closed]],

Cell[CellGroupData[{

Cell["References", "Subsubsection"],

Cell["\<\
[1] Gray, Alfred. Modern Differential Geometry of Curves and Surfaces. CRC \
PRESS, 1993 (German translation: Spektrum Akademischer Verlag 1994)\
\>", "Text"],

Cell["\<\
[2] Kuska, Jens-Peer. http://phong.informatik.uni-leipzig.de/ \
~kuska/mview3d.html.\
\>", "Text"],

Cell[TextData[{
  "[3] Sulanke, Rolf. M\[ODoubleDot]bius geometry V: Homogeneous surfaces in \
the M\[ODoubleDot]bius space ",
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)]],
  ". Colloquia Math. J. Bolyai 46, Topics in Differential Geometry, Debrecen \
1984, 1141-1154."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Keywords", "Subsubsection"],

Cell["\<\
vector objects, random vectors, rank,  orthoframes,  unitvectors, norming, \
cross product (general), outzero, nullvector, standard base ,  hyperplanes, \
stereographical projection, invers stereographical projection, 3-sphere, \
spheres, spheres through four points, inversion at hyperspheres, flat torus, \
geodesics on the flat torus, torus, Erhard Schmidt's othogonalization, \
orthogonal complement in the euclidean 4-space.  Parameter representation for \
n-spheres, spiral transformations, spiral surfaces, MathGl3d, ThreeScript.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Address", "Subsubsection"],

Cell["sulanke@mathematik.hu-berlin.de", "Text"],

Cell["\<\
The author will be very thankful for hints, corrections, comments.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[CellGroupData[{

Cell["The needed packages", "Subsection"],

Cell["\<\
If you want tu use all the notebooks of the item \"Spheres\", ensure that the \
packages
euvec.m
pseuvec.m
mspher.m
mcirc.m
spiralsf.m
and the master package init.m are in your working directory. These packages \
are contained in the item \"Spheres\".
 For the present notebook we only need the packages euvec.m,  spiralsf.m. \
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["3D graphics, animations", "Subsection"],

Cell[TextData[{
  "In the notebook command lines using ThreeScript, or MVShow3D appear. They \
may used for 3D-representations and animations of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Graphics3D objects. To apply them you must have the necessary software on \
your computer. Look in the last section. "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["For Windows", "Subsection"],

Cell[TextData[{
  "For the commands in this subsection:\n",
  StyleBox["DO NOT CONFIRM THE EVALUATION OF THE INITIALIZATION CELLS !!!",
    FontWeight->"Bold",
    FontColor->RGBColor[1, 0, 0]]
}], "Text"],

Cell[" If the result of the following command ", "Text"],

Cell[BoxData[
    \(Directory[]\)], "Input",
  CellLabel->"In[8]:="],

Cell["\<\
does not coincide with the working directory, e.g. \"C:\\Eigene \
Dateien\\mymath\\spheres.v4\", containing the needed packages, insert the \
Path of your working directory in the next command line and activate it \
first, without calling the initialization:\
\>", "Text"],

Cell[BoxData[
    \(SetDirectory["\<C:\\Eigene Dateien\\mymath\\spheres4\>"]\)], "Input",
  CellLabel->"In[7]:="],

Cell["\<\
If you want to use the package euvec.m only,  carry out the next command.\
\>", "Text"],

Cell[BoxData[
    \(Needs["\<euvec`\>"]\)], "Input",
  CellLabel->"In[1]:="],

Cell["\<\
If you want  that all commands contained in the packages of the item \
\"Spheres\" are available, activate\
\>", "Text"],

Cell[BoxData[
    \(Get["\<init.m\>"]\)], "Input",
  CellLabel->"In[9]:="],

Cell[TextData[StyleBox["Now: Activite Kernel/Evaluation/Evaluate \
Initialization from the menu!",
  FontSize->18,
  FontColor->RGBColor[1, 0, 0]]], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["For Linux (and other  systems: not tested!)", "Subsection"],

Cell[TextData[{
  "For the next command:\n",
  StyleBox["DO NOT CONFIRM THE EVALUATION OF THE INITIALIZATION CELLS !!!",
    FontWeight->"Bold",
    FontColor->RGBColor[1, 0, 0]]
}], "Text"],

Cell["\<\
If you want to use the package euvec.m only, remove the file init.m  from the \
working directory and carry out the next command.\
\>", "Text"],

Cell[BoxData[
    \(Needs["\<euvec`\>"]\)], "Input",
  CellLabel->"In[1]:="],

Cell[TextData[StyleBox["Now: Activite Kernel/Evaluation/Evaluate \
Initialization from the menu!",
  FontSize->18,
  FontColor->RGBColor[1, 0, 0]]], "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1 Introduction", "Section"],

Cell[CellGroupData[{

Cell["1.1 The 3-sphere", "Subsection"],

Cell["\<\
In this notebook we develop some  linear algebraic tools which can be applied \
for calculations in any dimension and for graphics in the threedimensional \
case. The commands appropriate for general use are collected in the packages \
euvec.m. The procedure for creating spiral surfaces is contained in the \
package spiralsf. m.  One has to fix the dimension dim of the underlying \
vector space, and its index ind; these global constants enter in almost all \
of the defined functions and modules. In our case their values are\
\>", "Text"],

Cell[BoxData[
    \(dim\  = \ 4; ind\  = \ 0;\)], "Input",
  CellLabel->"In[10]:=",
  InitializationCell->True],

Cell["\<\
For an elementary approach to the Riemannian geometry of the standard \
threedimensional sphere  (\"3-sphere\") it is convenient to consider the \
3-sphere as the submanifold of the unit vectors in the fourdimensional  \
euclidean space  (\"4-space\"):\
\>", "Text"],

Cell[BoxData[
    \(vec[x]\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(vec[x] . vec[x]\  == \ 1\)], "Input",
  CellLabel->"In[25]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["1.2 The usages", "Subsection"],

Cell[CellGroupData[{

Cell["Concepts in euvec.m", "Subsubsection"],

Cell["Valid for arbitrary values of the dimension dim are", "Text"],

Cell[BoxData[
    \(\(?nul\)\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(nul[]\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(nul[7]\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(\(?vec\)\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(vec[y]\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(\(?randomv\)\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(randomv[]\)], "Input",
  CellLabel->"In[23]:="],

Cell[BoxData[
    \(\(?stb\)\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(\(?cross\)\)], "Input",
  CellLabel->"In[25]:="],

Cell[BoxData[
    \(\(?dotnorme\)\)], "Input",
  CellLabel->"In[26]:="],

Cell["The next command generates an error message:", "Text"],

Cell[BoxData[
    \(dotnorme[nul[]]\)], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \(\(?outzero\)\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(\(?rank\)\)], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(\(?sterproj\)\)], "Input",
  CellLabel->"In[30]:="],

Cell[BoxData[
    \(\(?invsterproj\)\)], "Input",
  CellLabel->"In[31]:="],

Cell[BoxData[
    \(\(?hyperplane\)\)], "Input",
  CellLabel->"In[32]:="],

Cell[BoxData[
    \(\(?stb\)\)], "Input",
  CellLabel->"In[33]:="],

Cell[BoxData[
    \(stb[i]\)], "Input",
  CellLabel->"In[34]:="],

Cell["\<\
Since we defined dim = 4 the next command generates an error message:\
\>", "Text"],

Cell[BoxData[
    \(stb[5]\)], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(stb[5, 7]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(stb[4]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(hyperplane[Table[stb[j], {j, 4}]]\)], "Input",
  CellLabel->"In[38]:="],

Cell[BoxData[
    \(\(?unit\)\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(\(?sphericalreflection\)\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(\(?esorthonorm\)\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(\(?innerprod\)\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(\(?normed\)\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(\(?print\)\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(\(?neglect\)\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \(\(?renorm\)\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(\(?orthobase\)\)], "Input",
  CellLabel->"In[47]:="],

Cell[BoxData[
    \(orthobase[stb[5, 7]]\)], "Input",
  CellLabel->"In[48]:="],

Cell[BoxData[
    \(orthobase[nul[5]]\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(\(?smoothing\)\)], "Input",
  CellLabel->"In[50]:="],

Cell["The following commands work in dimensions  4 or 3  only:", "Text"],

Cell[BoxData[
    \(\(?ortho1\)\)], "Input",
  CellLabel->"In[51]:="],

Cell[BoxData[
    \(\(?orthoerg\)\)], "Input",
  CellLabel->"In[52]:="],

Cell[BoxData[
    \(\(?orthoframe\)\)], "Input",
  CellLabel->"In[53]:="],

Cell[BoxData[
    \(\(?tg4frame\)\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \(\(?sphere\)\)], "Input",
  CellLabel->"In[55]:="],

Cell[BoxData[
    \(\(?sphereplot3D\)\)], "Input",
  CellLabel->"In[56]:="],

Cell[BoxData[
    \(\(?subsphere\)\)], "Input",
  CellLabel->"In[57]:="],

Cell[BoxData[
    \(\(?subspheremf\)\)], "Input",
  CellLabel->"In[58]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Concepts in spiralsf.m", "Subsubsection"],

Cell["If the master package init.m is not in use first import", "Text"],

Cell[BoxData[
    \(Needs["\<spiralsf`\>"]\)], "Input",
  CellLabel->"In[3]:="],

Cell[BoxData[
    \(\(?spiral\)\)], "Input",
  CellLabel->"In[66]:="],

Cell[BoxData[
    \(\(?spiralsurface\)\)], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[
    \(\(?plotspiralsurface\)\)], "Input",
  CellLabel->"In[68]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Full definitions", "Subsubsection"],

Cell["\<\
For getting the usage together with the complete definition, enter e.g.\
\>", "Text"],

Cell[BoxData[
    \(?? sterproj\)], "Input",
  CellLabel->"In[69]:="],

Cell["\<\
Since the names are prepended by the ContextNames the definition obtained in \
this way are hard to read. Therefore you will find some of them in the \
original form in unevaluable cells.  It is recommended to look into the \
packages.\
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.3 Some tools", "Subsection"],

Cell[CellGroupData[{

Cell["Vector objects", "Subsubsection"],

Cell["In euvec.m a general vector is defined:", "Text"],

Cell[BoxData[
    \(vec[x_, \ n_:  dim]\  := \ Table[x[i], \ {i, \ n}]\)], "Input",
  CellLabel->"In[2]:=",
  Evaluatable->False],

Cell["Example", "Text"],

Cell[BoxData[
    \(vec[Sin]\)], "Input",
  CellLabel->"In[70]:="],

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[71]:="],

Cell["\<\
In euvec.m a  random vector with values in the unit n-cube is defined as\
\>", "Text"],

Cell[BoxData[
    \(randomv[n_:  dim] := \ Table[Random[], {i, n}]\)], "Input",
  CellLabel->"In[3]:=",
  Evaluatable->False],

Cell[BoxData[
    \(randomv[]\)], "Input",
  CellLabel->"In[72]:="],

Cell[BoxData[
    \(vec[randomv]\)], "Input",
  CellLabel->"In[73]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Reflections at arbitrary spheres (inversions)", "Subsubsection"],

Cell[TextData[{
  "A point pair x, y is",
  StyleBox[" ",
    FontColor->RGBColor[0, 1, 1]],
  StyleBox["in inversion",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[" ",
    FontWeight->"Bold"],
  "with respect to a hypersphere with radius r and center a iff a, x, y are \
collinear and fulfil the equation"
}], "Text"],

Cell[TextData[{
  "|y - a| |x - a| = ",
  Cell[BoxData[
      \(TraditionalForm\`r\^2\)]],
  "."
}], "Text",
  TextAlignment->Center],

Cell["Here |z| means the norm of the vector z.  We remember", "Text"],

Cell[BoxData[
    \(\(?Infinity\)\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(Infinity\)], "Input",
  CellLabel->"In[29]:="],

Cell["This leads to the definition (contained in euvec.m):", "Text"],

Cell[BoxData[
    \(\(?sphericalreflection\)\)], "Input",
  CellLabel->"In[68]:="],

Cell[BoxData[{
    \(\(sphericalreflection[{Infinity}, \ a_List, r_:  1]\  := \ 
        a;\)\), "\n", 
    \(sphericalreflection[x_List, \ a_List, \ r_:  1]\  := \ 
      If[Not[x == a], \n
        a\  + \ \((\((x\  - \ a)\)\ *\ 
                r^2)\)/\((\((x - a)\) . \((x - a)\))\), {Infinity}, \n
        a\  + \ \((\((x\  - \ a)\)\ *\ 
                r^2)\)/\((\((x - a)\) . \((x - a)\))\)]\)}], "Input",
  CellLabel->"In[44]:=",
  PageWidth->Infinity,
  Evaluatable->False],

Cell["sphericalreflection[{2, 2, 2},nul[3]]", "Input",
  CellLabel->"In[74]:=",
  PageWidth->Infinity],

Cell["sphericalreflection[vec[x],nul[]]", "Input",
  CellLabel->"In[15]:=",
  PageWidth->Infinity],

Cell["\<\
The reflection at the unit sphere with further examples will be considered in \
section 5.\
\>", "Text"],

Cell[BoxData[
    \(sphericalreflection[vec[x], \ vec[a], \ r]\)], "Input",
  CellLabel->"In[75]:="],

Cell[BoxData[
    \(\(vy\  = \ %;\)\)], "Input",
  CellLabel->"In[76]:="],

Cell["\<\
Test: The spherical reflections fulfill the following identities:\
\>", "Text"],

Cell[BoxData[
    \(Simplify[\((vy\  - vec[a])\) . \((vec[x]\  - \ vec[a])\) \[Equal] \ 
        r^2]\)], "Input",
  CellLabel->"In[77]:="],

Cell[BoxData[
    \(sphericalreflection[vec[a], \ vec[a], \ 
        r] \[Equal] {Infinity}\)], "Input",
  CellLabel->"In[78]:="],

Cell[BoxData[
    RowBox[{
      RowBox[{"sphericalreflection", "[", 
        RowBox[{
          RowBox[{"{", 
            InterpretationBox["\[Infinity]",
              DirectedInfinity[ 1]], "}"}], ",", " ", \(vec[a]\), ",", " ", 
          "r"}], "]"}], "\[Equal]", " ", \(vec[a]\)}]], "Input",
  CellLabel->"In[79]:="],

Cell[BoxData[
    \(Clear[vy]\)], "Input",
  CellLabel->"In[80]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Smoothing", "Subsubsection"],

Cell[TextData[{
  "The following function makes an almost zero  number to be zero; the \
sensibility can be regulated by the optional second argument, whose default \
value is ",
  Cell[BoxData[
      \(10\^\(-10\)\)]],
  ":"
}], "Text"],

Cell[BoxData[
    \(smoothing[a_, b_:  10^\((\(-10\))\)] := 
      Module[{c = a}, If[b \[Equal] \ 1, Return[c], If[Abs[c] < b, c = 0]]; 
        c]; Attributes[smoothing] = Listable;\)], "Input",
  CellLabel->"In[4]:=",
  Evaluatable->False],

Cell[BoxData[
    \(Attributes[smoothing]\)], "Input",
  CellLabel->"In[81]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Example: Chop and smoothing", "Subsubsection"],

Cell[TextData[{
  "There exists a similar ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " operation"
}], "Text"],

Cell[BoxData[
    \(?? Chop\)], "Input",
  CellLabel->"In[82]:="],

Cell["\<\
In spite of the fact that the attribute Listable is not reported, Chop seems \
to have this attribute. Therefore instead of smoothing[a, b] we could have \
used Chop[a, b]. The behavior of smoothing differs from that of Chop; for our \
purposes smoothing is sometimes more appropriate, see  the examples below.\
\>", "Text"],

Cell[BoxData[
    \(Chop[N[Exp[\(-40\)]]]\)], "Input",
  CellLabel->"In[83]:="],

Cell[BoxData[
    \(Chop[{N[Exp[\(-40\)]], \ N[Exp[\(-35\)]]}]\)], "Input",
  CellLabel->"In[84]:="],

Cell[BoxData[
    \(Chop[{N[Exp[\(-40\)]], \ Exp[\(-35\)]}]\)], "Input",
  CellLabel->"In[85]:="],

Cell[BoxData[
    \(Chop[{N[I*Exp[\(-40\)]], \ Exp[\(-35\)]}]\)], "Input",
  CellLabel->"In[86]:="],

Cell[BoxData[
    \(Chop[{N[Exp[\(-40\)]], I*Exp[\(-35\)]}]\)], "Input",
  CellLabel->"In[87]:="],

Cell["\<\
In the next example Chop produces an unexpected result; since t may be \
complex, the argument should have been returned:\
\>", "Text"],

Cell[BoxData[
    \(Chop[N[Pi*10^\((\(-19\))\)]*Cos[t]]\)], "Input",
  CellLabel->"In[88]:="],

Cell[BoxData[
    \(N[Pi*10^\((\(-19\))\)]*Cos[I*100. ]\)], "Input",
  CellLabel->"In[89]:="],

Cell["\<\
In difference to Chop smoothing evaluates the inserted expressions as \
expected:\
\>", "Text"],

Cell[BoxData[
    \(smoothing[{N[Exp[\(-40\)]], \ Exp[\(-35\)]}]\)], "Input",
  CellLabel->"In[90]:="],

Cell[BoxData[
    \(smoothing[{N[I*Exp[\(-40\)]], \ Exp[\(-35\)]}]\)], "Input",
  CellLabel->"In[91]:="],

Cell[BoxData[
    \(smoothing[{N[Exp[\(-40\)]], I*Exp[\(-35\)]}]\)], "Input",
  CellLabel->"In[92]:="],

Cell[BoxData[
    \(smoothing[N[Pi*10^\((\(-19\))\)]*Cos[t]]\)], "Input",
  CellLabel->"In[93]:="],

Cell[BoxData[
    \(smoothing[N[Pi*10^\((\(-19\))\)]]\)], "Input",
  CellLabel->"In[94]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["dotnorme", "Subsubsection"],

Cell["\<\
dotnorme normes a vector with respect to the Dot product. It can be applied \
also for symbolic calculations:\
\>", "Text"],

Cell[BoxData[
    \(\(?dotnorme\)\)], "Input",
  CellLabel->"In[113]:="],

Cell[BoxData[
    \(dotnorme[25*\(unit[4]\)[vec[s]]]\)], "Input",
  CellLabel->"In[110]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[111]:="],

Cell[BoxData[
    \(Simplify[% . %]\)], "Input",
  CellLabel->"In[112]:="],

Cell["Error message:", "Text"],

Cell[BoxData[
    \(dotnorme[nul[]]\)], "Input",
  CellLabel->"In[114]:="],

Cell["\<\
dot norme has been generalized to arbitrary, may be indefinite scalar \
products:\
\>", "Text"],

Cell[BoxData[
    \(\(?normalize\)\)], "Input",
  CellLabel->"In[115]:="],

Cell[BoxData[
    \(normalize[nul[]]\)], "Input",
  CellLabel->"In[116]:="]
}, Closed]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["2 Stereographic projection", "Section"],

Cell[CellGroupData[{

Cell["2.1  Definition of the general stereographic projection", "Subsection"],

Cell[TextData[{
  StyleBox["We want to visualize the geometry of the 3-sphere. By \
stereographic projection we go to the euclidean 3-space. Then we apply the \
3D-graphics tools of ",
    FontSize->14],
  StyleBox["Mathematica",
    FontSize->14,
    FontSlant->"Italic"],
  StyleBox[". Here is a definition of the general stereographic projection of \
the n-sphere \\ north pole onto the n-dimensional Euclidean space. We put the \
north pole at the point nul[n+1] + stb[n+1] and project onto the equatorial \
hyperplane; the north pole itself will be projected to Infinity.",
    FontSize->14]
}], "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman",
  FontSize->12,
  FontWeight->"Plain",
  FontSlant->"Plain",
  FontTracking->"Plain",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1],
  FontVariations->{"Underline"->False,
  "Outline"->False,
  "Shadow"->False}],

Cell[BoxData[
    \(\(?sterproj\)\)], "Input",
  CellLabel->"In[20]:="],

Cell["The definition contained in the package euvec.m is:", "Text"],

Cell[BoxData[
    \(sterproj[v_List] := 
      Module[{m = Length[v], l, w}, \[IndentingNewLine]l = 
          nul[m]; \[IndentingNewLine]w = v[\([m]\)]; \[IndentingNewLine]If[
          smoothing[w - 1] \[Equal] 0, 
          For[i = 1, i < m, \(i++\), l[\([i]\)] = Infinity], 
          l = Simplify[stb[m, m] + \((v - stb[m, m])\)/\((1 - w)\)], 
          l = Simplify[
              stb[m, m] + \((v - stb[m, m])\)/\((1 - 
                      w)\)]]; \[IndentingNewLine]l = 
          Take[l, m - 1]; \[IndentingNewLine]l]\)], "Input",
  Evaluatable->False],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Examples", "Subsubsection"]], "Subsubsection"],

Cell[BoxData[
    \(stb[1, 3]\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(sterproj[stb[1, 3]]\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(\(?unit\)\)], "Input",
  CellLabel->"In[23]:="],

Cell[BoxData[
    \(\(unit[2]\)[{s, t}]\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(sterproj[\(unit[2]\)[{s, t}]]\)], "Input",
  CellLabel->"In[25]:="],

Cell[BoxData[
    \(testvec[s_, t_]\  = \ AppendTo[%, \ 0]\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(ParametricPlot3D[
      testvec[s, t], {s, 0, 2  Pi}, \ {t, \(-\ Pi\)/2 + 0.0000001, 
        0}]\)], "Input",
  CellLabel->"In[33]:="],

Cell[BoxData[
    \(sterproj[\(unit[3]\)[{s, t, u}]]\)], "Input",
  CellLabel->"In[34]:="],

Cell["\<\
WARNING! Without any message sterproj gives generally incorrect results for \
incorrect, not normalized arguments:\
\>", "Text"],

Cell[BoxData[
    \(sterproj[stb[2]\  + stb[4]]\)], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(sterproj[\((stb[2]\  + stb[4])\)/Sqrt[2. ]]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(sterproj[nul[]]\)], "Input",
  CellLabel->"In[37]:="],

Cell["\<\
The image of the north pole stb[n,n]  is the infinite point, e.g. \
\>", "Text"],

Cell[BoxData[
    \(sterproj[stb[3, 3]]\)], "Input",
  CellLabel->"In[38]:="],

Cell["\<\
The other basis vectors are \"preserved\" in the equatorial hyperplane, \
e.g.\
\>", "Text"],

Cell[BoxData[
    \(stb[3, 7]\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(sterproj[stb[3, 7]]\)], "Input",
  CellLabel->"In[40]:="],

Cell["The general formula is: ", "Text"],

Cell[BoxData[
    \(vec[n]\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(sterproj[vec[n]]\)], "Input",
  CellLabel->"In[42]:="],

Cell["The elements of the equator remain fixed:", "Text"],

Cell[BoxData[
    \(vec[n]\  - \ n[4]*stb[4]\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(sterproj[\((vec[n]\  - \ n[4]*stb[4])\)]\)], "Input",
  CellLabel->"In[44]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["2.2 Example: Geodesics on the flat torus",
  FontFamily->"Helvetica"]], "Subsection",
  Evaluatable->False,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[TextData[StyleBox["We consider the torus as a direct product of two \
circles in orthogonal planes, producing a surface  in the unit 3-sphere in \
the euclidean 4-space. Obviously, its inner metric is flat. By  stereographic \
projection onto the euclidean 3-space we obtain the usual torus as a surface \
of revolution. Its parameter representation is (with 0 < c < 1):",
  FontFamily->"Times"]], "Text",
  FontFamily->"Times New Roman"],

Cell[TextData[StyleBox["tor[c_][s_, t_] := sterproj[{Cos[t], Sin[t], 0, 0}*c \
+\n{0, 0, Cos[s], Sin[s]}Sqrt[1 - c^2]]",
  FontFamily->"Courier"]], "Input",
  CellLabel->"In[11]:=",
  PageWidth->Infinity,
  InitializationCell->True,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(\(tor[c]\)[s, t]\)], "Input",
  CellLabel->"In[45]:=",
  FontSize->14],

Cell[BoxData[
    \(\(tor[0]\)[s, t]\)], "Input",
  CellLabel->"In[46]:=",
  FontSize->14],

Cell[BoxData[
    \(\(tor[1]\)[s, t]\)], "Input",
  CellLabel->"In[47]:=",
  FontSize->14],

Cell[BoxData[
    \(\(\(ParametricPlot3D[\(tor[0.8]\)[s, t], {s, 0, 2\ \[Pi]}, {t, 0, 
        2\ \[Pi]}, PlotRange \[Rule] All, 
      PlotPoints\  -> \ 50]\)\(\ \)\)\)], "Input",
  CellLabel->"In[50]:=",
  FontSize->14],

Cell["gr1= %", "Input",
  CellLabel->"In[51]:=",
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[TextData[{
  StyleBox["Th\[Epsilon] torus as submanifold of ",
    FontFamily->"Times"],
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)],
    FontFamily->"Times"],
  StyleBox[
  " is locally flat (local euclidean). Therefore its geodesics are the \
isogonal trajectories of its generating circles. Their images under the \
stereographic projection are:",
    FontFamily->"Times"]
}], "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"],

Cell[BoxData[
    StyleBox[\(\(kron[a_, \ c_]\)[t_] := 
        sterproj[{Cos[t], \ Sin[t], \ 0, \ 0}*
              c\  + \ \[IndentingNewLine]{0, \ 0, \ Cos[a\ t], \ Sin[a\ t]} 
              Sqrt[1\  - \ c^2]]\),
      FontFamily->"Courier"]], "Input",
  CellLabel->"In[12]:=",
  PageWidth->Infinity,
  InitializationCell->True,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(\(kron[a, \ c]\)[t]\)], "Input",
  CellLabel->"In[49]:=",
  FontSize->14],

Cell["\<\
Since stereographic projection is not an isometry, the curves kron[a,c][t] \
are in general not geodesics on the image torus in the 3-space.\
\>", "Text"],

Cell[BoxData[
    \(\(kron[ .5, \ 0.8]\)[t]\)], "Input",
  CellLabel->"In[52]:=",
  PageWidth->Infinity,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(ParametricPlot3D[\(kron[ .5,  .8]\)[t], {t, 0, 10\ \[Pi]}, 
      PlotRange \[Rule] All, PlotPoints \[Rule] 100]\)], "Input",
  CellLabel->"In[53]:=",
  PageWidth->Infinity,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell["gr2 = %", "Input",
  CellLabel->"In[54]:=",
  FontFamily->"Times New Roman",
  FontSize->14],

Cell["Show[{gr1, gr2}]", "Input",
  CellLabel->"In[55]:=",
  FontFamily->"Times New Roman",
  FontSize->14],

Cell["gr3 = %", "Input",
  CellLabel->"In[56]:=",
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[TextData[{
  "If you want to show or to animate this Graphics3D outside of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", use ThreeScript (see section 7.1), which has been imported with the \
Initialization."
}], "Text"],

Cell[BoxData[
    \(\(?ThreeScript\)\)], "Input",
  CellLabel->"In[12]:="],

Cell["ThreeScript[\"torexam.ts\", gr3]", "Input",
  CellLabel->"In[121]:=",
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[TextData[StyleBox[
"Unfortunately, ParametricPlot3D doesn't admit thickening the plot of the \
curve:",
  FontFamily->"Times"]], "Text",
  FontFamily->"Times New Roman"],

Cell["Options[ParametricPlot3D]", "Input",
  CellLabel->"In[122]:=",
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[TextData[StyleBox["But with wire frame geometry the geodesic is well \
visible looking at the ts-file torexam.ts under MathGL3D or MathLive.  \
Alternatively, link MathGL3d (see section 7.2) and apply the OpenGLViewer \
with MVShow3D[gr3, MVLineTubeSize -> 0.02]",
  FontFamily->"Times"]], "Text",
  FontFamily->"Times New Roman"],

Cell[BoxData[
    StyleBox[\(MVShow3D[gr3, \ MVLineTubeSize\  -> \ 0.02, 
        MVTextureMapType\  \[Rule] \ MVSphereMapping]\),
      FontFamily->"Times"]], "Input",
  CellLabel->"In[58]:="],

Cell["Now we show the torus by a family of Meridians", "Text"],

Cell[BoxData[
    \(Table[
      ParametricPlot3D[\(tor[0.8]\)[s, t], {s, 0, 2\ \[Pi]}], {t, 0, 
        2\ \[Pi],  .1}]\)], "Input",
  CellLabel->"In[59]:=",
  FontSize->14],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[60]:="],

Cell[BoxData[
    \(tnet\  = \ %\)], "Input",
  CellLabel->"In[61]:="],

Cell[BoxData[
    \(Show[{tnet, gr2}]\)], "Input",
  CellLabel->"In[62]:="],

Cell[BoxData[
    \(gr4\  = %\)], "Input",
  CellLabel->"In[63]:="],

Cell[BoxData[
    \(MVShow3D[gr4]\)], "Input",
  CellLabel->"In[64]:="],

Cell["\<\
Here one can see that the intersection angle along the curve is constant. \
(Remark: Taking Appearance -> WireFrame needs not to represent the images of \
the generating circles, as gr4 does.) \
\>", "Text"],

Cell[CellGroupData[{

Cell["PROOF of this statement.", "Subsubsection"],

Cell["The tangential vector of the curve is", "Text"],

Cell[BoxData[
    \(\(tangkron[a_, c_]\)[t_] := 
      D[\(kron[a, \ c]\)[t1], t1] /. t1 \[Rule] t\)], "Input",
  CellLabel->"In[13]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(tangkron[a, c]\)[t]\)], "Input",
  CellLabel->"In[87]:="],

Cell["\<\
The tangential vectors of the parameter lines of the torus are\
\>", "Text"],

Cell[BoxData[
    \(\(dttor[c_]\)[s_, t_] := \ 
      D[\(tor[c]\)[s, t1], t1] /. t1 \[Rule] t\)], "Input",
  CellLabel->"In[14]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(dttor[c]\)[s, t]\)], "Input",
  CellLabel->"In[94]:="],

Cell[BoxData[
    \(\(dstor[c_]\)[s_, t_] := \ 
      D[\(tor[c]\)[s1, t], s1] /. s1 \[Rule] s\)], "Input",
  CellLabel->"In[15]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(dstor[c]\)[s, t]\)], "Input",
  CellLabel->"In[96]:="],

Cell[BoxData[
    \(\(kron[a, c]\)[u]\)], "Input",
  CellLabel->"In[97]:="],

Cell[BoxData[
    \(\(tor[c]\)[s, t]\)], "Input",
  CellLabel->"In[98]:="],

Cell["\<\
The parameter representation of the curve kron[a,c][u] on the torus is s=a u, \
t = u:\
\>", "Text"],

Cell[BoxData[
    \(\(kron[a, c]\)[u] \[Equal] \(tor[c]\)[a*u, u]\)], "Input",
  CellLabel->"In[86]:="],

Cell["\<\
We show that the cosinus of the angle in which curve kron[a,c][u] intersects \
the parameter line is constant along the curve:\
\>", "Text"],

Cell[BoxData[
    \(testf[t_] := \ 
      dotnorme[\(tangkron[ .5,  .8]\)[t]] . 
        dotnorme[\(dstor[ .8]\)[\  .5*t, t]]\)], "Input",
  CellLabel->"In[87]:="],

Cell["Only numerical disturbations appear:", "Text"],

Cell[BoxData[
    \(Plot[testf[t], \ {t, 0, 2*Pi}]\)], "Input",
  CellLabel->"In[88]:="],

Cell[BoxData[
    \(Plot[Chop[testf[t] - testf[0]]\  + \ 1, \ {t, 0, 2*Pi}]\)], "Input",
  CellLabel->"In[89]:="],

Cell[BoxData[
    \(testf[0]\)], "Input",
  CellLabel->"In[90]:="],

Cell[BoxData[
    \(Clear[testf]\)], "Input",
  CellLabel->"In[91]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Test of the angle invariance", "Subsubsection"],

Cell[BoxData[
    RowBox[{\(\(flattor[c_]\)[s_, \ t_]\), " ", ":=", 
      StyleBox[\({Cos[t], \ Sin[t], \ 0, \ 0}*
            c\  + \n{0, \ 0, \ Cos[s], \ Sin[s]} Sqrt[1\  - \ c^2]\),
        FontFamily->"Courier"]}]], "Input",
  CellLabel->"In[66]:="],

Cell[BoxData[
    \(\(dsflatt[c_]\)[s_, t_]\  := \ 
      D[\(flattor[c]\)[s1, t], s1] /. s1 \[Rule] s\)], "Input",
  CellLabel->"In[69]:="],

Cell[BoxData[
    \(\(dsflatt[c]\)[s, t]\)], "Input",
  CellLabel->"In[70]:="],

Cell[BoxData[
    \(\(dtflatt[c_]\)[s_, t_] := \ 
      D[\(flattor[c]\)[s, t1], t1] /. t1 \[Rule] t\)], "Input",
  CellLabel->"In[71]:="],

Cell[BoxData[
    \(\(dtflatt[c]\)[s, t]\)], "Input",
  CellLabel->"In[72]:="],

Cell["\<\
The tangential basis of the euclidean torus has been defined with the \
initialization:\
\>", "Text"],

Cell[BoxData[
    \(\(dstor[c]\)[s, t]\)], "Input",
  CellLabel->"In[76]:="],

Cell[BoxData[
    \(\(dttor[c]\)[s, t]\)], "Input",
  CellLabel->"In[77]:="],

Cell[BoxData[
    \(\(dstor[c]\)[s, t] . \(dstor[c]\)[s, t]\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(Simplify[
      Together[%], \ \(\(s > 0 && t > 0\) && c > 0\) && c < 1]\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(\(g11[c_]\)[s_, 
        t_]\  := \((\((\(-1\) + c\^2)\)\ \((\(-3\) + 
                c\^2 - \((\(-1\) + c\^2)\)\ Cos[2\ s] + 
                4\ \@\(1 - c\^2\)\ Sin[s])\))\)/\((2\ \((1 - 
                4\ \@\(1 - c\^2\)\ Sin[s] - 
                6\ \((\(-1\) + c\^2)\)\ Sin[s]\^2 - 
                4\ \((1 - c\^2)\)\^\(3/2\)\ Sin[s]\^3 + \((\(-1\) + \
c\^2)\)\^2\ Sin[s]\^4)\))\)\)], "Input",
  CellLabel->"In[81]:="],

Cell[BoxData[
    \(\(dstor[c]\)[s, t] . \(dttor[c]\)[s, t]\)], "Input",
  CellLabel->"In[82]:="],

Cell[BoxData[
    \(\(dttor[c]\)[s, t] . \(dttor[c]\)[s, t]\)], "Input",
  CellLabel->"In[83]:="],

Cell[BoxData[
    \(\(g22[c_]\)[s_, 
        t_]\  := \ \(c\^2\ Cos[t]\^2\)\/\((1 - \@\(1 - c\^2\)\ Sin[s])\)\^2 + \
\(c\^2\ Sin[t]\^2\)\/\((1 - \@\(1 - c\^2\)\ Sin[s])\)\^2\)], "Input",
  CellLabel->"In[84]:="],

Cell["\<\
Dividing g11,g22 by the norm squares (1-c^2), resp. c^2 of the tangent \
vectors of flattor[c][s,t], and substracting:\
\>", "Text"],

Cell[BoxData[
    \(Simplify[\(g11[c]\)[s, t]/\((1 - c^2)\)\  - \ \(g22[c]\)[s, 
            t]/\((c^2)\)]\)], "Input",
  CellLabel->"In[85]:="],

Cell["\<\
and this proves the (well known) conformality of the map sterproj in this \
case.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["The isogonal trajectories are not geodesics", "Subsubsection"],

Cell["\<\
The last  result confirms the fact that the stereographical projection is a \
conformal map.  We show that the isogonal trajectories of the parameter \
lines, in general, are not  geodesics. To this aim we calulate the curvature \
vector of the curve kron[a,c][t]. The unit tangential vector is\
\>", "Text"],

Cell[BoxData[
    \(\(tkron[a_, c_]\)[t_]\  := \ dotnorme[\(tangkron[a, c]\)[t]]\)], "Input",\

  CellLabel->"In[92]:="],

Cell["\<\
The binormalvector of a regular curve cx[t] of the euclidean 3-space with an \
arbitrary parameter t is given by (see A. Gray [1]): \
\>", "Text"],

Cell[BoxData[
    \(\(binormal[cx_]\)[t_]\  := \ 
      Simplify[Cross[\[IndentingNewLine]D[cx[tt], tt], \ D[cx[tt], {tt, 2}]]]/
          Simplify[
              Factor[Cross[D[cx[tt], tt], D[cx[tt], {tt, 2}]] . 
                  Cross[D[cx[tt], tt], \ D[cx[tt], {tt, 2}]]]]^\((1/2)\)\  /. 
        tt \[Rule] t\)], "Input",
  CellLabel->"In[93]:="],

Cell["Now we consider the isogonal trajectory as defined above", "Text"],

Cell[BoxData[
    \(cx[t_]\  := \ \(kron[0.5, 0.8]\)[t]\)], "Input",
  CellLabel->"In[94]:="],

Cell[BoxData[
    \(\(binormal[cx]\)[t]\)], "Input",
  CellLabel->"In[95]:="],

Cell[BoxData[
    \(bincx[t_]\  = \ Simplify[Chop[%], \ t > 0]\)], "Input",
  CellLabel->"In[96]:="],

Cell["\<\
If kron[a,c][t] would be a geodesic, the tangential component of this vector \
should vanish; but the following calculation shows that this is not the case.\
\
\>", "Text"],

Cell[BoxData[
    \(ftest[t_] := \(dstor[ .8]\)[ .5*t, t] . bincx[t]\)], "Input",
  CellLabel->"In[97]:="],

Cell[BoxData[
    \(Plot[ftest[t], \ {t, 0, 2*Pi}]\)], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \(Clear[ftest]\)], "Input",
  CellLabel->"In[99]:="],

Cell[BoxData[
    \(ftest[t_] := \(dttor[ .8]\)[ .5*t, t] . bincx[t]\)], "Input",
  CellLabel->"In[100]:="],

Cell[BoxData[
    \(Plot[ftest[t], \ {t, 0, 2*Pi}]\)], "Input",
  CellLabel->"In[101]:="],

Cell["\<\
This shows that the isogonal trajectory cx is not a geodesic.\
\>", "Text"],

Cell[BoxData[
    \(Clear[ftest]\)], "Input",
  CellLabel->"In[102]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.3 The inverse stereographic projection", "Subsection",
  FontSize->14],

Cell["\<\
The following function maps an arbitrary point of the equatorial hyperplane \
to its preimage under the stereographic projection:\
\>", "Text"],

Cell[BoxData[
    \(\(?invsterproj\)\)], "Input",
  CellLabel->"In[103]:="],

Cell[BoxData[
    \(invsterproj[y_List] := \ 
      Module[{n = Length[y] + 1, x = Append[y, 0], \ 
          u = y . y}, \n\t\tx*2/\((1 + u)\)\  + 
          stb[n, n]*\((u - 1)\)/\((u + 1)\)]\)], "Input",
  CellLabel->"In[9]:=",
  Evaluatable->False],

Cell[BoxData[
    \(invsterproj[vec[y, 3]]\)], "Input",
  CellLabel->"In[104]:="],

Cell[BoxData[
    \(Simplify[% . %]\)], "Input",
  CellLabel->"In[105]:="],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["Simple tests", "Text"],

Cell[BoxData[
    \(invsterproj[nul[4]]\)], "Input",
  CellLabel->"In[106]:="],

Cell["Lines are mapped onto circles", "Text"],

Cell[BoxData[
    \(invsterproj[{1, t}]\)], "Input",
  CellLabel->"In[107]:="],

Cell[BoxData[
    \(ParametricPlot3D[{2\/\(2 + t\^2\), \(2\ t\)\/\(2 + t\^2\), 
        t\^2\/\(2 + t\^2\)}, \ {t, \ \(-100\), \ 100}, 
      PlotRange\  \[Rule] \ All, PlotPoints\  \[Rule] \ 1000]\)], "Input",
  CellLabel->"In[108]:="],

Cell[BoxData[
    \(invsterproj[stb[1] Cos[t] + stb[2] Sin[t] + stb[4]]\)], "Input",
  CellLabel->"In[109]:="],

Cell[BoxData[
    \(sterproj[
      invsterproj[stb[1] Cos[t] + stb[2] Sin[t] + stb[4]]]\)], "Input",
  CellLabel->"In[110]:="],

Cell[BoxData[
    \(sterproj[invsterproj[{a, b, c, d}]]\)], "Input",
  CellLabel->"In[111]:="],

Cell[BoxData[
    \(invsterproj[sterproj[%]]\)], "Input",
  CellLabel->"In[112]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[113]:="],

Cell[BoxData[
    \(Simplify[% /. a\^2 + b\^2 + c\^2\  \[Rule] \ 1 - d\^2]\)], "Input",
  CellLabel->"In[114]:="],

Cell["We remember the warning", "Text"],

Cell[BoxData[
    \(\(?sterproj\)\)], "Input",
  CellLabel->"In[115]:="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3 Euclidean representations of spheres", "Section"],

Cell[CellGroupData[{

Cell[TextData[{
  "3.1 A parameter representation of the unit n-sphere ",
  Cell[BoxData[
      \(TraditionalForm\`S\^n\)]]
}], "Subsection"],

Cell["\<\
We define parameter representations for the unit vectors in subspaces of the \
d-dimensional euclidean space:\
\>", "Text"],

Cell[BoxData[
    \(unit1[s_, \ d_:  dim]\  := \ 
      stb[1, d]*Cos[s] + \ stb[2, d]*Sin[s]\)], "Input",
  CellLabel->"In[16]:=",
  InitializationCell->True],

Cell[BoxData[
    \(unit2[s_, \ t_, \ d_:  dim]\  := \ 
      unit1[s, \ d]*Cos[t]\  + stb[3, d]*\ Sin[t]\)], "Input",
  CellLabel->"In[17]:=",
  InitializationCell->True],

Cell[BoxData[
    \(unit3[s_, t_, u_, \ d_:  dim]\  := \ 
      unit2[s, t, \ d]*Cos[u]\  + stb[4, d]*\ Sin[u]\)], "Input",
  CellLabel->"In[18]:=",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[TextData[{
  "3.1.1 A parameter representation of ",
  Cell[BoxData[
      \(TraditionalForm\`S\^n\)]],
  ", recursive Definition"
}], "Subsubsection"],

Cell["\<\
The following recursive definition, contained in euvec.m,  yields the usual \
parameter representation of the unit n-sphere as a hypersurface in the \
(n+1)-dimensional euclidean space. For n =1,2,3 it almost coincides with the \
definitions above. The parameters appear as the first n elements of the \
List-argument.\
\>", "Text"],

Cell[BoxData[
    \(\(unit[n_]\)[s_List]\  := \ 
      Module[{un, 
            i}, \n\t\tun[1] = {Cos[s[\([1]\)]], \ 
              Sin[s[\([1]\)]]}; \n\t\t\tFor[i = 2, \ 
            i <= \ n, \ \(i++\), \n\t\tun[i] = 
              AppendTo[un[i - 1], 0]*Cos[s[\([i]\)]] + 
                stb[1 + i, i + 1]*Sin[s[\([i]\)]]]; \n\t\t\t\tReturn[un[n]]] /; 
        Length[s] >= n\)], "Input",
  CellLabel->"In[20]:=",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    \(\(unit[2]\)[{s, t}]\)], "Input",
  CellLabel->"In[48]:="],

Cell["\<\
unit1,uni2,unit3 yield vectors of the fixed dimension d, completed with \
zeros, if necessary:\
\>", "Text"],

Cell[BoxData[
    \(unit2[s, t]\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(\(unit[2]\)[{s, t}]\)], "Input",
  CellLabel->"In[50]:="],

Cell[BoxData[
    \(TrueQ[unit2[s, t, 3]\  == \(unit[2]\)[{s, t}]]\)], "Input",
  CellLabel->"In[51]:="],

Cell[BoxData[
    \(\(unit[3]\)[{s, t, u}]\)], "Input",
  CellLabel->"In[52]:="],

Cell[BoxData[
    \(unit3[s, t, u]\)], "Input",
  CellLabel->"In[53]:="],

Cell[BoxData[
    \(\(unit[4]\)[vec[s]]\)], "Input",
  CellLabel->"In[54]:="],

Cell["\<\
If Length[s_List] > n only the first n components are considered:\
\>", "Text"],

Cell[BoxData[
    \(vec[s]\)], "Input",
  CellLabel->"In[55]:="],

Cell[BoxData[
    \(\(unit[2]\)[vec[s]]\)], "Input",
  CellLabel->"In[56]:="],

Cell["If Length[s_List] < n the evaluation failes:", "Text"],

Cell[BoxData[
    \(\(unit[5]\)[vec[s]]\)], "Input",
  CellLabel->"In[57]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
3.2 The standard parameter representation of spheres in the euclidean 3-space\
\
\>", "Subsection"],

Cell["\<\
The general sphere with radius r and centre  m has the parameter \
representation with geographical coordinates u, v, defined in euvec.m:\
\>", "Text"],

Cell[BoxData[
    \(\(?sphere\)\)], "Input",
  CellLabel->"In[25]:="],

Cell["\<\
sphere[m_List, r_][u_, v_] := {m[[1]] + r*Cos[u]*Cos[v], 
    m[[2]] + r*Cos[v]*Sin[u], m[[3]] + r*Sin[v]} /; Length[m] == 3\
\>", "Input",
  CellLabel->"In[27]:=",
  Evaluatable->False],

Cell["\<\
 Of course, the 2-spheres in the 3-space can directly be plotted; the sphere \
with centre m and radius r (optionally a part of it)  is\
\>", "Text"],

Cell[BoxData[
    \(\(?sphereplot3D\)\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(sphereplot3D[m_List, \ r_, \ u0_:  0, \ u1_:  2\ Pi, v0_: - Pi/2, 
        v1_:  Pi/2, \ opt___]\  := \ 
      ParametricPlot3D[\(sphere[m, r]\)[u, v], {u, u0, u1}, {v, v0, v1}, 
        opt]\)], "Input",
  CellLabel->"In[28]:=",
  Evaluatable->False],

Cell["\<\
Using unit[n][s], one easily generalizes the definition of the parameter \
representation to the case of arbitrary n-spheres:\
\>", "Text"],

Cell[BoxData[
    \(\(hypersphere[m_List, \ r_]\)[s_List]\  := \ 
      Module[{di\  = \ Length[m] - 1}, \n\t\tm\  + \ r*\(unit[di]\)[s]] /; 
        Length[s] \[Equal] Length[m] - 1\)], "Input",
  CellLabel->"In[19]:=",
  InitializationCell->True],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    \(\(sphere[nul[3], 1]\)[u, v]\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(sphereplot3D[nul[3], 1]\)], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \(sphereplot3D[{0, 0, 5}, \ 2]\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(sphereplot3D[{0, 0, 5}, \ 2, 0, \ Pi/2]\)], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(sphereplot3D[{0, 0, 5}, \ 2, 0, Pi/2, 0, Pi/2, \ Boxed\  -> \ False, 
      Axes\  -> \ False, \ PlotPoints\  \[Rule] \ 30]\)], "Input",
  CellLabel->"In[30]:="],

Cell[BoxData[
    \(\(hypersphere[{m1, m2}, r]\)[{s}]\)], "Input",
  CellLabel->"In[31]:="],

Cell[BoxData[
    \(\(hypersphere[{m1, m2, m3}, r]\)[{u, v}]\)], "Input",
  CellLabel->"In[32]:="],

Cell[BoxData[
    \(\(hypersphere[stb[1, 4], \ r]\)[{u, v, w}]\)], "Input",
  CellLabel->"In[33]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "3.3 Parameter representations of spheres in the 3-sphere ",
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)]]
}], "Subsection"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["orthobase",
  FontWeight->"Bold",
  FontSlant->"Plain",
  FontColor->RGBColor[0, 0, 1],
  FontVariations->{"CompatibilityType"->0}]], "Subsubsection"],

Cell[TextData[{
  "The parameter representation unit3 of ",
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)]],
  " is a vector function with values  in the 4-space; geometric objects in ",
  Cell[BoxData[
      \(TraditionalForm\`S\^\(\(\ \)\(3\)\)\)]],
  "can be plotted only after stereographic projection onto the 3-space. Each \
sphere (= hypersphere of ",
  Cell[BoxData[
      \(TraditionalForm\`S\^\(\(\ \)\(3\)\)\)]],
  ") is the intersection of ",
  Cell[BoxData[
      \(TraditionalForm\`S\^\(\(\ \)\(3\)\)\)]],
  " with a hyperplane"
}], "Text"],

Cell[BoxData[
    \(vec[n] . vec[x]\  == \ p\)], "Input",
  CellLabel->"In[34]:="],

Cell[TextData[{
  "with  normal vector  vec[n] \[Epsilon] ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(S\^\(\(\ \)\(3\)\)\)\)\)]],
  ", and distance p, -1 < p < 1, from the center. The ambiguity in the signs \
of vec[n] and p can be used to distinguish the orientations of the spheres. \
The manifold of spheres in ",
  Cell[BoxData[
      \(TraditionalForm\`S\^\(\(\ \)\(3\)\)\)]],
  " is 4-dimensional: three parameters of vec[n], and p ((n+1-dimensional in \
case of ",
  Cell[BoxData[
      \(TraditionalForm\`S\^\(\(\ \)\(n\)\)\)]],
  "), or centre and radius. To get a parameter representation of the sphere \
corresponding to vec[n] and p we need three vectors orthogonal to vec[n]. \
Applying ",
  "orthobase",
  " we shall get them."
}], "Text"],

Cell[BoxData[
    \(\(?orthobase\)\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(orthobase[vn_]\  := \ 
      Module[{d\  = \ Length[vn], \ tt\ , to, \ 
          v}, \ \[IndentingNewLine]tt\  = \ 
          IdentityMatrix[d]; \[IndentingNewLine]PrependTo[tt, \ 
          vn]; \[IndentingNewLine]to\  = \ 
          esorthonorm[tt]; \[IndentingNewLine]v\  = \ 
          to[\([1]\)]; \[IndentingNewLine]tt = 
          Take[to, \ \(-\((d - 1)\)\)]; \[IndentingNewLine]AppendTo[tt, \ 
          v]; \[IndentingNewLine]Return[tt]]\)], "Input",
  CellLabel->"In[13]:=",
  Evaluatable->False],

Cell["\<\
The Erhard Schmidt orthogonalization procedure esorthonorm is defined in the \
package euvec.m, see also section 4.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Simple cases", "Subsubsection"],

Cell[BoxData[
    \(orthobase[stb[1]]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(Det[%]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(orthobase[stb[4]]\)], "Input",
  CellLabel->"In[38]:="],

Cell[BoxData[
    \(Det[%]\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(orthobase[stb[2]*a\  + stb[4]*b]\)], "Input",
  CellLabel->"In[55]:="],

Cell[BoxData[
    \(Simplify[%, \ a > 0\  && b > 0]\)], "Input",
  CellLabel->"In[56]:="],

Cell[BoxData[
    \(Simplify[Det[%]]\)], "Input",
  CellLabel->"In[57]:="],

Cell[BoxData[
    \(orthobase[nul[]]\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(orthobase[stb[2, 7]]\)], "Input",
  CellLabel->"In[45]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Symbolic calculation", "Subsubsection"],

Cell[BoxData[
    \(vec[v]\)], "Input",
  CellLabel->"In[58]:="],

Cell[BoxData[
    \(of\  = orthobase[vec[v]]\)], "Input",
  CellLabel->"In[25]:="],

Cell[BoxData[
    \(Simplify[%, \(\(\(v[2]\^2 + v[3]\^2 + v[4]\^2\  == 1 - v[1]\^2 && 
              v[1] > 0\) && v[2] > 0\) && v[3] > 0\) && v[4] > 0]\)], "Input",\

  CellLabel->"In[22]:="],

Cell["Control of the scalar products:", "Text"],

Cell[BoxData[
    \(Simplify[Table[of[\([i]\)] . of[\([j]\)], {i, 4}, {j, 4}]]\)], "Input",
  CellLabel->"In[25]:="],

Cell["or", "Text"],

Cell[BoxData[
    \(Simplify[of . Transpose[of]]\)], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \(Det[of]\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(Simplify[%, v[1] > 0 && v[2] > 0 && v[3] > 0 && v[4] > 0]\)], "Input",
  CellLabel->"In[29]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Direct Definition", "Subsubsection"],

Cell["\<\
Using the formula for orthobase[vec[n]] as definition one could try defining \
orthobase directly, as a function of  vec[n].\
\>", "Text"],

Cell[BoxData[
    \(orthobasis4[wx_List] := 
      Module[{c = wx . wx, \ vx}, \[IndentingNewLine]If[
          Length[wx]\  \[NotEqual] \ 4, \ 
          Print[wx, \ "\< Incorrect argument!\>"]; 
          Return[]]; \[IndentingNewLine]If[c \[Equal] 0, 
          Return[IdentityMatrix[4]]]; \[IndentingNewLine]If[c \[Equal] 1, 
          vx\  = \ wx, vx\  = \ wx/Sqrt[c], 
          vx\  = \ wx/Sqrt[c]]; \[IndentingNewLine]{{Sqrt[
              vx[\([2]\)]^2 + vx[\([3]\)]^2 + 
                vx[\([4]\)]^2], \(-\((\((vx[\([1]\)]*vx[\([2]\)])\)/
                  Sqrt[vx[\([2]\)]^2 + vx[\([3]\)]^2 + 
                      vx[\([4]\)]^2])\)\), \(-\((\((vx[\([1]\)]*
                      vx[\([3]\)])\)/
                  Sqrt[vx[\([2]\)]^2 + vx[\([3]\)]^2 + 
                      vx[\([4]\)]^2])\)\), \(-\((\((vx[\([1]\)]*
                      vx[\([4]\)])\)/
                  Sqrt[vx[\([2]\)]^2 + vx[\([3]\)]^2 + 
                      vx[\([4]\)]^2])\)\)}, {0, 
            Sqrt[\((vx[\([3]\)]^2 + vx[\([4]\)]^2)\)/\((vx[\([2]\)]^2 + 
                    vx[\([3]\)]^2 + 
                    vx[\([4]\)]^2)\)], \(-\((\((vx[\([2]\)]*vx[\([3]\)]*
                      Sqrt[\((vx[\([3]\)]^2 + 
                              vx[\([4]\)]^2)\)/\((vx[\([2]\)]^2 + 
                              vx[\([3]\)]^2 + 
                              vx[\([4]\)]^2)\)])\)/\((vx[\([3]\)]^2 + 
                      vx[\([4]\)]^2)\))\)\), \(-\((\((vx[\([2]\)]*vx[\([4]\)]*
                      Sqrt[\((vx[\([3]\)]^2 + 
                              vx[\([4]\)]^2)\)/\((vx[\([2]\)]^2 + 
                              vx[\([3]\)]^2 + 
                              vx[\([4]\)]^2)\)])\)/\((vx[\([3]\)]^2 + 
                      vx[\([4]\)]^2)\))\)\)}, {0, 0, 
            Sqrt[vx[\([4]\)]^2/\((vx[\([3]\)]^2 + 
                    vx[\([4]\)]^2)\)], \(-\((\((vx[\([3]\)]*
                      Sqrt[vx[\([4]\)]^2/\((vx[\([3]\)]^2 + 
                              vx[\([4]\)]^2)\)])\)/
                  vx[\([4]\)])\)\)}, {vx[\([1]\)], vx[\([2]\)], vx[\([3]\)], 
            vx[\([4]\)]}}]\)], "Input",
  CellLabel->"In[27]:="],

Cell["\<\
WARNING! Doing so, for vectors with vx[[4]] == 0 we obtain indeterminate \
expressions:\
\>", "Text"],

Cell[BoxData[
    \(orthobasis4[stb[1] + \ stb[2]]\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(orthobasis4[stb[1] + \ stb[2] + \ stb[3]]\)], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(orthobasis4[stb[1] + \ stb[2] + \ stb[4]]\)], "Input",
  CellLabel->"In[30]:="],

Cell[BoxData[
    \(orthobasis4[{a, b, c, 0}]\)], "Input",
  CellLabel->"In[31]:="],

Cell[BoxData[
    \(Clear[a, b, c]\)], "Input",
  CellLabel->"In[32]:="],

Cell[BoxData[
    \(orthobasis4[{a, b, c, x}]\)], "Input",
  CellLabel->"In[33]:="],

Cell["We can determine the basis as  a limit:", "Text"],

Cell[BoxData[
    \(Limit[%, \ x\  \[Rule] \ 0]\)], "Input",
  CellLabel->"In[34]:="],

Cell["Thus, we try to refine the definition as follows:", "Text"],

Cell[BoxData[
    \(Clear[orthobasis4]\)], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(orthobasis4[wx_List] := 
      Module[{c = wx . wx, \ vx, \ ex\  = \ 0, \ ob, \ main, sing, 
          t}, \[IndentingNewLine]If[Length[wx]\  \[NotEqual] \ 4, \ 
          Print[wx, \ "\< Incorrect argument!\>"]; 
          Return[]]; \[IndentingNewLine]If[c \[Equal] 0, 
          Return[IdentityMatrix[4]]]; \[IndentingNewLine]If[c \[Equal] 1, 
          vx\  = \ wx, vx\  = \ wx/Sqrt[c], 
          vx\  = \ wx/Sqrt[c]]; \[IndentingNewLine]If[
          vx[\([4]\)]\  \[Equal] \ 0, \ ex\  = \ 1; \ 
          Goto[sing]\ ]; \[IndentingNewLine]Label[
          main]; \[IndentingNewLine]ob\  = \[IndentingNewLine]{{Sqrt[
                vx[\([2]\)]^2 + vx[\([3]\)]^2 + 
                  vx[\([4]\)]^2], \(-\((\((vx[\([1]\)]*vx[\([2]\)])\)/
                    Sqrt[vx[\([2]\)]^2 + vx[\([3]\)]^2 + 
                        vx[\([4]\)]^2])\)\), \(-\((\((vx[\([1]\)]*
                        vx[\([3]\)])\)/
                    Sqrt[vx[\([2]\)]^2 + vx[\([3]\)]^2 + 
                        vx[\([4]\)]^2])\)\), \(-\((\((vx[\([1]\)]*
                        vx[\([4]\)])\)/
                    Sqrt[vx[\([2]\)]^2 + vx[\([3]\)]^2 + 
                        vx[\([4]\)]^2])\)\)}, {0, 
              Sqrt[\((vx[\([3]\)]^2 + vx[\([4]\)]^2)\)/\((vx[\([2]\)]^2 + 
                      vx[\([3]\)]^2 + 
                      vx[\([4]\)]^2)\)], \(-\((\((vx[\([2]\)]*vx[\([3]\)]*
                        Sqrt[\((vx[\([3]\)]^2 + 
                                vx[\([4]\)]^2)\)/\((vx[\([2]\)]^2 + 
                                vx[\([3]\)]^2 + 
                                vx[\([4]\)]^2)\)])\)/\((vx[\([3]\)]^2 + 
                        vx[\([4]\)]^2)\))\)\), \(-\((\((vx[\([2]\)]*
                        vx[\([4]\)]*
                        Sqrt[\((vx[\([3]\)]^2 + 
                                vx[\([4]\)]^2)\)/\((vx[\([2]\)]^2 + 
                                vx[\([3]\)]^2 + 
                                vx[\([4]\)]^2)\)])\)/\((vx[\([3]\)]^2 + 
                        vx[\([4]\)]^2)\))\)\)}, {0, 0, 
              Sqrt[vx[\([4]\)]^2/\((vx[\([3]\)]^2 + 
                      vx[\([4]\)]^2)\)], \(-\((\((vx[\([3]\)]*
                        Sqrt[vx[\([4]\)]^2/\((vx[\([3]\)]^2 + 
                                vx[\([4]\)]^2)\)])\)/
                    vx[\([4]\)])\)\)}, {vx[\([1]\)], vx[\([2]\)], 
              vx[\([3]\)], vx[\([4]\)]}}; \[IndentingNewLine]If[
          ex \[Equal] 0, \ Return[ob]]; \[IndentingNewLine]If[
          ex \[Equal] 1, \ 
          Return[Limit[ob, \ t\  \[Rule] \ 0]]]; \[IndentingNewLine]Label[
          sing]; \[IndentingNewLine]vx\  = 
          vx /. vx[\([4]\)] -> \ t; \[IndentingNewLine]Goto[
          main];\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(orthobasis4[stb[1]]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(Table[%[\([i]\)] . %[\([j]\)], \ {i, 4}, {j, 4}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[38]:="],

Cell["This is not the simplest solution:", "Text"],

Cell[BoxData[
    \(orthobase[stb[1]]\)], "Input",
  CellLabel->"In[39]:="],

Cell["\<\
With the new definition we do not get indeterminate expressions:\
\>", "Text"],

Cell[BoxData[
    \(orthobasis4[stb[1] + \ stb[2]]\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(orthobasis4[stb[1] + \ stb[2] + \ stb[3]]\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(orthobasis4[stb[1] + \ stb[2] + \ stb[4]]\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(orthobasis4[{a, b, c, 0}]\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(Clear[a, b, c]\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(orthobasis4[{a, b, c, x}]\)], "Input",
  CellLabel->"In[45]:="],

Cell["Timings:", "Text"],

Cell[BoxData[
    \(Timing[orthobase[{a, b, c, 0}]]\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(Timing[orthobasis4[{a, b, c, 0}]]\)], "Input",
  CellLabel->"In[47]:="],

Cell["Very often orthobasis4 is much faster than orthobase:", "Text"],

Cell[BoxData[
    \(Timing[orthobase[vec[vv]]]\)], "Input",
  CellLabel->"In[48]:="],

Cell[BoxData[
    \(Timing[orthobasis4[vec[vv]]]\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(Simplify[Together[%]]\)], "Input",
  CellLabel->"In[50]:="],

Cell[BoxData[
    \(Simplify[
      Table[\(%[\([2]\)]\)[\([i]\)] . \(%[\([2]\)]\)[\([j]\)], {i, 4}, {j, 
          4}]]\)], "Input",
  CellLabel->"In[51]:="],

Cell[BoxData[
    \(Remove[of]\)], "Input",
  CellLabel->"In[52]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Sphere given by vec[n], p", "Subsubsection"],

Cell["\<\
Let vec[n], p be as at the beginning of  3.3. Obviously, vec[n]*p is the \
center and Sqrt[1-p^2] is the radius of the sphere in the hyperplane \
vec[n].x=p.  A parameter representation of this sphere as a surface in the \
4-space is given by subsphere[n,p][v,w].\
\>", "Text"],

Cell[BoxData[
    \(\(?subsphere\)\)], "Input",
  CellLabel->"In[53]:="],

Cell[BoxData[
    \(\(subsphere[stb[1]*Cos[t]\  + stb[2]*Sin[t], \ 1/6]\)[v, w]\)], "Input",\

  CellLabel->"In[57]:="],

Cell[BoxData[
    \(\((% - \((stb[1]*Cos[t]\  + stb[2]*Sin[t])\)/
            6)\) . \((% - \((stb[1]*Cos[t]\  + stb[2]*Sin[t])\)/
            6)\)\)], "Input",
  CellLabel->"In[58]:="],

Cell[BoxData[
    \(Simplify[Together[%]]\)], "Input",
  CellLabel->"In[59]:="],

Cell["Error message :", "Text"],

Cell[BoxData[
    \(\(subsphere[stb[5, 7], 0]\)[v, w]\)], "Input",
  CellLabel->"In[60]:="],

Cell["\<\
The next command gives the unit sphere with center in the origin in the \
hyperplane spanned by stb[2], stb[3], stb[4]; the same gives the next \
command.\
\>", "Text"],

Cell[BoxData[
    \(\(subsphere[nul[], 0]\)[v, w]\)], "Input",
  CellLabel->"In[61]:="],

Cell[BoxData[
    \(\(subsphere[stb[1], 0]\)[v, w]\)], "Input",
  CellLabel->"In[62]:="],

Cell[BoxData[
    \(\(subsphere[stb[1], p]\)[v, w]\)], "Input",
  CellLabel->"In[63]:="],

Cell[BoxData[
    \(Simplify[\((% - stb[1]*p)\) . \((% - stb[1]*p)\)]\)], "Input",
  CellLabel->"In[64]:="],

Cell[BoxData[
    \(\(subsphere[stb[1], 0.5]\)[v, w]\)], "Input",
  CellLabel->"In[65]:="],

Cell[BoxData[
    \(\(subsphere[stb[1]*n[1] + \ stb[2]*n[2], p]\)[v, w]\)], "Input",
  CellLabel->"In[66]:="],

Cell[BoxData[
    \(\(subsphere[{1, 1, 0, 0}/Sqrt[2], 1/2]\)[v, w]\)], "Input",
  CellLabel->"In[69]:="],

Cell[BoxData[
    \(\(subsphere[{1, 0, 0, 0}, 1/2]\)[v, w]\)], "Input",
  CellLabel->"In[70]:="],

Cell[BoxData[
    \(\(subsphere[stb[4], \  .9]\)[v, w]\)], "Input",
  CellLabel->"In[71]:="],

Cell[BoxData[
    \(Timing[\(subsphere[stb[2]*Cos[t] + stb[3]*Sin[t], Random[]]\)[v, 
        w]]\)], "Input",
  CellLabel->"In[72]:="],

Cell[BoxData[
    \(Timing[\(subsphere[stb[2]*Cos[t] + stb[3]*Sin[t], 0.8]\)[v, 
        w]]\)], "Input",
  CellLabel->"In[73]:="],

Cell["Symbolic calculations", "Text"],

Cell[BoxData[
    \(\(subsphere[vec[n], \ p]\)[v, w]\  /. 
      vec[n] . vec[n]\  -> \ 1\)], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[
    \(Simplify[%, \ n[1] > 0 && n[2] > 0 && n[3] > 0 && n[4] > 0]\)], "Input",\

  CellLabel->"In[68]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Tangential framing", "Subsubsection"],

Cell[TextData[{
  "At first we only analyze the situation and find formulas, which  then will \
be used, in a simplified way, to define the frame explicitely as a function \
of the geographical coordinates s,t,u. The frame is a natural tangential \
frame at unit3[s,t,u]. We  derive  the parameter representation of ",
  Cell[BoxData[
      \(TraditionalForm\`S\^\(\(3\)\(\ \)\)\)]],
  ", and norm the tangential vectors. We use"
}], "Text"],

Cell[BoxData[
    \(\(\(dotnorme[b]\)\(\ \)\)\)], "Input",
  CellLabel->"In[74]:="],

Cell[BoxData[
    \(\(\(dotnorme[vec[n]]\)\(\ \)\)\)], "Input",
  CellLabel->"In[75]:="],

Cell[BoxData[
    \(unit3[s, \ t, u]\)], "Input",
  CellLabel->"In[81]:="],

Cell[BoxData[
    \(tg1[s_, t_, u_]\  := \ D[unit3[x, \ t, u], x] /. x\  -> \ s\)], "Input",\

  CellLabel->"In[76]:="],

Cell[BoxData[
    \(tg1[s, t, u]\)], "Input",
  CellLabel->"In[77]:="],

Cell[BoxData[
    \(% . %\)], "Input",
  CellLabel->"In[78]:="],

Cell[BoxData[
    \(ntg1[s_, t_, u_] := \ dotnorme[tg1[s, t, u]]\)], "Input",
  CellLabel->"In[79]:="],

Cell[BoxData[
    \(ntg1[s, t, u]\)], "Input",
  CellLabel->"In[83]:="],

Cell[BoxData[
    \(Simplify[%, \ 
      s \[Element] Reals && t \[Element] Reals && 
        u \[Element] Reals]\)], "Input",
  CellLabel->"In[84]:="],

Cell["\<\
Since t, u fulfil the inequality -Pi/2< t, u < Pi/2 we may set\
\>", "Text"],

Cell[BoxData[{
    \(\(Clear[ntg1];\)\), "\[IndentingNewLine]", 
    \(ntg1[s_, t_, u_] := \ {\(-Sin[s]\), \ Cos[s], 0, 0}\)}], "Input",
  CellLabel->"In[91]:="],

Cell[BoxData[
    \(tg2[s_, t_, u_]\  := \ D[unit3[s, \ x, u], x] /. x\  -> \ t\)], "Input",\

  CellLabel->"In[87]:="],

Cell[BoxData[
    \(ntg2[s_, t_, u_] := \ dotnorme[tg2[s, t, u]]\)], "Input",
  CellLabel->"In[88]:="],

Cell[BoxData[
    \(ntg2[s, t, u]\)], "Input",
  CellLabel->"In[89]:="],

Cell[BoxData[
    \(Simplify[%, \ 
      s \[Element] Reals && t \[Element] Reals && 
        u \[Element] Reals]\)], "Input",
  CellLabel->"In[90]:="],

Cell["Since u fulfils the inequality -Pi/2< u < Pi/2 we may set", "Text"],

Cell[BoxData[{
    \(\(Clear[ntg2];\)\), "\[IndentingNewLine]", 
    \(ntg2[s_, t_, u_] := \ {\(-Cos[s]\)*Sin[t], \ \(-Sin[s]\)*Sin[t], \ 
        Cos[t], 0}\)}], "Input",
  CellLabel->"In[100]:="],

Cell[BoxData[
    \(tg3[s_, t_, u_]\  := \ D[unit3[s, \ t, x], x] /. x\  -> \ u\)], "Input",\

  CellLabel->"In[95]:="],

Cell[BoxData[
    \(ntg3[s_, t_, u_] := \ dotnorme[tg3[s, t, u]]\)], "Input",
  CellLabel->"In[96]:="],

Cell[BoxData[
    \(Simplify[ntg3[s, t, u], \ 
      s \[Element] Reals && t \[Element] Reals && 
        u \[Element] Reals]\)], "Input",
  CellLabel->"In[97]:="],

Cell["We set", "Text"],

Cell[BoxData[{
    \(\(Clear[ntg3];\)\), "\[IndentingNewLine]", 
    \(ntg3[s_, t_, 
        u_] := \ {\(-Cos[s]\)\ Cos[t]\ Sin[u], \(-Cos[t]\)\ Sin[s]\ Sin[
            u], \(-Sin[t]\)\ Sin[u], Cos[u]}\)}], "Input",
  CellLabel->"In[102]:="],

Cell[BoxData[
    \(Clear[tgframe]\)], "Input",
  CellLabel->"In[106]:="],

Cell[BoxData[
    \(tgframe[s_, t_, u_]\  := \ {ntg1[s, t, u], ntg2[s, t, u], 
        ntg3[s, t, u], unit3[s, t, u]}\)], "Input",
  CellLabel->"In[107]:="],

Cell[BoxData[
    \(tgframe[s, t, u] // MatrixForm\)], "Input",
  CellLabel->"In[108]:="],

Cell["\<\
Having this result we can formulate the direct definition (remember 0<=s< \
2Pi, -Pi/2<= t,u <= Pi/2).\
\>", "Text"],

Cell["This definition  contained in the package euvec.m is:", "Text"],

Cell[BoxData[
    \(\(?tg4frame\)\)], "Input",
  CellLabel->"In[109]:="],

Cell[BoxData[
    \(MatrixForm[tg4frame[s, t, u]]\)], "Input",
  CellLabel->"In[110]:="],

Cell["tg4frame yields a negatively oriented frame:", "Text"],

Cell[BoxData[
    \(Simplify[Det[tg4frame[s, t, u]]]\)], "Input",
  CellLabel->"In[114]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Another parameter representation of the spheres, using the tangential \
frames\
\>", "Subsubsection"],

Cell["\<\
The tangential framing gives the possibility of the following parameter \
representation of arbitrary spheres, defined in euvec.m:\
\>", "Text"],

Cell[BoxData[
    \(\(?subspheremf\)\)], "Input",
  CellLabel->"In[118]:="],

Cell[TextData[{
  "We called this procedure subspheremf since the argumants s,t,u,p may serve \
as coordinates on the 4-dimensional manifold of all subspheres of ",
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)]],
  ".  The procedure is much faster, and more effectiv,  than the composition \
of subsphere with unit[3]:"
}], "Text"],

Cell[BoxData[
    \(Timing[\(subspheremf[s, t, u, \ p]\)[v, w]]\)], "Input",
  CellLabel->"In[117]:="],

Cell[BoxData[
    \(Timing[\(subsphere[\(unit[3]\)[{s, t, u}], p]\)[v, w]]\)], "Input",
  CellLabel->"In[118]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    \(unit3[0, 0, 0]\)], "Input",
  CellLabel->"In[119]:="],

Cell["For Abs[p] > 1 we get an imaginary sphere", "Text"],

Cell[BoxData[
    \(\(subspheremf[0, \(-Pi\)/2, 0, 2]\)[v, w]\)], "Input",
  CellLabel->"In[120]:="],

Cell["and for Abs[p] = 1 we get a point only:", "Text"],

Cell[BoxData[
    \(\(subspheremf[0, \(-Pi\)/2, 0, \(-1\)]\)[v, w]\)], "Input",
  CellLabel->"In[121]:="],

Cell[BoxData[
    \(unit3[0, t, Pi/2]\)], "Input",
  CellLabel->"In[122]:="],

Cell[BoxData[
    \(unit3[s, t, Pi/2]\)], "Input",
  CellLabel->"In[123]:="],

Cell[BoxData[
    \(\(subspheremf[s, t, Pi/2, p]\)[v, w]\)], "Input",
  CellLabel->"In[129]:="],

Cell["\<\
We may plot the sphere in the intersecting hyperplane by e.g.\
\>", "Text"],

Cell[BoxData[
    \(Timing[
      ParametricPlot3D[
        Take[\(subspheremf[0.4, 0.3, Pi/2, 0.9]\)[v, w], 3], \ {v, \ 0, \ 
          2\ Pi}, {w, \ \(-Pi\)/2, Pi/2}]]\)], "Input",
  CellLabel->"In[125]:="],

Cell[BoxData[{
    \(a = Random[]; b = Random[]; c = Random[]; 
    Print[{a, b, c}];\), "\[IndentingNewLine]", 
    \(ParametricPlot3D[
      Take[\(subspheremf[a, b, Pi/2, c]\)[v, w], 3], \ {v, \ 0, \ 
        2\ Pi}, {w, \ \(-Pi\)/2, Pi/2}]\)}], "Input",
  CellLabel->"In[126]:="],

Cell["\<\
Now we apply the stereographical projection to plot the corresponding sphere \
in the euclidean 3-space:\
\>", "Text"],

Cell[BoxData[{
    \(a = Random[]; b = Random[]; c = Random[]; 
    Print[{a, b, c}];\), "\[IndentingNewLine]", 
    \(ParametricPlot3D[
      sterproj[\(subspheremf[a, b, c, 0.3]\)[v, w]], \ {v, \ 0, \ 
        2\ Pi}, {w, \ \(-Pi\)/2, Pi/2}, PlotRange\  \[Rule] \ All, 
      PlotPoints\  \[Rule] 60]\)}], "Input",
  CellLabel->"In[140]:="],

Cell[BoxData[
    \(Clear[a, b, c]\)], "Input",
  CellLabel->"In[142]:="],

Cell[BoxData[
    \(hyperplane[Table[stb[i], \ {i, 4}]]\)], "Input",
  CellLabel->"In[158]:="],

Cell[TextData[{
  "It follows that the north pole stb[4] of the standard unit hypersphere ",
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)]],
  " belongs to the sphere"
}], "Text"],

Cell[BoxData[
    \(\(subsphere[{1, 1, 1, 1}/2, 1/2]\)[v, w]\)], "Input",
  CellLabel->"In[143]:="],

Cell[BoxData[
    \(ParametricPlot3D[
      sterproj[\(subsphere[{1, 1, 1, 1}/2, 1/2]\)[v, w]], \ {v, \ 0, \ 
        2\ Pi}, \ {w, \(-Pi\)/2, \ Pi/2}]\)], "Input",
  CellLabel->"In[144]:="],

Cell[TextData[{
  "Since the sphere containes the north pole of the hypersphere ",
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)]],
  " we got by stereographic projection sterproj  a plane of the euclidean \
3-space. The next example is the standard unit sphere in the 3-space (the \
equatorial sphere of ",
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(\(subsphere[{0, 0, 0, 1}, 0]\)[v, w]\)], "Input",
  CellLabel->"In[159]:="],

Cell[BoxData[
    \(sterproj[\(subsphere[{0, 0, 0, 1}, 0]\)[v, w]]\)], "Input",
  CellLabel->"In[160]:="],

Cell[BoxData[
    \(ParametricPlot3D[
      sterproj[\(subsphere[{0, 0, 0, 1}, 0]\)[v, w]], \ {v, \ 0, \ 
        2\ Pi}, \ {w, \(-Pi\)/2, \ Pi/2}]\)], "Input",
  CellLabel->"In[161]:="],

Cell[BoxData[
    \(sterproj[\(subspheremf[s, t, u, \ p]\)[v, w]\ ]\)], "Input",
  CellLabel->"In[162]:="],

Cell["Now we create a configuration of spheres:", "Text"],

Cell[BoxData[
    \(Table[
      ParametricPlot3D[
        sterproj[\(subspheremf[i\ *2*Pi/5, 0, 0, \ 0.9]\)[v, 
            w]], \n\t\t\t{v, \ 0, \ 2  Pi}, \ {w, \(-Pi\)/2, Pi/2}], \ {i, 
        5}]\)], "Input",
  CellLabel->"In[163]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[164]:="],

Cell[BoxData[
    \(sph5v\  = \ %\)], "Input",
  CellLabel->"In[165]:="],

Cell[BoxData[
    \(MVShow3D[sph5v]\)], "Input",
  CellLabel->"In[167]:="],

Cell[BoxData[
    \(ThreeScript["\<sph5.ts\>", \ sph5v]\)], "Input",
  CellLabel->"In[35]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.4 The sphere through four points", "Subsection"],

Cell[CellGroupData[{

Cell["Hyperplanes through four points", "Subsubsection"],

Cell["\<\
Four points of the euclidean 4-space in general position define a hyperplane \
hp[n,p], n, p as in  3.3, namely\
\>", "Text"],

Cell[BoxData[
    \(\(?hyperplane\)\)], "Input",
  CellLabel->"In[158]:="],

Cell[BoxData[
    \(hyperplane[pt_List]\  := \ 
      Module[{n, q, p, d = Length[pt]}, \n\t\tq\  = \ 
          Table[pt[\([i]\)]\  - \ pt[\([d]\)], \ {i, 
              d\  - \ 1}]; \n\t\t\tn\  = \ cross[q]; \n\t
        If[Length[n]\  != \ d, \ Goto[fin]]; \n\t\tIf[n\  === \ nul[], 
          Return[\ Print["\<The points \>", \ 
              pt, \ "\< are not in general position!\>"]]]; \
\[IndentingNewLine]n\  = \ n/Sqrt[n . n]; \t\t\[IndentingNewLine]p\  = \ 
          n . pt[\([1]\)]; \nReturn[{n, \ p}]; \n\tLabel[fin]; \n
        Return[]]\)], "Input",
  Evaluatable->False],

Cell["\<\
Remark: The definition \"hyperplane\" is valid in any dimension.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Tests", "Subsubsection"]], "Subsubsection"],

Cell["\<\
For dim = 4, in case -1 < p < 1, the pair {n,p} defines the sphere with \
parameter representation subsphere[n, p][v,w].\
\>", "Text"],

Cell[BoxData[
    \(\(?subsphere\)\)], "Input",
  CellLabel->"In[168]:="],

Cell[BoxData[
    \(hyperplane[{stb[1], \ stb[2], \ 0.9*stb[3], \ stb[4]}]\)], "Input",
  CellLabel->"In[169]:="],

Cell[BoxData[
    \(\(subsphere[%[\([1]\)], \ %[\([2]\)]]\)[v, w]\)], "Input",
  CellLabel->"In[170]:="],

Cell[BoxData[
    \(ParametricPlot3D[
      sterproj[%], \ {v, 0, 2\ Pi}, {w, \ \(-Pi\)/2, \ Pi/2}]\)], "Input",
  CellLabel->"In[171]:="],

Cell[BoxData[
    \(hyperplane[{stb[1], \ stb[2], \ stb[3], \ \(-stb[3]\)}]\)], "Input",
  CellLabel->"In[172]:="],

Cell[BoxData[
    \(\(subsphere[%[\([1]\)], \ %[\([2]\)]]\)[v, w]\)], "Input",
  CellLabel->"In[173]:="],

Cell[BoxData[
    \(ParametricPlot3D[
      sterproj[%], \ {v, 0, 2\ Pi}, {w, \ \(-Pi\)/2, \ Pi/2}]\)], "Input",
  CellLabel->"In[174]:="],

Cell["The next two examples return only messages...", "Text"],

Cell[BoxData[
    \(hyperplane[{stb[1], \ stb[2], \ stb[2], \ \(-stb[3]\)}]\)], "Input",
  CellLabel->"In[175]:="],

Cell[BoxData[
    \(hyperplane[{stb[1, 5], \ stb[2, 5], \ 
        stb[3, 5], \ \(-stb[3, 5]\)}]\)], "Input",
  CellLabel->"In[176]:="],

Cell[BoxData[
    \(hyperplane[{stb[1, 5], \ stb[2, 5], \ stb[3, 5], \ \(-stb[3, 5]\), 
        stb[5, 5]}]\)], "Input",
  CellLabel->"In[177]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Sphere through four points via hyperplane", "Subsubsection"],

Cell["\<\
The following Module combines  stereographical projection, hyperplane and \
subsphere to get a parameter representation of the sphere through four points \
of the euclidean 3-space: \
\>", "Text"],

Cell[BoxData[
    \(\(sphere4pts[pt_List]\)[v_, w_] := 
      Module[{a, 
          b}, \n\t\t\tIf[\((Length[pt]\  != 
                4)\)\  || \((Length[pt[\([1]\)]]\  != \ 3)\), \ 
          Print["\<Incorrect arguments!!!\>"]; \tReturn[]]; \n\t\t\ta\  = \ 
          Table[invsterproj[pt[\([i]\)]], \ {i, 4}]; \n\t\t\tb\  = \ 
          hyperplane[a]; \n\t\tIf[b == \ Null, \ \tReturn[]]; \n\t\t\tReturn[
          sterproj[\(subsphere[b[\([1]\)], \ b[\([2]\)]]\)[v, 
              w]]]\n]\)], "Input",
  CellLabel->"In[179]:="],

Cell["This gives, of course, the standard unit sphere:", "Text"],

Cell[BoxData[
    \(\(sphere4pts[{stb[1, \ 3], \ stb[2, 3], stb[3, 3], \ \(-stb[3, 3]\)}]\)[
      v, w]\)], "Input",
  CellLabel->"In[180]:="],

Cell[BoxData[
    \(ParametricPlot3D[%, {v, \ 0, \ 2\ Pi}, \ {w, \ \(-Pi\)/2, \ Pi/2}, \ 
      PlotRange\  -> \ All, \ \nPlotPoints\  -> \ 50]\)], "Input",
  CellLabel->"In[181]:="],

Cell[BoxData[
    \(ptslist = \ {stb[1, \ 3], \ stb[2, 3], stb[3, 3], \ 
        2\ \ stb[3, 3]}\)], "Input",
  CellLabel->"In[182]:="],

Cell[BoxData[
    \(pts\  = Map[Point, ptslist]\)], "Input",
  CellLabel->"In[183]:="],

Cell[BoxData[
    \(gr4pt\  = \ \ Graphics3D[Join[{PointSize[0.04]}, pts]]\)], "Input",
  CellLabel->"In[184]:="],

Cell[BoxData[
    \(Show[gr4pt]\)], "Input",
  CellLabel->"In[185]:="],

Cell[BoxData[
    \(grpts\  = \ %\)], "Input",
  CellLabel->"In[186]:="],

Cell[BoxData[
    \(\(sphere4pts[ptslist]\)[v, w]\)], "Input",
  CellLabel->"In[187]:="],

Cell[BoxData[
    \(ParametricPlot3D[%, {v, \ 0, \ 2\ Pi}, \ {w, \ \(-Pi\)/2, \ Pi/2}, \ 
      PlotRange\  -> \ All, \ \nPlotPoints\  -> \ 40]\)], "Input",
  CellLabel->"In[188]:="],

Cell[BoxData[
    \(sph4pt\  = \ %\)], "Input",
  CellLabel->"In[189]:="],

Cell[BoxData[
    \(Show[{sph4pt, grpts}]\)], "Input",
  CellLabel->"In[190]:="],

Cell[BoxData[
    \(t4pt\  = \ %\)], "Input",
  CellLabel->"In[191]:="],

Cell[BoxData[
    \(MVShow3D[t4pt, MVPointSphereSize\  \[Rule] \ 0.8]\)], "Input",
  CellLabel->"In[192]:="],

Cell["\<\
Apparently, by numerical reasons, one point seems not to lay on the sphere.\
\>", "Text"],

Cell["The next three examples return only messages...", "Text"],

Cell[BoxData[
    \(\(sphere4pts[{stb[1, \ 3], \ stb[2, 3], stb[3, 3], \ stb[3, 3]}]\)[v, 
      w]\)], "Input",
  CellLabel->"In[193]:="],

Cell[BoxData[
    \(\(sphere4pts[{stb[1], \ stb[3], stb[3], \ \(-stb[3]\)}]\)[v, 
      w]\)], "Input",
  CellLabel->"In[194]:="],

Cell[BoxData[
    \(\(sphere4pts[{stb[1], \ stb[3], stb[3]}]\)[v, w]\)], "Input",
  CellLabel->"In[195]:="],

Cell["\<\
If wanted one can insert sphere4pts[] in a plot command and get the picture \
of the sphere through four points by\
\>", "Text"],

Cell[BoxData[
    \(pt4sphere[pt_List, opt___]\  := 
      ParametricPlot3D[
        Evaluate[\(sphere4pts[pt]\)[v, w]], \n\t\t{v, \ 0, \ 
          2\ Pi}, {w, \ \(-Pi\)/2, \ Pi/2}, \ opt]\)], "Input",
  CellLabel->"In[196]:="],

Cell[BoxData[
    \(pt4sphere[{{0.5, \ 1, \ 2. }, \ {\(-1\), \ 1.5, \ 0.7}, \ {2. , \ 
          1. , \ 0. }, \ stb[3, 3]}, \nPlotPoints\  -> \ 50, 
      PlotRange\  \[Rule] \ All]\)], "Input",
  CellLabel->"In[197]:="],

Cell[BoxData[
    \(ttgr\  = \ %\)], "Input",
  CellLabel->"In[198]:="],

Cell[BoxData[
    \(pts\  = 
      Map[Point, {{0.5, \ 1, \ 2. }, \ {\(-1\), \ 1.5, \ 0.7}, \ {2. , \ 
            1. , \ 0. }, \ stb[3, 3]}]\)], "Input",
  CellLabel->"In[199]:="],

Cell[BoxData[
    \(gr4ptt\  = \ \ Graphics3D[Join[{PointSize[0.04]}, pts]]\)], "Input",
  CellLabel->"In[200]:="],

Cell[BoxData[
    \(Show[gr4ptt]\)], "Input",
  CellLabel->"In[201]:="],

Cell[BoxData[
    \(grptts\  = \ %\)], "Input",
  CellLabel->"In[202]:="],

Cell[BoxData[
    \(Show[{ttgr, grptts}]\)], "Input",
  CellLabel->"In[205]:="],

Cell[BoxData[
    \(testgrtt\  = \ %\)], "Input",
  CellLabel->"In[206]:="],

Cell[BoxData[
    \(MVShow3D[testgrtt, MVPointSphereSize\  \[Rule] \ 1.4]\)], "Input",
  CellLabel->"In[207]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["A direct approach ", "Subsubsection"],

Cell["\<\
Another, maybe simpler way to plot the sphere through four points in the \
3-space is first to calculate its centre and radius and then to apply sphere.\
\
\>", "Text"],

Cell[BoxData[
    \(centre[pt_List] := 
      Module[{p = Length[pt], d, i, j, k, ab, at, mat, mb, \ c}, \n\t\tIf[
          p\  != \ 4, \ Print["\<Incorrect argument!\>"]; \ 
          Goto[fin]]; \n\t\t\tFor[i = 1, \ i < 4, \ \(i++\), 
          at[i] = pt[\([i + 1]\)] - pt[\([i]\)]; \n\t\t\t\tab[
              i] = \((\((pt[\([i + 1]\)] . 
                      pt[\([i + 1]\)])\) - \((pt[\([i]\)] . pt[\([i]\)])\))\)/
              2]; \n\t\t\t\tmat\  = \ Table[at[j], {j, 1, 3}]; \n\t\td\  = \ 
          Det[mat]; \n\t\tIf[d == \ 0. , \ 
          Print["\<Points lie on a plane!\>"]; \[IndentingNewLine]Goto[
            fin]]; \t\t\n\t\tmb\  = \ 
          Table[ab[k], {k, 1, 3}]; \n\t\t\t\tc\  = \ 
          LinearSolve[mat, mb]; \n\tReturn[c]; \n\tLabel[fin]; \n\t
        Return[]]\)], "Input",
  CellLabel->"In[20]:=",
  InitializationCell->True],

Cell[BoxData[
    \(radius[pt_List] := \ 
      Module[{c\  = centre[pt]}\ , \[IndentingNewLine]If[c \[Equal] \ Null, \ 
          Return[]]; \[IndentingNewLine]Sqrt[\((pt[\([1]\)] - 
                c)\) . \((pt[\([1]\)] - c)\)]]\)], "Input",
  CellLabel->"In[21]:=",
  InitializationCell->True],

Cell[BoxData[
    \(sphere4ptsplot3D[p_List, s0_:  0, \ s1_:  2\ Pi, t0_: - Pi/2, \ 
        t1_:  Pi/2, \ opt___]\  := 
      Module[{m, r}, \[IndentingNewLine]m\  = \ 
          centre[p]; \[IndentingNewLine]If[m\  \[Equal] Null, \ 
          Return[]]; \[IndentingNewLine]r\  = \ radius[p]; \n\t
        sphereplot3D[m, r, s0, \ s1, t0, \ t1, \ opt]]\)], "Input",
  CellLabel->"In[22]:=",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples, tests", "Subsubsection"],

Cell[BoxData[
    \(sqr\  = {{0.5, \ 1, \ 2. }, \ {\(-1\), \ 1.5, \ 0.7}, \ {2. , \ 1. , \ 
          0. }, \ stb[3, 3]}\)], "Input",
  CellLabel->"In[259]:="],

Cell[BoxData[
    \(centre[sqr]\)], "Input",
  CellLabel->"In[260]:="],

Cell[BoxData[
    \(cpt\  = \ %\)], "Input",
  CellLabel->"In[261]:="],

Cell[BoxData[
    \(centre[sqr]\)], "Input",
  CellLabel->"In[262]:="],

Cell[BoxData[
    \(sqr[\([1]\)]\)], "Input",
  CellLabel->"In[263]:="],

Cell[BoxData[
    \(Table[\((cpt - sqr[\([j]\)])\) . \((cpt - sqr[\([j]\)])\), \ {j, 1, 
        4}]\)], "Input",
  CellLabel->"In[264]:="],

Cell[BoxData[
    \(radius[sqr]\)], "Input",
  CellLabel->"In[265]:="],

Cell[BoxData[
    \(radius[sqr]\)], "Input",
  CellLabel->"In[266]:="],

Cell[BoxData[
    \(%^2\)], "Input",
  CellLabel->"In[267]:="],

Cell[BoxData[
    \(sphere4ptsplot3D[sqr]\)], "Input",
  CellLabel->"In[268]:="],

Cell[BoxData[
    \(testgrtt\  = \ %\)], "Input",
  CellLabel->"In[269]:="],

Cell[BoxData[
    \(Show[{testgrtt, grptts}]\)], "Input",
  CellLabel->"In[270]:="],

Cell[BoxData[
    \(testgrtt1\  = \ %\)], "Input",
  CellLabel->"In[271]:="],

Cell[BoxData[
    \(MVShow3D[testgrtt1, MVPointSphereSize\  \[Rule] \ 1.4]\)], "Input",
  CellLabel->"In[272]:="],

Cell["Now the sphere fits much better to the points!", "Text"],

Cell[BoxData[
    \(p2\  = {nul[3], stb[1, 3], stb[2, 3], \ stb[3, 3]}\)], "Input",
  CellLabel->"In[273]:="],

Cell[BoxData[
    \(centre[p2]\)], "Input",
  CellLabel->"In[274]:="],

Cell[BoxData[
    \(cp2\  = \ %\)], "Input",
  CellLabel->"In[275]:="],

Cell[BoxData[
    \(Table[\((cp2 - p2[\([i]\)])\) . \((cp2 - p2[\([i]\)])\), \ {i, 1, 
        4}]\)], "Input",
  CellLabel->"In[276]:="],

Cell[BoxData[
    \(radius[p2]\)], "Input",
  CellLabel->"In[277]:="],

Cell[BoxData[
    \(sphere4ptsplot3D[p2, \ 0, \ Pi, \ \(-Pi\)/2, Pi/2, \ 
      PlotPoints \[Rule] \ 40]\)], "Input",
  CellLabel->"In[337]:="],

Cell["Error messages", "Text"],

Cell[BoxData[
    \(sphere4ptsplot3D[Take[p2, 2]]\)], "Input",
  CellLabel->"In[333]:="],

Cell[BoxData[
    \(p3\  = {nul[3], stb[1, 3], stb[2, 3], \ 2*stb[2, 3]}\)], "Input",
  CellLabel->"In[280]:="],

Cell[BoxData[
    \(centre[p3]\)], "Input",
  CellLabel->"In[334]:="],

Cell[BoxData[
    \(radius[p3]\)], "Input",
  CellLabel->"In[335]:="],

Cell[BoxData[
    \(sphere4ptsplot3D[p3]\)], "Input",
  CellLabel->"In[336]:="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4 Orthogonality", "Section"],

Cell[CellGroupData[{

Cell["4.1 Erhard Schmidt's orthogonalization", "Subsection"],

Cell[CellGroupData[{

Cell["The procedure", "Subsubsection"],

Cell["\<\
esorthonorm follows Erhard Schmidt's orthogonalization, in difference to \
which the given vector sequence m_List needs not to be linearly independent. \
If a vector linearly depends on the foregoing vectors, esorthonorm generates \
the zero vector and eliminates it.  The module esorthonorm is contained in \
the package euvec.m.The numerical behavior of esorthonorm can be influenced \
by the option neglect, which sets the parameter of smoothing. 
WARNING: Because of the partly numerical character of the procedure, an \
inappropriate setting of the option neglect may cause false results. One \
always should control the result of esorthonorm checking the orthogonality \
and completeness of the produced sequence!\
\>", "Text"],

Cell[BoxData[
    \(Options[esorthonorm]\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(\(?smoothing\)\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(\(?neglect\)\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(\(?print\)\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(\(?esorthonorm\)\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(\(\(esorthonorm[m_List, 
        opts___]\)\(\ \)\(:=\)\(\ \)\(\n\)\(\ \ \)\(Module[\n\t\t{ma = \ m, 
          mb = {}, k\  = \ Length[m], rd = Length[m], \ j\  = \ 0, \ i, ne, 
          prod, \ pr, \ bn, \ sm, aus, \ h, \ 
          f}, \n\t\t{prod, nor, pr, \ 
            bn}\  = \ \n\t\t\t\({innerprod, normed, \ print, \ 
                neglect} /. {opts}\) /. 
            Options[esorthonorm]; \[IndentingNewLine]If[
          Head[m[\([1]\)]\ ] \[Equal] \ List, 
          rd\  = \ Length[m[\([1]\)]]]; \n\t
        f[h_]\  := \ smoothing[h, 10^bn]; \n\t
        sm[aus_]\  := \ MapAll[f, aus]; \n\t\tLabel[
          ne]; \[IndentingNewLine]If[
          Length[mb] \[GreaterEqual] \ Min[rd, k], \ 
          Return[mb]]; \n\t\tj\  = \ j + 1; \[IndentingNewLine]If[j > k, 
          Return[mb]]; \n\t\t\tma[\([j]\)] = 
          sm[Simplify[
              ma[\([j]\)]\  - \n\t
                Sum[mb[\([i]\)]*
                    prod[ma[\([j]\)], \ mb[\([i]\)]]/
                      prod[mb[\([i]\)], \ mb[\([i]\)]], \ \n\t{i, 1, \ 
                    Length[mb]}]]]; \ \t\t\n
        If[pr\  == True, Print[ma[\([j]\)]]]; \n
        If[prod[ma[\([j]\)], \ ma[\([j]\)]] == \ 0, 
          Goto[ne], \n\ \ \ \ \ \ \ \ If[nor\  == \ True, 
            AppendTo[mb, 
              sm[Simplify[
                  ma[\([j]\)]/
                    Sqrt[prod[ma[\([j]\)], 
                        ma[\([j]\)]]]]]], \n\t\t\t\tAppendTo[mb, 
              Simplify[ma[\([j]\)]]]], \n\t\t\t\tIf[
            nor\  == True, \n\t\t\t\tAppendTo[mb, 
              sm[Simplify[
                  ma[\([j]\)]/
                    Sqrt[prod[ma[\([j]\)], ma[\([j]\)]]]]]], \ \t\t\t\n
            AppendTo[mb, Simplify[ma[\([j]\)]]]]]; \t\t\n\t\tGoto[
          ne]]\)\(\n\)\)\)], "Input",
  CellLabel->"In[18]:=",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Simple tests", "Subsubsection"],

Cell["First we give some simple examples.", "Text"],

Cell[BoxData[
    \(esorthonorm[{nul[]}]\)], "Input",
  CellLabel->"In[95]:="],

Cell[BoxData[
    \(esorthonorm[Table[stb[i]*i, \ {i, 4}], \ print\  -> \ True]\)], "Input",\

  CellLabel->"In[96]:="],

Cell[BoxData[
    \(esorthonorm[Table[stb[i]*\((i - 1)\), \ {i, 4}], normed\  -> \ False, 
      print\  -> \ True]\)], "Input",
  CellLabel->"In[97]:="],

Cell["Linearly dependent vectors are omitted:", "Text"],

Cell[BoxData[
    \(at\  = 
      AppendTo[Table[randomv[]*\((j - 2)\), {j, 7}], \ nul[]]\)], "Input",
  CellLabel->"In[98]:="],

Cell["neglect -> 0 amd neglect -> default give the same results:", "Text"],

Cell[BoxData[
    \(ot1 = esorthonorm[at, print\  -> \ True, neglect\  -> \ 0]\)], "Input",
  CellLabel->"In[99]:="],

Cell[BoxData[
    \(Table[
        smoothing[ot1[\([i]\)] . ot1[\([j]\)], \ 1], {i, Length[ot1]}, \ {j, 
          Length[ot1]}] // MatrixForm\)], "Input",
  CellLabel->"In[100]:="],

Cell[BoxData[
    \(Chop[%]\)], "Input",
  CellLabel->"In[101]:="],

Cell[BoxData[
    \(ot2 = esorthonorm[at, print\  -> \ True]\)], "Input",
  CellLabel->"In[102]:="],

Cell[BoxData[
    \(ot2 - ot1\)], "Input",
  CellLabel->"In[103]:="],

Cell["\<\
Also with normed -> False (and neglect -> 0) one gets the same results:\
\>", "Text"],

Cell[BoxData[
    \(at\  = 
      AppendTo[\ Table[randomv[]*\((j - 2)\), {j, 7}], \ nul[]]\)], "Input",
  CellLabel->"In[104]:="],

Cell[BoxData[
    \(ot3 = 
      esorthonorm[at, normed\  -> \ False, \ print\  -> \ True, 
        neglect\  -> \ 0]\)], "Input",
  CellLabel->"In[105]:="],

Cell[BoxData[
    \(Table[
        smoothing[ot3[\([i]\)] . ot3[\([j]\)], \ 1], {i, Length[ot3]}, \ {j, 
          Length[ot3]}] // MatrixForm\)], "Input",
  CellLabel->"In[106]:="],

Cell[BoxData[
    \(Chop[%]\)], "Input",
  CellLabel->"In[107]:="],

Cell["Now again with the default value of neglect:", "Text"],

Cell[BoxData[
    \(ot4 = 
      esorthonorm[at, normed\  -> \ False, \ print\  -> \ True]\)], "Input",
  CellLabel->"In[108]:="],

Cell[BoxData[
    \(ot3 - ot4\)], "Input",
  CellLabel->"In[109]:="]
}, Closed]],

Cell[CellGroupData[{

Cell[" The Legendre polynomials with esorthonorm", "Subsubsection"],

Cell["\<\
The procedure esorthonorm  may be applied also for function spaces, for which \
a positive semidefinite scalar product must be defined in terms of \
Mathematica.\
\>", "Text"],

Cell[BoxData[
    \(prodint[f_, g_]\  := \ Integrate[f*g, {x, \ \(-1\), 1}]\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(prodint[1, x^2]\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(seq[h_]\  := \ Table[x^i, {i, 0, h}]\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(seq[3]\)], "Input",
  CellLabel->"In[87]:="],

Cell[BoxData[
    \(esorthonorm[seq[1], innerprod\  -> \ prodint]\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(esorthonorm[seq[2], \ innerprod\  -> \ prodint]\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(esorthonorm[seq[3], \ innerprod\  -> \ prodint]\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(esorthonorm[seq[4], innerprod\  -> \ prodint]\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(esorthonorm[seq[5], innerprod\  -> \ prodint]\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \(esorthonorm[seq[6], \ innerprod\  -> \ prodint]\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(esorthonorm[seq[7], \ innerprod\  -> \ prodint]\)], "Input",
  CellLabel->"In[47]:="],

Cell[BoxData[
    \(Timing[
      esorthonorm[seq[20], \ innerprod\  -> \ prodint, \ 
        print\  -> \ True]]\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(Length[%[\([2]\)]]\)], "Input",
  CellLabel->"In[23]:="],

Cell[BoxData[
    \(\(eslegpol20\  = \ %%[\([2]\)];\)\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(Table[
        prodint[eslegpol20[\([i]\)], eslegpol20[\([j]\)]], {i, 21}, {j, 21}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[25]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Comparison with the implemented LegendreP", "Subsubsection"],

Cell[BoxData[
    \(\(?LegendreP\)\)], "Input",
  CellLabel->"In[26]:="],

Cell[TextData[{
  "The functions LegendreP implemented in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " are normed by the condition"
}], "Text"],

Cell[BoxData[
    \(LegendreP[_, 1]\)], "Input",
  CellLabel->"In[46]:="],

Cell["\<\
Thus they are orthogonal but not normed with respect to prodint:\
\>", "Text"],

Cell[BoxData[
    \(\(\(legpol20 = 
        Table[LegendreP[j, x], \ {\ j, 0, 20}];\)\(\ \)\)\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(Table[prodint[legpol20[\([i]\)], legpol20[\([j]\)]], {i, 21}, {j, 21}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(LegendreP[20, 1]\)], "Input",
  CellLabel->"In[30]:="],

Cell[BoxData[
    \(eslegpol20[\([21]\)] /. x \[Rule] 1\)], "Input",
  CellLabel->"In[33]:="],

Cell["\<\
Up to the norming they coincide with the orthonormal polynomials obtained by \
esorthonorm:\
\>", "Text"],

Cell[BoxData[
    \(Table[
      LegendreP[i, x] - 
        eslegpol20[\([i + 1]\)]/\((eslegpol20[\([i + 1]\)] /. 
              x \[Rule] 1)\), \ {i, 0, 20}]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[35]:="]
}, Closed]],

Cell[CellGroupData[{

Cell[" A last example", "Subsubsection"],

Cell["\<\
Sometimes sequences containing numerical and functional vectors, cause \
difficulties. The next example runs normally\
\>", "Text"],

Cell[BoxData[
    \(v1\  = \ randomv[3]\)], "Input",
  CellLabel->"In[107]:="],

Cell[BoxData[
    \(Timing[
      esorthonorm[
        Join[{{0, Cos[t], Sin[t]}, v1}, Table[stb[i, 3], \ {i, 2}]], \ 
        print\  -> \ True, \ normed\  \[Rule] \ False]]\)], "Input",
  CellLabel->"In[114]:="],

Cell[BoxData[
    \(ov\  = Chop[%[\([2]\)]]\)], "Input",
  CellLabel->"In[115]:="],

Cell[BoxData[
    \(\(\(Table[
      Simplify[ov[\([i]\)] . ov[\([j]\)]], {i, Length[ov]}, {j, 
        Length[ov]}]\)\(\ \)\)\)], "Input",
  CellLabel->"In[116]:="],

Cell[BoxData[
    \(Chop[%]\)], "Input",
  CellLabel->"In[117]:="],

Cell[BoxData[
    \(\(prodsov[t_] = %;\)\)], "Input",
  CellLabel->"In[118]:="],

Cell[BoxData[
    \(prodsov[Random[]]\)], "Input",
  CellLabel->"In[122]:="],

Cell["\<\
Remember that we used the option normed -> False. Why we first have got small \
imaginary parts?\
\>", "Text"],

Cell[BoxData[
    \(v1\  = \ randomv[3]\)], "Input",
  CellLabel->"In[125]:="],

Cell[BoxData[
    \(Timing[
      esorthonorm[
        Join[{{0, Cos[t], Sin[t]}, v1}, Table[stb[i, 3], \ {i, 2}]], \ 
        print\  -> \ False]]\)], "Input",
  CellLabel->"In[126]:="],

Cell[BoxData[
    \(\(abc[t_]\  = \ %[\([2]\)];\)\)], "Input",
  CellLabel->"In[127]:="],

Cell[BoxData[
    \(Chop[abc[t]]\)], "Input",
  CellLabel->"In[128]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[129]:="],

Cell[BoxData[
    \(Length[abc[t]]\)], "Input",
  CellLabel->"In[130]:="],

Cell[BoxData[
    \(Table[
      Dot[\(abc[t]\)[\([i]\)], \(abc[t]\)[\([j]\)]], {i, \ 3}, {j, \ 
        3}]\)], "Input",
  CellLabel->"In[131]:="],

Cell[BoxData[
    \(Chop[%]\)], "Input",
  CellLabel->"In[132]:="],

Cell[BoxData[
    \(\(prods[t_]\  = \ %;\)\)], "Input",
  CellLabel->"In[133]:="],

Cell[BoxData[
    \(prods[Random[]]\)], "Input",
  CellLabel->"In[134]:="],

Cell[BoxData[
    \(Chop[%]\)], "Input",
  CellLabel->"In[135]:="],

Cell[BoxData[
    \(Chop[prods[6*Random[]]]\)], "Input",
  CellLabel->"In[139]:="],

Cell[BoxData[
    \(Remove[ov, prods, abc]\)], "Input",
  CellLabel->"In[160]:="],

Cell["The same with GramSchmidt, a procedure defined in", "Text"],

Cell[BoxData[
    \(<< "\<LinearAlgebra/Orthogonalization.m\>"\)], "Input",
  CellLabel->"In[104]:=",
  FontSize->15],

Cell[BoxData[
    \(\(?GramSchmidt\)\)], "Input",
  CellLabel->"In[105]:="],

Cell[BoxData[
    \(Options[GramSchmidt]\)], "Input",
  CellLabel->"In[106]:="],

Cell["\<\
We recommend the user to load this package, and to compare GramSchmidt with \
esorthonorm, running the same tests including the timing for both procedures, \
e.g.\
\>", "Text"],

Cell[BoxData[
    \(v1\  = \ randomv[3]\)], "Input",
  CellLabel->"In[161]:="],

Cell[BoxData[
    \(Timing[
      GramSchmidt[
        Join[{{0, Cos[t], Sin[t]}, v1}, 
          Table[stb[i, 3], \ {i, 2}]]]]\)], "Input",
  CellLabel->"In[162]:="],

Cell[BoxData[
    \(Simplify[%[\([2]\)]]\)], "Input",
  CellLabel->"In[163]:="],

Cell[BoxData[
    \(\(ov1[t_] = \ %;\)\)], "Input",
  CellLabel->"In[164]:="],

Cell[BoxData[
    \(Chop[ov1[t]]\)], "Input",
  CellLabel->"In[165]:="],

Cell[BoxData[
    \(\(ov2[t_] = %;\)\)], "Input",
  CellLabel->"In[166]:="],

Cell[BoxData[
    \(Length[%]\)], "Input",
  CellLabel->"In[143]:="],

Cell["The last vector is zero. ", "Text"],

Cell[BoxData[
    \(prods[t_] := 
      Table[Simplify[\(ov2[t]\)[\([i]\)] . \(ov2[t]\)[\([j]\)]], {i, 3}, {j, 
          3}]\)], "Input",
  CellLabel->"In[168]:="],

Cell[BoxData[
    \(prods[t]\)], "Input",
  CellLabel->"In[169]:="],

Cell[BoxData[
    \(Chop[%]\)], "Input",
  CellLabel->"In[170]:="],

Cell[BoxData[
    \(\((% /. t \[Rule] Random[])\) // MatrixForm\)], "Input",
  CellLabel->"In[171]:="],

Cell[BoxData[
    \(Chop[prods[Random[]]]\)], "Input",
  CellLabel->"In[174]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2 The Euclidean orthogonal complement in the 4-space", "Subsection",
  Evaluatable->False],

Cell["\<\
For later applications in e.g. M\[ODoubleDot]bius geometry:
To construct the circle which corresponds to the intersection of two \
hyperplanes in the 4-space and the unit hypersphere we need an orthonormal \
solution of
v.x == 0, w.x == 0.
This is given by orthoerg defined in euvec.m:\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(\(?orthoerg\)\)], "Input",
  CellLabel->"In[17]:="],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  Evaluatable->False],

Cell[BoxData[
    \(orthoerg[stb[3], stb[1]]\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(orthoerg[{0, Cos[t], Sin[t], 0}, {1, 0, 0, 0}]\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(Simplify[%, \ t > 0]\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(Clear[v1, v2]\)], "Input",
  CellLabel->"In[81]:="],

Cell[BoxData[
    \(v1\  = {Cos[t], 0, Sin[t], 0}; v2 = {1, 0, 0, 1};\)], "Input",
  CellLabel->"In[82]:="],

Cell[BoxData[
    \(orthoerg[v1, v2]\)], "Input",
  CellLabel->"In[83]:="],

Cell[BoxData[
    \({v1 . %[\([1]\)], \ v2 . %[\([2]\)], v1 . %[\([2]\)], \ 
      v2 . %[\([1]\)], \ 
      Table[%[\([i]\)] . %[\([j]\)], \ {i, 2}, {j, 2}] // 
        MatrixForm}\)], "Input",
  CellLabel->"In[84]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[85]:="],

Cell[BoxData[
    \(Clear[v1, v2]\)], "Input",
  CellLabel->"In[86]:="],

Cell[BoxData[
    \(v1\  = \ randomv[]; v2\  = \ randomv[];\)], "Input",
  CellLabel->"In[87]:="],

Cell[BoxData[
    \(Timing[orthoerg[v1, v2]]\)], "Input",
  CellLabel->"In[88]:="],

Cell[BoxData[
    \(%[\([2]\)]\)], "Input",
  CellLabel->"In[89]:="],

Cell[BoxData[
    \({v1 . %[\([1]\)], \ v2 . %[\([2]\)], v1 . %[\([2]\)], \ 
      v2 . %[\([1]\)], \ 
      Table[%[\([i]\)] . %[\([j]\)], \ {i, 2}, {j, 2}] // 
        MatrixForm}\)], "Input",
  CellLabel->"In[90]:="],

Cell[BoxData[
    \(MapAll[smoothing, %]\)], "Input",
  CellLabel->"In[91]:="],

Cell[BoxData[
    \(Clear[v1]\)], "Input",
  CellLabel->"In[93]:="],

Cell[BoxData[
    \(v1\  = \ {0, Cos[t], Sin[t], 0}\)], "Input",
  CellLabel->"In[94]:="],

Cell[BoxData[
    \(v2\)], "Input",
  CellLabel->"In[95]:="],

Cell[BoxData[
    \(orthoerg[v1, v2]\)], "Input",
  CellLabel->"In[96]:="],

Cell[BoxData[
    \(\(vo\  = \ %;\)\)], "Input",
  CellLabel->"In[97]:="],

Cell[BoxData[
    \(\(\(Simplify[{v1 . vo[\([1]\)], \ v2 . vo[\([2]\)], v1 . vo[\([2]\)], \ 
        v2 . vo[\([1]\)]}]\)\(\ \)\)\)], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \(Chop[%]\)], "Input",
  CellLabel->"In[99]:="],

Cell[BoxData[
    \(Plot[%[\([2]\)]\  + \ 1, {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[100]:="],

Cell[TextData[{
  "Obviously, also the second term vanishes, but ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " does not recognize this."
}], "Text",
  CellLabel->"In[77]:="],

Cell[BoxData[
    \(Table[
      Chop[Simplify[vo[\([i]\)] . vo[\([j]\)]]], {i, 2}, {j, 2}]\)], "Input",
  CellLabel->"In[101]:="],

Cell["\<\
Here all is ok. Sometimes the simplification is unsatisfactory, what depends \
on the random vector. The varry ou the next (or similar) commands\
\>", "Text"],

Cell[BoxData[
    \(Plot[%[\([2, 2]\)], {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[79]:="],

Cell[BoxData[
    \(Plot[Chop[%%[\([2, 2]\)] - 1] + 1, {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[80]:="],

Cell["OK, up to  small numerical errors.", "Text"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["5 Reflection at  spheres", "Section",
  Evaluatable->False],

Cell["\<\
Already in 1.3 we considered inversions at arbitrary hyperspheres. The \
reflection at the unit sphere can be defined much easier:\
\>", "Text"],

Cell["refl[x_List] := x/(x.x)", "Input",
  CellLabel->"In[23]:=",
  PageWidth->Infinity,
  InitializationCell->True],

Cell[CellGroupData[{

Cell["5.1 Examples", "Subsection"],

Cell[CellGroupData[{

Cell["Tests", "Subsubsection"],

Cell[BoxData[
    \(refl[{5, 5, 0, 0}]\)], "Input",
  CellLabel->"In[84]:="],

Cell[BoxData[
    \(refl[{2, 2, 2. }]\)], "Input",
  CellLabel->"In[85]:="],

Cell[BoxData[
    \(refl[vec[x]]\)], "Input",
  CellLabel->"In[86]:="],

Cell[BoxData[
    \(Simplify[Together[\((% . %)\)*\((vec[x] . vec[x])\)\  - 1]]\)], "Input",\

  CellLabel->"In[87]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Planes and spheres are transformed in planes or spheres", \
"Subsubsection"],

Cell["\<\
The unit square in the plane z = 1 is transformed into a spherical region:\
\>", "Text"],

Cell[BoxData[
    \(ParametricPlot3D[
      refl[{u, \ v, \ 1}], \ {u, \ 0, \ 1}, \ {v, \ 0, \ 1}]\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(tgr1\  = \ %\)], "Input",
  CellLabel->"In[27]:="],

Cell["\<\
Table[ParametricPlot3D[refl[{u, v, k}], {u, 0, 1}, {v, 0, 1}], 
{k, 1, 6}]\
\>", "Input",
  CellLabel->"In[28]:=",
  PageWidth->Infinity],

Cell["Show[%]", "Input",
  CellLabel->"In[29]:=",
  PageWidth->Infinity],

Cell[BoxData[
    \(tgr2\  = %\)], "Input",
  CellLabel->"In[30]:="],

Cell["The infinite point is mapped in the origin:", "Text"],

Cell[BoxData[
    \(Limit[refl[{0, 0, t}], \ t \[Rule] \ Infinity]\)], "Input",
  CellLabel->"In[31]:="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["6 Spiral surfaces", "Section"],

Cell[CellGroupData[{

Cell["6.1 The spiral  groups in the 3-Space", "Subsection",
  Evaluatable->False],

Cell["\<\
In what follows we need the package (automatically supplied by init.m)\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(Needs["\<spiralsf`\>"]\)], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \(\(?spiral\)\)], "Input",
  CellLabel->"In[17]:="],

Cell["\<\
The spiral groups are 1-parameter subgroups of the conformal group of the \
euclidean 3-space depending on a  parameter a:\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(MatrixForm[spiral[t, a]]\)], "Input",
  CellLabel->"In[59]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["6.2 Fixed Points and Orbits", "Subsection",
  Evaluatable->False],

Cell["The only fixed point is the origin:", "Text",
  Evaluatable->False],

Cell["D[spiral[t, a], t]", "Input",
  CellLabel->"In[19]:=",
  PageWidth->Infinity],

Cell["m0 = % /. t -> 0", "Input",
  CellLabel->"In[20]:=",
  PageWidth->Infinity],

Cell["Solve[m0.vec[x,3] == nul[3], vec[x,3]]", "Input",
  CellLabel->"In[21]:=",
  PageWidth->Infinity],

Cell["\<\
The orbit of a point pt is the loxodromic spiral on the circular conus \
generated by the line through the origin and pt.\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(ParametricPlot3D[
      spiral[t, \ 0.1] . {1, \ 0, \ 1}, \ {t, \ \(-8\)\ Pi, \ 2\ Pi}, \n
      PlotPoints\  -> \ 150]\)], "Input",
  CellLabel->"In[52]:="],

Cell[BoxData[
    \(conspir = \ %\)], "Input",
  CellLabel->"In[53]:="],

Cell[BoxData[
    \(con[t_, u_] := spiral[t, \ 0.1] . {1, \ 0, \ 1} u\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(ax\  = \ con[t, u]\)], "Input",
  CellLabel->"In[25]:="],

Cell[BoxData[
    \(ax[\([1]\)]^2 + ax[\([2]\)]^2 - ax[\([3]\)]^2\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \(conb[t_, u_] := spiral[t, \ 0.1] . {1, \ 0, b} u\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(bx\  = \ conb[t, u]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(bx[\([1]\)]^2 + bx[\([2]\)]^2 - \((bx[\([3]\)]^2)\)/b^2\)], "Input",
  CellLabel->"In[38]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[39]:="],

Cell["Here is the corresponding cone: ", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(ParametricPlot3D[
      con[t, u], \ {t, \ \(-6\)\ Pi, \ 0}, {u, \ 0, \ 2. }, \n
      PlotPoints\  -> \ 30]\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(conus\  = \ %\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(Show[{conus, conspir}]\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \(conspir2\  = \ %\)], "Input",
  CellLabel->"In[55]:="],

Cell["\<\
The circlar conus can be created as usual, independent of the spiral:\
\>", "Text"],

Cell[BoxData[
    \(ParametricPlot3D[{Cos[t], \ Sin[t], 1}*u, \ {t, 0, 2\ Pi}, {u, 0, 
        2. }]\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(conus2\  = \ %\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \(Show[{conus2, conspir}]\)], "Input",
  CellLabel->"In[57]:="],

Cell[BoxData[
    \(conspir3\  = \ %\)], "Input",
  CellLabel->"In[58]:="],

Cell["\<\
Apply MathGL3d (or MathLive via ThreeScript) for animating these graphics! \
(See section 7)\
\>", "Text"],

Cell[BoxData[
    \(MVShow3D[conspir2, \ MVLineTubeSize\  -> \ 0.01]\)], "Input",
  CellLabel->"In[61]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["6.3 The spiral cylinder", "Subsection",
  Evaluatable->False],

Cell["A cylinder erected on the spiral is given by", "Text"],

Cell["spy[u_, v_, r_:10] := {Cos[u], Sin[u], 1/v} Exp[u/r]", "Input",
  CellLabel->"In[5]:=",
  PageWidth->Infinity],

Cell["Of course, that is a comparatively simple surface:", "Text"],

Cell["\<\
ParametricPlot3D[spy[u, v,15], {u, 0, 8 Pi}, {v, 0.5, 5}, 
PlotPoints \[Rule] 90]\
\>", "Input",
  CellLabel->"In[6]:=",
  PageWidth->Infinity],

Cell[BoxData[
    \(spc\  = \ %\)], "Input",
  CellLabel->"In[64]:="],

Cell["\<\
An apparently simpler (the usual) parameter representation of a spiral  \
cylinder  is given by\
\>", "Text"],

Cell["\<\
linspy[u_, v_, r_:10] := {Cos[u], Sin[u], 0} Exp[u/r]+ stb[3,3]*v\
\>", "Input",
  CellLabel->"In[7]:=",
  PageWidth->Infinity],

Cell["Of course, that is a comparatively simple surface:", "Text"],

Cell["\<\
ParametricPlot3D[linspy[u, v,15], {u, 0, 8 Pi}, {v, 0, 5}, 
PlotPoints \[Rule] {110, 20}]\
\>", "Input",
  CellLabel->"In[8]:=",
  PageWidth->Infinity],

Cell["\<\
The spiral cylinder is of special interest in M\[ODoubleDot]bius geometry: it \
is a homogeneous surface. That means: the subgroup of the M\[ODoubleDot]bius \
group preserving the cylinder acts transitively on this manifold. Moreover, \
it admits an invariant metric of constant negative curvature. 
For visualizing the behavior at the infinite point, we reflect the spiral \
cylinder at the unit sphere\
\>", "Text"],

Cell["reflspy[u_, v_, r_:10] := refl[spy[u, v, r]]", "Input",
  CellLabel->"In[28]:=",
  PageWidth->Infinity],

Cell["refllinspy[u_, v_, r_:10] := refl[linspy[u, 1/v, r]]", "Input",
  CellLabel->"In[29]:=",
  PageWidth->Infinity],

Cell["\<\
and obtain a conformally equivalent, interesting surface (zoom the created  \
Graphics3D!):\
\>", "Text"],

Cell[BoxData[
    \(reflspy[u, v]\)], "Input",
  CellLabel->"In[30]:="],

Cell["\<\
ParametricPlot3D[reflspy[u, v], {u, 0, 8 Pi}, {v, -.5, .5}, 
PlotPoints -> 90]\
\>", "Input",
  CellLabel->"In[31]:=",
  PageWidth->Infinity],

Cell[BoxData[
    \(reflspcy\  = \ %\)], "Input",
  CellLabel->"In[77]:="],

Cell["\<\
ParametricPlot3D[refllinspy[u, v,15], {u, 0, 8 Pi}, {v, -.5, .5}, 
PlotPoints -> 90]\
\>", "Input",
  CellLabel->"In[32]:=",
  PageWidth->Infinity],

Cell[BoxData[
    \(refllinc\  = \ %\)], "Input",
  CellLabel->"In[33]:="],

Cell["\<\
Apply MathGl3d (or MathLive via ThreeScript) for animating this graphics! \
(See section 7)\
\>", "Text"],

Cell[BoxData[
    \(MVShow3D[reflspcy]\)], "Input",
  CellLabel->"In[88]:="],

Cell[BoxData[
    \(MVShow3D[refllinc]\)], "Input",
  CellLabel->"In[35]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["6.4 Spiral Surfaces", "Subsection",
  Evaluatable->False,
  FontSize->16],

Cell["\<\
The definition of the spiral surface generated by a curve xz[u] in the x, \
z-plane is given in spirals.m:\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(\(?spiralsurface\)\)], "Input",
  CellLabel->"In[60]:="],

Cell[BoxData[
    \(\(spiralsurface[func_, \ a_:  0.1]\)[u_, \ 
        t_]\  := \ \n\ \ \(\(spiral[t, \ a]\  . \ {\(func[u]\)[\([1]\)], \ 
          0, \ \(func[u]\)[\([2]\)]}\)\(\ \)\)\)], "Input",
  CellLabel->"In[88]:=",
  Evaluatable->False],

Cell["\<\
The corresponding plotting command is also imported from spirals.m:\
\>", "Text"],

Cell[BoxData[
    \(\(\(\ \ \)\(\(\(plotspiralsurface[func_, \ a_:  0.1]\)[\ t0_, \ t1_, \ 
        u0_, \ u1_, \ 
        opt___]\)\(\ \)\(:=\)\(\ \)\(\n\)\(\t\)\(ParametricPlot3D[
        Evaluate[\(spiralsurface[func, \ a]\)[u, \ t]], \ \n\t{t, \ t0, \ 
          t1}, {u, \ u0, \ u1}, \ opt]\)\(\ \)\)\)\)], "Input",
  CellLabel->"In[117]:=",
  Evaluatable->False],

Cell[CellGroupData[{

Cell["Examples generated by these definitions", "Subsubsection"],

Cell[BoxData[
    \(f1[s_] := 1\  + \ Sin[s]; f2[s_] := \ Cos[s]; 
    fct[s_] := \ {f1[s], f2[s]}\)], "Input",
  CellLabel->"In[61]:="],

Cell[BoxData[
    \(\(spiralsurface[fct, 2]\)[u, t]\)], "Input",
  CellLabel->"In[62]:="],

Cell[BoxData[
    \(\(spiralsurface[fct]\)[u, t]\)], "Input",
  CellLabel->"In[63]:="],

Cell[BoxData[
    \(\(plotspiralsurface[fct]\)[\(-4\)\ Pi, 2  Pi, 0, \ 2\ Pi, \ 
      PlotPoints\  \[Rule] \ 60]\)], "Input",
  CellLabel->"In[64]:="],

Cell[BoxData[
    \(ulitka\  = \ %\)], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \(MVShow3D[ulitka]\)], "Input",
  CellLabel->"In[99]:="],

Cell[BoxData[
    \(ThreeScript["\<ulitka.ts\>", \ ulitka]\)], "Input",
  CellLabel->"In[102]:="],

Cell[BoxData[
    \(xz[s_]\  := \ {3\  + s^2\ , 1 + s^3}\)], "Input",
  CellLabel->"In[144]:="],

Cell[BoxData[
    \(\(spiralsurface[xz, \  .5]\)[u, t]\)], "Input",
  CellLabel->"In[145]:="],

Cell[BoxData[
    \(\(plotspiralsurface[xz,  .22]\)[\(-2\)*Pi, \ 2*Pi, \ \(-2\), \ 2, \n
      PlotRange\  -> \ All, PlotPoints \[Rule] 50]\)], "Input",
  CellLabel->"In[149]:="],

Cell[BoxData[
    \(scubesf = \ %\)], "Input",
  CellLabel->"In[150]:="],

Cell[BoxData[
    \(MVShow3D[scubesf]\)], "Input",
  CellLabel->"In[151]:="],

Cell[BoxData[
    \(fxz[u_]\  := \ {1\  + \ 0.2*Cos[u], \ 0.3*Sin[u]}\)], "Input",
  CellLabel->"In[131]:="],

Cell[BoxData[
    \(\(spiralsurface[fxz, \  .3]\)[u, t]\)], "Input",
  CellLabel->"In[135]:="],

Cell[BoxData[
    \(\(plotspiralsurface[fxz, \  .17]\)[\(-2\)\ Pi, \ 4\ Pi, \ 0, \ 2\ Pi, \ 
      PlotPoints\  -> \ 60, \ PlotRange\  -> \ All]\)], "Input",
  CellLabel->"In[153]:=",
  FontFamily->"Times"],

Cell[BoxData[
    \(spsf1 = \ %\)], "Input",
  CellLabel->"In[154]:=",
  FontFamily->"Times"],

Cell[BoxData[
    \(MVShow3D[spsf1]\)], "Input",
  CellLabel->"In[156]:="],

Cell[BoxData[
    \(ThreeScript["\<spsf1.ts\>", spsf1]\)], "Input",
  CellLabel->"In[157]:="],

Cell["This plots a spiral section of a circular cone, 1 turn:", "Text",
  Evaluatable->False,
  FontWeight->"Plain",
  FontSlant->"Plain",
  FontTracking->"Plain",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1],
  FontVariations->{"Underline"->False,
  "Outline"->False,
  "Shadow"->False}],

Cell[BoxData[{
    \(\(fxz[u_]\  := {u, u};\)\), "\n", 
    \(\(plotspiralsurface[fxz, 0.2]\)[0, 2*Pi, \ 0, \ 1, \ 
      PlotPoints\  -> \ 45, \ PlotRange\  -> \ All]\)}], "Input",
  CellLabel->"In[158]:=",
  FontFamily->"Times"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
6.5 Spiral surfaces generated by Alfred Gray's curves collection\
\>", "Subsection",
  Evaluatable->False,
  FontSize->16],

Cell["\<\
To generate other spiral surfaces we use parameter representations of certain \
plane curves. The parameter representations of these and other curves can be \
found in   A. Gray\.b4s book [1].\
\>", "Text",
  Evaluatable->False,
  FontWeight->"Plain",
  FontSlant->"Plain",
  FontTracking->"Plain",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1],
  FontVariations->{"Underline"->False,
  "Outline"->False,
  "Shadow"->False}],

Cell[BoxData[{
    \(\ \(\(astroid[a_]\)[t_] := \ {a*Cos[t]^3, 
          a*Sin[t]^3}\ ;\)\ \ \ \ \ \ \ \ \ \ \ \ \ \n\), "\n", 
    \(\ \ \ \ \(\(astroid[a_, b_]\)[t_] := \ {a*Cos[t]^3, 
          b*Sin[t]^3};\)\n\), "\n", 
    \(\ \ \ \ \(\(astroid[n_, a_, b_]\)[t_] := \ {a*Cos[t]^n, 
          b*Sin[t]^n};\)\n\)}], "Input",
  CellLabel->"In[160]:="],

Cell[BoxData[
    \(ParametricPlot[\(astroid[1]\)[u], \ {u, \ 0, \ 2\ Pi}, \ 
      AspectRatio\  -> \ Automatic]\)], "Input",
  CellLabel->"In[163]:="],

Cell["\<\

spiralsurface[astroid[1]][u, t]
\
\>", "Input",
  CellLabel->"In[164]:=",
  PageWidth->Infinity,
  FontFamily->"Times",
  FontSize->15,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell["\<\

plotspiralsurface[astroid[1], 0.06][ 0, 4 Pi,0, 2*Pi,
PlotPoints -> 60, PlotRange -> All]
\
\>", "Input",
  CellLabel->"In[165]:=",
  PageWidth->Infinity,
  FontFamily->"Times",
  FontSize->15,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell[BoxData[
    \(ast\  = \ %\)], "Input",
  CellLabel->"In[166]:="],

Cell[BoxData[
    \(MVShow3D[ast]\)], "Input",
  CellLabel->"In[170]:="],

Cell[BoxData[
    \(\(fct[u_] := \ {2, \ 4}\  + \ 0.5\ \(astroid[1]\)[u];\)\)], "Input",
  CellLabel->"In[171]:="],

Cell[BoxData[
    \(\(plotspiralsurface[fct, \ 0.06]\)[\ 0, \ 4\ Pi, 0, \ 2*Pi, \n
      PlotPoints\  -> \ 40, \ PlotRange\  -> \ All]\)], "Input",
  CellLabel->"In[175]:="],

Cell[BoxData[
    \(spsf2 = %\)], "Input",
  CellLabel->"In[176]:="],

Cell[BoxData[
    \(MVShow3D[spsf2]\)], "Input",
  CellLabel->"In[177]:="],

Cell[BoxData[
    \(Clear[fct]\)], "Input",
  CellLabel->"In[178]:="],

Cell[BoxData[
    \(\(fct[u_] := \ {1.5\  + \ Sin[u], 
          1.5\  + \ Cos[u]\ Sin[u]};\)\)], "Input",
  CellLabel->"In[179]:="],

Cell[BoxData[
    \(\(plotspiralsurface[fct, \ 0.2]\)[\ \(-2\)\ Pi, \ 4\ Pi\ , \ 
      0, \ \ 2\ Pi, \ \nPlotRange\  -> \ All, \ 
      PlotPoints\  -> \ {25, \ 45}]\)], "Input",
  CellLabel->"In[180]:="],

Cell[BoxData[
    \(spsf3 = %\)], "Input",
  CellLabel->"In[84]:="],

Cell[BoxData[
    \(\(\(\ \)\(eight[
        u_] := \ {2, 2} + {Sin[u], Sin[u]*Cos[u]}\)\)\)], "Input",
  CellLabel->"In[181]:="],

Cell[BoxData[
    \(\(plotspiralsurface[eight, \ 0.3]\)[0, \ 4\ Pi, \ 0, \ 2\ Pi, \ \n\ 
      PlotRange\  -> \ All, \ PlotPoints\  -> \ {25, \ 45}]\)], "Input",
  CellLabel->"In[182]:="],

Cell[BoxData[
    \(spsf4 = \ %\)], "Input",
  CellLabel->"In[87]:="],

Cell[BoxData[{
    \(\(\(diamond[a_]\)[
          t_] := \ {2, \ 2}\  + \n\ \ \ \ \ \ \ \ {a*Cos[t]*Sqrt[Cos[t]^2], 
            a*Sin[t]*Sqrt[Sin[t]^2]};\)\n\), "\n", 
    \(\ \ \ \ \(\(diamond[n_, a_, b_]\)[
          t_] := \ {2, 
            2}\  + \n\ \ \ \ \ \ \ \ {a*Sqrt[Cos[t]^2]^\((n\  - \ 1)\)*
              Cos[t], \n\ \ \ \ \ \ \ \ \ b*Sqrt[Sin[t]^2]^\((n\  - \ 1)\)*
              Sin[t]};\)\)}], "Input",
  CellLabel->"In[183]:="],

Cell[BoxData[
    \(ParametricPlot[\(diamond[2, \ 1, \ 1]\)[u] // Evaluate, \ {u, \ 0.1, \ 
        2\ Pi\  + \ 0.1}, \nAspectRatio\  -> \ Automatic]\)], "Input",
  CellLabel->"In[185]:="],

Cell[BoxData[
    \(ParametricPlot[\(diamond[5, \ 1, \ 1]\)[u] // Evaluate, \ {u, \ 0.1, \ 
        2\ Pi\  + \ 0.1}, \nAspectRatio\  -> \ Automatic]\)], "Input",
  CellLabel->"In[91]:="],

Cell[BoxData[
    \(\(plotspiralsurface[diamond[4, \ 1, \ 1], \ 0.3]\)[\ 0, \ 4\ Pi, \ 
      0.1, \ 2\ Pi\  + \ 0.1, \ \n\ PlotRange\  -> \ All, \ 
      PlotPoints\  -> \ {25, \ 45}]\)], "Input",
  CellLabel->"In[186]:="],

Cell[BoxData[
    \(spsf5 = %\)], "Input",
  CellLabel->"In[93]:="],

Cell[BoxData[
    \(\(\(\ \)\(\(clothoidprime[n_, a_]\)[
        t_] := \ \n\ \ \ \ \ \ \ \ \ {a*Sin[t^\((n + 1)\)/\((n + 1)\)], 
        a*Cos[t^\((n + 1)\)/\((n + 1)\)]}\)\)\)], "Input",
  CellLabel->"In[187]:="],

Cell[BoxData[
    \(\(\(\ \)\(\(clothoid[n_, a_]\)[
        t_] := {1.8, \ 
          1.8}\  + \n\ \ \ \ \ \ \ \ \ Integrate[\(clothoidprime[n, a]\)[
            tt], {tt, 0, t}]\)\)\)], "Input",
  CellLabel->"In[188]:="],

Cell[BoxData[
    \(ParametricPlot[\(clothoid[1, \ 1]\)[u] // Evaluate, \ {u, \ \(-5\), \ 
        5}, \ AspectRatio\  -> \ Automatic]\)], "Input",
  CellLabel->"In[189]:="],

Cell[BoxData[
    \(\(plotspiralsurface[clothoid[\ 1, \ 1], \ 0.3]\)[\ \(-6\)\ Pi, \ \(-\ 
        Pi\), \(-6\), \ 6, \ \n\ PlotRange\  -> \ All, \ 
      PlotPoints\  -> \ {45, \ 45}]\)], "Input",
  CellLabel->"In[190]:="],

Cell[BoxData[
    \(spsf6\  = %\)], "Input",
  CellLabel->"In[191]:="],

Cell[BoxData[
    \(MVShow3D[%]\)], "Input",
  CellLabel->"In[192]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["7 ThreeScript, MathGL3d", "Section"],

Cell[TextData[{
  "If you want to generate *.ts files for animations outside of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", activate the next section. To use the animation program MathGL3d in this \
notebook, activate section 7.2."
}], "Text"],

Cell[CellGroupData[{

Cell["7.1 Threescript ", "Subsection"],

Cell["\<\
 The ThreeScript commands generate *.ts files appropriate  for certain \
animation programs. The good freeware program  MathGL3d  created by Jens-Peer \
Kuska (for Windows95, Linux and other platforms) can be downloaded from \
[2].\
\>", "Text",
  Evaluatable->False],

Cell["\<\
Under Windows the commercial 3D-Animation programs MathLive and \
DynamicVisualizer are available. To produce .ts-files one \
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(Needs["\<Graphics`ThreeScript`\>"]\)], "Input",
  CellLabel->"In[100]:="],

Cell[BoxData[
    \(\(?ThreeScript\)\)], "Input",
  CellLabel->"In[101]:="],

Cell["\<\
The standard extension of these files is ts: \"filename.ts\" or \
\"Path/filename.ts\"\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(ThreeScript["\<isofam.ts\>", \ isofam]\)], "Input",
  CellLabel->"In[408]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["7.2 MathGL3d", "Subsection"],

Cell[TextData[{
  "The free animation program MathGL3d by Jens-Peer Kuska [2] can be used \
from a command line and within a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session. A new version of this program is in preparation. Jens-Peer Kuska \
has sent me a new startup file OpenGLViewer.m, and friendly permitted to add \
it to the item \"Spheres\". Replace the old file OpenGLViewer.m by this new \
OpenGLViewer.m. If you did this, changes of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s ParametricPlot3D, especially the sometimes disturbing generating of \
MeshGraphics3D objects, will not take place.\nEstablish the link to the \
current ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session with the command:"
}], "Text"],

Cell[BoxData[
    \(Get["\<MathGL3d`OpenGLViewer`\>"]\)], "Input",
  CellLabel->"In[34]:="],

Cell["If you want to close the link, enter", "Text"],

Cell["MVClose[]", "Text",
  CellLabel->"In[44]:="],

Cell["\<\
The following command clears the Viewer window; this and other MathGL3d \
commands you may enter also with  palettes.\
\>", "Text"],

Cell[BoxData[{
    \(\(MVClear[];\)\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(MVShow3D[\[Placeholder], MVPolygonShading \[Rule] MVSmooth, 
      MVGrayBackground \[Rule] False, MVTexture \[Rule] None, 
      MVTextureMapType \[Rule] MVMeshUVMapping, 
      MVScaleTexture \[Rule] {1, 1}, 
      MVTranslateTextureTarget \[Rule] Automatic, 
      MVRotateTextureTarget \[Rule] Automatic, 
      MVScaleTextureTarget \[Rule] Automatic]\)}], "Input",
  CellLabel->"In[2]:="],

Cell[BoxData[
    \(MVShow3D[gre1\[IndentingNewLine], MVPointSphereSize \[Rule] 0.8, \ 
      MVLineTubeSize\  -> \ 0.03]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(MVShow3D[conspir3, \ MVLineTubeSize\  -> \ 0.03]\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(MVShow3D[tgr3, \ MVPointSphereSize\  \[Rule] \ 1.8]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(MVShow3D[ex123, MVPointSphereSize \[Rule] 1.8]\)], "Input",
  CellLabel->"In[69]:="],

Cell[BoxData[
    \(MVShow3D[tgr2]\)], "Input",
  CellLabel->"In[99]:="],

Cell[BoxData[
    \(Options[MVShow3D]\)], "Input",
  CellLabel->"In[56]:="],

Cell[BoxData[
    \(MVClose[]\)], "Input",
  CellLabel->"In[36]:="]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
AutoGeneratedPackage->Automatic,
WindowToolbars->{},
WindowSize->{889, 646},
WindowMargins->{{1, Automatic}, {Automatic, 2}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
StyleDefinitions -> "TutorialBook.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 158, 4, 70, "Title",
  Evaluatable->False],
Cell[1900, 57, 149, 4, 32, "Subtitle",
  Evaluatable->False],
Cell[2052, 63, 179, 4, 27, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[2256, 71, 144, 4, 36, "Subsubsection",
  Evaluatable->False],
Cell[2403, 77, 1604, 24, 240, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[4044, 106, 35, 0, 30, "Subsubsection"],
Cell[4082, 108, 168, 3, 48, "Text"],
Cell[4253, 113, 108, 3, 28, "Text"],
Cell[4364, 118, 286, 7, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[4687, 130, 33, 0, 30, "Subsubsection"],
Cell[4723, 132, 560, 8, 88, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[5320, 145, 32, 0, 30, "Subsubsection"],
Cell[5355, 147, 47, 0, 28, "Text"],
Cell[5405, 149, 90, 2, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[5532, 156, 33, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[5590, 160, 41, 0, 45, "Subsection"],
Cell[5634, 162, 350, 11, 252, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[6021, 178, 45, 0, 39, "Subsection"],
Cell[6069, 180, 334, 7, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[6440, 192, 33, 0, 39, "Subsection"],
Cell[6476, 194, 205, 5, 58, "Text"],
Cell[6684, 201, 56, 0, 27, "Text"],
Cell[6743, 203, 68, 2, 32, "Input"],
Cell[6814, 207, 282, 5, 46, "Text"],
Cell[7099, 214, 113, 2, 32, "Input"],
Cell[7215, 218, 97, 2, 27, "Text"],
Cell[7315, 222, 76, 2, 32, "Input"],
Cell[7394, 226, 130, 3, 27, "Text"],
Cell[7527, 231, 74, 2, 32, "Input"],
Cell[7604, 235, 156, 3, 31, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7797, 243, 65, 0, 45, "Subsection"],
Cell[7865, 245, 190, 5, 61, "Text"],
Cell[8058, 252, 153, 3, 28, "Text"],
Cell[8214, 257, 76, 2, 31, "Input"],
Cell[8293, 261, 156, 3, 32, "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8498, 270, 33, 0, 92, "Section"],

Cell[CellGroupData[{
Cell[8556, 274, 38, 0, 45, "Subsection"],
Cell[8597, 276, 553, 8, 88, "Text"],
Cell[9153, 286, 111, 3, 31, "Input",
  InitializationCell->True],
Cell[9267, 291, 276, 5, 48, "Text"],
Cell[9546, 298, 64, 2, 31, "Input"],
Cell[9613, 302, 82, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[9732, 309, 36, 0, 39, "Subsection"],

Cell[CellGroupData[{
Cell[9793, 313, 44, 0, 37, "Subsubsection"],
Cell[9840, 315, 67, 0, 28, "Text"],
Cell[9910, 317, 66, 2, 31, "Input"],
Cell[9979, 321, 63, 2, 31, "Input"],
Cell[10045, 325, 64, 2, 31, "Input"],
Cell[10112, 329, 66, 2, 31, "Input"],
Cell[10181, 333, 64, 2, 31, "Input"],
Cell[10248, 337, 70, 2, 31, "Input"],
Cell[10321, 341, 67, 2, 31, "Input"],
Cell[10391, 345, 66, 2, 31, "Input"],
Cell[10460, 349, 68, 2, 31, "Input"],
Cell[10531, 353, 71, 2, 31, "Input"],
Cell[10605, 357, 60, 0, 28, "Text"],
Cell[10668, 359, 73, 2, 31, "Input"],
Cell[10744, 363, 70, 2, 31, "Input"],
Cell[10817, 367, 67, 2, 31, "Input"],
Cell[10887, 371, 71, 2, 31, "Input"],
Cell[10961, 375, 74, 2, 31, "Input"],
Cell[11038, 379, 73, 2, 31, "Input"],
Cell[11114, 383, 66, 2, 31, "Input"],
Cell[11183, 387, 64, 2, 31, "Input"],
Cell[11250, 391, 93, 2, 28, "Text"],
Cell[11346, 395, 64, 2, 31, "Input"],
Cell[11413, 399, 67, 2, 31, "Input"],
Cell[11483, 403, 64, 2, 31, "Input"],
Cell[11550, 407, 91, 2, 31, "Input"],
Cell[11644, 411, 67, 2, 31, "Input"],
Cell[11714, 415, 82, 2, 31, "Input"],
Cell[11799, 419, 74, 2, 31, "Input"],
Cell[11876, 423, 72, 2, 31, "Input"],
Cell[11951, 427, 69, 2, 31, "Input"],
Cell[12023, 431, 68, 2, 31, "Input"],
Cell[12094, 435, 70, 2, 31, "Input"],
Cell[12167, 439, 69, 2, 31, "Input"],
Cell[12239, 443, 72, 2, 31, "Input"],
Cell[12314, 447, 78, 2, 31, "Input"],
Cell[12395, 451, 75, 2, 31, "Input"],
Cell[12473, 455, 72, 2, 31, "Input"],
Cell[12548, 459, 72, 0, 28, "Text"],
Cell[12623, 461, 69, 2, 31, "Input"],
Cell[12695, 465, 71, 2, 31, "Input"],
Cell[12769, 469, 73, 2, 31, "Input"],
Cell[12845, 473, 71, 2, 31, "Input"],
Cell[12919, 477, 69, 2, 31, "Input"],
Cell[12991, 481, 75, 2, 31, "Input"],
Cell[13069, 485, 72, 2, 31, "Input"],
Cell[13144, 489, 74, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[13255, 496, 47, 0, 37, "Subsubsection"],
Cell[13305, 498, 71, 0, 28, "Text"],
Cell[13379, 500, 79, 2, 31, "Input"],
Cell[13461, 504, 69, 2, 31, "Input"],
Cell[13533, 508, 76, 2, 31, "Input"],
Cell[13612, 512, 80, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[13729, 519, 41, 0, 31, "Subsubsection"],
Cell[13773, 521, 95, 2, 28, "Text"],
Cell[13871, 525, 69, 2, 31, "Input"],
Cell[13943, 529, 314, 7, 48, "Text",
  Evaluatable->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[14306, 542, 36, 0, 39, "Subsection"],

Cell[CellGroupData[{
Cell[14367, 546, 39, 0, 36, "Subsubsection"],
Cell[14409, 548, 55, 0, 28, "Text"],
Cell[14467, 550, 129, 3, 31, "Input",
  Evaluatable->False],
Cell[14599, 555, 23, 0, 28, "Text"],
Cell[14625, 557, 66, 2, 31, "Input"],
Cell[14694, 561, 62, 2, 31, "Input"],
Cell[14759, 565, 96, 2, 28, "Text"],
Cell[14858, 569, 125, 3, 31, "Input",
  Evaluatable->False],
Cell[14986, 574, 67, 2, 31, "Input"],
Cell[15056, 578, 70, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[15163, 585, 70, 0, 30, "Subsubsection"],
Cell[15236, 587, 335, 11, 30, "Text"],
Cell[15574, 600, 133, 6, 28, "Text"],
Cell[15710, 608, 69, 0, 28, "Text"],
Cell[15782, 610, 71, 2, 31, "Input"],
Cell[15856, 614, 66, 2, 31, "Input"],
Cell[15925, 618, 68, 0, 28, "Text"],
Cell[15996, 620, 82, 2, 31, "Input"],
Cell[16081, 624, 481, 11, 100, "Input",
  Evaluatable->False],
Cell[16565, 637, 102, 2, 35, "Input"],
Cell[16670, 641, 98, 2, 35, "Input"],
Cell[16771, 645, 114, 3, 28, "Text"],
Cell[16888, 650, 100, 2, 31, "Input"],
Cell[16991, 654, 73, 2, 31, "Input"],
Cell[17067, 658, 89, 2, 28, "Text"],
Cell[17159, 662, 139, 3, 31, "Input"],
Cell[17301, 667, 129, 3, 31, "Input"],
Cell[17433, 672, 322, 8, 31, "Input"],
Cell[17758, 682, 67, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[17862, 689, 34, 0, 30, "Subsubsection"],
Cell[17899, 691, 237, 7, 48, "Text"],
Cell[18139, 700, 242, 5, 77, "Input",
  Evaluatable->False],
Cell[18384, 707, 79, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[18500, 714, 52, 0, 30, "Subsubsection"],
Cell[18555, 716, 123, 5, 28, "Text"],
Cell[18681, 723, 65, 2, 31, "Input"],
Cell[18749, 727, 334, 5, 68, "Text"],
Cell[19086, 734, 79, 2, 31, "Input"],
Cell[19168, 738, 100, 2, 31, "Input"],
Cell[19271, 742, 97, 2, 31, "Input"],
Cell[19371, 746, 99, 2, 31, "Input"],
Cell[19473, 750, 97, 2, 31, "Input"],
Cell[19573, 754, 145, 3, 28, "Text"],
Cell[19721, 759, 93, 2, 31, "Input"],
Cell[19817, 763, 93, 2, 31, "Input"],
Cell[19913, 767, 105, 3, 28, "Text"],
Cell[20021, 772, 102, 2, 31, "Input"],
Cell[20126, 776, 104, 2, 31, "Input"],
Cell[20233, 780, 102, 2, 31, "Input"],
Cell[20338, 784, 98, 2, 31, "Input"],
Cell[20439, 788, 91, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[20567, 795, 33, 0, 30, "Subsubsection"],
Cell[20603, 797, 133, 3, 28, "Text"],
Cell[20739, 802, 72, 2, 31, "Input"],
Cell[20814, 806, 91, 2, 31, "Input"],
Cell[20908, 810, 70, 2, 31, "Input"],
Cell[20981, 814, 74, 2, 31, "Input"],
Cell[21058, 818, 30, 0, 28, "Text"],
Cell[21091, 820, 74, 2, 31, "Input"],
Cell[21168, 824, 105, 3, 28, "Text"],
Cell[21276, 829, 73, 2, 31, "Input"],
Cell[21352, 833, 75, 2, 31, "Input"]
}, Closed]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[21488, 842, 45, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[21558, 846, 77, 0, 44, "Subsection"],
Cell[21638, 848, 883, 24, 104, "Text",
  Evaluatable->False],
Cell[22524, 874, 71, 2, 31, "Input"],
Cell[22598, 878, 67, 0, 28, "Text"],
Cell[22668, 880, 563, 11, 192, "Input",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[23256, 895, 70, 0, 37, "Subsubsection"],
Cell[23329, 897, 67, 2, 31, "Input"],
Cell[23399, 901, 77, 2, 31, "Input"],
Cell[23479, 905, 67, 2, 31, "Input"],
Cell[23549, 909, 77, 2, 31, "Input"],
Cell[23629, 913, 87, 2, 31, "Input"],
Cell[23719, 917, 96, 2, 31, "Input"],
Cell[23818, 921, 155, 4, 31, "Input"],
Cell[23976, 927, 90, 2, 31, "Input"],
Cell[24069, 931, 138, 3, 28, "Text"],
Cell[24210, 936, 85, 2, 31, "Input"],
Cell[24298, 940, 101, 2, 31, "Input"],
Cell[24402, 944, 73, 2, 31, "Input"],
Cell[24478, 948, 90, 2, 28, "Text"],
Cell[24571, 952, 77, 2, 31, "Input"],
Cell[24651, 956, 102, 3, 28, "Text"],
Cell[24756, 961, 67, 2, 31, "Input"],
Cell[24826, 965, 77, 2, 31, "Input"],
Cell[24906, 969, 40, 0, 28, "Text"],
Cell[24949, 971, 64, 2, 31, "Input"],
Cell[25016, 975, 74, 2, 31, "Input"],
Cell[25093, 979, 57, 0, 28, "Text"],
Cell[25153, 981, 82, 2, 31, "Input"],
Cell[25238, 985, 98, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[25385, 993, 180, 4, 36, "Subsection",
  Evaluatable->False],
Cell[25568, 999, 443, 6, 68, "Text"],
Cell[26014, 1007, 281, 7, 51, "Input",
  InitializationCell->True],
Cell[26298, 1016, 90, 3, 29, "Input"],
Cell[26391, 1021, 90, 3, 29, "Input"],
Cell[26484, 1026, 90, 3, 29, "Input"],
Cell[26577, 1031, 221, 5, 49, "Input"],
Cell[26801, 1038, 97, 3, 34, "Input"],
Cell[26901, 1043, 452, 13, 48, "Text",
  Evaluatable->False],
Cell[27356, 1058, 380, 10, 49, "Input",
  InitializationCell->True],
Cell[27739, 1070, 93, 3, 29, "Input"],
Cell[27835, 1075, 164, 3, 28, "Text"],
Cell[28002, 1080, 153, 5, 30, "Input"],
Cell[28158, 1087, 242, 6, 30, "Input"],
Cell[28403, 1095, 98, 3, 34, "Input"],
Cell[28504, 1100, 107, 3, 34, "Input"],
Cell[28614, 1105, 98, 3, 34, "Input"],
Cell[28715, 1110, 238, 6, 48, "Text"],
Cell[28956, 1118, 74, 2, 31, "Input"],
Cell[29033, 1122, 124, 3, 34, "Input"],
Cell[29160, 1127, 174, 4, 28, "Text"],
Cell[29337, 1133, 117, 3, 34, "Input"],
Cell[29457, 1138, 335, 5, 48, "Text"],
Cell[29795, 1145, 193, 4, 32, "Input"],
Cell[29991, 1151, 62, 0, 28, "Text"],
Cell[30056, 1153, 174, 5, 29, "Input"],
Cell[30233, 1160, 65, 2, 31, "Input"],
Cell[30301, 1164, 70, 2, 31, "Input"],
Cell[30374, 1168, 75, 2, 31, "Input"],
Cell[30452, 1172, 67, 2, 31, "Input"],
Cell[30522, 1176, 71, 2, 31, "Input"],
Cell[30596, 1180, 217, 4, 48, "Text"],

Cell[CellGroupData[{
Cell[30838, 1188, 49, 0, 37, "Subsubsection"],
Cell[30890, 1190, 53, 0, 28, "Text"],
Cell[30946, 1192, 164, 4, 31, "Input",
  InitializationCell->True],
Cell[31113, 1198, 79, 2, 31, "Input"],
Cell[31195, 1202, 86, 2, 28, "Text"],
Cell[31284, 1206, 160, 4, 31, "Input",
  InitializationCell->True],
Cell[31447, 1212, 76, 2, 31, "Input"],
Cell[31526, 1216, 160, 4, 31, "Input",
  InitializationCell->True],
Cell[31689, 1222, 76, 2, 31, "Input"],
Cell[31768, 1226, 75, 2, 31, "Input"],
Cell[31846, 1230, 74, 2, 31, "Input"],
Cell[31923, 1234, 110, 3, 28, "Text"],
Cell[32036, 1239, 103, 2, 31, "Input"],
Cell[32142, 1243, 150, 3, 28, "Text"],
Cell[32295, 1248, 163, 4, 31, "Input"],
Cell[32461, 1254, 52, 0, 28, "Text"],
Cell[32516, 1256, 88, 2, 31, "Input"],
Cell[32607, 1260, 113, 2, 31, "Input"],
Cell[32723, 1264, 66, 2, 31, "Input"],
Cell[32792, 1268, 70, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[32899, 1275, 53, 0, 37, "Subsubsection"],
Cell[32955, 1277, 254, 5, 54, "Input"],
Cell[33212, 1284, 140, 3, 31, "Input"],
Cell[33355, 1289, 78, 2, 31, "Input"],
Cell[33436, 1293, 138, 3, 31, "Input"],
Cell[33577, 1298, 78, 2, 31, "Input"],
Cell[33658, 1302, 111, 3, 28, "Text"],
Cell[33772, 1307, 76, 2, 31, "Input"],
Cell[33851, 1311, 76, 2, 31, "Input"],
Cell[33930, 1315, 97, 2, 31, "Input"],
Cell[34030, 1319, 130, 3, 31, "Input"],
Cell[34163, 1324, 443, 9, 95, "Input"],
Cell[34609, 1335, 97, 2, 31, "Input"],
Cell[34709, 1339, 97, 2, 31, "Input"],
Cell[34809, 1343, 211, 4, 63, "Input"],
Cell[35023, 1349, 142, 3, 28, "Text"],
Cell[35168, 1354, 144, 3, 31, "Input"],
Cell[35315, 1359, 105, 3, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[35457, 1367, 68, 0, 37, "Subsubsection"],
Cell[35528, 1369, 318, 5, 68, "Text"],
Cell[35849, 1376, 120, 3, 31, "Input"],
Cell[35972, 1381, 156, 3, 28, "Text"],
Cell[36131, 1386, 351, 7, 100, "Input"],
Cell[36485, 1395, 72, 0, 28, "Text"],
Cell[36560, 1397, 93, 2, 31, "Input"],
Cell[36656, 1401, 77, 2, 31, "Input"],
Cell[36736, 1405, 100, 2, 31, "Input"],
Cell[36839, 1409, 182, 4, 48, "Text"],
Cell[37024, 1415, 106, 2, 31, "Input"],
Cell[37133, 1419, 88, 2, 31, "Input"],
Cell[37224, 1423, 70, 2, 31, "Input"],
Cell[37297, 1427, 107, 2, 31, "Input"],
Cell[37407, 1431, 89, 2, 31, "Input"],
Cell[37499, 1435, 85, 2, 28, "Text"],
Cell[37587, 1439, 71, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[37707, 1447, 78, 1, 42, "Subsection"],
Cell[37788, 1450, 153, 3, 28, "Text"],
Cell[37944, 1455, 75, 2, 31, "Input"],
Cell[38022, 1459, 252, 6, 54, "Input",
  Evaluatable->False],
Cell[38277, 1467, 81, 2, 31, "Input"],
Cell[38361, 1471, 74, 2, 31, "Input"],

Cell[CellGroupData[{
Cell[38460, 1477, 33, 0, 37, "Subsubsection"],
Cell[38496, 1479, 28, 0, 28, "Text"],
Cell[38527, 1481, 78, 2, 31, "Input"],
Cell[38608, 1485, 45, 0, 28, "Text"],
Cell[38656, 1487, 78, 2, 31, "Input"],
Cell[38737, 1491, 236, 4, 75, "Input"],
Cell[38976, 1497, 110, 2, 31, "Input"],
Cell[39089, 1501, 127, 3, 31, "Input"],
Cell[39219, 1506, 94, 2, 31, "Input"],
Cell[39316, 1510, 83, 2, 31, "Input"],
Cell[39402, 1514, 70, 2, 31, "Input"],
Cell[39475, 1518, 113, 2, 32, "Input"],
Cell[39591, 1522, 39, 0, 28, "Text"],
Cell[39633, 1524, 72, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[39766, 1533, 57, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[39848, 1537, 141, 4, 44, "Subsection"],
Cell[39992, 1543, 133, 3, 27, "Text"],
Cell[40128, 1548, 159, 4, 32, "Input",
  InitializationCell->True],
Cell[40290, 1554, 171, 4, 32, "Input",
  InitializationCell->True],
Cell[40464, 1560, 176, 4, 32, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[40665, 1568, 156, 5, 36, "Subsubsection"],
Cell[40824, 1575, 342, 6, 65, "Text"],
Cell[41169, 1583, 440, 10, 152, "Input",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[41646, 1598, 33, 0, 36, "Subsubsection"],
Cell[41682, 1600, 77, 2, 32, "Input"],
Cell[41762, 1604, 118, 3, 27, "Text"],
Cell[41883, 1609, 69, 2, 32, "Input"],
Cell[41955, 1613, 77, 2, 32, "Input"],
Cell[42035, 1617, 104, 2, 32, "Input"],
Cell[42142, 1621, 80, 2, 32, "Input"],
Cell[42225, 1625, 72, 2, 32, "Input"],
Cell[42300, 1629, 77, 2, 32, "Input"],
Cell[42380, 1633, 89, 2, 27, "Text"],
Cell[42472, 1637, 64, 2, 32, "Input"],
Cell[42539, 1641, 77, 2, 32, "Input"],
Cell[42619, 1645, 60, 0, 27, "Text"],
Cell[42682, 1647, 77, 2, 32, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[42808, 1655, 109, 3, 38, "Subsection"],
Cell[42920, 1660, 161, 3, 27, "Text"],
Cell[43084, 1665, 69, 2, 32, "Input"],
Cell[43156, 1669, 196, 5, 61, "Input",
  Evaluatable->False],
Cell[43355, 1676, 159, 3, 27, "Text"],
Cell[43517, 1681, 75, 2, 32, "Input"],
Cell[43595, 1685, 271, 6, 80, "Input",
  Evaluatable->False],
Cell[43869, 1693, 149, 3, 27, "Text"],
Cell[44021, 1698, 248, 5, 56, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[44294, 1707, 33, 0, 36, "Subsubsection"],
Cell[44330, 1709, 85, 2, 32, "Input"],
Cell[44418, 1713, 81, 2, 32, "Input"],
Cell[44502, 1717, 86, 2, 32, "Input"],
Cell[44591, 1721, 97, 2, 32, "Input"],
Cell[44691, 1725, 181, 3, 56, "Input"],
Cell[44875, 1730, 91, 2, 32, "Input"],
Cell[44969, 1734, 98, 2, 32, "Input"],
Cell[45070, 1738, 100, 2, 32, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[45219, 1746, 146, 4, 38, "Subsection"],

Cell[CellGroupData[{
Cell[45390, 1754, 174, 4, 37, "Subsubsection"],
Cell[45567, 1760, 556, 15, 48, "Text"],
Cell[46126, 1777, 82, 2, 31, "Input"],
Cell[46211, 1781, 763, 18, 88, "Text"],
Cell[46977, 1801, 72, 2, 31, "Input"],
Cell[47052, 1805, 528, 11, 192, "Input",
  Evaluatable->False],
Cell[47583, 1818, 139, 3, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[47759, 1826, 37, 0, 31, "Subsubsection"],
Cell[47799, 1828, 75, 2, 31, "Input"],
Cell[47877, 1832, 64, 2, 31, "Input"],
Cell[47944, 1836, 75, 2, 31, "Input"],
Cell[48022, 1840, 64, 2, 31, "Input"],
Cell[48089, 1844, 90, 2, 31, "Input"],
Cell[48182, 1848, 89, 2, 31, "Input"],
Cell[48274, 1852, 74, 2, 31, "Input"],
Cell[48351, 1856, 74, 2, 31, "Input"],
Cell[48428, 1860, 78, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[48543, 1867, 45, 0, 31, "Subsubsection"],
Cell[48591, 1869, 64, 2, 31, "Input"],
Cell[48658, 1873, 82, 2, 31, "Input"],
Cell[48743, 1877, 192, 4, 55, "Input"],
Cell[48938, 1883, 47, 0, 28, "Text"],
Cell[48988, 1885, 116, 2, 31, "Input"],
Cell[49107, 1889, 18, 0, 28, "Text"],
Cell[49128, 1891, 86, 2, 31, "Input"],
Cell[49217, 1895, 65, 2, 31, "Input"],
Cell[49285, 1899, 115, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[49437, 1906, 42, 0, 31, "Subsubsection"],
Cell[49482, 1908, 148, 3, 28, "Text"],
Cell[49633, 1913, 2120, 38, 468, "Input"],
Cell[51756, 1953, 111, 3, 28, "Text"],
Cell[51870, 1958, 88, 2, 31, "Input"],
Cell[51961, 1962, 99, 2, 31, "Input"],
Cell[52063, 1966, 99, 2, 31, "Input"],
Cell[52165, 1970, 83, 2, 31, "Input"],
Cell[52251, 1974, 72, 2, 31, "Input"],
Cell[52326, 1978, 83, 2, 31, "Input"],
Cell[52412, 1982, 55, 0, 28, "Text"],
Cell[52470, 1984, 85, 2, 31, "Input"],
Cell[52558, 1988, 65, 0, 28, "Text"],
Cell[52626, 1990, 76, 2, 31, "Input"],
Cell[52705, 1994, 2743, 48, 675, "Input"],
Cell[55451, 2044, 77, 2, 31, "Input"],
Cell[55531, 2048, 127, 3, 31, "Input"],
Cell[55661, 2053, 50, 0, 28, "Text"],
Cell[55714, 2055, 75, 2, 31, "Input"],
Cell[55792, 2059, 88, 2, 28, "Text"],
Cell[55883, 2063, 88, 2, 31, "Input"],
Cell[55974, 2067, 99, 2, 31, "Input"],
Cell[56076, 2071, 99, 2, 31, "Input"],
Cell[56178, 2075, 83, 2, 31, "Input"],
Cell[56264, 2079, 72, 2, 31, "Input"],
Cell[56339, 2083, 83, 2, 31, "Input"],
Cell[56425, 2087, 24, 0, 28, "Text"],
Cell[56452, 2089, 89, 2, 31, "Input"],
Cell[56544, 2093, 91, 2, 31, "Input"],
Cell[56638, 2097, 69, 0, 28, "Text"],
Cell[56710, 2099, 84, 2, 31, "Input"],
Cell[56797, 2103, 86, 2, 31, "Input"],
Cell[56886, 2107, 79, 2, 31, "Input"],
Cell[56968, 2111, 158, 4, 31, "Input"],
Cell[57129, 2117, 68, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[57234, 2124, 50, 0, 31, "Subsubsection"],
Cell[57287, 2126, 287, 5, 48, "Text"],
Cell[57577, 2133, 72, 2, 31, "Input"],
Cell[57652, 2137, 119, 3, 31, "Input"],
Cell[57774, 2142, 185, 4, 54, "Input"],
Cell[57962, 2148, 79, 2, 31, "Input"],
Cell[58044, 2152, 31, 0, 28, "Text"],
Cell[58078, 2154, 91, 2, 31, "Input"],
Cell[58172, 2158, 178, 4, 48, "Text"],
Cell[58353, 2164, 87, 2, 31, "Input"],
Cell[58443, 2168, 88, 2, 31, "Input"],
Cell[58534, 2172, 88, 2, 31, "Input"],
Cell[58625, 2176, 107, 2, 31, "Input"],
Cell[58735, 2180, 90, 2, 31, "Input"],
Cell[58828, 2184, 109, 2, 31, "Input"],
Cell[58940, 2188, 104, 2, 31, "Input"],
Cell[59047, 2192, 96, 2, 31, "Input"],
Cell[59146, 2196, 92, 2, 31, "Input"],
Cell[59241, 2200, 135, 3, 31, "Input"],
Cell[59379, 2205, 130, 3, 31, "Input"],
Cell[59512, 2210, 37, 0, 28, "Text"],
Cell[59552, 2212, 127, 3, 31, "Input"],
Cell[59682, 2217, 119, 3, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[59838, 2225, 43, 0, 31, "Subsubsection"],
Cell[59884, 2227, 441, 8, 68, "Text"],
Cell[60328, 2237, 83, 2, 31, "Input"],
Cell[60414, 2241, 88, 2, 31, "Input"],
Cell[60505, 2245, 74, 2, 31, "Input"],
Cell[60582, 2249, 119, 3, 31, "Input"],
Cell[60704, 2254, 70, 2, 31, "Input"],
Cell[60777, 2258, 63, 2, 31, "Input"],
Cell[60843, 2262, 102, 2, 31, "Input"],
Cell[60948, 2266, 71, 2, 31, "Input"],
Cell[61022, 2270, 151, 4, 31, "Input"],
Cell[61176, 2276, 86, 2, 28, "Text"],
Cell[61265, 2280, 161, 3, 54, "Input"],
Cell[61429, 2285, 119, 3, 31, "Input"],
Cell[61551, 2290, 102, 2, 31, "Input"],
Cell[61656, 2294, 71, 2, 31, "Input"],
Cell[61730, 2298, 151, 4, 31, "Input"],
Cell[61884, 2304, 73, 0, 28, "Text"],
Cell[61960, 2306, 197, 4, 54, "Input"],
Cell[62160, 2312, 119, 3, 31, "Input"],
Cell[62282, 2317, 102, 2, 31, "Input"],
Cell[62387, 2321, 163, 4, 31, "Input"],
Cell[62553, 2327, 22, 0, 28, "Text"],
Cell[62578, 2329, 242, 5, 77, "Input"],
Cell[62823, 2336, 73, 2, 31, "Input"],
Cell[62899, 2340, 156, 3, 54, "Input"],
Cell[63058, 2345, 89, 2, 31, "Input"],
Cell[63150, 2349, 126, 3, 28, "Text"],
Cell[63279, 2354, 69, 0, 28, "Text"],
Cell[63351, 2356, 72, 2, 31, "Input"],
Cell[63426, 2360, 88, 2, 31, "Input"],
Cell[63517, 2364, 60, 0, 28, "Text"],
Cell[63580, 2366, 91, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[63708, 2373, 111, 3, 31, "Subsubsection"],
Cell[63822, 2378, 154, 3, 28, "Text"],
Cell[63979, 2383, 75, 2, 31, "Input"],
Cell[64057, 2387, 334, 7, 48, "Text"],
Cell[64394, 2396, 102, 2, 31, "Input"],
Cell[64499, 2400, 113, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[64649, 2407, 33, 0, 37, "Subsubsection"],
Cell[64685, 2409, 73, 2, 31, "Input"],
Cell[64761, 2413, 57, 0, 28, "Text"],
Cell[64821, 2415, 100, 2, 31, "Input"],
Cell[64924, 2419, 55, 0, 28, "Text"],
Cell[64982, 2421, 105, 2, 31, "Input"],
Cell[65090, 2425, 76, 2, 31, "Input"],
Cell[65169, 2429, 76, 2, 31, "Input"],
Cell[65248, 2433, 95, 2, 31, "Input"],
Cell[65346, 2437, 85, 2, 28, "Text"],
Cell[65434, 2441, 208, 5, 77, "Input"],
Cell[65645, 2448, 283, 6, 77, "Input"],
Cell[65931, 2456, 128, 3, 28, "Text"],
Cell[66062, 2461, 342, 7, 77, "Input"],
Cell[66407, 2470, 73, 2, 31, "Input"],
Cell[66483, 2474, 94, 2, 31, "Input"],
Cell[66580, 2478, 182, 5, 28, "Text"],
Cell[66765, 2485, 99, 2, 31, "Input"],
Cell[66867, 2489, 190, 4, 54, "Input"],
Cell[67060, 2495, 378, 10, 48, "Text"],
Cell[67441, 2507, 95, 2, 31, "Input"],
Cell[67539, 2511, 105, 2, 31, "Input"],
Cell[67647, 2515, 186, 4, 54, "Input"],
Cell[67836, 2521, 106, 2, 31, "Input"],
Cell[67945, 2525, 57, 0, 28, "Text"],
Cell[68005, 2527, 239, 6, 77, "Input"],
Cell[68247, 2535, 66, 2, 31, "Input"],
Cell[68316, 2539, 72, 2, 31, "Input"],
Cell[68391, 2543, 74, 2, 31, "Input"],
Cell[68468, 2547, 93, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[68610, 2555, 56, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[68691, 2559, 56, 0, 37, "Subsubsection"],
Cell[68750, 2561, 135, 3, 28, "Text"],
Cell[68888, 2566, 74, 2, 31, "Input"],
Cell[68965, 2570, 588, 11, 261, "Input",
  Evaluatable->False],
Cell[69556, 2583, 88, 2, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[69681, 2590, 67, 0, 37, "Subsubsection"],
Cell[69751, 2592, 144, 3, 28, "Text"],
Cell[69898, 2597, 73, 2, 31, "Input"],
Cell[69974, 2601, 113, 2, 31, "Input"],
Cell[70090, 2605, 104, 2, 31, "Input"],
Cell[70197, 2609, 138, 3, 31, "Input"],
Cell[70338, 2614, 114, 2, 31, "Input"],
Cell[70455, 2618, 104, 2, 31, "Input"],
Cell[70562, 2622, 138, 3, 31, "Input"],
Cell[70703, 2627, 61, 0, 28, "Text"],
Cell[70767, 2629, 114, 2, 31, "Input"],
Cell[70884, 2633, 135, 3, 31, "Input"],
Cell[71022, 2638, 146, 3, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[71205, 2646, 66, 0, 31, "Subsubsection"],
Cell[71274, 2648, 206, 4, 48, "Text"],
Cell[71483, 2654, 532, 10, 192, "Input"],
Cell[72018, 2666, 64, 0, 28, "Text"],
Cell[72085, 2668, 143, 3, 31, "Input"],
Cell[72231, 2673, 182, 3, 54, "Input"],
Cell[72416, 2678, 135, 3, 31, "Input"],
Cell[72554, 2683, 86, 2, 31, "Input"],
Cell[72643, 2687, 113, 2, 31, "Input"],
Cell[72759, 2691, 70, 2, 31, "Input"],
Cell[72832, 2695, 72, 2, 31, "Input"],
Cell[72907, 2699, 88, 2, 31, "Input"],
Cell[72998, 2703, 182, 3, 54, "Input"],
Cell[73183, 2708, 73, 2, 31, "Input"],
Cell[73259, 2712, 80, 2, 31, "Input"],
Cell[73342, 2716, 71, 2, 31, "Input"],
Cell[73416, 2720, 108, 2, 31, "Input"],
Cell[73527, 2724, 99, 2, 28, "Text"],
Cell[73629, 2728, 63, 0, 28, "Text"],
Cell[73695, 2730, 138, 3, 31, "Input"],
Cell[73836, 2735, 129, 3, 31, "Input"],
Cell[73968, 2740, 107, 2, 31, "Input"],
Cell[74078, 2744, 138, 3, 28, "Text"],
Cell[74219, 2749, 228, 5, 77, "Input"],
Cell[74450, 2756, 220, 4, 54, "Input"],
Cell[74673, 2762, 71, 2, 31, "Input"],
Cell[74747, 2766, 180, 4, 54, "Input"],
Cell[74930, 2772, 114, 2, 31, "Input"],
Cell[75047, 2776, 71, 2, 31, "Input"],
Cell[75121, 2780, 73, 2, 31, "Input"],
Cell[75197, 2784, 79, 2, 31, "Input"],
Cell[75279, 2788, 75, 2, 31, "Input"],
Cell[75357, 2792, 112, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[75506, 2799, 43, 0, 31, "Subsubsection"],
Cell[75552, 2801, 178, 4, 48, "Text"],
Cell[75733, 2807, 862, 16, 307, "Input",
  InitializationCell->True],
Cell[76598, 2825, 296, 6, 77, "Input",
  InitializationCell->True],
Cell[76897, 2833, 419, 8, 169, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[77353, 2846, 40, 0, 37, "Subsubsection"],
Cell[77396, 2848, 159, 3, 31, "Input"],
Cell[77558, 2853, 70, 2, 31, "Input"],
Cell[77631, 2857, 70, 2, 31, "Input"],
Cell[77704, 2861, 70, 2, 31, "Input"],
Cell[77777, 2865, 71, 2, 31, "Input"],
Cell[77851, 2869, 139, 3, 31, "Input"],
Cell[77993, 2874, 70, 2, 31, "Input"],
Cell[78066, 2878, 70, 2, 31, "Input"],
Cell[78139, 2882, 62, 2, 31, "Input"],
Cell[78204, 2886, 80, 2, 31, "Input"],
Cell[78287, 2890, 75, 2, 31, "Input"],
Cell[78365, 2894, 83, 2, 31, "Input"],
Cell[78451, 2898, 76, 2, 31, "Input"],
Cell[78530, 2902, 113, 2, 31, "Input"],
Cell[78646, 2906, 62, 0, 28, "Text"],
Cell[78711, 2908, 109, 2, 31, "Input"],
Cell[78823, 2912, 69, 2, 31, "Input"],
Cell[78895, 2916, 70, 2, 31, "Input"],
Cell[78968, 2920, 137, 3, 31, "Input"],
Cell[79108, 2925, 69, 2, 31, "Input"],
Cell[79180, 2929, 143, 3, 31, "Input"],
Cell[79326, 2934, 30, 0, 28, "Text"],
Cell[79359, 2936, 88, 2, 31, "Input"],
Cell[79450, 2940, 111, 2, 31, "Input"],
Cell[79564, 2944, 69, 2, 31, "Input"],
Cell[79636, 2948, 69, 2, 31, "Input"],
Cell[79708, 2952, 79, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[79848, 2961, 34, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[79907, 2965, 60, 0, 44, "Subsection"],

Cell[CellGroupData[{
Cell[79992, 2969, 38, 0, 37, "Subsubsection"],
Cell[80033, 2971, 743, 11, 140, "Text"],
Cell[80779, 2984, 78, 2, 31, "Input"],
Cell[80860, 2988, 72, 2, 31, "Input"],
Cell[80935, 2992, 70, 2, 31, "Input"],
Cell[81008, 2996, 68, 2, 31, "Input"],
Cell[81079, 3000, 74, 2, 31, "Input"],
Cell[81156, 3004, 1853, 39, 652, "Input",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[83046, 3048, 37, 0, 37, "Subsubsection"],
Cell[83086, 3050, 51, 0, 28, "Text"],
Cell[83140, 3052, 78, 2, 31, "Input"],
Cell[83221, 3056, 119, 3, 31, "Input"],
Cell[83343, 3061, 153, 3, 31, "Input"],
Cell[83499, 3066, 55, 0, 28, "Text"],
Cell[83557, 3068, 127, 3, 31, "Input"],
Cell[83687, 3073, 74, 0, 28, "Text"],
Cell[83764, 3075, 116, 2, 31, "Input"],
Cell[83883, 3079, 181, 4, 54, "Input"],
Cell[84067, 3085, 66, 2, 31, "Input"],
Cell[84136, 3089, 99, 2, 31, "Input"],
Cell[84238, 3093, 68, 2, 31, "Input"],
Cell[84309, 3097, 95, 2, 28, "Text"],
Cell[84407, 3101, 130, 3, 31, "Input"],
Cell[84540, 3106, 156, 4, 31, "Input"],
Cell[84699, 3112, 181, 4, 54, "Input"],
Cell[84883, 3118, 66, 2, 31, "Input"],
Cell[84952, 3122, 60, 0, 28, "Text"],
Cell[85015, 3124, 129, 3, 31, "Input"],
Cell[85147, 3129, 68, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[85252, 3136, 67, 0, 37, "Subsubsection"],
Cell[85322, 3138, 185, 4, 48, "Text"],
Cell[85510, 3144, 113, 2, 31, "Input"],
Cell[85626, 3148, 73, 2, 31, "Input"],
Cell[85702, 3152, 94, 2, 31, "Input"],
Cell[85799, 3156, 64, 2, 31, "Input"],
Cell[85866, 3160, 103, 2, 31, "Input"],
Cell[85972, 3164, 105, 2, 31, "Input"],
Cell[86080, 3168, 105, 2, 31, "Input"],
Cell[86188, 3172, 103, 2, 31, "Input"],
Cell[86294, 3176, 103, 2, 31, "Input"],
Cell[86400, 3180, 105, 2, 31, "Input"],
Cell[86508, 3184, 105, 2, 31, "Input"],
Cell[86616, 3188, 151, 4, 31, "Input"],
Cell[86770, 3194, 76, 2, 31, "Input"],
Cell[86849, 3198, 91, 2, 31, "Input"],
Cell[86943, 3202, 162, 4, 54, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[87142, 3211, 66, 0, 37, "Subsubsection"],
Cell[87211, 3213, 72, 2, 31, "Input"],
Cell[87286, 3217, 157, 5, 28, "Text"],
Cell[87446, 3224, 73, 2, 31, "Input"],
Cell[87522, 3228, 88, 2, 28, "Text"],
Cell[87613, 3232, 131, 3, 31, "Input"],
Cell[87747, 3237, 149, 3, 54, "Input"],
Cell[87899, 3242, 74, 2, 31, "Input"],
Cell[87976, 3246, 93, 2, 31, "Input"],
Cell[88072, 3250, 115, 3, 28, "Text"],
Cell[88190, 3255, 196, 5, 54, "Input"],
Cell[88389, 3262, 69, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[88495, 3269, 40, 0, 37, "Subsubsection"],
Cell[88538, 3271, 141, 3, 28, "Text"],
Cell[88682, 3276, 78, 2, 31, "Input"],
Cell[88763, 3280, 213, 5, 54, "Input"],
Cell[88979, 3287, 82, 2, 31, "Input"],
Cell[89064, 3291, 165, 4, 31, "Input"],
Cell[89232, 3297, 66, 2, 31, "Input"],
Cell[89301, 3301, 79, 2, 31, "Input"],
Cell[89383, 3305, 76, 2, 31, "Input"],
Cell[89462, 3309, 120, 3, 28, "Text"],
Cell[89585, 3314, 78, 2, 31, "Input"],
Cell[89666, 3318, 186, 5, 54, "Input"],
Cell[89855, 3325, 88, 2, 31, "Input"],
Cell[89946, 3329, 71, 2, 31, "Input"],
Cell[90020, 3333, 70, 2, 31, "Input"],
Cell[90093, 3337, 73, 2, 31, "Input"],
Cell[90169, 3341, 147, 4, 31, "Input"],
Cell[90319, 3347, 66, 2, 31, "Input"],
Cell[90388, 3351, 81, 2, 31, "Input"],
Cell[90472, 3355, 74, 2, 31, "Input"],
Cell[90549, 3359, 66, 2, 31, "Input"],
Cell[90618, 3363, 82, 2, 31, "Input"],
Cell[90703, 3367, 81, 2, 31, "Input"],
Cell[90787, 3371, 65, 0, 28, "Text"],
Cell[90855, 3373, 117, 3, 30, "Input"],
Cell[90975, 3378, 75, 2, 31, "Input"],
Cell[91053, 3382, 79, 2, 31, "Input"],
Cell[91135, 3386, 186, 4, 48, "Text"],
Cell[91324, 3392, 78, 2, 31, "Input"],
Cell[91405, 3396, 166, 5, 54, "Input"],
Cell[91574, 3403, 79, 2, 31, "Input"],
Cell[91656, 3407, 77, 2, 31, "Input"],
Cell[91736, 3411, 71, 2, 31, "Input"],
Cell[91810, 3415, 75, 2, 31, "Input"],
Cell[91888, 3419, 68, 2, 31, "Input"],
Cell[91959, 3423, 41, 0, 28, "Text"],
Cell[92003, 3425, 164, 4, 31, "Input"],
Cell[92170, 3431, 67, 2, 31, "Input"],
Cell[92240, 3435, 66, 2, 31, "Input"],
Cell[92309, 3439, 102, 2, 31, "Input"],
Cell[92414, 3443, 80, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[92543, 3451, 98, 1, 44, "Subsection",
  Evaluatable->False],
Cell[92644, 3454, 331, 8, 144, "Text",
  Evaluatable->False],
Cell[92978, 3464, 71, 2, 31, "Input"],

Cell[CellGroupData[{
Cell[93074, 3470, 55, 1, 37, "Subsubsection",
  Evaluatable->False],
Cell[93132, 3473, 82, 2, 31, "Input"],
Cell[93217, 3477, 104, 2, 31, "Input"],
Cell[93324, 3481, 78, 2, 31, "Input"],
Cell[93405, 3485, 71, 2, 31, "Input"],
Cell[93479, 3489, 107, 2, 31, "Input"],
Cell[93589, 3493, 74, 2, 31, "Input"],
Cell[93666, 3497, 219, 5, 54, "Input"],
Cell[93888, 3504, 69, 2, 31, "Input"],
Cell[93960, 3508, 71, 2, 31, "Input"],
Cell[94034, 3512, 97, 2, 31, "Input"],
Cell[94134, 3516, 82, 2, 31, "Input"],
Cell[94219, 3520, 68, 2, 31, "Input"],
Cell[94290, 3524, 219, 5, 54, "Input"],
Cell[94512, 3531, 78, 2, 31, "Input"],
Cell[94593, 3535, 67, 2, 31, "Input"],
Cell[94663, 3539, 89, 2, 31, "Input"],
Cell[94755, 3543, 60, 2, 31, "Input"],
Cell[94818, 3547, 74, 2, 31, "Input"],
Cell[94895, 3551, 73, 2, 31, "Input"],
Cell[94971, 3555, 167, 3, 31, "Input"],
Cell[95141, 3560, 65, 2, 31, "Input"],
Cell[95209, 3564, 98, 2, 31, "Input"],
Cell[95310, 3568, 186, 6, 28, "Text"],
Cell[95499, 3576, 130, 3, 31, "Input"],
Cell[95632, 3581, 168, 3, 28, "Text"],
Cell[95803, 3586, 92, 2, 31, "Input"],
Cell[95898, 3590, 107, 2, 31, "Input"],
Cell[96008, 3594, 50, 0, 28, "Text"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[96119, 3601, 65, 1, 60, "Section",
  Evaluatable->False],
Cell[96187, 3604, 154, 3, 27, "Text"],
Cell[96344, 3609, 116, 3, 36, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[96485, 3616, 34, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[96544, 3620, 30, 0, 37, "Subsubsection"],
Cell[96577, 3622, 76, 2, 31, "Input"],
Cell[96656, 3626, 75, 2, 31, "Input"],
Cell[96734, 3630, 70, 2, 31, "Input"],
Cell[96807, 3634, 119, 3, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[96963, 3642, 82, 1, 31, "Subsubsection"],
Cell[97048, 3645, 98, 2, 28, "Text"],
Cell[97149, 3649, 136, 3, 31, "Input"],
Cell[97288, 3654, 70, 2, 31, "Input"],
Cell[97361, 3658, 147, 5, 59, "Input"],
Cell[97511, 3665, 72, 2, 35, "Input"],
Cell[97586, 3669, 68, 2, 31, "Input"],
Cell[97657, 3673, 59, 0, 28, "Text"],
Cell[97719, 3675, 104, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[97884, 3684, 36, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[97945, 3688, 81, 1, 44, "Subsection",
  Evaluatable->False],
Cell[98029, 3691, 116, 3, 28, "Text",
  Evaluatable->False],
Cell[98148, 3696, 79, 2, 31, "Input"],
Cell[98230, 3700, 69, 2, 31, "Input"],
Cell[98302, 3704, 168, 4, 28, "Text",
  Evaluatable->False],
Cell[98473, 3710, 82, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[98592, 3717, 71, 1, 44, "Subsection",
  Evaluatable->False],
Cell[98666, 3720, 73, 1, 28, "Text",
  Evaluatable->False],
Cell[98742, 3723, 83, 2, 35, "Input"],
Cell[98828, 3727, 81, 2, 35, "Input"],
Cell[98912, 3731, 103, 2, 35, "Input"],
Cell[99018, 3735, 167, 4, 28, "Text",
  Evaluatable->False],
Cell[99188, 3741, 177, 4, 54, "Input"],
Cell[99368, 3747, 71, 2, 31, "Input"],
Cell[99442, 3751, 107, 2, 31, "Input"],
Cell[99552, 3755, 76, 2, 31, "Input"],
Cell[99631, 3759, 103, 2, 31, "Input"],
Cell[99737, 3763, 69, 2, 31, "Input"],
Cell[99809, 3767, 106, 2, 31, "Input"],
Cell[99918, 3771, 77, 2, 31, "Input"],
Cell[99998, 3775, 113, 2, 31, "Input"],
Cell[100114, 3779, 69, 2, 31, "Input"],
Cell[100186, 3783, 70, 1, 28, "Text",
  Evaluatable->False],
Cell[100259, 3786, 166, 4, 54, "Input"],
Cell[100428, 3792, 71, 2, 31, "Input"],
Cell[100502, 3796, 80, 2, 31, "Input"],
Cell[100585, 3800, 74, 2, 31, "Input"],
Cell[100662, 3804, 93, 2, 28, "Text"],
Cell[100758, 3808, 138, 3, 31, "Input"],
Cell[100899, 3813, 72, 2, 31, "Input"],
Cell[100974, 3817, 81, 2, 31, "Input"],
Cell[101058, 3821, 74, 2, 31, "Input"],
Cell[101135, 3825, 116, 3, 28, "Text"],
Cell[101254, 3830, 106, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[101397, 3837, 67, 1, 44, "Subsection",
  Evaluatable->False],
Cell[101467, 3840, 60, 0, 28, "Text"],
Cell[101530, 3842, 116, 2, 35, "Input"],
Cell[101649, 3846, 66, 0, 28, "Text"],
Cell[101718, 3848, 153, 5, 59, "Input"],
Cell[101874, 3855, 69, 2, 31, "Input"],
Cell[101946, 3859, 119, 3, 28, "Text"],
Cell[102068, 3864, 137, 4, 35, "Input"],
Cell[102208, 3870, 66, 0, 28, "Text"],
Cell[102277, 3872, 161, 5, 59, "Input"],
Cell[102441, 3879, 427, 7, 80, "Text"],
Cell[102871, 3888, 109, 2, 35, "Input"],
Cell[102983, 3892, 117, 2, 35, "Input"],
Cell[103103, 3896, 115, 3, 28, "Text"],
Cell[103221, 3901, 71, 2, 31, "Input"],
Cell[103295, 3905, 151, 5, 59, "Input"],
Cell[103449, 3912, 74, 2, 31, "Input"],
Cell[103526, 3916, 157, 5, 59, "Input"],
Cell[103686, 3923, 74, 2, 31, "Input"],
Cell[103763, 3927, 115, 3, 28, "Text"],
Cell[103881, 3932, 76, 2, 31, "Input"],
Cell[103960, 3936, 76, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[104073, 3943, 79, 2, 39, "Subsection",
  Evaluatable->False],
Cell[104155, 3947, 152, 4, 28, "Text",
  Evaluatable->False],
Cell[104310, 3953, 76, 2, 31, "Input"],
Cell[104389, 3957, 246, 5, 54, "Input",
  Evaluatable->False],
Cell[104638, 3964, 91, 2, 28, "Text"],
Cell[104732, 3968, 368, 7, 77, "Input",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[105125, 3979, 64, 0, 37, "Subsubsection"],
Cell[105192, 3981, 136, 3, 31, "Input"],
Cell[105331, 3986, 89, 2, 31, "Input"],
Cell[105423, 3990, 86, 2, 31, "Input"],
Cell[105512, 3994, 151, 3, 31, "Input"],
Cell[105666, 3999, 72, 2, 31, "Input"],
Cell[105741, 4003, 74, 2, 31, "Input"],
Cell[105818, 4007, 97, 2, 31, "Input"],
Cell[105918, 4011, 95, 2, 31, "Input"],
Cell[106016, 4015, 93, 2, 31, "Input"],
Cell[106112, 4019, 178, 3, 54, "Input"],
Cell[106293, 4024, 72, 2, 31, "Input"],
Cell[106368, 4028, 76, 2, 31, "Input"],
Cell[106447, 4032, 108, 2, 31, "Input"],
Cell[106558, 4036, 94, 2, 31, "Input"],
Cell[106655, 4040, 206, 4, 32, "Input"],
Cell[106864, 4046, 93, 3, 32, "Input"],
Cell[106960, 4051, 74, 2, 31, "Input"],
Cell[107037, 4055, 93, 2, 31, "Input"],
Cell[107133, 4059, 297, 9, 44, "Text",
  Evaluatable->False],
Cell[107433, 4070, 230, 5, 56, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[107712, 4081, 132, 4, 45, "Subsection",
  Evaluatable->False],
Cell[107847, 4087, 442, 13, 64, "Text",
  Evaluatable->False],
Cell[108292, 4102, 354, 7, 146, "Input"],
Cell[108649, 4111, 152, 3, 31, "Input"],
Cell[108804, 4116, 223, 11, 96, "Input"],
Cell[109030, 4129, 281, 12, 119, "Input"],
Cell[109314, 4143, 70, 2, 31, "Input"],
Cell[109387, 4147, 72, 2, 31, "Input"],
Cell[109462, 4151, 114, 2, 31, "Input"],
Cell[109579, 4155, 173, 3, 54, "Input"],
Cell[109755, 4160, 68, 2, 31, "Input"],
Cell[109826, 4164, 74, 2, 31, "Input"],
Cell[109903, 4168, 69, 2, 31, "Input"],
Cell[109975, 4172, 132, 3, 31, "Input"],
Cell[110110, 4177, 205, 4, 54, "Input"],
Cell[110318, 4183, 67, 2, 31, "Input"],
Cell[110388, 4187, 129, 3, 31, "Input"],
Cell[110520, 4192, 187, 3, 54, "Input"],
Cell[110710, 4197, 69, 2, 31, "Input"],
Cell[110782, 4201, 446, 9, 146, "Input"],
Cell[111231, 4212, 188, 3, 54, "Input"],
Cell[111422, 4217, 187, 3, 54, "Input"],
Cell[111612, 4222, 223, 4, 54, "Input"],
Cell[111838, 4228, 67, 2, 31, "Input"],
Cell[111908, 4232, 214, 4, 54, "Input"],
Cell[112125, 4238, 221, 5, 54, "Input"],
Cell[112349, 4245, 173, 3, 54, "Input"],
Cell[112525, 4250, 223, 4, 54, "Input"],
Cell[112751, 4256, 70, 2, 31, "Input"],
Cell[112824, 4260, 70, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[112943, 4268, 42, 0, 60, "Section"],
Cell[112988, 4270, 259, 6, 48, "Text"],

Cell[CellGroupData[{
Cell[113272, 4280, 38, 0, 44, "Subsection"],
Cell[113313, 4282, 277, 6, 48, "Text",
  Evaluatable->False],
Cell[113593, 4290, 170, 4, 28, "Text",
  Evaluatable->False],
Cell[113766, 4296, 93, 2, 31, "Input"],
Cell[113862, 4300, 75, 2, 31, "Input"],
Cell[113940, 4304, 132, 4, 28, "Text",
  Evaluatable->False],
Cell[114075, 4310, 97, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[114209, 4317, 34, 0, 38, "Subsection"],
Cell[114246, 4319, 771, 17, 120, "Text"],
Cell[115020, 4338, 91, 2, 31, "Input"],
Cell[115114, 4342, 52, 0, 28, "Text"],
Cell[115169, 4344, 50, 1, 28, "Text"],
Cell[115222, 4347, 141, 3, 28, "Text"],
Cell[115366, 4352, 481, 9, 146, "Input"],
Cell[115850, 4363, 159, 3, 54, "Input"],
Cell[116012, 4368, 106, 2, 31, "Input"],
Cell[116121, 4372, 109, 2, 31, "Input"],
Cell[116233, 4376, 104, 2, 31, "Input"],
Cell[116340, 4380, 72, 2, 31, "Input"],
Cell[116415, 4384, 75, 2, 31, "Input"],
Cell[116493, 4388, 67, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

