(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    202340,       7166]*)
(*NotebookOutlinePosition[    203819,       7208]*)
(*  CellTagsIndexPosition[    203775,       7204]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[" Geometry of Circles", "Title",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center],

Cell["by", "Subsubtitle",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell[CellGroupData[{

Cell["Rolf Sulanke", "Subtitle",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center],

Cell["April 14, 1999, last revision November 30, 2000", "Subsubtitle",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center],

Cell[CellGroupData[{

Cell["Summary", "Subsubsection",
  TextAlignment->Left],

Cell[TextData[{
  "This notebook describes the Moebius invariants for pairs of circles in the \
3-sphere, or, by stereographical projection, in the euclidean 3-space. \
Section 2 contains the needed concepts for the euclidean geometry of circles \
in the 3-space (or in the 3-sphere). On the level of the euclidean 3-space we \
give  a parametrization of the 6-dimensional space of circles. We construct \
two plot commands which plot the circles corresponding to these parameters. \
Furthermore, we define functions giving the circle, and the euclidean \
invariants radius, center and position vector of the circle through three \
points on the euclidean level. As an application, a plot command for tubes of \
general space curves is developed. Section 3 gives the basic concepts for \
circles in the 3-dimensional M\[ODoubleDot]bius space. The circles are \
represented by  2-dimensional euclidean subspaces of the 5-dimensional \
pseudo-euclidean vector space of index 1, which are defined by orthonormal \
pairs of vectors. By stereographical projection of the 3-sphere onto the \
euclidean 3-space the circles  become usual euclidean 3D-graphics (important \
for considering the mutual position of circles in the space). We construct \
procedures giving an adapted frame of the euclidean 2-space in terms of the \
euclidean circle invariants, and vice versa giving the circle as function of \
the euclidean 2-space.  In Section 4 we define a complete system of M\
\[ODoubleDot]bius-geometric invariants for pairs of circles in the 3-sphere, \
and try to find out their geometric meaning. Section 5 gives the normal forms \
of the circle pairs in relation to this system of invariants. Geodesics  are \
studied as circle orbits and plotted in section 6.  The last section imports  \
ThreeScript and MathGL3d, which can be applied for animations of the created \
3D-graphics. \nMany, but not all constructs developed in this notebook are \
collected in the package mcirc.m.\nThis notebook is related to the notebooks \
eusphere.nb, which contains the metric geometry of spheres, and the notebook \
mspheres.nb, devoted to the M\[ODoubleDot]bius geometry of spheres. It uses \
tools of  pseudo-euclidean linear algebra, developed in the notebook \
pseuklid.nb. \nFor differential geometry with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " in the euclidean 3-space see the textbook A. Gray [1]. The concepts of M\
\[ODoubleDot]bius geometry applied here are contained in the paper [4], see \
also my homepage [6]. A systematic description of the circle space  can be \
found  in the diploma paper of U. May [3]."
}], "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["References", "Subsubsection"],

Cell[TextData[{
  "[1] Alfred Gray. Modern Differential Geometry of Curves and Surfaces. CRC \
PRESS, 1993 (German translation: Spektrum Akademischer Verlag 1994)\n[2] \
Jens-Peer Kuska. http://phong.informatik.uni-leipzig.de/ ~kuska/mview3d.html.\
\n[3] U. May. Der Raum der Kreise in der Sph\[ADoubleDot]re. Diplomarbeit, \
Humboldt-Universit\[ADoubleDot]t, Berlin, 1993.\n[4] R. Sulanke. M\
\[ODoubleDot]bius invariants for pairs (",
  Cell[BoxData[
      \(TraditionalForm\`S\_1\%m\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`S\_2\%l\)]],
  ") of spheres in the M\[ODoubleDot]bius space ",
  Cell[BoxData[
      \(TraditionalForm\`S\^n\)]],
  ". Beitr\[ADoubleDot]ge zur Algebra und Geometrie.41 (2000), No.1, 233-246. \
Preprint No. 373, SFB 288, Berlin 1999.  \
http://xxx.lanl.gov/abs/math.MG/9902131. \n[5] R. Sulanke. Lie Algebras. \
http://www.mathsource.com/cgi-bin/msitem?0210-845\n[6] R. Sulanke. Homepage. \
http://www-irm.mathematik.hu-berlin.de/~sulanke/"
}], "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Keywords", "Subsubsection"],

Cell["\<\
pseudo-euclidean geometry,  Moebius geometry, 3D-circles, circles defined by \
subspaces, orthogonal circles, isospherical circles, stationary angles, \
eigenspheres, invariants of pairs of circles, normal forms of pairs of \
circles, geodesics in the circle space, MathGL3d.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Address", "Subsubsection"],

Cell["sulanke@mathematik.hu-berlin.de", "Text"],

Cell["\<\
The author will be very thankful for hints, corrections, comments.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[CellGroupData[{

Cell["The needed packages", "Subsection"],

Cell["\<\
Before initializing the present notebook ensure that the packages
euvec.m
pseuvec.m
mspher.m
mcirc.m
and the master package init.m are in your working directory. These packages \
are contained in the item \"Spheres\".
 For the present notebook the package spiralsf.m is not needed.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["3D graphics, animations", "Subsection"],

Cell[TextData[{
  "In the notebook command lines using ThreeScript, or MVShow3D appear. They \
may used for 3D-representations and animations of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Graphics3D objects. To apply them you must have the necessary software on \
your computer. Look in the last section. "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["For Windows", "Subsection"],

Cell[TextData[{
  "For the commands in this subsection:\n",
  StyleBox["DO NOT CONFIRM THE EVALUATION OF THE INITIALIZATION CELLS !!!",
    FontWeight->"Bold",
    FontColor->RGBColor[1, 0, 0]]
}], "Text"],

Cell[" If the result of the following command ", "Text"],

Cell[BoxData[
    \(Directory[]\)], "Input",
  CellLabel->"In[1]:="],

Cell["\<\
does not coincide with the working directory, e.g. \"C:\\Eigene \
Dateien\\mymath\\spheres.v4\", containing the needed packages, insert the \
Path of your working directory in the next command line and activate it \
first, without calling the initialization:\
\>", "Text"],

Cell[BoxData[
    \(SetDirectory["\<C:\\Eigene Dateien\\mymath\\spheres4\>"]\)], "Input",
  CellLabel->"In[33]:="],

Cell["\<\
If you want to use the package euvec.mpseuvec.m,mspher.m only,  carry out the \
next command.\
\>", "Text"],

Cell[BoxData[
    \(Needs["\<mspher`\>"]\)], "Input",
  CellLabel->"In[1]:="],

Cell["\<\
If you want  that all commands contained in the packages of the item \
\"Spheres\" are available, activate\
\>", "Text"],

Cell[BoxData[
    \(Get["\<init.m\>"]\)], "Input",
  CellLabel->"In[2]:="],

Cell[TextData[StyleBox["Now: Activite Kernel/Evaluation/Evaluate \
Initialization from the menu!",
  FontSize->18,
  FontColor->RGBColor[1, 0, 0]]], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["For Linux (and other  systems: not tested!)", "Subsection"],

Cell[TextData[StyleBox[" Activite Kernel/Evaluation/Evaluate Initialization \
from the menu!",
  FontSize->18,
  FontColor->RGBColor[1, 0, 0]]], "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1 Preliminaries", "Section"],

Cell[CellGroupData[{

Cell["1.1 Pseudo-euclidean vector algebra and other prerequisites", 
  "Subsection",
  FontFamily->"Times New Roman"],

Cell["\<\
M\[ODoubleDot]bius geometry is treated by tools of euclidean and \
pseudo-euclidean linear algebra. The basic concepts are developed in  the \
notebooks eusphere.nb,  pseuklid.nb and contained in the packages euvec.m,  \
pseuvec.m. Basic concepts of M\[ODoubleDot]bius geometry, in particular the M\
\[ODoubleDot]bius geometry of spheres, can be found in the notebook \
mspheres.nb and the package mspher.m. All constructs can be called as part of \
the master package init.m of this directory. The symbols are listed below, \
see section 1.2.\
\>", "Text"],

Cell["\<\
Before using them one has to fix the dimension dim of the underlying vector \
space and the index  ind of the (non-degenerate, symmetric) scalar product. \
These basic constants are now defined by\
\>", "Text"],

Cell[BoxData[
    \(dim\  = \ 5; \ ind\  = \ 1;\)], "Input",
  CellLabel->"In[1]:=",
  InitializationCell->True],

Cell["\<\
because in this notebook the 3-dimensional sphere is treated as the \
elliptical hyperquadric in the four-dimensional projective  space, which \
belongs to the five-dimensional real vector space. You can change these \
constants only before starting the evaluation of the initialization cells  by \
changing this command line.. But remember that these constants enter in \
almost all of the treated concepts; therefore they should be defined as \
global constants before using the packages.\
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"],

Cell["\<\
The matrix of the scalar products of any orthonormed basis of the \
pseudo-euclidean vector space is\
\>", "Text"],

Cell[BoxData[
    \(MatrixForm[Table[pssp[stb[i], \ stb[j]], {i, 5}, {j, 5}]]\)], "Input",
  CellLabel->"In[2]:=",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["1.2 The usages", "Subsection"],

Cell["\<\
After a package has been loaded you can get information about the  listed \
concepts in the usual way, e.g.\
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(\(?orthonorm\)\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(Context[orthonorm]\)], "Input",
  CellLabel->"In[15]:="],

Cell["\<\
For getting the usage and the complete definition, enter e.g.\
\>", "Text"],

Cell[BoxData[
    \(?? spt\)], "Input",
  CellLabel->"In[13]:="],

Cell["\<\
Since the names are prepended by the ContextNames the definition obtained in \
this way are hard to read. Therefore you will find some of them below in the \
original form in unevaluable cells.  It is recommended to look in the \
packages.\
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"],

Cell[CellGroupData[{

Cell["Symbols in mcirc.m", "Subsubsection"],

Cell["\<\
Here you can get information about the commands which are defined in mcirc.m:\
\
\>", "Text"],

Cell[BoxData[
    \(TableForm[{"\<adaptsplframe\>", "\<center\>", "\<posvec\>", \
"\<radius\>", "\<center3pts\>", "\<circle2spv\>", "\<circle3D\>", \
"\<circle3pts\>", "\<circlefamily\>", "\<circleplane\>", \
"\<circlespacevectors\>", "\<control\>", "\<gencircle\>", \
"\<plotcircle2spv\>", "\<plotcircle3D\>", "\<plotcircle3pts\>", \
"\<plotgencircle\>", "\<posvec3pts\>", "\<pp\>", "\<ppdet\>", "\<pptr\>", \
"\<pscomplement\>", "\<radius3pts\>", "\<threepoints\>", "\<tube\>", \
"\<unitvec\>", "\<vspace\>"}]\)], "Input",
  CellLabel->"In[15]:="],

Cell["To get the usages activate the next cell!", "Text"],

Cell[BoxData[
    \(\({\(?"\<adaptsplframe\>"\), \(?"\<center\>"\), \(?"\<posvec\>"\), \(?"\
\<radius\>"\), \(?"\<center3pts\>"\), \ \(?"\<circle2spv\>"\), \ \
\(?"\<circle3D\>"\), \(?"\<circle3pts\>"\), \(?"\<circlefamily\>"\), \
\(?"\<circleplane\>"\), \(?"\<circlespacevectors\>"\), \ \(?"\<control\>"\), \
\ \(?"\<gencircle\>"\), \ \ \ \(?"\<plotcircle2spv\>"\), \
\(?"\<plotcircle3D\>"\), \ \(?"\<plotgencircle\>"\), \
\(?"\<plotcircle3pts\>"\), \(?"\<posvec3pts\>"\), \ \(?"\<pp\>"\), \
\(?"\<ppdet\>"\), \ \(?"\<pptr\>"\), \(?"\<pscomplement\>"\), \
\(?"\<radius3pts\>"\), \(?"\<threepoints\>"\), \ \n\ \(?"\<tube\>"\), \ \
\(?"\<unitvec\>"\), \ \(?"\<vspace\>"\)};\)\)], "Input",
  CellLabel->"In[405]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Symbols in euvec.m", "Subsubsection"],

Cell[BoxData[
    \(TableForm[{"\<cross\>", "\<hyperplane\>", "\<invsterproj\>", \
"\<dotnorme\>", "\<esorthonorm\>", "\<innerprod\>", "\<normed\>", \
"\<print\>", "\<neglect\>", "\<nul\>", "\<ortho1\>", "\<orthobase\>", \
"\<orthoerg\>", "\<orthoframe\>", "\<outzero\>", "\<randomv\>", "\<rank\>", "\
\<renorm\>", "\<smoothing\>", "\<sphere\>", "\<sphereplot3D\>", \ \
"\<subsphere\>", "\<subspheremf\>", "\<sphereplot3D\>", \
"\<sphericalreflection\>", "\<stb\>", "\<sterproj\>", "\<tg4frame\>", "\<unit\
\>", "\<vec\>"}]\)], "Input",
  CellLabel->"In[19]:="],

Cell["To get the usages activate the next cell!", "Text"],

Cell[BoxData[
    \(\({\(?"\<cross\>"\), \(?"\<hyperplane\>"\), \(?"\<invsterproj\>"\), \
\(?"\<dotnorme\>"\), \(?"\<esorthonorm\>"\), \(?"\<innerprod\>"\), \
\(?"\<normed\>"\), \(?"\<print\>"\), \(?"\<neglect\>"\), \(?"\<nul\>"\), \(?"\
\<ortho1\>"\), \(?"\<orthobase\>"\), \(?"\<orthoerg\>"\), \(?"\<orthoframe\>"\
\), \(?"\<outzero\>"\), \(?"\<randomv\>"\), \(?"\<rank\>"\), \
\(?"\<renorm\>"\), \(?"\<smoothing\>"\), \(?"\<sphere\>"\), \
\(?"\<subsphere\>"\), \(?"\<subspheremf\>"\), \(?"\<sphereplot3D\>"\), \
\(?"\<sphericalreflection\>"\), \(?"\<stb\>"\), \(?"\<sterproj\>"\), \
\(?"\<tg4frame\>"\), \(?"\<unit\>"\), \(?"\<vec\>"\)};\)\)], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[
    \(?? invsterproj\)], "Input",
  CellLabel->"In[36]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Symbols in pseuvec.m", "Subsubsection"],

Cell[BoxData[
    \(\(\(TableForm[{"\<ch\>", "\<dual\>", "\<ide\>", "\<normalize\>", \
"\<orthonorm\>", "\<orthopair\>", "\<pr\>", "\<psCross\>", "\<pscross\>", \
"\<psfilter\>", "\<pssp\>"}]\)\(\ \)\)\)], "Input",
  CellLabel->"In[17]:="],

Cell["To get the usages activate the next cell!", "Text"],

Cell[BoxData[
    \(\({\(?"\<ch\>"\), \(?"\<dual\>"\), \(?"\<ide\>"\), \
\(?"\<normalize\>"\), \(?"\<orthonorm\>"\), \(?"\<orthopair\>"\), \(?"\<pr\>"\
\), \(?"\<psCross\>"\), \(?"\<pscross\>"\), \(?"\<psfilter\>"\), \
\(?"\<pssp\>"\)};\)\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(\(?psfilter\)\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(?? pscross\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(?? psCross\)], "Input",
  CellLabel->"In[16]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Symbols in mspher.m", "Subsubsection"],

Cell[BoxData[
    \(\(\(TableForm[{"\<confinv\>", \ "\<gencos\>", \ "\<invstepro\>", \ \
"\<planev\>", \ "\<plotsphere\>", \n"\<euklidsphere\>", \ "\<pstg5frame\>", \ \
"\<euklidsphereplot3D\>", \ "\<vradius\>", \ "\<vcenter\>", \ "\<spacevec\>", \
\n"\<sphvec\>", \ "\<spt\>", "\<spunit\>", \ "\<stepro\>"}]\)\(\ \)\)\)], \
"Input",
  CellLabel->"In[27]:="],

Cell["To get the usages activate the next cell!", "Text"],

Cell[BoxData[
    \(\({\(?"\<confinv\>"\), \ \(?"\<gencos\>"\), \ \(?"\<invstepro\>"\), \ \
\(?"\<planev\>"\), \ \(?"\<plotsphere\>"\), \n\(?"\<euklidsphere\>"\), \ \(?"\
\<pstg5frame\>"\), \ \(?"\<euklidsphereplot3D\>"\), \ \(?"\<vradius\>"\), \ \
\(?"\<vcenter\>"\), \ \(?"\<spacevec\>"\), \n\(?"\<sphvec\>"\), \ \
\(?"\<spt\>"\), \(?"\<spunit\>"\), \ \(?"\<stepro\>"\)};\)\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(?? spunit\)], "Input",
  CellLabel->"In[23]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.3 Some tools", "Subsection"],

Cell[CellGroupData[{

Cell["A general vector, random vectors", "Subsubsection"],

Cell[BoxData[
    \(vec[x_, \ n_:  dim]\  := \ Table[x[i], \ {i, \ n}]\)], "Input",
  CellLabel->"In[27]:=",
  Evaluatable->False,
  FontSize->15],

Cell[BoxData[
    \(vec[Sin]\)], "Input",
  CellLabel->"In[33]:="],

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(randomv[n_:  dim]\  := \ Table[Random[], {i, \ n}]\)], "Input",
  CellLabel->"In[33]:=",
  Evaluatable->False],

Cell[BoxData[
    \(randomv[]\)], "Input",
  CellLabel->"In[35]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Finding orthogonal pairs", "Subsubsection"],

Cell["The following definition is given in pseuklid.m:", "Text"],

Cell[BoxData[
    \(\(?orthopair\)\)], "Input",
  CellLabel->"In[36]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    \(orthopair[{0.5, 0.5, 1, 0, 0}]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(orthopair[{0.5, 0.5, 1, 1, 1}]\)], "Input",
  CellLabel->"In[38]:="],

Cell[BoxData[
    \(ow = %\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(Table[pssp[ow[\([i]\)], ow[\([j]\)]], {i, 2}, {j, 2}]\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(orthopair[stb[1]]\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(orthopair[stb[5]]\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(Context[orthopair]\)], "Input",
  CellLabel->"In[43]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Orthogonal complement in the euclidean 4-space", "Subsubsection"],

Cell[BoxData[
    \(\(?orthoerg\)\)], "Input",
  CellLabel->"In[44]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Control of orthonormality", "Subsubsection"],

Cell["If necessary,  one can apply the following control:", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(\(?control\)\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \(control[v_List, w_List, \ prod_:  pssp] := 
      Simplify[\((prod[v, v] - 1)\)\^2 + 
          prod[v, w]\^2 + \((prod[w, w] - 1)\)\^2]\)], "Input",
  CellLabel->"In[5]:=",
  Evaluatable->False],

Cell["\<\
If all is correct the result must be 0, or numerically very small. If the \
result depends on a parameter, integrate!\
\>", "Text",
  Evaluatable->False],

Cell["EXAMPLE:", "Text"],

Cell[BoxData[
    \(orthoerg[{Cos[t], Sin[t], \ 0, 0}, \ stb[2, 4]]\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(control[%[\([1]\)], %[\([2]\)], \ Dot]\)], "Input",
  CellLabel->"In[47]:="],

Cell[BoxData[
    \(control[stb[1], \ stb[5]]\)], "Input",
  CellLabel->"In[48]:="],

Cell[BoxData[
    \(\((va\  = \ randomv[]; \ Print[va, "\<, \>", \ ch[va]]; 
      vb\  = orthopair[va]; 
      Print[vb]; \[IndentingNewLine]control[vb[\([1]\)], \ 
        vb[\([2]\)]])\)\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(Remove[va, \ vb]\)], "Input",
  CellLabel->"In[50]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Smoothing", "Subsubsection"],

Cell[TextData[{
  "The following function makes an almost zero  number to be zero; the \
sensibility can be regulated by the optional second argument, whose default \
value is ",
  Cell[BoxData[
      \(10\^\(-10\)\)]],
  "; for the difference to Chop see the remark in the notebook eusphere.nb."
}], "Text"],

Cell[BoxData[
    \(\(smoothing[a_, b_:  10^\(-10\)] := 
        Module[{c = a}, If[b \[Equal] 1, Return[c], If[Abs[c] < b, c = 0]]; \ 
          c];\)\)], "Input",
  Evaluatable->False],

Cell[BoxData[
    \(Attributes[smoothing]\)], "Input",
  CellLabel->"In[51]:="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2 Euclidean geometry of circles in the 3-space ", "Section",
  FontSize->19],

Cell[CellGroupData[{

Cell["2.1 Parametrisation of the circle space. The general circle", \
"Subsection"],

Cell[CellGroupData[{

Cell["\<\
The unit sphere with center o and its tangential unit vectors\
\>", "Subsubsection"],

Cell["\<\
unit[2][{u,v}] is the usual parameter representation of the unit sphere in \
the 3-space with the geographical coordinates u, v:    0 <= u < 2Pi,  -Pi/2 < \
v < Pi/2.\
\>", "Text"],

Cell[BoxData[
    \(\(unit[2]\)[{u, v}]\)], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \(ParametricPlot3D[\(unit[2]\)[{u, v}], {u, \ 0, 2\ Pi}, \ {v, \(-Pi\)/2, 
        Pi/2}]\)], "Input",
  CellLabel->"In[28]:="],

Cell["For all values of u,v,t the following vector has norm 1:", "Text"],

Cell[BoxData[
    \(\(?unitvec\)\)], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(Context[unitvec]\)], "Input",
  CellLabel->"In[30]:="],

Cell[BoxData[
    \(\(unitvec[u, v]\)[t]\)], "Input",
  CellLabel->"In[47]:="],

Cell[BoxData[
    \(\(unitvec[u, v]\)[t] . \(unitvec[u, v]\)[t]\)], "Input",
  CellLabel->"In[32]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[33]:="],

Cell["unitvec[u,v][t] is orthogonal to unit[2][{u,v}]:", "Text"],

Cell[BoxData[
    \(\(unitvec[u, v]\)[t] . \(unit[2]\)[{u, v}]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[35]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell["\<\
For u,v fix, t variable, unitvec[u,v][t] is the parameter representation of \
the unit circle in the tangent vector space of the unit sphere at the point \
with parameters u,v. \
\>", "Text"],

Cell[BoxData[
    \(\(tangcircle[u_, v_]\)[
        t_]\  := \(unit[2]\)[{u, v}]\  + \ \(unitvec[u, v]\)[t]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(ParametricPlot3D[\(unitvec[0, Pi/3]\)[t], {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(ParametricPlot3D[\(tangcircle[0, Pi/3]\)[t], {t, 0, 2\ Pi}]\)], "Input",\

  CellLabel->"In[38]:="],

Cell[BoxData[
    \(Show[{%%, %}]\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(ParametricPlot3D[\(unitvec[0, Pi/2]\)[t], {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(ParametricPlot3D[\(unitvec[Pi/2, Pi/4]\)[t], {t, 0, 2\ Pi}]\)], "Input",\

  CellLabel->"In[41]:="],

Cell[BoxData[
    \(ParametricPlot3D[\(tangcircle[Pi/2, Pi/4]\)[t], {t, 0, 
        2\ Pi}]\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(Show[{%%, %}]\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(cp1\  = \ %\)], "Input",
  CellLabel->"In[44]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Definition of circle3D", "Subsubsection"],

Cell[BoxData[
    \(\(?circle3D\)\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \(\(circle3D[x_, y_, z_, u_, v_, r_]\)[t_] := {x, \ y, z}\  + \ 
        \(unitvec[u, v]\)[t]*r\)], "Input",
  CellLabel->"In[20]:=",
  Evaluatable->False],

Cell[BoxData[
    \(\(circle3D[x, y, z, u, v, r]\)[t]\)], "Input",
  CellLabel->"In[46]:="],

Cell[TextData[{
  "A similar, but more general  definition of Circle3D as a special case of \
an ellipse in the 3-dimensional space is contained in the package \
Hyperbolic/Circle3D.m by Sylvio Levy  (mathsource 1992). Furthermore, it \
includes a plot definition. This package seems to run only under  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " v. 2."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Example", "Subsubsection"],

Cell[BoxData[
    \(ParametricPlot3D[\(circle3D[2, 0, 0, Pi/2, \ Pi/4, \ 2]\)[t], {t, 0, 
        2\ Pi}]\)], "Input",
  CellLabel->"In[48]:="],

Cell[BoxData[
    \(\((u1\  = \ 2\ Pi\ Random[]; \ v1\  = \ Random[]\ Pi/2; \n
      ParametricPlot3D[\(circle3D[0, 0, 0, u1, \ v1, \ 1]\)[t], {t, 0, 
          2\ Pi}])\)\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(Table[\((u1\  = \ 2\ Pi\ Random[]; \ v1\  = \ Random[]\ Pi/2; \n
        ParametricPlot3D[\(circle3D[0, 0, 1, u1, \ v1, \ 1]\)[t], {t, 0, 
            2\ Pi}])\), {i, 10}]\)], "Input",
  CellLabel->"In[50]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[51]:="],

Cell[BoxData[
    \(Clear[u1, v1]\)], "Input",
  CellLabel->"In[53]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Another definition: gencircle", "Subsubsection"],

Cell["\<\
In the next definition the radius of the circle is the norm of v:\
\>", "Text"],

Cell[BoxData[
    \(\(?gencircle\)\)], "Input",
  CellLabel->"In[123]:="],

Cell[BoxData[
    \(\(gencircle[p_List, v_List]\)[t_] := 
      Module[{a\  = \ v . v, b1, \ b2}, \n\t\t\tIf[
          Length[p] != 3 || Length[v] != 3, \ 
          Return[Print["\<Incorrect arguments!!!\>"]]]; \n\t\ \ \tIf[
          a == \ 0, \ Return[p]]; \n\t\t\tIf[
          v[\([1]\)]\  == \ 0\  && \ v[\([2]\)]\  == 0, \ 
          Return[p\  + \ {Cos[t], \ Sin[t], \ 0}*
                Sqrt[a]]]; \n\t\t\tb1 = {\(-v[\([2]\)]\), \ v[\([1]\)], \ 0}/
            Sqrt[a\  - \ v[\([3]\)]^2]; \n\t\t\tb2\  = \ 
          Cross[v, \ b1]; \n\t\t\tReturn[
          p + b1*Sqrt[a]*Cos[t]\  + b2\ Sin[t]]]\)], "Input",
  CellLabel->"In[22]:=",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    \(ParametricPlot3D[\(gencircle[{2, 0, 0}, 2*\(unit[2]\)[{Pi/2, Pi/4}]]\)[
        t], {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[55]:="],

Cell[BoxData[
    \(randomv[3]\)], "Input",
  CellLabel->"In[56]:="],

Cell[BoxData[
    \(\((p1\  = \ randomv[3]; \ v1\  = \(-randomv[3]\); \n\t
      Print[p1, "\<, \>", \ v1, "\<, \>", \ Sqrt[v1 . v1]]; \ \n
      ParametricPlot3D[\(gencircle[p1, v1]\)[t], {t, 0, 2\ Pi}])\)\)], "Input",\

  CellLabel->"In[60]:="],

Cell[BoxData[
    \(Clear[p1, v1]\)], "Input",
  CellLabel->"In[61]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.2 The Plot definitions", "Subsection"],

Cell[BoxData[
    \(\(?circle3D\)\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(plotcircle3D[x_, y_, z_, u_, v_, r_, \ opts___] := \ 
      ParametricPlot3D[\n\t\tEvaluate[\(circle3D[x, y, z, u, v, r]\)[
            t]], \ {t, \ 0, \ 2\ Pi}, \ opts]\)], "Input",
  CellLabel->"In[26]:=",
  Evaluatable->False],

Cell[BoxData[
    \(\(?gencircle\)\)], "Input",
  CellLabel->"In[134]:="],

Cell[BoxData[
    \(plotgencircle[p_List, v_List, \ opts___] := \ 
      ParametricPlot3D[\n\t\tEvaluate[\(gencircle[p, v]\)[t]], \ {t, \ 0, \ 
          2\ Pi}, \ opts]\)], "Input",
  CellLabel->"In[27]:=",
  Evaluatable->False],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    \(plotgencircle[nul[3], \ stb[3, 3]]\)], "Input",
  CellLabel->"In[62]:="],

Cell[BoxData[
    \(plotgencircle[nul[3], \ 25. *stb[2, 3]]\)], "Input",
  CellLabel->"In[63]:="],

Cell[BoxData[
    \(plotgencircle[\ 2  randomv[3], 5\ randomv[3]]\)], "Input",
  CellLabel->"In[64]:="],

Cell[BoxData[
    \(plotcircle3D[1, 1, 1, Pi/2, 0, \ 2]\)], "Input",
  CellLabel->"In[65]:="],

Cell[BoxData[
    \(plotcircle3D[1, 5\ Random[], 1, Pi/2, Pi\ Random[], \ 2]\)], "Input",
  CellLabel->"In[66]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.3 Circles and lines through three points", "Subsection"],

Cell[CellGroupData[{

Cell["Definitions: Position vector, center, radius, and circle", \
"Subsubsection"],

Cell["\<\
The position vector of a circle is a unit vector orthogonal to the plane of \
the circle; it is uniquely defined up to sign, and it can be calculated from \
three points of the circle as follows:\
\>", "Text"],

Cell[BoxData[
    \(posvec3pts[p1_List, p2_List, p3_List]\  := \n\t
      Module[{\ a}, \n\t\ta\  = \ Cross[p2 - p1, \ p3 - p1]; \n\t\tIf[
          a\  \[NotEqual] nul[3], Return[smoothing[a/Sqrt[a . a]]], \ 
          Return[nul[3]], Return[smoothing[a/Sqrt[a . a]]]]]\)], "Input",
  CellLabel->"In[4]:=",
  Evaluatable->False],

Cell["\<\
For the calculation of the center m of the circle through three points we \
start with  the  formula\
\>", "Text"],

Cell[BoxData[
    \(m\  = \ 
      p3\  + \ \((p1\  - \ p3)\)\ x\  + \ \((p2\  - \ p3)\)\ y\)], "Input",
  Evaluatable->False],

Cell["Evaluating the conditions", "Text"],

Cell[BoxData[{
    \(\(\((m - p3)\) . \((m - p3)\) \[Equal] \ \((m - p1)\) . \((m - 
              p1)\);\)\), "\[IndentingNewLine]", 
    \(\(\((m - p3)\) . \((m - p3)\) \[Equal] \ \((m - p2)\) . \((m - 
              p2)\);\)\)}], "Input",
  Evaluatable->False],

Cell["\<\
we get a linear system for the unknowns x, y. By the \
Cauchy-Schwarz-inequality its determinant dn is zero if and only if the \
points p1, p2, p3 belong to a line. By definition,  then the center is {\
\[Infinity],\[Infinity],\[Infinity]}. Otherwise, Cramer\.b4s rule yields the \
solution.  These considerations lead to the following module:\
\>", "Text"],

Cell[BoxData[
    \(center3pts[p1_List, p2_List, p3_List] := \ 
      Module[{q1 = \ p1\  - \ p3, \ q2\  = p2\  - \ p3, dn, dn1, 
          dn2}, \[IndentingNewLine]dn\  = \ 
          Det[{{q1 . q1, \ q1 . q2}, {q1 . q2, 
                q2 . q2}}]; \[IndentingNewLine]If[dn\  \[Equal] \ 0, \ 
          Return[{Infinity, \ Infinity, 
              Infinity}]]; \[IndentingNewLine]dn1\  = \ 
          Det[{{q1 . q1/2, \ q1 . q2}, {q2 . q2/2, 
                q2 . q2}}]; \[IndentingNewLine]dn2\  = \ 
          Det[{{q1 . q1, \ q1 . q1/2}, {q1 . q2, 
                q2 . q2/2}}]; \[IndentingNewLine]Return[
          p3 + \((dn1*q1\  + \ dn2*q2)\)/dn]]\)], "Input",
  CellLabel->"In[5]:=",
  Evaluatable->False],

Cell["Knowing the center it is easy to calculate the radius:", "Text"],

Cell[BoxData[
    \(\(\(radius3pts[p1_List, p2_List, p3_List]\  := \n\t
      Module[{\ a}, \n\t\ta\  = \ center3pts[p1, p2, p3]; \n\t\tIf[
          a\  \[Equal] {Infinity, Infinity, Infinity}, \ 
          Return[Infinity]]; \n\t\tReturn[\ 
          Sqrt[\((p1 - a)\) . \((p1\  - \ 
                  a)\)]]]\)\(\n\)\(\t\t\)\)\)], "Input",
  CellLabel->"In[6]:=",
  Evaluatable->False],

Cell[TextData[{
  StyleBox["WARNING",
    FontColor->RGBColor[1, 0, 0]],
  ": Always, if there exist a line through the three points, e.g. if the \
points coincide, center3pts and radius3pts are infinite!"
}], "Text"],

Cell[BoxData[
    \(center3pts[stb[1, 3], stb[1, 3], stb[1, 3]]\)], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[
    \(radius3pts[stb[1, 3], stb[1, 3], stb[1, 3]]\)], "Input",
  CellLabel->"In[68]:="],

Cell["\<\
Now we  give a definition of a 3D-circle through three points. The circle may \
eventually be a line (circle of infinite radius):\
\>", "Text"],

Cell[BoxData[
    \(\(?circle3pts\)\)], "Input",
  CellLabel->"In[69]:="],

Cell[BoxData[
    \(\(circle3pts[p1_List, p2_List, p3_List]\)[t_]\  := \n\t
      Module[{q2\ , \ q3\ , \ a, \ m, r}, \[IndentingNewLine]If[
          p1\  \[Equal] \ {Infinity, Infinity, \ Infinity}, 
          Return[p2\  + \ t*\((p3 - p2)\)]]; \[IndentingNewLine]If[
          p2\  \[Equal] \ {Infinity, Infinity, \ Infinity}, 
          Return[p1\  + \ t*\((p3 - p1)\)]]; \[IndentingNewLine]If[
          p3\  \[Equal] \ {Infinity, Infinity, \ Infinity}, 
          Return[p1\  + \ t*\((p2 - p1)\)]]; \n
        m\  = \ center3pts[p1, p2, p3]; \[IndentingNewLine]If[
          m\  == \ {Infinity, Infinity, Infinity}, 
          If[smoothing[p2\  - p1]\  \[Equal] nul[3], \ 
            Return[p1\  + \ t*\((p3\  - \ p1)\)], \ 
            Return[p1\  + \ 
                t*\((p2\  - \ p1)\)]]]; \[IndentingNewLine]r\  = \ 
          Sqrt[\((p1 - m)\) . \((p1\  - \ m)\)]; \t\[IndentingNewLine]\t
        q2\  = \ p2 - p1; \ q3\  = \ p3 - p1; 
        a\  = \ Cross[q2, q3]; \n\t\tq3\  = \ Cross[a, q2]; \n\t\tq2\ \  = \ 
          q2/Sqrt[q2 . q2]; \n\t\tq3\  = \ \ q3/
            Sqrt[q3 . q3]; \n\t\tm\  + \((q2\ *Cos[t]\  + \ q3*Sin[t])\)*
            r]\)], "Input",
  CellLabel->"In[7]:=",
  Evaluatable->False],

Cell["Here is the corresponding plot command:", "Text"],

Cell[BoxData[
    \(plotcircle3pts[p1_List, p2_List, p3_List, opt___] := \ 
      ParametricPlot3D[
        Evaluate[\(circle3pts[p1, p2, p3]\)[t]], {t, \ \(-Pi\), \ Pi}, 
        opt]\)], "Input",
  CellLabel->"In[52]:=",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    \(\(circle3pts[{1, 0, 0}, {0, 1, 0}, {0, \(-1\), 0}]\)[t]\)], "Input",
  CellLabel->"In[70]:=",
  Evaluatable->True],

Cell[BoxData[
    \(plotcircle3pts[{1, 0, 0}, {0, 1, 0}, {0, \(-1\), 0}]\)], "Input",
  CellLabel->"In[71]:="],

Cell[BoxData[
    \(\(circle3pts[{3, 0, 0}, {3, 1, 0}, {3, \(-1\), 0}]\)[t]\)], "Input",
  CellLabel->"In[72]:=",
  Evaluatable->True],

Cell[BoxData[
    \(plotcircle3pts[{3, 0, 0}, {3, 1, 0}, {3, \(-1\), 0}]\)], "Input",
  CellLabel->"In[73]:="],

Cell[BoxData[{
    \(a = Random[]; \ b = Random[]; \ c = Random[];\), "\n", 
    \(\(Print[{a, b, c}];\)\), "\n", 
    \(pa = {a, 0, 0}; pb = {0, b, 0}; pc = {0, 0, c};\), "\n", 
    \(\(gr1 = plotcircle3pts[pa, pb, pc];\)\), "\n", 
    \(\(gr2\  = \ 
        Graphics3D[{PointSize[0.05], Point[pa], Point[pb], 
            Point[pc]}];\)\), "\n", 
    \(Show[{gr1, gr2}]\)}], "Input",
  CellLabel->"In[74]:=",
  Evaluatable->True],

Cell[BoxData[
    \(center3pts[pa, pb, pc]\)], "Input",
  CellLabel->"In[80]:="],

Cell[BoxData[
    \(radius3pts[pa, pb, pc]\)], "Input",
  CellLabel->"In[81]:="],

Cell[BoxData[
    \(posvec3pts[pa, pb, pc]\)], "Input",
  CellLabel->"In[82]:="],

Cell[BoxData[
    \(% . %\)], "Input",
  CellLabel->"In[83]:="],

Cell[BoxData[{
    \(a = Random[]; \ b = Random[];\), "\n", 
    \(\(Print[{a, b}];\)\), "\n", 
    \(pa = {a, 2*b, 0}; pb = {0, \(-b\), a}; 
    pc = \((pa\  + \ pb)\)/2;\), "\n", 
    \(\(gr1 = plotcircle3pts[pa, pb, pc];\)\), "\n", 
    \(\(gr2\  = \ 
        Graphics3D[{PointSize[0.03], Point[pa], Point[pb], 
            Point[pc]}];\)\), "\n", 
    \(Show[{gr1, gr2}]\)}], "Input",
  CellLabel->"In[84]:=",
  Evaluatable->True],

Cell[BoxData[
    \({pa, pb, pc}\)], "Input",
  CellLabel->"In[90]:="],

Cell["If the three points coincide, circle3pts returns the point:", "Text"],

Cell[BoxData[
    \(\(circle3pts[{1, 0, 0}, {1, 0, 0}, {1, 0, 0}]\)[t]\)], "Input",
  CellLabel->"In[91]:="],

Cell[BoxData[
    \(plotcircle3pts[{1, 0, 0}, {1, 0, 0}, {1, 0, 0}]\)], "Input",
  CellLabel->"In[92]:="],

Cell["\<\
If only two points coincide we get the line through these points:\
\>", "Text"],

Cell[BoxData[
    \(\(circle3pts[{1, 0, 0}, {1, 0, 0}, {2, 0, 0}]\)[t]\)], "Input",
  CellLabel->"In[93]:="],

Cell[BoxData[
    \(plotcircle3pts[{1, 0, 0}, {1, 0, 0}, {2, 0, 0}]\)], "Input",
  CellLabel->"In[94]:="],

Cell[BoxData[
    \(plotcircle3pts[{1, 3, 0}, {1, 3, 0}, {2, 0, 1}]\)], "Input",
  CellLabel->"In[95]:="],

Cell[BoxData[
    \(\(circle3pts[{1, 3, 0}, {1, 3, 0}, {2, 0, 1}]\)[t]\)], "Input",
  CellLabel->"In[96]:="],

Cell[BoxData[
    \(\(circle3pts[{1, 0, 0}, {2, 0, 0}, {1, 0, 0}]\)[t]\)], "Input",
  CellLabel->"In[97]:="],

Cell[BoxData[
    \(\(circle3pts[{1, 0, 0}, {2, 0, 0}, {2, 0, 0}]\)[t]\)], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \(posvec3pts[{1, 0, 0}, {0, 1, 0}, {0, \(-1\), 0}]\)], "Input",
  CellLabel->"In[99]:=",
  Evaluatable->True],

Cell[BoxData[
    \(center3pts[{1, 0, 0}, {0, 1, 0}, {0, \(-1\), 0}]\)], "Input",
  CellLabel->"In[100]:=",
  Evaluatable->True],

Cell[BoxData[
    \(radius3pts[{1, 0, 0}, {0, 1, 0}, {0, \(-1\), 0}]\)], "Input",
  CellLabel->"In[101]:=",
  Evaluatable->True],

Cell[BoxData[
    \(posvec3pts[{3, 0, 0}, {3, 1, 0}, {3, \(-1\), 0}]\)], "Input",
  CellLabel->"In[102]:=",
  Evaluatable->True],

Cell[BoxData[
    \(center3pts[{3, 0, 0}, {3, 1, 0}, {3, \(-1\), 0}]\)], "Input",
  CellLabel->"In[103]:=",
  Evaluatable->True],

Cell[BoxData[
    \(radius3pts[{3, 0, 0}, {3, 1, 0}, {3, \(-1\), 0}]\)], "Input",
  CellLabel->"In[104]:=",
  Evaluatable->True],

Cell[BoxData[{
    \(a = 4*Random[]; \ b = 4*Random[]; \ c = 4*Random[];\), "\n", 
    \(\(Print[{a, b, c}];\)\), "\n", 
    \(pa = {a, 0, 0}; pb = {0, b, 0}; pc = {0, 0, c};\), "\n", 
    \(\(pv\  = \ \ posvec3pts[pa, pb, pc];\)\), "\n", 
    \(\(pm\  = \ \ center3pts[pa, pb, pc];\)\), "\n", 
    \(\(Print["\<Position vector = \>", \ pv, "\<, \>", "\<center = \>", 
        pm, "\<, \>", "\<radius = \>", radius3pts[pa, pb, pc]];\)\), "\n", 
    \(\(gr1 = plotcircle3pts[pa, pb, pc];\)\), "\n", 
    \(\(gr2\  = \ 
        Graphics3D[{PointSize[0.05], Point[pa], Point[pb], 
            Point[pc]}];\)\), "\n", 
    \(\(gr3\  = \ Graphics3D[Line[{pm, pm\  + pv}]];\)\), "\n", 
    \(Show[{gr1, gr2, gr3}]\)}], "Input",
  CellLabel->"In[158]:=",
  Evaluatable->True],

Cell[BoxData[{
    \(a = Random[]; \ b = Random[];\), "\n", 
    \(\(Print[{a, b}];\)\), "\n", 
    \(pa = {a, b, 0}; pb = {Random[], b, Random[]*a}; 
    pc = \((pa\  + \ pb)\)/2;\), "\n", 
    \(\(Print[center3pts[pa, pb, pc], "\<, \>", 
        radius3pts[pa, pb, pc]];\)\), "\n", 
    \(\(gr1 = plotcircle3pts[pa, pb, pc];\)\), "\n", 
    \(\(gr2\  = \ 
        Graphics3D[{PointSize[0.05], Point[pa], Point[pb], 
            Point[pc]}];\)\), "\n", 
    \(Show[{gr1, gr2}]\)}], "Input",
  CellLabel->"In[226]:=",
  Evaluatable->True],

Cell[BoxData[{
    \(pa = 2*randomv[3]; pb = \(-3\)*randomv[3]; 
    pc = \((pa\  + \ pb)\)/2;\), "\n", 
    \(\(Print[center3pts[pa, pb, pc], "\<, \>", 
        radius3pts[pa, pb, pc]];\)\), "\n", 
    \(\(gr1 = plotcircle3pts[pa, pb, pc];\)\), "\n", 
    \(\(gr2\  = \ 
        Graphics3D[{PointSize[0.05], Point[pa], Point[pb], 
            Point[pc]}];\)\), "\n", 
    \(Show[{gr1, gr2}]\)}], "Input",
  CellLabel->"In[238]:=",
  Evaluatable->True],

Cell[BoxData[
    \(Remove[a, b, c, pa, pb, pc, pv, pm, gr1, gr2, gr3]\)], "Input",
  CellLabel->"In[243]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.4 An application: Tubes of a space curve", "Subsection"],

Cell["We plot a tube around the helix.", "Text"],

Cell[BoxData[
    \(\(?circle3D\)\)], "Input",
  CellLabel->"In[157]:="],

Cell[BoxData[
    \(c1\  = ArcSin[0.5/Sqrt[1.25]]\)], "Input",
  CellLabel->"In[245]:="],

Cell[BoxData[
    \(ParametricPlot3D[\(circle3D[Cos[s], \ Sin[s], \ 0.5\ s, s + Pi/2, c1, 
          0.1]\)[t], {s, \(-6\), 6}, {t, \ 0, \ 2\ Pi}, 
      PlotPoints\  -> \ 60]\)], "Input",
  CellLabel->"In[246]:="],

Cell[BoxData[
    \(heltube = \ %\)], "Input",
  CellLabel->"In[247]:="],

Cell["\<\
Animation: Before using ThreeScript first import the package (see section 7.1 \
below)!\
\>", "Text"],

Cell[BoxData[
    \(ThreeScript["\<heltube.ts\>", \ heltube]\)], "Input",
  CellLabel->"In[117]:="],

Cell[BoxData[
    \(MVShow3D[heltube]\)], "Input",
  CellLabel->"In[250]:="],

Cell["\<\
The following example will be generalized in the next definition:\
\>", "Text"],

Cell[BoxData[
    \(ParametricPlot3D[\(gencircle[{Cos[s], \ Sin[s], \ 
            0.5\ s}, {\(-Sin[s]\), Cos[s], 0.5}*0.3]\)[t], {s, \(-8\), 
        8}, \n\t{t, \ 0, \ 2\ Pi}, PlotPoints\  -> \ 40]\)], "Input",
  CellLabel->"In[254]:="],

Cell[BoxData[
    \(heltube2 = %\)], "Input",
  CellLabel->"In[255]:="],

Cell[BoxData[
    \(ThreeScript["\<heltube2.ts\>", \ heltube2]\)], "Input",
  CellLabel->"In[120]:="],

Cell[BoxData[
    \(MVShow3D[heltube2]\)], "Input",
  CellLabel->"In[256]:="],

Cell["\<\
The following definition is more general as that given in Alfred Gray's book \
[1], because we do not need the normal vectors; regularity is not assumed, \
but applied implicitely. The parameter representation of the curve must be \
one times differentiable.\
\>", "Text"],

Cell[BoxData[
    \(\(?tube\)\)], "Input",
  CellLabel->"In[257]:="],

Cell[BoxData[
    \(tube[x_, \ a_, b_, r_, \ opts___]\  := \ 
      Module[{t, s, \ w, v, \ u}, \n\t\t\t\t\tv[s_]\  := \ 
          D[x[u], u] /. u\  -> \ s; \n\t\t\t\ \ \ \ w[s_] := 
          If[v[s] . v[s] === \ 0. , 
            Return[v[s]], \n\t\t\t\t\ \ Return[\ 
              v[s]/Sqrt[
                  v[s] . v[
                      s]]]]; \n\t\t\t\t\t\t\tParametricPlot3D[\(gencircle[
              x[s], r*w[s]]\)[t], \ {s, a, b}, {t, 0, 2\ Pi}, 
          opts]]\)], "Input",
  CellLabel->"In[39]:=",
  Evaluatable->False],

Cell[BoxData[
    \(helix1[s_]\  := \ {Cos[s], \ Sin[s], \ 0.5*s}\)], "Input",
  CellLabel->"In[258]:="],

Cell[BoxData[
    \(tube[helix1, 0, \ 6\ Pi, 0.3, \ PlotPoints\  -> \ 50]\)], "Input",
  CellLabel->"In[259]:="],

Cell[TextData[{
  "One finds ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " definitions of numerous examples of space curves  in Appendix A.3 of \
Alfred Gray's book [1].  We construct tubes of some of these space curves."
}], "Text"],

Cell[CellGroupData[{

Cell["Tubes of ast3d", "Subsubsection"],

Cell["\<\
Here is a curve ast3d with singularities, a space curve analog of the \
astroid:\
\>", "Text"],

Cell[BoxData[
    \(curv[s_]\  := \ {Cos[s]\^3, 1.5`\ Sin[s]\^3, Cos[2\ s]}\)], "Input",
  CellLabel->"In[260]:="],

Cell[BoxData[
    \(ParametricPlot3D[curv[s], \ {s, 0, 10}]\)], "Input",
  CellLabel->"In[261]:="],

Cell[BoxData[
    \(tube[curv, 0, \ 2\ Pi, 0.08, \ PlotRange\  -> \ All]\)], "Input",
  CellLabel->"In[262]:="],

Cell[BoxData[
    \(ast3\  = \ %\)], "Input",
  CellLabel->"In[263]:="],

Cell[BoxData[
    \(ThreeScript["\<ast3.ts\>", \ %]\)], "Input",
  CellLabel->"In[130]:="],

Cell[BoxData[
    \(MVShow3D[ast3]\)], "Input",
  CellLabel->"In[264]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Tubes of a bicilinder curve", "Subsubsection"],

Cell["\<\
The next curve bicylinder is the intersection of two cylinders:\
\>", "Text"],

Cell[BoxData[
    \(curv[s_]\  := \ {Cos[s], Sin[s], \@\(4 - Sin[s]\^2\)}\)], "Input",
  CellLabel->"In[265]:="],

Cell[BoxData[
    \(ParametricPlot3D[curv[s], \ {s, 0, 10}]\)], "Input",
  CellLabel->"In[266]:="],

Cell[BoxData[
    \(tube[curv, 0, \ 2\ Pi, 0.08, \ PlotRange\  -> \ All]\)], "Input",
  CellLabel->"In[267]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Tubes of a loxodrome", "Subsubsection"],

Cell["Now we shall consider a loxodrome:", "Text"],

Cell[BoxData[
    \(curv[
        s_]\  := \ {\(2\ s\ Cos[Log[s]]\)\/\(1 + s\^2\), \(2\ s\ \
Sin[Log[s]]\)\/\(1 + s\^2\), \(1 - s\^2\)\/\(1 + s\^2\)}\)], "Input",
  CellLabel->"In[268]:="],

Cell[BoxData[
    \(Simplify[curv[s] . curv[s], \ s > 0]\)], "Input",
  CellLabel->"In[269]:="],

Cell[BoxData[
    \(N[Log[30]]\)], "Input",
  CellLabel->"In[270]:="],

Cell[BoxData[
    \(ParametricPlot3D[curv[s], \ {s, 0.1, 30}, 
      PlotRange\  \[Rule] \ All]\)], "Input",
  CellLabel->"In[271]:="],

Cell[BoxData[
    \(ParametricPlot3D[curv[s], \ {s, 0.0001, 10}]\)], "Input",
  CellLabel->"In[272]:="],

Cell[BoxData[
    \(tube[curv, 0.0001, \ 20, 0.02, PlotPoints\  \[Rule] \ 40, \ 
      PlotRange\  \[Rule] \ All]\)], "Input",
  CellLabel->"In[273]:="],

Cell["\<\
The following parameter transformation gives simpler formulas for the \
evaluations:\
\>", "Text"],

Cell[BoxData[
    \(Simplify[curv[s] /. \ s \[Rule] Exp[t], \ 
      t \[Element] \ Reals]\)], "Input",
  CellLabel->"In[274]:="],

Cell[BoxData[
    \(curve[t_]\  = \ %\)], "Input",
  CellLabel->"In[275]:="],

Cell[BoxData[
    \(ParametricPlot3D[curve[s], \ {s, \(-10\), 10}, 
      PlotRange\  \[Rule] \ All]\)], "Input",
  CellLabel->"In[276]:="],

Cell[BoxData[
    \(tube[curve, \(-3\)*Pi, \ 3\ *Pi, 0.02, \ PlotRange\  -> \ All, 
      PlotPoints\  \[Rule] \ {40, 20}]\)], "Input",
  CellLabel->"In[277]:="],

Cell[BoxData[
    \(loxotube\  = \ %\)], "Input",
  CellLabel->"In[278]:="],

Cell[BoxData[
    \(MVShow3D[loxotube]\)], "Input",
  CellLabel->"In[279]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Tubes of torus knots", "Subsubsection"],

Cell["Now we introduce a torus knot:", "Text"],

Cell[BoxData[
    \(curv[
        s_]\  := \ {Cos[
            2\ s]\ \((1 + 2\ Cos[3\ s])\), \((1 + 2\ Cos[3\ s])\)\ Sin[2\ s], 
        Sin[3\ s]}\)], "Input",
  CellLabel->"In[280]:="],

Cell[BoxData[
    \(ParametricPlot3D[curv[s], \ {s, 0. , 2\ Pi}]\)], "Input",
  CellLabel->"In[281]:="],

Cell[BoxData[
    \(knot1\  = \ %\)], "Input",
  CellLabel->"In[282]:="],

Cell[BoxData[
    \(tube[curv, 0, \ 2\ Pi, 0.02, \ PlotRange\  -> \ All, \ 
      PlotPoints \[Rule] {50, 25}]\)], "Input",
  CellLabel->"In[283]:="],

Cell[BoxData[
    \(knot = \ %\)], "Input",
  CellLabel->"In[284]:="],

Cell[BoxData[
    \(ThreeScript["\<knot.ts\>", knot]\)], "Input",
  CellLabel->"In[44]:="],

Cell["\<\
Animation: Before using the next command first link MathGL3d (see section 7.2 \
below)!\
\>", "Text"],

Cell[BoxData[
    \(MVShow3D[knot]\)], "Input",
  CellLabel->"In[285]:="],

Cell[BoxData[
    \(MVShow3D[knot1, \ MVLineTubeSize\  -> \ 0.005]\)], "Input",
  CellLabel->"In[288]:="],

Cell[BoxData[
    \(Clear[curv]\)], "Input",
  CellLabel->"In[289]:="],

Cell["The general circular torus is the surface of revolution", "Text"],

Cell[BoxData[
    \(\(torussf[a_, b_]\)[u_, 
        v_] := \ {\((a + b*Cos[u])\)*Cos[v], \((a + b*Cos[u])\)*Sin[v], 
        b*Sin[u]}\)], "Input",
  CellLabel->"In[3]:=",
  InitializationCell->True],

Cell[BoxData[
    \(ParametricPlot3D[\(torussf[1, \ 1/3]\)[u, v], \ {u, 0, 2*Pi}, \ {v, 0, 
        2*Pi}]\)], "Input",
  CellLabel->"In[290]:="],

Cell["\<\
A torus knot is a curve on the torus with the parameter representation\
\>", "Text"],

Cell[BoxData[
    \(\(\(torkn[a_, b_]\)[p_, q_]\)[
        t_]\  := {\((a + b*Cos[p*t])\)*Cos[q*t], \((a + b*Cos[p*t])\)*
          Sin[q*t], b*Sin[p*t]}\)], "Input",
  CellLabel->"In[4]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(?LCM\)\)], "Input",
  CellLabel->"In[291]:="],

Cell[BoxData[
    \(\(?GCD\)\)], "Input",
  CellLabel->"In[292]:="],

Cell["\<\
If p,q, are integers torkn is a closed curve, i.e. it is periodic with the \
smallest period 2*Pi/GCD[p,q].\
\>", "Text"],

Cell[BoxData[
    \(ParametricPlot3D[\(\(torkn[1, 1/3]\)[2, 3]\)[t], \ {t, 0, 2*Pi}, \ 
      PlotPoints\  \[Rule] \ 100]\)], "Input",
  CellLabel->"In[293]:="],

Cell[BoxData[
    \(kn23 = %\)], "Input",
  CellLabel->"In[294]:="],

Cell[BoxData[
    \(ThreeScript["\<kn23.ts\>", kn23]\)], "Input",
  CellLabel->"In[72]:="],

Cell[BoxData[
    \(MVShow3D[kn23, \ MVLineTubeSize\  \[Rule] \  .02]\)], "Input",
  CellLabel->"In[295]:="],

Cell[BoxData[
    \(curv[t_]\  := \(\(torkn[1, 1/3]\)[2, 3]\)[t]\)], "Input",
  CellLabel->"In[296]:="],

Cell[BoxData[
    \(tube[curv, 0, 2*Pi, 0.04, PlotPoints\  \[Rule] {80, 20}]\)], "Input",
  CellLabel->"In[297]:="],

Cell[BoxData[
    \(kntub23\  = \ %\)], "Input",
  CellLabel->"In[298]:="],

Cell[BoxData[
    \(MVShow3D[kntub23]\)], "Input",
  CellLabel->"In[299]:="],

Cell[BoxData[
    \(ThreeScript["\<kntub23.ts\>", kntub23]\)], "Input",
  CellLabel->"In[71]:="],

Cell[BoxData[
    \(ParametricPlot3D[\(\(torkn[1, 1/3]\)[5, 2]\)[t], \ {t, 0, 2*Pi}, 
      PlotPoints\  \[Rule] \ 100]\)], "Input",
  CellLabel->"In[80]:="],

Cell[BoxData[
    \(kn52\  = \ %\)], "Input",
  CellLabel->"In[81]:="],

Cell[BoxData[
    \(ThreeScript["\<kn52.ts\>", kn52]\)], "Input",
  CellLabel->"In[89]:="],

Cell[BoxData[
    \(Clear[curv]\)], "Input",
  CellLabel->"In[83]:="],

Cell[BoxData[
    \(curv[t_] := \(\(torkn[1, 1/3]\)[5, 2]\)[t]\)], "Input",
  CellLabel->"In[84]:="],

Cell[BoxData[
    \(tube[curv, 0, 2*Pi, 0.03, PlotPoints\  \[Rule] {90, 20}]\)], "Input",
  CellLabel->"In[85]:="],

Cell[BoxData[
    \(kntub52 = %\)], "Input",
  CellLabel->"In[86]:="],

Cell[BoxData[
    \(ThreeScript["\<kntub52.ts\>", kntub52]\)], "Input",
  CellLabel->"In[88]:="],

Cell[BoxData[
    \(ParametricPlot3D[\(\(torkn[1, 1/3]\)[7, 3]\)[t], \ {t, 0, 2*Pi}, 
      PlotPoints\  \[Rule] \ 100]\)], "Input",
  CellLabel->"In[300]:="],

Cell[BoxData[
    \(kn73\  = \ %\)], "Input",
  CellLabel->"In[301]:="],

Cell[BoxData[
    \(ThreeScript["\<kn73.ts\>", kn73]\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(Clear[curv]\)], "Input",
  CellLabel->"In[302]:="],

Cell[BoxData[
    \(curv[t_] := \(\(torkn[1, 1/3]\)[7, 3]\)[t]\)], "Input",
  CellLabel->"In[303]:="],

Cell[BoxData[
    \(tube[curv, 0, 2*Pi, 0.03, PlotPoints\  \[Rule] {90, 20}]\)], "Input",
  CellLabel->"In[304]:="],

Cell[BoxData[
    \(kntub73 = %\)], "Input",
  CellLabel->"In[305]:="],

Cell[BoxData[
    \(MVShow3D[kntub73]\)], "Input",
  CellLabel->"In[306]:="],

Cell[BoxData[
    \(ThreeScript["\<kntub73.ts\>", kntub73]\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(Clear[curv]\)], "Input",
  CellLabel->"In[222]:="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
3 Circles defined by 2-dimensional euclidean vector subspaces (dim = 5, ind = \
1)\
\>", "Section",
  FontSize->19],

Cell["\<\
In M\[ODoubleDot]bius geometry a circle on the 3-sphere, or by \
stereographical projection, a circle or a line in the euclidean 3-space, is \
defined by a 3-dimensional pseudo-euclidean vector subspace: the generators \
of the isotropic cone of this 3-space are the (projective) points of the \
circle. Equivalently it can be defined by the orthogonal complement, that is \
a 2-dimensional euclidean subspace of the 5-dimensional pseudo-euclidean \
vector space. The sets of these spaces are open orbits of the \
M\[ODoubleDot]bius group in the corresponding  Grassmann manifolds. The M\
\[ODoubleDot]bius group can be identified with two components of the \
pseudo-othogonal group O(5,1). It is well known that the orbits have a \
natural invariant structure of a pseudo-Riemannian symmetric space.\
\>", "Text"],

Cell[CellGroupData[{

Cell["3.1 From the subspace to the circle", "Subsection"],

Cell["\<\
In this section we start with an arbitrary basis of an euclidean 2-space, \
construct a pseudo-orthonormal basis of its orthogonal complement and obtain, \
using this basis and stereographical projection,  parameter representation \
and plots of the corresponding circle.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
The definitions pscomplement, threepoints, circle2spv, plotcircle2spv\
\>", "Subsubsection"],

Cell["\<\
Let v, w be a pair of  vectors spanning an euclidean 2-space in the \
pseudo-euclidean 5-space. Then one obtaines an pseudo-orthonormal basis of \
the wanted orthogonal complementary 3-space as follows:\
\>", "Text"],

Cell[BoxData[
    \(\(?pscomplement\)\)], "Input",
  CellLabel->"In[307]:="],

Cell[BoxData[
    \(\(\(pscomplement[v_List, 
        w_List]\)\(:=\)\(\ \)\(Module[{j = 0, w1, w2, w3, 
          chj}, \[IndentingNewLine]Label[chj]; \[IndentingNewLine]If[
          j \[Equal] dim, \ 
          Print["\<pscompl not found! Vectors do not span an euclidean \
2-space?\>"]; Return[]; Break[], 
          j = j + 1]; \ \ \ \ \ \ \ \ \ \ \ \ \ \ \[IndentingNewLine]\ 
        w1\  = \ pscross[{v, w, stb[j], stb[5]}]; \[IndentingNewLine]If[
          w1\  \[Equal] \ nul[5], \ Goto[chj], \ 
          w2\  = \ pscross[{v, w, w1, stb[5]}], \ 
          w2\  = \ pscross[{v, w, w1, stb[5]}]]; \[IndentingNewLine]w3\  = \ 
          pscross[{v, w, w1, w2}]; \[IndentingNewLine]Return[{w1/
              Sqrt[pssp[w1, w1]], w2/Sqrt[pssp[w2, w2]], 
            w3/Sqrt[\(-pssp[w3, 
                    w3]\)]}]]\)\(\[IndentingNewLine]\)\)\)], "Input",
  CellLabel->"In[194]:=",
  Evaluatable->False],

Cell["\<\
 The following Modules yield the parameter representation of  the \
corresponding circle in the euclidean 3-space, and the corresponding plot \
command; all these definition are protected and contained in mcirc.m.\
\>", "Text"],

Cell[BoxData[
    \(\(circle2spv[v_List, w_List]\)[t_] := \ 
      Module[{psc}, \[IndentingNewLine]psc\  = \ 
          pscomplement[v, w]; \[IndentingNewLine]If[psc\  \[Equal] \ Null, \ 
          Return[]]; \[IndentingNewLine]stepro[\(-psc[\([1]\)]\)*Cos[t]\  - 
            psc[\([2]\)]*Sin[t]\  + \ psc[\([3]\)]]]\)], "Input",
  CellLabel->"In[26]:=",
  Evaluatable->False],

Cell[BoxData[
    \(\(?plotcircle2spv\)\)], "Input",
  CellLabel->"In[308]:="],

Cell[BoxData[
    \(plotcircle2spv[v_List, w_List, opt___] := \ 
      ParametricPlot3D[\(circle2spv[v, w]\)[t], \ {t, 0, 2*Pi}, \ 
        opt]\)], "Input",
  Evaluatable->False],

Cell["\<\
Another way: we take three  points of the circle corresponding to the \
subspace [v,w]. \
\>", "Text"],

Cell[BoxData[
    \(threepoints[v_, w_] := \ 
      Module[{b = pscomplement[v, w]}, \n\t
        If[b \[Equal] Null, Return[]; 
          Break[]]; \t\t\[IndentingNewLine]{stepro[
            b[\([3]\)] + \ \ b[\([1]\)]], \ 
          stepro[b[\([3]\)]\  - \ b[\([1]\)]], \ 
          stepro[b[\([3]\)] + \ b[\([2]\)]]}]\)], "Input",
  CellLabel->"In[5]:=",
  Evaluatable->False],

Cell["\<\
Combining threepoints with circle3pts, plotcircle3pts of section 2 one \
obtaines a parameter representation and the plotting procedure plotcircle:\
\>", "Text"],

Cell[BoxData[
    \(\(?circle3pts\)\)], "Input",
  CellLabel->"In[49]:="],

Cell["OBSOLET:", "Text"],

Cell[BoxData[
    \(\(oldcircle2spv[v_, \ w_]\)[t_]\  := \ 
      Module[{p1, p2, p3, b}, \n\t\tb\  = 
          threepoints[v, w]; \[IndentingNewLine]\t\ \ \ \ If[b \[Equal] Null, 
          Return[]; Break[]]; \n\t\tp1 = \ \ b[\([1]\)]; \n\t\tp2\  = 
          b[\([2]\)]; \n\t\tp3\  = \ \ b[\([3]\)]; \n\t\t\(circle3pts[p1, p2, 
            p3]\)[t]]\)], "Input",
  CellLabel->"In[309]:="],

Cell[BoxData[
    \(\(?plotcircle3pts\)\)], "Input",
  CellLabel->"In[310]:="],

Cell[BoxData[
    \(oldplotcircle2spv[\ v_, w_, \ opt___] := \ 
      Module[{p1, p2, p3, b}, \n\t\tb\  = 
          threepoints[v, w]; \[IndentingNewLine]\ 
        If[b \[Equal] Null, Return[]; 
          Break[]]; \n\t\tp1 = \ \ b[\([1]\)]; \n\t\tp2\  = 
          b[\([2]\)]; \n\t\tp3\  = \ \ b[\([3]\)]; \n\t\tplotcircle3pts[p1, 
          p2, p3, opt]]\)], "Input",
  CellLabel->"In[311]:="],

Cell["The possible options are those of ParametricPlot3D:", "Text"],

Cell[BoxData[
    \(Options[ParametricPlot3D]\)], "Input",
  CellLabel->"In[26]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.2 Examples, tests", "Subsection"],

Cell[CellGroupData[{

Cell["First examples", "Subsubsection"],

Cell[BoxData[
    \(pscomplement[stb[2], \ stb[3]]\)], "Input",
  CellLabel->"In[312]:="],

Cell[BoxData[
    \(pscomplement[stb[2], \ stb[5]]\)], "Input",
  CellLabel->"In[313]:="],

Cell[BoxData[
    \(pscomplement[stb[2], \ stb[2]]\)], "Input",
  CellLabel->"In[314]:="],

Cell[BoxData[
    \(pscomplement[stb[2], \ stb[2] + stb[4]]\)], "Input",
  CellLabel->"In[315]:="],

Cell[BoxData[
    \(Timing[\(circle2spv[stb[2], \ stb[3]]\)[t]]\)], "Input",
  CellLabel->"In[316]:="],

Cell[BoxData[
    \(plotcircle2spv[stb[2], \ stb[3]]\)], "Input",
  CellLabel->"In[317]:="],

Cell[BoxData[
    \(Timing[\(circle2spv[stb[3], \ stb[4]]\)[t]]\)], "Input",
  CellLabel->"In[318]:="],

Cell[BoxData[
    \(Timing[plotcircle2spv[stb[3], \ stb[4]]]\)], "Input",
  CellLabel->"In[319]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["The Frame Circles", "Subsubsection",
  Evaluatable->False],

Cell["The unit circle in the x,y-plane, centered at o:", "Text"],

Cell[BoxData[
    \(grc1 = plotcircle2spv[stb[3], \ stb[4]]\)], "Input",
  CellLabel->"In[5]:=",
  InitializationCell->True],

Cell["\<\
This is the circle of radius 1 in the x, z-plane with center at the origin:\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(grc2 = plotcircle2spv[stb[2], stb[4]]\)], "Input",
  CellLabel->"In[321]:="],

Cell["\<\
This is the circle of radius 1 in the y, z-plane with center at the origin:\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(grc3 = plotcircle2spv[stb[1], stb[4]]\)], "Input",
  CellLabel->"In[322]:="],

Cell["This is the x-axis:", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(grc4 = plotcircle2spv[stb[2], stb[3]]\)], "Input",
  CellLabel->"In[348]:="],

Cell["This is the y-axis:", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(grc5 = plotcircle2spv[stb[1], stb[3]]\)], "Input",
  CellLabel->"In[324]:="],

Cell["This is the z-axis:", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(grc6 = plotcircle2spv[stb[1], stb[2]]\)], "Input",
  CellLabel->"In[325]:="],

Cell[BoxData[
    \(Show[{grc1, grc2, grc3, grc4, grc5, grc6}, \ 
      PlotRange\  \[Rule] \ {{\(-1.5\), 1.5}, {\(-1.5\), 1.5}, {\(-1.5\), 
            1.5}}, Boxed\  \[Rule] \ False, \ 
      Axes\  \[Rule] \ False]\)], "Input",
  CellLabel->"In[342]:="],

Cell[BoxData[
    \(framecir\  = \ %\)], "Input",
  CellLabel->"In[343]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Further examples", "Subsubsection"],

Cell["\<\
In this example circle2spv, plotcircle2spv are shown to be stable against \
linear combination of the arguments; it would give an incorrect result in \
case a=0, or b = 0:\
\>", "Text"],

Cell[BoxData[
    \(pscomplement[stb[1]*b + stb[4]*c, stb[4]*a]\)], "Input",
  CellLabel->"In[349]:="],

Cell["Timing[circle2spv[stb[1]*b + stb[4]*c, stb[4]*a][t]]", "Input",
  CellLabel->"In[350]:="],

Cell["plotcircle2spv[stb[4]*c, stb[4]*a]", "Input",
  CellLabel->"In[351]:="],

Cell["plotcircle2spv[stb[1]+stb[4]*c, stb[4]*0]", "Input",
  CellLabel->"In[352]:="],

Cell["\<\
Of course, with unspecified parameters plotcircle may not work, in general:\
\>", "Text"],

Cell["plotcircle2spv[stb[1]*b + stb[4]*c, stb[4]*a]", "Input",
  CellLabel->"In[353]:="],

Cell["Here the parmeter c is unessential:", "Text"],

Cell["plotcircle2spv[stb[1] + stb[4]*c, stb[4]]", "Input",
  CellLabel->"In[354]:="],

Cell["\<\
Moreover, the last example shows  that circle2spv does not require neither \
orthogonality nor norming; generally, in  all  modules using circle2spv or \
the 2-space spanned by v, w, these vectors may be an arbitrary basis of the \
2-space.\
\>", "Text"],

Cell[BoxData[
    \(threepoints[stb[3], \ stb[4]]\)], "Input",
  CellLabel->"In[355]:="],

Cell[BoxData[
    \(Timing[plotcircle2spv[\ stb[3], \ stb[4]]]\)], "Input",
  CellLabel->"In[356]:="],

Cell[BoxData[
    \(threepoints[stb[1], \ stb[2]]\)], "Input",
  CellLabel->"In[357]:="],

Cell[BoxData[
    \(Timing[plotcircle2spv[\ stb[1], stb[2]]]\)], "Input",
  CellLabel->"In[358]:="],

Cell[BoxData[
    \(\((wr\  = 
        orthopair[{2*Random[], \(-2\)*Random[], 1. , 3*Random[], \ 
            Random[]}]; \ v\  = \ wr[\([1]\)]; w\  = \ wr[\([2]\)]; 
      mwr = threepoints[v, w]; \npa = mwr[\([1]\)]; pb = mwr[\([2]\)]; 
      pc = mwr[\([3]\)]; \ngr1 = plotcircle3pts[pa, pb, pc]; \n
      gr2\  = \ Graphics3D[{PointSize[0.05], Point[pa], Point[pb], 
            Point[pc]}]; \nShow[{gr1, gr2}])\)\)], "Input",
  CellLabel->"In[359]:="],

Cell[BoxData[
    \(Timing[plotcircle2spv[\ v, w]]\)], "Input",
  CellLabel->"In[360]:="],

Cell[BoxData[
    \(v = v + \ 0.1*stb[1]; \ w\  = w + \ 0.01*stb[5];\)], "Input",
  CellLabel->"In[361]:="],

Cell[BoxData[
    \(Timing[plotcircle2spv[\ v, w]]\)], "Input",
  CellLabel->"In[362]:="],

Cell[BoxData[
    \(Remove[wr, v, w, mwr, pa, pb, pc, gr1, gr2]\)], "Input",
  CellLabel->"In[363]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.3 From the circle to the subspace", "Subsection"],

Cell["\<\
For the geometrical interpretations of the concepts and results defined or \
calculated in the pseudo-euclidean 5-space it is interesting to have \
expressions of the spacelike vectors as functions of the center {x,y,z}, the \
radius r, and the angles u,v which define the position vector of the circle \
in euclidean 3-space. In this and the following sections we find and apply \
such functions.  We begin with a very simple, direct definition which is best \
appropriate for all applications. Defining a distinguished, from the \
euclidean point of view, basis of the euclidean 2-space corresponding to a \
given circle, the vectors of which depend on the mentioned  euclidean \
invariants we get what we wanted.  The choice of such a basis means the \
choice of two spheres which intersect orthogonally in the given circle.  We \
have two such \"spheres\" canonical associated to the circle: The plane which \
containes the circle, and the sphere with the same radius and center \
containing the circle. Let us consider an arbitrary circle\
\>", "Text"],

Cell[BoxData[
    \(Clear[a, b, c, x, y, z, u, v, r, t]\)], "Input",
  CellLabel->"In[364]:="],

Cell[BoxData[
    \(\(circle3D[x, y, z, u, v, r]\)[t]\)], "Input",
  CellLabel->"In[365]:="],

Cell["\<\
The spacelike vectors defining the associated spheres are orthogonal and span \
the spacelike 2-space of the circle. The vectors belonging to general planes \
and spheres are calculated in the notebook mspheres.nb. For a plane we have\
\>", "Text"],

Cell[BoxData[
    \(\(?planev\)\)], "Input",
  CellLabel->"In[366]:="],

Cell[BoxData[
    \(planev[a, b, c, p]\)], "Input",
  CellLabel->"In[367]:="],

Cell[BoxData[
    \(spt[stb[4]\  + \ stb[5]]\)], "Input",
  CellLabel->"In[368]:="],

Cell[BoxData[
    \(stepro[stb[4]\  + \ stb[5]]\)], "Input",
  CellLabel->"In[369]:="],

Cell[BoxData[
    \(pssp[vec[nv], stb[4]\  + \ stb[5]]\)], "Input",
  CellLabel->"In[370]:="],

Cell[BoxData[
    \(pssp[planev[a, b, c, p], stb[4]\  + \ stb[5]]\)], "Input",
  CellLabel->"In[371]:="],

Cell["\<\
We know that the property nv[[4]] == nv[[5]] is characteristic for those \
spacelike vectors nv which represent planes. The reason is that all the \
subspheres through the north pole spt[stb[4] + stb[5]] of the 3-sphere, and \
only these, are projected onto planes by the stereographic projection.  Since\
\
\>", "Text"],

Cell[BoxData[
    \({a, b, c} . {a, b, c}\  \[Equal] \ 
      pssp[{a, b, c, p, p}, {a, b, c, p, p}]\)], "Input",
  CellLabel->"In[372]:="],

Cell["\<\
for all values of p we may define the normed vector nv easily if we found \
Hesse's normal form \
\>", "Text"],

Cell[BoxData[
    \({a, b, c} . vec[x, 3]\  \[Equal] \ p\)], "Input",
  CellLabel->"In[373]:="],

Cell["of the plane containing the circle, with", "Text"],

Cell[BoxData[
    \({a, b, c} . {a, b, c}\  \[Equal] \ 1\)], "Input",
  CellLabel->"In[374]:="],

Cell["\<\
Here vec[x,3] denotes the running point of the plane.  We write down the \
equation of the plane containing the circle:\
\>", "Text"],

Cell[BoxData[
    \(\((vec[x, 3] - {x, y, z})\) . \(unit[2]\)[{u, v}]\  \[Equal] \ 
      0\)], "Input",
  CellLabel->"In[375]:="],

Cell["and recognize", "Text"],

Cell[BoxData[
    \({a, b, c}\  = \ \(unit[2]\)[{u, v}]\)], "Input",
  CellLabel->"In[376]:="],

Cell[BoxData[
    \(p\  = \ \(unit[2]\)[{u, v}] . {x, y, z}\)], "Input",
  CellLabel->"In[377]:="],

Cell["This leads to the definition", "Text"],

Cell[BoxData[
    \(circleplane[x_, y_, z_, u_, v_]\  := {Cos[u]\ Cos[v], Cos[v]\ Sin[u], 
        Sin[v], x\ Cos[u]\ Cos[v] + y\ Cos[v]\ Sin[u] + z\ Sin[v], 
        x\ Cos[u]\ Cos[v] + y\ Cos[v]\ Sin[u] + z\ Sin[v]}\)], "Input",
  CellLabel->"In[21]:=",
  Evaluatable->False],

Cell["\<\
We emphasize that circleplane does not depend on the radius.  The second \
vector we know already:\
\>", "Text"],

Cell[BoxData[
    \(\(?spacevec\)\)], "Input",
  CellLabel->"In[378]:="],

Cell["\<\
Thus, a  vector corresponding to the sphere with the same radius and center \
as the circle does not depend on the position variables u,v; it is\
\>", "Text"],

Cell[BoxData[
    \(spacevec[x, y, z, r]\)], "Input",
  CellLabel->"In[379]:="],

Cell[BoxData[
    \(circlespacevectors[x_, y_, z_, u_, v_, 
        r_]\  := {circleplane[x, y, z, u, v], 
        spacevec[x, y, z, r]}\)], "Input",
  CellLabel->"In[22]:=",
  Evaluatable->False],

Cell[BoxData[
    \(Clear[a, b, c, p]\)], "Input",
  CellLabel->"In[380]:="],

Cell[TextData[{
  "Obviously, the circlespacevectors are explicitly defined in terms of the \
euclidean invariants of the circle. By its geometric definition as spacelike \
unit vectors such that the first corresponds to the plane of the circle they \
form  an orthonormal basis of the associated spacelike 2-space which is \
uniquely defined up to multiplications with  -1. If they have the mentioned \
property we call them ",
  StyleBox["adapted spacelike vectors",
    FontColor->RGBColor[1, 0, 0]],
  " of the circle. For a line any \"sphere\" containing it must be a plane. \
Therefore each vector v of the corresponding spacelike 2-space has the \
characteristic property ",
  Cell[BoxData[
      \(TraditionalForm\`v\_4\)]],
  " == ",
  Cell[BoxData[
      \(TraditionalForm\`v\_5\)]],
  ", and any orthonormal basis of this 2-space will be considered as adapted \
to the line."
}], "Text"],

Cell[CellGroupData[{

Cell["Tests", "Subsubsection"],

Cell["Let us check our considerations:", "Text"],

Cell[BoxData[
    \(Clear[x, y, z, u, v, r]\)], "Input",
  CellLabel->"In[381]:="],

Cell[BoxData[
    \(\(\(\ \)\(circlespacevectors[x, y, z, u, v, r]\)\)\)], "Input",
  CellLabel->"In[382]:="],

Cell[BoxData[
    \(Table[Simplify[pssp[%[\([i]\)], %[\([j]\)]]], {i, 2}, \ {j, 2}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[383]:="],

Cell["\<\
Now we show that the euclidean plotcircle3D defined in section 2 and the plot \
generated by the spacelike vectors coincide:\
\>", "Text"],

Cell[BoxData[
    \(rv\  = \ randomv[6]\)], "Input",
  CellLabel->"In[384]:="],

Cell[BoxData[
    \(tv\  = \ 
      circlespacevectors[rv[\([1]\)], \(-rv[\([2]\)]\), \ rv[\([3]\)], \ 
        rv[\([4]\)], \ rv[\([5]\)], \ 10*rv[\([6]\)]]\)], "Input",
  CellLabel->"In[385]:="],

Cell[BoxData[
    \(plotcircle3D[rv[\([1]\)], \(-rv[\([2]\)]\), \ rv[\([3]\)], \ 
      rv[\([4]\)], \ rv[\([5]\)], \ 10*rv[\([6]\)]]\)], "Input",
  CellLabel->"In[386]:="],

Cell[BoxData[
    \(plotcircle2spv[tv[\([1]\)], \ tv[\([2]\)]]\)], "Input",
  CellLabel->"In[387]:="],

Cell[BoxData[
    \(Remove[rv, tv]\)], "Input",
  CellLabel->"In[388]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Example: A circular cylinder", "Subsubsection"],

Cell["\<\
We consider the circles generating a cylinder of radius 1 around the \
z-axis:\
\>", "Text"],

Cell[BoxData[
    \(zz[u_] := \ circlespacevectors[0, 0, u, 0, Pi/2, 1]\)], "Input",
  CellLabel->"In[398]:="],

Cell[BoxData[
    \(zz[u]\)], "Input",
  CellLabel->"In[399]:="],

Cell[BoxData[
    \(Table[
      pssp[\(zz[u]\)[\([i]\)], \ \(zz[u]\)[\([j]\)]], \ {i, 2}, \ {j, 
        2}]\)], "Input",
  CellLabel->"In[400]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[401]:="],

Cell[BoxData[
    \(\(circle2spv[\(zz[u]\)[\([1]\)], \(zz[u]\)[\([2]\)]]\)[t]\)], "Input",
  CellLabel->"In[391]:="],

Cell[BoxData[
    \(Simplify[%, \ u > 0]\)], "Input",
  CellLabel->"In[392]:="],

Cell[BoxData[
    \(Timing[
      Table[plotcircle2spv[\(zz[u]\)[\([1]\)], \(zz[u]\)[\([2]\)]], {u, \ 0, 
          5, 1/2}]]\)], "Input",
  CellLabel->"In[393]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[394]:="],

Cell[" Plotting the cylinder:", "Text"],

Cell[BoxData[
    \(Timing[
      ParametricPlot3D[
        Evaluate[\(circle2spv[\(zz[u]\)[\([1]\)], \(zz[u]\)[\([2]\)]]\)[
            t]], \ {u, \ 0. , 5}, {t, \ 0.01, \ 2\ Pi}, \ 
        PlotPoints\  \[Rule] \ 50]]\)], "Input",
  CellLabel->"In[395]:="],

Cell[BoxData[
    \(Remove[zz]\)], "Input",
  CellLabel->"In[396]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.4 Adapting the frame", "Subsection"],

Cell["\<\
Often one meets the following situation: one has the euclidean 2-space given \
by an arbitrary basis, and one wants to find the corresponding circle. One \
way to do that is the following: Transform the given frame in an adapted \
frame (defined in the last section), and then read of the euclidean \
parameters from the adapted frame with the help of which one may get the \
parameter representation and the plot of the circle.  The adaption of the \
frame may be done by the following module:\
\>", "Text"],

Cell[BoxData[
    \(adaptsplframe[vx_List, vy_List]\  := \ 
      Module[{aob, d1, d2, q\  = \ 0, v, nor}, \[IndentingNewLine]aob\  = \ 
          orthonorm[{vx, vy}, \ 
            normed\  \[Rule] \ False]; \[IndentingNewLine]If[
          Length[aob] < 
            2, \[IndentingNewLine]Print["\<The span is not 2-dimensional and \
spacelike!\>"]; Return[]]; \[IndentingNewLine]If[
          Simplify[pssp[aob[\([1]\)], \ aob[\([1]\)]]] \[LessEqual] \ 0\  || 
            Simplify[pssp[aob[\([2]\)], \ aob[\([2]\)]]] \[LessEqual] \ 
              0\ , \[IndentingNewLine]Print["\<The span is not 2-dimensional \
and spacelike!\>"]; Return[]]; \[IndentingNewLine]d1 = 
          Simplify[\(aob[\([1]\)]\)[\([5]\)]\  - \ \
\(aob[\([1]\)]\)[\([4]\)]]; \[IndentingNewLine]d2 = \ 
          Simplify[\(aob[\([2]\)]\)[\([5]\)]\  - \ \
\(aob[\([2]\)]\)[\([4]\)]]; \[IndentingNewLine]If[d1 \[Equal] \ 0, \ q = 1; 
          Goto[nor]]; \[IndentingNewLine]If[d2 \[Equal] 0, 
          v = aob[\([1]\)]; \ aob[\([1]\)]\  = \ aob[\([2]\)]; 
          aob[\([2]\)] = \ v; q = 1]; \[IndentingNewLine]Label[
          nor]; \[IndentingNewLine]aob[\([1]\)]\  = \ 
          aob[\([1]\)]/
            Sqrt[pssp[aob[\([1]\)], \ 
                aob[\([1]\)]]]; \[IndentingNewLine]aob[\([2]\)]\  = \ 
          aob[\([2]\)]/
            Sqrt[pssp[aob[\([2]\)], \ aob[\([2]\)]]]; \[IndentingNewLine]If[
          q \[Equal] 1, Return[Simplify[aob]]]; \[IndentingNewLine]d1 = 
          Simplify[\(aob[\([1]\)]\)[\([5]\)]\  - \ \
\(aob[\([1]\)]\)[\([4]\)]]; \[IndentingNewLine]d2 = \ 
          Simplify[\(aob[\([2]\)]\)[\([5]\)]\  - \ \
\(aob[\([2]\)]\)[\([4]\)]]; \[IndentingNewLine]q\  = \ 
          Sqrt[d1^2\  + \ d2^2]; \[IndentingNewLine]v\  = \ 
          aob[\([1]\)]; \ \[IndentingNewLine]aob[\([1]\)]\  = \ 
          Simplify[\ \(-v\)*d2/q\  + \ 
              aob[\([2]\)]*d1/q]; \[IndentingNewLine]aob[\([2]\)]\  = 
          Simplify[\ 
            v*d1/q\  + \ aob[\([2]\)]*d2/q]; \[IndentingNewLine]Return[
          aob]]\)], "Input",
  CellLabel->"In[9]:=",
  Evaluatable->False],

Cell[CellGroupData[{

Cell["Tests", "Subsubsection"],

Cell[BoxData[
    \(adaptsplframe[stb[1], \ stb[5]]\)], "Input",
  CellLabel->"In[406]:="],

Cell[BoxData[
    \(adaptsplframe[stb[1], \ stb[1]]\)], "Input",
  CellLabel->"In[407]:="],

Cell[BoxData[
    \(adaptsplframe[stb[1], \ stb[4]]\)], "Input",
  CellLabel->"In[408]:="],

Cell[BoxData[
    \(adaptsplframe[{0, Sqrt[2]*Cos[t], Sqrt[2]*Sin[t], 0, 1}, {1, 0, 0, 0, 
        0}]\)], "Input",
  CellLabel->"In[409]:="],

Cell[BoxData[
    \(adaptsplframe[
      stb[1], {0, Sqrt[2]*Cos[t], Sqrt[2]*Sin[t], 0, 1}]\)], "Input",
  CellLabel->"In[410]:="],

Cell[BoxData[
    \(adaptsplframe[stb[1], {0, Cos[t], Sin[t], 0, 1}]\)], "Input",
  CellLabel->"In[411]:="],

Cell[BoxData[
    \(adaptsplframe[stb[1], {a, Cos[t], Sin[t], 0, a}]\)], "Input",
  CellLabel->"In[412]:="],

Cell[BoxData[
    \(adaptsplframe[{Cosh[s], 0, 0, 0, Sinh[s]}, {a, Cos[t], Sin[t], 0, 
        a}]\)], "Input",
  CellLabel->"In[413]:="],

Cell[BoxData[
    \(Simplify[%, \ s > 0 && a > 0]\)], "Input",
  CellLabel->"In[414]:="],

Cell["\<\
This is not a direct result, but it coincides with the direct one:\
\>", "Text"],

Cell[BoxData[
    \(Simplify[% /. \ a\  \[Rule] \ 0]\)], "Input",
  CellLabel->"In[415]:="],

Cell[BoxData[
    \(adaptsplframe[{Sqrt[1 + s^2], 0, 0, 0, s}, {0, Cos[t], Sin[t], 0, 
        0}]\)], "Input",
  CellLabel->"In[416]:="],

Cell[BoxData[
    \(vx\  = \ randomv[]; \ vy\  = \ randomv[]; \ 
    ot\  = adaptsplframe[vx, vy]; ot // MatrixForm\)], "Input",
  CellLabel->"In[417]:="],

Cell[BoxData[
    \(Table[pssp[%[\([i]\)], \ %[\([j]\)]], \ {i, 2}, \ {j, 2}]\)], "Input",
  CellLabel->"In[418]:="],

Cell[BoxData[
    \(plotcircle2spv[vx, vy]\)], "Input",
  CellLabel->"In[419]:="],

Cell[BoxData[
    \(plotcircle2spv[ot[\([1]\)], ot[\([2]\)]]\)], "Input",
  CellLabel->"In[420]:="],

Cell[BoxData[
    \(Remove[vx, vy, ot, s1, t1]\)], "Input",
  CellLabel->"In[422]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
3.5  Radius, center, and position vector as functions of the subspace\
\>", "Subsection"],

Cell["\<\
In this section we calculate the basic euclidean parameters radius, center, \
and position vector of the circle as functions of the  vectors v,w spanning \
an euclidean 2-space in  the pseudo-euclidean 5-space. There are two ways to \
calculate these invariants: one can use the adaption procedure adaptsplframe, \
or one can use threepoints, and the concepts of section 2.3. For numerical \
calculations the speed of both methods is nearly the same. We have chosen the \
first method with the corresponding definitions as standard concepts, and \
included them into the package mcirc.m, since it seems to be better \
appropriate for symbolic calculations.\
\>", "Text"],

Cell[CellGroupData[{

Cell["The euclidean invariants with  frame adaption", "Subsubsection"],

Cell[BoxData[
    \(\(?circlespacevectors\)\)], "Input",
  CellLabel->"In[74]:="],

Cell[BoxData[
    \(circlespacevectors[x, y, z, u, v, r]\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(posvec[vx_List, vy_List]\  := \ 
      Module[{ab}, \n
        ab = adaptsplframe[vx, 
            vy]; \[IndentingNewLine]If[\(ab[\([2]\)]\)[\([4]\)]\  \[Equal] \ \
\(ab[\([2]\)]\)[\([5]\)], \ Return[{0, 0, 0}]]; \[IndentingNewLine]Return[
          Take[ab[\([1]\)], 3]]]\)], "Input",
  CellLabel->"In[10]:=",
  Evaluatable->False],

Cell[BoxData[
    \(center[vx_List, vy_List]\  := \ 
      Module[{ab}, \n
        ab = adaptsplframe[vx, 
            vy]; \[IndentingNewLine]If[\(ab[\([2]\)]\)[\([4]\)]\  \[Equal] \ \
\(ab[\([2]\)]\)[\([5]\)], \ 
          Return[{Infinity, Infinity, Infinity}]]; \[IndentingNewLine]Return[
          Take[ab[\([2]\)], 
              3]/\((\(ab[\([2]\)]\)[\([5]\)]\  - \ \
\(ab[\([2]\)]\)[\([4]\)])\)]]\)], "Input",
  CellLabel->"In[11]:=",
  Evaluatable->False],

Cell[BoxData[
    \(radius[vx_List, vy_List]\  := \ 
      Module[{ab}, \n
        ab = adaptsplframe[vx, 
            vy]; \[IndentingNewLine]If[\(ab[\([2]\)]\)[\([4]\)]\  \[Equal] \ \
\(ab[\([2]\)]\)[\([5]\)], \ Return[Infinity]]; \[IndentingNewLine]Return[
          1/\((\(ab[\([2]\)]\)[\([5]\)]\  - \ \
\(ab[\([2]\)]\)[\([4]\)])\)]]\)], "Input",
  CellLabel->"In[12]:=",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["The euclidean invariants with threepoints", "Subsubsection"],

Cell["\<\
We take three points of circle2spv[v,w][t] with fixed parameter values, and \
calculate the invariants on the base of these three points applying the \
functions defined in section 2.3.\
\>", "Text"],

Cell[BoxData[
    \(\(?threepoints\)\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    RowBox[{\(posvec1[v_List, w_List]\), ":=", " ", 
      RowBox[{"Module", "[", 
        RowBox[{\({p1, p2, p3, b}\), ",", "\n", "\t\t", 
          
          RowBox[{\(b = threepoints[v, w]\), ";", "\n", 
            " ", \(If[b \[Equal] Null, Return[]; Break[]]\), ";", 
            "\[IndentingNewLine]", \(p1 = \ b[\([1]\)]\), ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"p1", "\[Equal]", " ", 
                  RowBox[{"{", 
                    RowBox[{
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]]}], "}"}]}], ",", 
                " ", \(Return[{0, 0, 0}]\)}], "]"}], ";", "\n", 
            "\t\t", \(p2\  = \ b[\([2]\)]\), ";", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"p2", "\[Equal]", " ", 
                  RowBox[{"{", 
                    RowBox[{
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]]}], "}"}]}], ",", 
                " ", \(Return[{0, 0, 0}]\)}], "]"}], ";", "\n", 
            "\t\t", \(p3\  = \ \ b[\([3]\)]\), ";", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"p3", "\[Equal]", " ", 
                  RowBox[{"{", 
                    RowBox[{
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]]}], "}"}]}], ",", 
                " ", \(Return[{0, 0, 0}]\)}], "]"}], ";", "\n", 
            "\t\t", \(posvec3pts[p1, p2, p3]\)}]}], "]"}], "\n"}]], "Input",
  CellLabel->"In[6]:=",
  InitializationCell->True],

Cell[BoxData[
    RowBox[{\(center1[v_List, w_List]\), ":=", " ", 
      RowBox[{"Module", "[", 
        RowBox[{\({p1, p2, p3, b}\), ",", "\n", "\t\t", 
          
          RowBox[{\(b = threepoints[v, w]\), ";", "\n", 
            " ", \(If[b \[Equal] Null, Return[]; Break[]]\), ";", "\t", 
            "\[IndentingNewLine]", \(p1 = \ b[\([1]\)]\), ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"p1", "\[Equal]", " ", 
                  RowBox[{"{", 
                    RowBox[{
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]]}], "}"}]}], ",", 
                " ", \(Return[p1]\)}], "]"}], ";", "\n", 
            "\t\t", \(p2\  = \ b[\([2]\)]\), ";", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"p2", "\[Equal]", " ", 
                  RowBox[{"{", 
                    RowBox[{
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]]}], "}"}]}], ",", 
                " ", \(Return[p2]\)}], "]"}], ";", "\n", 
            "\t\t", \(p3\  = \ \ b[\([3]\)]\), ";", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"p3", "\[Equal]", " ", 
                  RowBox[{"{", 
                    RowBox[{
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]], ",", 
                      InterpretationBox["\[Infinity]",
                        DirectedInfinity[ 1]]}], "}"}]}], ",", 
                " ", \(Return[p3]\)}], "]"}], ";", "\n", 
            "\t", \(center3pts[p1, p2, p3]\)}]}], " ", "]"}], "\n"}]], "Input",\

  CellLabel->"In[7]:=",
  InitializationCell->True],

Cell[BoxData[
    RowBox[{\(radius1[v_List, w_List]\), ":=", " ", 
      RowBox[{"Module", "[", 
        RowBox[{\({p1, p2, p3, a, \ b}\), ",", "\n", "\t\t\t", 
          
          RowBox[{\(b = threepoints[v, w]\), ";", "\[IndentingNewLine]", 
            " ", \(If[b \[Equal] Null, Return[]; Break[]]\), ";", "\t", "\n", 
            "\t\t", \(p1 = \ b[\([1]\)]\), ";", \(p2\  = \ b[\([2]\)]\), 
            ";", \(p3\  = b[\([3]\)]\), ";", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"p1", "\[Equal]", " ", 
                    RowBox[{"{", 
                      RowBox[{
                        InterpretationBox["\[Infinity]",
                          DirectedInfinity[ 1]], ",", 
                        InterpretationBox["\[Infinity]",
                          DirectedInfinity[ 1]], ",", 
                        InterpretationBox["\[Infinity]",
                          DirectedInfinity[ 1]]}], "}"}]}], "||", 
                  RowBox[{"p2", "\[Equal]", " ", 
                    RowBox[{"{", 
                      RowBox[{
                        InterpretationBox["\[Infinity]",
                          DirectedInfinity[ 1]], ",", 
                        InterpretationBox["\[Infinity]",
                          DirectedInfinity[ 1]], ",", 
                        InterpretationBox["\[Infinity]",
                          DirectedInfinity[ 1]]}], "}"}]}], "||", 
                  RowBox[{"p3", "\[Equal]", " ", 
                    RowBox[{"{", 
                      RowBox[{
                        InterpretationBox["\[Infinity]",
                          DirectedInfinity[ 1]], ",", 
                        InterpretationBox["\[Infinity]",
                          DirectedInfinity[ 1]], ",", 
                        InterpretationBox["\[Infinity]",
                          DirectedInfinity[ 1]]}], "}"}]}]}], ",", 
                " ", \(Return[Infinity]\)}], "]"}], ";", 
            "\n", \(a\  = \ center3pts[p1, p2, p3]\), ";", "\n", 
            "\t\t", \(If[a\  == \ {Infinity, Infinity, Infinity}, 
              Return[Infinity], Sqrt[\((a - p1)\) . \((a - p1)\)], 
              Sqrt[\((a - p1)\) . \((a - p1)\)]]\)}]}], "]"}]}]], "Input",
  CellLabel->"In[8]:=",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    \(posvec[stb[3], stb[4]]\)], "Input",
  CellLabel->"In[423]:="],

Cell[BoxData[
    \(posvec[stb[1], stb[3]]\)], "Input",
  CellLabel->"In[424]:="],

Cell[BoxData[
    \(center[stb[3], stb[4]]\)], "Input",
  CellLabel->"In[425]:="],

Cell[BoxData[
    \(center[stb[2], stb[1]]\)], "Input",
  CellLabel->"In[426]:="],

Cell[BoxData[
    \(radius[stb[3], stb[4]]\)], "Input",
  CellLabel->"In[427]:="],

Cell[BoxData[
    \(radius[\(-stb[4]\), stb[1]]\)], "Input",
  CellLabel->"In[428]:="],

Cell[BoxData[
    \(radius[stb[3], stb[1]]\)], "Input",
  CellLabel->"In[429]:="],

Cell[BoxData[
    \(\((wr\  = 
        orthopair[{2*Random[], \(-2\)*Random[], 1. , 3*Random[], \ 
            Random[]}]; \ v\  = \ wr[\([1]\)]; 
      w\  = \ wr[\([2]\)])\)\)], "Input",
  CellLabel->"In[430]:="],

Cell[BoxData[
    \(Timing[posvec[v, w]]\)], "Input",
  CellLabel->"In[431]:="],

Cell[BoxData[
    \(Timing[posvec1[v, w]]\)], "Input",
  CellLabel->"In[432]:="],

Cell[BoxData[
    \(Timing[center[v, w]]\)], "Input",
  CellLabel->"In[433]:="],

Cell[BoxData[
    \(Timing[center1[v, w]]\)], "Input",
  CellLabel->"In[434]:="],

Cell[BoxData[
    \(Timing[radius[v, w]]\)], "Input",
  CellLabel->"In[435]:="],

Cell[BoxData[
    \(Timing[radius1[v, w]]\)], "Input",
  CellLabel->"In[436]:="],

Cell[BoxData[
    \(Remove[v, w]\)], "Input",
  CellLabel->"In[437]:="],

Cell[BoxData[
    \(Timing[\((wr\  = 
          orthopair[{2*Random[], \(-2\)*Random[], 1. , 3*Random[], \ 
              Random[]}]; \ v\  = \ wr[\([1]\)]; w\  = \ wr[\([2]\)]; 
        mwr = threepoints[v, w]; \npa = mwr[\([1]\)]; pb = mwr[\([2]\)]; 
        pc = mwr[\([3]\)]; \npv\  = \ \ posvec1[v, w]; \n
        pm\  = \ \ center1[v, w]; \n
        Print["\<Position vector = \>", \ pv, "\<, \>", 
          pv . pv, "\<, \>", "\<center = \>", 
          N[pm], "\<, \>", "\<radius = \>", radius1[v, w]]; \n
        gr1 = plotcircle2spv[v, w]; \n
        gr2\  = \ 
          Graphics3D[{PointSize[0.05], Point[pa], Point[pb], Point[pc]}]; \n
        gr3\  = \ Graphics3D[Line[{pm, \ smoothing[pm\  + pv]}]]; \n
        Show[{gr3, gr1, gr2}])\)]\)], "Input",
  CellLabel->"In[438]:="],

Cell[BoxData[{
    \(\(pv\  = \ \ posvec[v, w];\)\), "\n", 
    \(\(pm\  = \ \ center[v, w];\)\), "\n", 
    \(\(Print["\<Position vector = \>", \ pv, "\<, \>", 
        pv . pv, "\<, \>", "\<center = \>", N[pm], "\<, \>", "\<radius = \>", 
        radius[v, w]];\)\), "\n", 
    \(\(gr1 = plotcircle2spv[v, w];\)\), "\n", 
    \(\(gr2\  = \ 
        Graphics3D[{PointSize[0.05], Point[pa], Point[pb], 
            Point[pc]}];\)\), "\n", 
    \(\(gr3\  = \ Graphics3D[Line[{pm, \ smoothing[pm\  + pv]}]];\)\), "\n", 
    \(Show[{gr3, gr1, gr2}]\)}], "Input",
  CellLabel->"In[439]:="],

Cell[BoxData[
    \(Remove[wr, v, w, mwr, pa, pb, pc, pv, pm, gr1, gr2, gr3]\)], "Input",
  CellLabel->"In[446]:="],

Cell[BoxData[
    \(Timing[\((wr\  = 
          orthopair[{2*Random[], \(-2\)*Random[], 1. , 3*Random[], \ 
              Random[]}]; \ v\  = \ wr[\([1]\)]; w\  = \ wr[\([2]\)]; 
        mwr = threepoints[v, w]; \npa = mwr[\([1]\)]; pb = mwr[\([2]\)]; 
        pc = mwr[\([3]\)]; \npv\  = \ \ posvec[v, w]; \n
        pm\  = \ \ center[v, w]; \n
        Print["\<Position vector = \>", \ pv, "\<, \>", 
          pv . pv, "\<, \>", "\<center = \>", 
          N[pm], "\<, \>", "\<radius = \>", radius[v, w]]; \n
        gr1 = plotcircle2spv[v, w]; \n
        gr2\  = \ 
          Graphics3D[{PointSize[0.05], Point[pa], Point[pb], Point[pc]}]; \n
        gr3\  = \ Graphics3D[Line[{pm, \ smoothing[pm\  + pv]}]]; \n
        Show[{gr3, gr1, gr2}])\)]\)], "Input",
  CellLabel->"In[447]:="],

Cell[BoxData[
    \(Timing[\((wr\  = 
          orthopair[{2*Random[], \(-2\)*Random[], 1. , 3*Random[], \ 
              Random[]}]; \ v\  = \ wr[\([1]\)]; w\  = \ wr[\([2]\)]; 
        mwr = threepoints[v, w]; \npa = mwr[\([1]\)]; pb = mwr[\([2]\)]; 
        pc = mwr[\([3]\)]; \npv\  = \ \ posvec1[v, w]; \n
        pm\  = \ \ center1[v, w]; \n
        Print["\<Position vector = \>", \ pv, "\<, \>", 
          pv . pv, "\<, \>", "\<center = \>", 
          N[pm], "\<, \>", "\<radius = \>", radius1[v, w]]; \n
        gr1 = plotcircle2spv[v, w]; \n
        gr2\  = \ 
          Graphics3D[{PointSize[0.05], Point[pa], Point[pb], Point[pc]}]; \n
        gr3\  = \ Graphics3D[Line[{pm, \ smoothing[pm\  + pv]}]]; \n
        Show[{gr3, gr1, gr2}])\)]\)], "Input",
  CellLabel->"In[448]:="],

Cell[BoxData[
    \(Remove[wr, v, w, mwr, pa, pb, pc, pv, pm, gr1, gr2, gr3]\)], "Input",
  CellLabel->"In[449]:="],

Cell[BoxData[{
    \(s1\  = \ Random[]; \ t1\  = \ Random[]*Pi;\), "\[IndentingNewLine]", 
    \(\(ot\  = \ 
        adaptsplframe[{Sqrt[1 + s1^2], 0, 0, 0, s1}, {0, Cos[t1], Sin[t1], 0, 
            0}];\)\), "\[IndentingNewLine]", 
    \(\(gt1\  = \ 
        plotcircle2spv[ot[\([1]\)], \ 
          ot[\([2]\)]];\)\), "\[IndentingNewLine]", 
    \(\(gt2\  = 
        euklidsphereplot3D[
          ot[\([1]\)], \(-Abs[radius1[ot[\([1]\)], ot[\([2]\)]]]\), 
          3*Abs[radius1[ot[\([1]\)], ot[\([2]\)]]], \(-2\)*
            Abs[radius1[ot[\([1]\)], ot[\([2]\)]]], 
          2*Abs[radius1[ot[\([1]\)], 
                ot[\([2]\)]]]];\)\), "\[IndentingNewLine]", 
    \(\(gt3 = euklidsphereplot3D[ot[\([2]\)]];\)\), "\[IndentingNewLine]", 
    \(Show[{gt1, gt2, gt3}]\)}], "Input",
  CellLabel->"In[450]:="],

Cell[BoxData[
    \(gtest\  = \ %\)], "Input",
  CellLabel->"In[456]:="],

Cell[BoxData[
    \(MVShow3D[gtest]\)], "Input",
  CellLabel->"In[457]:="],

Cell[BoxData[
    \(Remove[ot, s1, t1]\)], "Input",
  CellLabel->"In[458]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
3.6 From  three points to the subspace of the corresponding circle\
\>", "Subsection"],

Cell[CellGroupData[{

Cell["Definitions", "Subsubsection"],

Cell["\<\
Let p1, p2, p3 be three pairwise distinct points in the euclidean 3-space. \
Their preimages under the stereographic projection define uniquely a circle \
on the 3-sphere, and the corresponding isotropic vectors span a \
pseudo-euclidean 3-space, the orthogonal complement of which is the euclidean \
subspace we ask for.\
\>", "Text"],

Cell[BoxData[
    \(\(?vspace\)\)], "Input",
  CellLabel->"In[62]:="],

Cell[BoxData[
    \(vspace[p1_, p2_, p3_] := 
      Module[{q1, q2, q3, j = 1, w1, w2, chj}, 
        q1 = Simplify[invstepro[p1]]; \[IndentingNewLine]q2 = 
          Simplify[invstepro[p2]]; \[IndentingNewLine]q3 = 
          Simplify[invstepro[p3]]; \[IndentingNewLine]Label[
          chj]; \[IndentingNewLine]If[j \[Equal] dim, 
          Print["\<vspace not found! Points not in general position?\>"]; 
          Return[], j = j + 1]; \[IndentingNewLine]w1 = 
          Simplify[pscross[{q1, q2, q3, stb[j]}]]; \[IndentingNewLine]If[
          smoothing[w1] \[Equal] nul[dim], Goto[chj], 
          w1 = Simplify[w1/Sqrt[pssp[w1, w1]]], 
          w1 = Simplify[w1/Sqrt[pssp[w1, w1]]]]; \ \ \ \ \ \ w2 = 
          Simplify[pscross[{q1, q2, q3, w1}]]; \[IndentingNewLine]If[
          w2 \[Equal] nul[dim], Goto[chj]]; \[IndentingNewLine]w2 = 
          Simplify[w2/Sqrt[pssp[w2, w2]]]; \t\tReturn[
          smoothing[{w1, w2}]]]\)], "Input",
  CellLabel->"In[28]:=",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    \(vspace[stb[1, 3], \ {0, 0, 0}, \ {2, 0, 0}]\)], "Input",
  CellLabel->"In[490]:="],

Cell[BoxData[
    \(vspace[stb[1, 3], \ {1, 0, 0}, \ {2, 0, 0}]\)], "Input",
  CellLabel->"In[491]:="],

Cell[BoxData[
    \(vspace[stb[1], \ {1, 0, 0}, \ {2, 0, 0}]\)], "Input",
  CellLabel->"In[492]:="],

Cell[BoxData[
    \(Timing[
      plotcircle3pts[{1, 0, 0}, \ {0, 1, 0}, {1, \(-1\), 0}]]\)], "Input",
  CellLabel->"In[493]:="],

Cell[BoxData[
    \(vspace[{1, 0, 0}, \ {0, 1, 0}, {1, \(-1\), 0}]\)], "Input",
  CellLabel->"In[494]:="],

Cell[BoxData[
    \(vs = %\)], "Input",
  CellLabel->"In[495]:="],

Cell[BoxData[
    \(Timing[plotcircle2spv[vs[\([1]\)], vs[\([2]\)]]]\)], "Input",
  CellLabel->"In[496]:="],

Cell[BoxData[
    \(radius[vs[\([1]\)], vs[\([2]\)]]\)], "Input",
  CellLabel->"In[497]:="],

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[498]:="],

Cell[BoxData[
    \(N[center[vs[\([1]\)], vs[\([2]\)]]]\)], "Input",
  CellLabel->"In[499]:="],

Cell[BoxData[
    \(%\  - {1, \ 0, \ 0}\)], "Input",
  CellLabel->"In[500]:="],

Cell[BoxData[
    \(Sqrt[% . %]\)], "Input",
  CellLabel->"In[501]:="],

Cell[BoxData[
    \(N[posvec[vs[\([1]\)], vs[\([2]\)]]]\)], "Input",
  CellLabel->"In[502]:="],

Cell[BoxData[
    \(vspace[{1, 0, 0}, \ {0, 0, 0}, {\(-1\), 0, 0}]\)], "Input",
  CellLabel->"In[503]:="],

Cell[BoxData[
    \(vs\  = \ %\)], "Input",
  CellLabel->"In[504]:="],

Cell[BoxData[
    \(plotcircle2spv[vs[\([1]\)], vs[\([2]\)]]\)], "Input",
  CellLabel->"In[505]:="],

Cell[BoxData[
    \(radius[vs[\([1]\)], vs[\([2]\)]]\)], "Input",
  CellLabel->"In[506]:="],

Cell[BoxData[
    \(center[vs[\([1]\)], vs[\([2]\)]]\)], "Input",
  CellLabel->"In[507]:="],

Cell[BoxData[
    \(plotcircle3pts[{1, 0, 0}, \ {0, 0, 0}, {\(-1\), 0, 0}]\)], "Input",
  CellLabel->"In[508]:="],

Cell[BoxData[
    \(posvec[vs[\([1]\)], vs[\([2]\)]]\)], "Input",
  CellLabel->"In[509]:="],

Cell[BoxData[
    \(\((a\  = \ Random[]; \nb\  = \ 2*Random[]; \nc\  = \ \(-\ Random[]\); \ 
      Print[TableForm[{a, b, c}]]; \n
      Timing[plotcircle3pts[{a, 0, 0}, \ {0, b, 0}, {0, 0, 
            c}]]\n)\)\)], "Input",
  CellLabel->"In[510]:="],

Cell[BoxData[
    \(\((a\  = \ Random[]; \nb\  = \ 2*Random[]; \nc\  = \ \(-\ Random[]\); \ 
      Print[TableForm[{a, b, c}]]; \n
      Timing[vspace[{a, 0, 0}, \ {0, b, 0}, {0, 0, c}]]\n)\)\)], "Input",
  CellLabel->"In[511]:="],

Cell[BoxData[
    \(\(\(\t\)\(vs\  = \ %[\([2]\)]\)\)\)], "Input",
  CellLabel->"In[512]:="],

Cell[BoxData[
    \(Timing[plotcircle2spv[vs[\([1]\)], vs[\([2]\)]]]\)], "Input",
  CellLabel->"In[513]:="],

Cell[BoxData[
    \(\(\(\t\t\)\(Timing[
      plotcircle3pts[{a, 0, 0}, \ {0, b, 0}, {0, 0, c}]]\)\)\)], "Input",
  CellLabel->"In[514]:="],

Cell[BoxData[
    \(Remove[a, b, c, vs]\)], "Input",
  CellLabel->"In[515]:="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["4 Position and invariants of two circles",
  FontFamily->"Helvetica"]], "Section",
  FontFamily->"Courier",
  FontSize->19],

Cell[TextData[{
  "In this section we apply the general theory, see the paper [4], to the \
case of circles in the 3-sphere and, of course, the euclidean 3-space. For \
the M\[ODoubleDot]bius geometry of spheres see the notebook  mspheres.nb. \
Invariants for the other possible pairings (",
  Cell[BoxData[
      \(TraditionalForm\`S\^k\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`S\^m\)]],
  "), 0 \[LessEqual] k, m \[LessEqual] 3, are treated in the notebook \
pairs.nb."
}], "Text"],

Cell[CellGroupData[{

Cell["4.1 Case discussion", "Subsection"],

Cell["\<\
Notations: C0, C1 two distinct circles, W0, W1 the corresponding \
3-dimensional pseudo-euclidean subspaces, U0, U1 their orthogonal (euclidean) \
complements.
Problem: Express the geometric properties and invariants of C0, C1 by \
algebraic properties and invariants of U0, U1.
Since                                  Dim(W0\[Intersection]W1) + Dim(W0 + \
W1) = Dim W0 + Dim W1= 6\t\t\t\t\t\t\t(1)
we distinguish (remember: orthogonal complement[W0\[Intersection]W1] = U0 + \
U1)
1.  Dim(W0\[Intersection]W1) = 1\t<=>Dim(U0 +U1)=4\t\t\t\t
2. Dim(W0\[Intersection]W1) = 2\t<=>Dim(U0 +U1)=3\t\t\t\t\t\t\t\t\t\t\t\t(2)
The geometric meaning of these cases is, because of (1),
Case 1 <=> Dim(W0 + W1) = 5 <=> there does not exist any 2-sphere containing \
C0\[Union]C1 <=> Dim(U0\[Intersection]U1) = 0.\t\t\t(3)
Case 2 <=> Dim(W0 + W1) = 4 <=> there exists a 2-sphere containing \
C0\[Union]C1  <=> Dim(U0\[Intersection]U1) = 1.\t\t\t\t(4)
In each of these cases we have three subcases with obvious geometric \
interpretations:
a)W0\[Intersection]W1 euclidean <=> C0\[Intersection]C1 is empty and C0, C1 \
can be separated by spheres <=> U0 + U1 is pseudo-euclidean.\t\t(5)
case 1 and b)W0\[Intersection]W1 pseudo-euclidean <=> C0\[Intersection]C1 is \
empty, C0, C1 are interlaced <=> U0 + U1 is euclidean, 4-dimensional.\t(6)
case 2 and b)W0\[Intersection]W1 pseudo-euclidean <=> C0\[Intersection]C1 \
={p0,p1} - two points <=> U0 + U1 is euclidean, 3-dimensional.\t\t(7)
c)W0\[Intersection]W1 isotropic <=> C0\[Intersection]C1 = {p} - one point <=> \
U0 + U1 is isotropic.\t\t\t\t\t\t\t\t(8)\
\>", "Text"],

Cell["\<\
Here \"separated by spheres\" means that there exist disjoint spheres S0, S1 \
with C0\[Subset]S0, C1\[Subset]S1; \"interlaced\" means that \
C0\[Intersection]C1 is empty, and such spheres do not exist: any two spheres \
S0,S1 with C0\[Subset]S0, C1\[Subset]S1 must intersect.
In the following we discuss the M\[ODoubleDot]bius invariants of the pairs in \
each of the cases and give examples.  \
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2 Invariants for pairs of euclidean 2-subspaces", "Subsection"],

Cell[CellGroupData[{

Cell["Case criterion", "Subsubsection"],

Cell["\<\
If U0 and U1 are the euclidean vector subspaces spanned by the orthonormal \
bases {v1, v2}, {w1, w2} respectively, then an important equivariant entity \
for the case characterization is the pseudo-orthogonal cross product \
\>", "Text"],

Cell["C0xC1 := pscross[{v1, v2, w1, w2}].", "Text",
  CellLabel->"In[26]:="],

Cell["\<\
By  (3) we have case 1 iff  C0xC1 does not vanish; from (5) to (8) and the \
orthogonality of C0xC1 to U0 + U1 we see, that the following disjunction \
holds:
case 1.a) <==> ch[C0xC1] = 1;
case 1.b) <==> ch[C0xC1]  = -1;
case 1.c) <==> ch[C0xC1] = 0;
case 2  <==> C0xC1 = o.
Remember:\
\>", "Text"],

Cell[BoxData[
    \(\(?ch\)\)], "Input",
  CellLabel->"In[12]:="],

Cell["Case 2 leads to", "Text"],

Cell[BoxData[
    \(ch[nul[]]\)], "Input",
  CellLabel->"In[13]:="],

Cell["\<\
From the general properties of the cross product one easily concludes: 
C0xC1 is independent of the choice of the oriented orthonormal bases spanning \
U0 and U1; it changes the sign if the bases correspond to different \
orientations of U1 or U1. ch[C0xC1] depends only of the (non-oriented) \
circles C0,C1.\
\>", "Text"],

Cell[BoxData[
    \(\(\(ftest[
        t_]\)\(\ \)\(:=\)\(\ \)\(pscross[{vec[x]\ Cos[t] + \ 
            vec[y]\ Sin[t], \(-vec[x]\) Sin[t]\  + vec[y] Cos[t], \ vec[u], \ 
          vec[v]}]\)\(\ \)\)\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \(D[ftest[t], \ t]\)], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[
    \(Remove[ftest]\)], "Input",
  CellLabel->"In[16]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Example: Random pair, case 1.a", "Subsubsection"],

Cell["\<\
We define  four vector functions such that {va1[t], va2[s]}, {wa1[u], wa2[v]} \
are orthonormal pairs for any values of s,t,u,v:\
\>", "Text"],

Cell[BoxData[
    \(\((va1[t_]\  := \ stb[3]*Cos[t]\  + \ stb[4]*Sin[t]; \ 
      va2\ [s_] := \ stb[5]*\ Sinh[s]\  + \ stb[2]*\ Cosh[s]; \n\t
      wa1[s_]\  := \ stb[3]*Cosh[s]\  + \ stb[5]*Sinh[s]; \ 
      wa2[t_]\  := \ stb[2]*Sin[t]\  + \ stb[4]*\ Cos[t]; \n
      C0xC1\  = \ pscross[{va1[t], va2[s], wa1[s], wa2[t]}])\)\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(Simplify[ch[%]]\)], "Input",
  CellLabel->"In[19]:="],

Cell["\<\
Remark: For s = 0 or t=(2k+1]Pi/4 we have case 2. For all other values of t,s \
we have case 1. a : Non-intersecting, separated circles.\
\>", "Text"],

Cell[BoxData[{
    \(gr1 = plotcircle2spv[va1[Random[]*Pi], \ va2[Random[]*Pi]]\), "\n", 
    \(gr2\  = \ 
      plotcircle2spv[wa1[Random[]*Pi], \ wa2[Random[]*Pi]]\), "\n", 
    \(Show[{gr1, gr2}, \ PlotRange\  -> \ All]\)}], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(separ1\  = \ %\)], "Input",
  CellLabel->"In[23]:="],

Cell["Apply MathGL3D to visualize the mutual position!", "Text"],

Cell[BoxData[
    \(MVShow3D[separ1]\)], "Input",
  CellLabel->"In[25]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Example: Random pair, case 1.b", "Subsubsection"],

Cell["\<\
We define other four vector functions such that {vb1[t], vb2[s]}, {wb1[u], \
wb2[v]} are orthonormal pairs for any values of s,t,u,v:\
\>", "Text"],

Cell[BoxData[
    \(\((vb1[t_]\  := \ stb[3]*Cos[t]\  + \ stb[4]*Sin[t]; \ 
      vb2\ [s_] := \ \(-stb[1]\)*\ Sin[s]\  + \ stb[2]*\ Cos[s]; \n\t
      wb1[s_]\  := \ stb[1]*Cos[s]\  + \ stb[2]*Sin[s]; \ 
      wb2[t_]\  := \ \(-stb[3]\)*Sin[t]\  + \ stb[4]*\ Cos[t]; \n
      C0xC1\  = \ pscross[{vb1[t], vb2[s], wb1[s], wb2[t]}])\)\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \(ch[%]\)], "Input",
  CellLabel->"In[28]:="],

Cell["\<\
It follows: Any two circles generated by the orthonormal pairs {v1[t], \
v2[s]}, {w1[u], w2[v]} are  interlaced. One can test it evaluating the next \
four cells and applying MathGL3d, see section 7.2 below. (By numerical \
reasons it sometimes happens that the random circles have \"corners\"; try to \
avoid that increasing the number of plotpoints.)\
\>", "Text"],

Cell[BoxData[{
    \(gr1 = plotcircle2spv[vb1[Random[]*Pi], \ vb2[Random[]*Pi]]\), "\n", 
    \(gr2\  = \ 
      plotcircle2spv[wb1[Random[]*Pi], \ wb2[Random[]*Pi]]\), "\n", 
    \(Show[{gr1, gr2}, \ PlotRange\  -> \ All]\)}], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(inter1\  = \ %\)], "Input",
  CellLabel->"In[32]:="],

Cell[BoxData[
    \(MVShow3D[inter1]\)], "Input",
  CellLabel->"In[33]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Example: Random pair, case 1.c", "Subsubsection"],

Cell["We shall define random circles intersecting at the origin:", "Text"],

Cell[BoxData[
    \(\((pa = randomv[3]; pb\  = \ randomv[3]; \ qa\  = \ 2*randomv[3]; \ 
      qb\  = \ 2*randomv[3]; \n\tmvp\  = \ vspace[nul[3], pa, pb]; \ 
      mvq\  = vspace[nul[3], qa, qb]; \n\t
      Print[smoothing[
          pscross[{mvp[\([1]\)], mvp[\([2]\)], mvq[\([1]\)], 
              mvq[\([2]\)]}]]]; \n\t\tPrint[
        ch[pscross[{mvp[\([1]\)], mvp[\([2]\)], mvq[\([1]\)], 
              mvq[\([2]\)]}]]]; \n\tgr1\  = \ plotcircle3pts[nul[3], pa, pb]; 
      gr2\  = \ plotcircle3pts[nul[3], qa, qb]; \n\t
      Show[{gr1, gr2}, \ PlotRange\  -> \ All])\)\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(onept\  = \ %\)], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(ParametricPlot3D[{x, y, 0}, {x, \(-1\), 2}, \ {y, \ \(-1\), 
        2}]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(Show[{onept, %}]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(onept1\  = \ %\)], "Input",
  CellLabel->"In[38]:="],

Cell[BoxData[
    \(MVShow3D[onept1, \ MVLineTubeSize\  \[Rule] \ 0.01]\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(Remove[pa, pb, qa, qb, mvp, \ mvq, \ gr1, gr2]\)], "Input",
  CellLabel->"In[41]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Stationary angles", "Subsubsection"],

Cell["\<\
Now we are going to describe the basic invariants for pairs of circles under \
the M\[ODoubleDot]bius group. In the next sections we then shall give further \
examples for which we calculate these invariants. As for stationary angles of \
subspaces of the euclidean space the invariants are found by considering the \
projections defined by the corresponding spacelike subspaces, see [4]. \
\>", "Text"],

Cell[TextData[
"pp[v1, v2, w1, w2]  is the matrix of the double projection pp= p2\
\[SmallCircle] p1, the composition of the orthogonal projections\n\t\tp1: U0 \
-> U1, and p2: U1 -> U0, \nwith respect to the base {v1, v2}."], "Text"],

Cell[BoxData[
    \(\(\ pp[v1_, \ v2_, \ w1_, \ w2_]\  := \ \n
      \ \ \ \ {{pssp[v1, \ w1]^2\  + \ pssp[v1, \ w2]^2, \ \n\ \ \ \ \ \ 
          pssp[v1, \ w1]*pssp[v2, \ w1]\  + \ pssp[v1, \ w2]*pssp[v2, \ w2]}, 
        \ \n\ \ \ \ \ {
          pssp[v1, \ w1]*pssp[v2, \ w1]\  + \ pssp[v1, \ w2]*pssp[v2, \ w2], 
          \ \n\ \ \ \ \ \ pssp[v2, \ w1]^2\  + \ pssp[v2, \ w2]^2}}\)\)], 
  "Input",
  CellLabel->"In[97]:=",
  Evaluatable->False],

Cell["The invariants of pp are the determinant and the trace:", "Text"],

Cell[BoxData[
    \(ppdet[v1_, \ v2_, \ w1_, \ w2_]\  := \ 
      Det[pp[v1, \ v2, \ w1, \ w2]]\)], "Input",
  CellLabel->"In[5]:=",
  Evaluatable->False],

Cell[BoxData[
    \(\(\ \ 
    pptr[v1_, \ v2_, \ w1_, \ w2_]\  := \ 
      Sum[\(pp[v1, \ v2, \ w1, \ w2]\)[\([i, i]\)], \ {i, \ 1, \ 2}]\)\)], 
  "Input",
  CellLabel->"In[6]:=",
  Evaluatable->False],

Cell[BoxData[
    \(pp[v1, v2, w1, w2]\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(ppdet[v1, v2, w1, w2]\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(pptr[v1, v2, w1, w2]\)], "Input",
  CellLabel->"In[45]:="],

Cell["\<\
We show that  ppdet  is the square of the determinant of the scalar products \
pssp[xi, yj], i, j = 1, 2, and therefore always non negative:\
\>", "Text"],

Cell[BoxData[
    \(ppdet[x1, x2, y1, 
        y2]\  - \((\ 
          pssp[x1, \ y1]*pssp[x2, \ y2]\  - \ 
            pssp[x1, \ y2]*pssp[x2, \ y1])\)^2\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[47]:="],

Cell[TextData[{
  "It follows ppdet[v1_,v2_,w1_,w2_]>= 0,  but ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " does not now that our square is a real number:"
}], "Text"],

Cell[BoxData[
    \(TrueQ[ppdet[v1_, v2_, w1_, w2_] >= \ 0]\)], "Input",
  CellLabel->"In[48]:="],

Cell[BoxData[
    \(TrueQ[Simplify[ppdet[v1_, v2_, w1_, w2_]] >= \ 0]\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(Eigenvalues[pp[v1, \ v2, \ w1, \ w2]]\)], "Input",
  CellLabel->"In[50]:="],

Cell["Using the vector object", "Text"],

Cell[BoxData[
    \(vec[w1]\)], "Input",
  CellLabel->"In[51]:="],

Cell["\<\
and the fact that the M\[ODoubleDot]bius group acts transitively on the \
circles, we can always assume that the first circle is the unit circle grc1\
\>", "Text"],

Cell[BoxData[
    \(Eigenvalues[pp[stb[3], \ stb[4], \ vec[w1], vec[w2]]]\)], "Input",
  CellLabel->"In[52]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[53]:="],

Cell["\<\
For shortness we write a,b,c,d instead of w1[3]=pssp[v1,w1], w2[4]= \
pssp[v2,w2],  w2[3]=pssp[v1,w2],  w1[4]=pssp[v2,w1], and consider the \
argument of Sqrt:\
\>", "Text"],

Cell[BoxData[
    \(poly[a_, b_, c_, 
        d_]\  := \ \(-4\)*\((c*d\  - \ a*b)\)^2\  + \ \((a^2\  + b^2\  + 
              c^2\  + d^2)\)^2\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \(Factor[poly[a, b, c, d]]\)], "Input",
  CellLabel->"In[55]:="],

Cell[TextData[{
  "Since the square of the first terms in Simplify[Eigenvalues] above \
coincides with ",
  Cell[BoxData[
      \(\((a\^2 + b\^2 + c\^2 + d\^2)\)\^2\)]],
  " and the first term is non negative we conclude for the eigenvalues \
\[Alpha]1, \[Alpha]2 of pp: "
}], "Text"],

Cell[TextData[
"\[Alpha]1 \[GreaterEqual] \[Alpha]2 \[GreaterEqual] 0."], "Text",
  TextAlignment->Center],

Cell[BoxData[
    \(Simplify[\((a\^2 + b\^2 + c\^2 + d\^2)\)\^2\  - \ \((a\^2 + 2\ a\ b + 
              b\^2 + c\^2 - 2\ c\ d + d\^2)\)\ \((a\^2 - 2\ a\ b + b\^2 + 
              c\^2 + 2\ c\ d + d\^2)\)]\)], "Input",
  CellLabel->"In[56]:="],

Cell[TextData[{
  "If U0, U1 are mutually orthogonal, we say that the corresponding circles \
are ",
  StyleBox["orthogonal",
    FontColor->RGBColor[1, 0, 1]],
  ". Obviously, then we have case 1.b: the circles are interlaced, and the \
eigenvalues fulfil \[Alpha]1 = \[Alpha]2 = 0. Two circles are called",
  StyleBox[" isogonal",
    FontColor->RGBColor[1, 0, 1]],
  ", if the eigenvalues satisfy \[Alpha]1 = \[Alpha]2. The question about the \
existence  of non-orthogonal isogonal circles will be answered positively in \
section 4.4 with the construction of examples. Obviously, orthogonal circles \
exist:"
}], "Text"],

Cell[BoxData[
    \(gr1\  = \ plotcircle2spv[stb[1], \ stb[2]]; 
    gr2\  = \ plotcircle2spv[stb[3], stb[4]]; Show[{gr1, gr2}]\)], "Input",
  CellLabel->"In[57]:="],

Cell[BoxData[
    \(pp[stb[1], stb[2], stb[3], stb[4]] // MatrixForm\)], "Input",
  CellLabel->"In[59]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.3 Case 1.a: disjoint,  non-interlaced circles", "Subsection"],

Cell[CellGroupData[{

Cell["Example", "Subsubsection"],

Cell["\<\
Take two disjoint spheres and a circle in each. There are lot of examples. \
\>", "Text"],

Cell[BoxData[
    \(\((v1\  = \ {1, \ 0, \ 1, \ \(-1\), \ 0}/Sqrt[3]; \ 
      v2\  = \ stb[2]\ ; \n\tw1\  = \ {3\/\@2, 1\/\@2, 0, 1\/\@2, 3\/\@2}; \ 
      w2\  = \ stb[3]\ ; \n
      C0xC1\  = \ pscross[{v1, v2, w1, w2}])\)\)], "Input",
  CellLabel->"In[60]:="],

Cell[BoxData[
    \(ch[%]\)], "Input",
  CellLabel->"In[61]:="],

Cell["\<\
It follows: case 1.a; the circles are disjoint and separated.\
\>", "Text"],

Cell[BoxData[
    \({plotcircle2spv[v1, v2], plotcircle2spv[w1, w2]}\)], "Input",
  CellLabel->"In[62]:="],

Cell[BoxData[
    \(Show[%, PlotRange\  \[Rule] All]\)], "Input",
  CellLabel->"In[63]:="],

Cell[BoxData[
    \(testgr\  = \ %\)], "Input",
  CellLabel->"In[64]:="],

Cell[BoxData[
    \(pp[v1, v2, w1, \ w2]\)], "Input",
  CellLabel->"In[65]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[66]:="],

Cell[BoxData[
    \(Eigenvalues[pp[v1, v2, w1, \ w2]]\)], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[68]:="],

Cell[BoxData[
    \(Sqrt[%]\)], "Input",
  CellLabel->"In[69]:="],

Cell["\<\
As it follows from the general theory [4], for case 1.a is characteristic \
that the maximal eigenvalue must be greater than 1, and has multiplicity 1. \
Therefore, in case 1.a may not exist isogonal circle pairs.\
\>", "Text"],

Cell[BoxData[
    \(Eigenvectors[pp[v1, v2, w1, \ w2]]\)], "Input",
  CellLabel->"In[70]:="],

Cell["\<\
Therefore the \"eigenspheres\" (spheres corresponding to the eigenvectors) \
containing the circle C0 are\
\>", "Text"],

Cell[BoxData[
    \(eigenv1\  = \ 
      normalize[\(-v1\)*\(\(-3\) + \@57\)\/\(4\ \@3\)\  + \ v2]\)], "Input",
  CellLabel->"In[71]:="],

Cell[BoxData[
    \(eigenv2\  = \ 
      normalize[v1\ *\(3 + \@57\)\/\(4\ \@3\)\  + \ v2]\)], "Input",
  CellLabel->"In[72]:="],

Cell["\<\
They intersect orthogonally in the circle C0. The spheres containing the \
circles C0, C1 are the one-parameter families S0[t]\[Superset]C0, S1[s]\
\[Superset] C1, respectively, defined by the spacelike unit vectors\
\>", "Text"],

Cell[BoxData[{
    \(\(uv0[t_] := \ eigenv1\ Cos[t]\  + \ \ eigenv2\ Sin[t];\)\ \), "\n", 
    \(\(uv1[s_]\  := \ w1\ Cos[s]\  + \ w2\ Sin[s];\)\)}], "Input",
  CellLabel->"In[73]:="],

Cell["\<\
 For the generalized cosinus of the intersection angle (see the notebook \
mspheres.nb) we calculate\
\>", "Text"],

Cell[BoxData[
    \(pssp[uv0[t], \ uv1[s]]\)], "Input",
  CellLabel->"In[75]:="],

Cell[BoxData[
    \(Simplify[Together[%]]\)], "Input",
  CellLabel->"In[76]:="],

Cell[BoxData[
    \(fctr[s_, t_]\  = \ %\)], "Input",
  CellLabel->"In[77]:="],

Cell[BoxData[
    \(Plot3D[fctr[s, t], {s, 0, \ 2\ Pi}, {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[78]:="],

Cell["\<\
The first eigenvector  of pp[v1,v2,w1,w2] corresponds to t=0 or Pi, where the \
spheres uv1[s] have the following generalized cosinus with the sphere eigenv1 \
.\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ 0], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[79]:="],

Cell["\<\
For t = Pi/2, 3Pi/2 we get correspondingly for the second eigenvector:\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ Pi/2], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[80]:="],

Cell["\<\
The amplitudes correspond to the square roots of the corresponding \
eigenvalues.\
\>", "Text"],

Cell["The two distinguished spheres are", "Text"],

Cell[BoxData[
    \({euklidsphereplot3D[eigenv1], euklidsphereplot3D[eigenv2]}\)], "Input",
  CellLabel->"In[82]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[83]:="],

Cell[BoxData[
    \(eigensph\  = \ %\)], "Input",
  CellLabel->"In[84]:="],

Cell[BoxData[
    \(MVShow3D[eigensph]\)], "Input",
  CellLabel->"In[122]:="],

Cell[BoxData[
    \(pssp[eigenv1, eigenv2]\)], "Input",
  CellLabel->"In[85]:="],

Cell[BoxData[
    \(Simplify[Together[%]]\)], "Input",
  CellLabel->"In[86]:="],

Cell["The intersection is the circle C0.", "Text"],

Cell[BoxData[
    \(plotcircle2spv[v1, v2]\)], "Input",
  CellLabel->"In[87]:="],

Cell[BoxData[
    \(Show[{%, \ eigensph}]\)], "Input",
  CellLabel->"In[88]:="],

Cell[BoxData[
    \(testgr0\  = \ %\)], "Input",
  CellLabel->"In[89]:="],

Cell[BoxData[
    \(MVShow3D[testgr0, \ MVLineTubeSize\  \[Rule] \ 0.02]\)], "Input",
  CellLabel->"In[123]:="],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[uv1[s\ *2\ Pi], \(-1\), 1, \(-1\), 1, \ 
        PlotPoints\  -> \ 30], \ {s, \ 0.3, \ 0.5, \ 0.1}]\)], "Input",
  CellLabel->"In[90]:="],

Cell[BoxData[
    \(testgr1 = \ %\)], "Input",
  CellLabel->"In[91]:="],

Cell[BoxData[
    \(Show[testgr1]\)], "Input",
  CellLabel->"In[92]:="],

Cell[BoxData[
    \(testgr1\  = \ %\)], "Input",
  CellLabel->"In[93]:="],

Cell[BoxData[
    \(euklidsphereplot3D[eigenv2]\)], "Input",
  CellLabel->"In[94]:="],

Cell[BoxData[
    \(testgr2 = \ Flatten[{testgr1, %}]\)], "Input",
  CellLabel->"In[95]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[96]:="],

Cell[BoxData[
    \(testgr2\  = \ %\)], "Input",
  CellLabel->"In[97]:="],

Cell[BoxData[
    \(radius[w1, w2]\)], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[99]:="],

Cell[BoxData[
    \(N[center[v1, v2]]\)], "Input",
  CellLabel->"In[100]:="],

Cell[BoxData[
    \(N[posvec[v1, v2]]\)], "Input",
  CellLabel->"In[101]:="],

Cell["There are many possibilities for experimental graphics...", "Text"],

Cell["\<\
The projection of the eigenvalue eigenv2 onto the 2-space U1 is\
\>", "Text"],

Cell[BoxData[
    \(preigenv2\  = \ 
      w1*pssp[w1, eigenv2]\  + \ w2*pssp[w2, eigenv2]\)], "Input",
  CellLabel->"In[102]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[103]:="],

Cell[BoxData[
    \(npreigenv2\  = \ normalize[preigenv2]\)], "Input",
  CellLabel->"In[104]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[105]:="],

Cell[BoxData[
    \(N[pssp[npreigenv2, eigenv2]]\)], "Input",
  CellLabel->"In[106]:="],

Cell["\<\
This is the square root of the eigenvalue. Since it is greater 1, the spheres \
defined by these vectors are disjoint and separate the circles\
\>", "Text"],

Cell[BoxData[
    \(plotcircle2spv[v1, v2]\)], "Input",
  CellLabel->"In[107]:="],

Cell[BoxData[
    \(c1\  = \ %\)], "Input",
  CellLabel->"In[108]:="],

Cell[BoxData[
    \(euklidsphereplot3D[eigenv2]\)], "Input",
  CellLabel->"In[109]:="],

Cell[BoxData[
    \(s1\  = \ %\)], "Input",
  CellLabel->"In[110]:="],

Cell[BoxData[
    \(Show[{c1, s1}]\)], "Input",
  CellLabel->"In[111]:="],

Cell[BoxData[
    \(sc1\  = \ %\)], "Input",
  CellLabel->"In[112]:="],

Cell[BoxData[
    \(plotcircle2spv[w1, w2]\)], "Input",
  CellLabel->"In[113]:="],

Cell[BoxData[
    \(c2\  = \ %\)], "Input",
  CellLabel->"In[114]:="],

Cell[BoxData[
    \(euklidsphereplot3D[npreigenv2]\)], "Input",
  CellLabel->"In[115]:="],

Cell[BoxData[
    \(s2\  = \ %\)], "Input",
  CellLabel->"In[116]:="],

Cell[BoxData[
    \(Show[{c2, s2}]\)], "Input",
  CellLabel->"In[117]:="],

Cell[BoxData[
    \(sc2\  = \ %\)], "Input",
  CellLabel->"In[118]:="],

Cell[BoxData[
    \(Show[{sc1, sc2}]\)], "Input",
  CellLabel->"In[119]:="],

Cell[BoxData[
    \(test\  = \ %\)], "Input",
  CellLabel->"In[120]:="],

Cell["Now link MathGL3d (see section 7.2), apply", "Text"],

Cell[BoxData[
    \(MVShow3D[test, MVLineTubeSize\  -> \ 0.01]\)], "Input",
  CellLabel->"In[121]:="],

Cell["\<\
and look at the picture with Appearance/WireFrame, spin it around the axis. \
You will see the eigensphere eigenv2, containing the circle C0, and the \
sphere corresponding to the projection npreigenv2, containing C1. The two \
spheres separate the circles.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Another example", "Subsubsection"],

Cell[BoxData[
    \(nv = N[circlespacevectors[2, 0, 0, Pi/4, Pi/4, \ 1]]\)], "Input",
  CellLabel->"In[124]:="],

Cell[BoxData[
    \(\((v1\  = \ stb[2]\ ; \ v2\  = \ stb[4]\ ; \n\tw1\  = nv[\([1]\)]; \ 
      w2\  = \ \ nv[\([2]\)]; \n
      C0xC1\  = \ pscross[{v1, v2, w1, w2}])\)\)], "Input",
  CellLabel->"In[125]:="],

Cell[BoxData[
    \(ch[%]\)], "Input",
  CellLabel->"In[126]:="],

Cell["It follows: case 1.a. ", "Text"],

Cell[BoxData[
    \({plotcircle2spv[v1, v2], plotcircle2spv[w1, w2]}\)], "Input",
  CellLabel->"In[127]:="],

Cell[BoxData[
    \(Show[\ %]\)], "Input",
  CellLabel->"In[128]:="],

Cell[BoxData[
    \(pp[v1, \ v2, w1, \ w2]\)], "Input",
  CellLabel->"In[129]:="],

Cell[BoxData[
    \(Eigenvalues[%]\)], "Input",
  CellLabel->"In[130]:="],

Cell[BoxData[
    \(Eigenvectors[pp[v1, \ v2, w1, \ w2]]\)], "Input",
  CellLabel->"In[131]:="],

Cell[BoxData[
    \(eigenv = \ %\)], "Input",
  CellLabel->"In[132]:="],

Cell[BoxData[
    \(eigenv1\  = \ 
      normalize[\(-v1\)*0.25666793515702424` - \ 
          v2*0.9664996487646695`]\)], "Input",
  CellLabel->"In[133]:="],

Cell[BoxData[
    \(eigenv2\  = \ 
      normalize[
        v2*0.25666793515702424` - \ v1*0.9664996487646695`]\)], "Input",
  CellLabel->"In[134]:="],

Cell[TextData[
"The spheres containing the circles C0, C1 are the one-parameter families \
S0[t]\[Superset]C0, S1[s]\[Superset] C1 defined by the spacelike unit \
vectors"], "Text"],

Cell[BoxData[{
    \(\(uv0[t_] := \ eigenv1\ Cos[t]\  + \ eigenv2\ \ Sin[t];\)\ \), "\n", 
    \(\(uv1[s_]\  := \ w1\ Cos[s]\  + \ w2\ Sin[s];\)\)}], "Input",
  CellLabel->"In[135]:="],

Cell["\<\
For the generalized cosinus of the intersection angle (see the notebook \
mspheres.nb) we calculate\
\>", "Text"],

Cell[BoxData[
    \(pssp[uv0[t], \ uv1[s]]\)], "Input",
  CellLabel->"In[137]:="],

Cell[BoxData[
    \(fctr[s_, t_]\  = \ Chop[%]\)], "Input",
  CellLabel->"In[138]:="],

Cell[BoxData[
    \(Plot3D[fctr[s, t], {s, 0, \ 2\ Pi}, {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[139]:="],

Cell[BoxData[
    \(Plot[fctr[s, \ 0], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[140]:="],

Cell["\<\
Maximum and Minimum are defined by the square root of the eigenvalue:\
\>", "Text"],

Cell[BoxData[
    \(Sqrt[2.1327822185373186`]\)], "Input",
  CellLabel->"In[141]:="],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[uv1[s\ *2\ Pi], \(-1\), 1, \(-1\), 1, \ 
        PlotPoints\  -> \ 30], \ {s, \ 0.3, \ 0.5, \ 0.1}]\)], "Input",
  CellLabel->"In[142]:="],

Cell[BoxData[
    \(testgr1 = \ %\)], "Input",
  CellLabel->"In[143]:="],

Cell[BoxData[
    \(euklidsphereplot3D[eigenv1]\)], "Input",
  CellLabel->"In[144]:="],

Cell[BoxData[
    \(testgr\  = \ Flatten[{testgr1, \ %}]\)], "Input",
  CellLabel->"In[145]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[146]:="],

Cell[BoxData[
    \(testgr\  = \ %\)], "Input",
  CellLabel->"In[147]:="],

Cell["\<\
The other eigenvector of pp[v1,v2,w1,w2]   corresponds to t=Pi/2 or 3 Pi/2, \
where the spheres uv1[s] intersect the eigensphere   in different angles, \
whose cosinus are defined by:\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ Pi/2], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[148]:="],

Cell["\<\
Maximum and Minimum are defined by the square root of the eigenvalue:\
\>", "Text"],

Cell[BoxData[
    \(Sqrt[0.11721778146268136`]\)], "Input",
  CellLabel->"In[149]:="],

Cell[BoxData[
    \(euklidsphereplot3D[eigenv2]\)], "Input",
  CellLabel->"In[150]:="],

Cell["There are many possibilities for experimental graphics...", "Text"],

Cell[BoxData[
    \(testgr\  = \ Flatten[{testgr1, \ %}]\)], "Input",
  CellLabel->"In[151]:="],

Cell[BoxData[
    \(Table[Show[{testgr[\([i]\)], \ \ testgr[\([4]\)]}], {i, 3}]\)], "Input",\

  CellLabel->"In[152]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[153]:="],

Cell[BoxData[
    \(testgr2\  = \ %\)], "Input",
  CellLabel->"In[154]:="],

Cell[BoxData[
    \(Remove[v1, v2, w1, w2, fctr, testgr, \ grsph1, uv1, uv0]\)], "Input",
  CellLabel->"In[156]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.4 Case 1.b: Interlaced circles", "Subsection"],

Cell["\<\
Already at the end of section 4.2 we have seen the standard unit circle in \
the x,y-plane and the z-axis are interlaced \"circles\". We give now examples \
of isogonal and non-isogonal interlaced circle pairs.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Example: Isogonal circles", "Subsubsection"],

Cell["A first example  of an isogonal pair: ", "Text"],

Cell[BoxData[
    \(\((w1\  = {1, 0, 1, 1, 0}/Sqrt[3]; \ 
      w2\  = \ {0, 1, \(-1\), 1, 0}/Sqrt[3]; \n
      griso\  = \ plotcircle2spv[w1, w2])\)\)], "Input",
  CellLabel->"In[157]:="],

Cell[BoxData[
    \(Show[{grc1, griso}]\)], "Input",
  CellLabel->"In[158]:="],

Cell[BoxData[
    \(isosph = %\)], "Input",
  CellLabel->"In[159]:="],

Cell["\<\
We again got an interlaced pair of circles; for seeing this clearer apply \
MathGL3d, section 7.2.  The unique eigenvalue is\
\>", "Text"],

Cell[BoxData[
    \(pp[stb[3], \ stb[4], w1, \ w2]\)], "Input",
  CellLabel->"In[160]:="],

Cell[BoxData[
    \(Eigenvalues[%]\)], "Input",
  CellLabel->"In[161]:="],

Cell[TextData[
"The spheres containing the circles C0, C1 are the one-parameter families \
S0[t]\[Superset]C0, S1[s]\[Superset] C1 defined by the spacelike unit \
vectors"], "Text"],

Cell[BoxData[{
    \(\(uv0[t_] := \ stb[3]\ Cos[t]\  + \ \ stb[4]\ Sin[t];\)\ \), "\n", 
    \(\(uv1[s_]\  := \ w1\ Cos[s]\  + \ w2\ Sin[s];\)\)}], "Input",
  CellLabel->"In[162]:="],

Cell["\<\
Any pair (S0[t], S1[s]]) intersects in a certain circle, since (C0,C1) are \
interlaced. For the cosinus of the intersection angle (see the notebook \
mspheres.nb) we calculate\
\>", "Text"],

Cell[BoxData[
    \(pssp[uv0[t], \ uv1[s]]\)], "Input",
  CellLabel->"In[164]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[165]:="],

Cell[BoxData[
    \(fctr[s_, t_]\  = \ %\)], "Input",
  CellLabel->"In[166]:="],

Cell[BoxData[
    \(Plot3D[fctr[s, t], {t, 0, 2\ Pi}, \ {s, 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[167]:="],

Cell["\<\
We do not have distinguished eigenspheres: since the eigenvalue has \
multiplicity 2, any sphere S0[t] \[Superset] C0 generates the same angle \
function with the family (S1[s]) (up to a phase translation). Therefore we \
called the circle pair (C0, C1) isogonal. In concordance with the  general \
theory  the maximum is the square root of the eigenvalue:\
\>", "Text"],

Cell[BoxData[
    \(fctr[0, \ Pi/4]\)], "Input",
  CellLabel->"In[170]:="],

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[171]:="],

Cell[BoxData[
    \(fctr[0, \ 5\ Pi/4]\)], "Input",
  CellLabel->"In[169]:="],

Cell["\<\
Since pssp[-n, m] = - pssp[n, m], we get all relative minimal values as the \
negatives of the relative maximal values, changing the orientation of one of \
the spheres.  \
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Family of circles,  isogonal to the unit circle grc1", "Subsubsection"],

Cell["\<\
Now we construct a family of isogonal pairs containing examples for any \
eigenvalue between 0 and 1. By the general theory it follows, that any pair \
of isogonal circles is M\[ODoubleDot]bius equivalent to a uniquely defined \
member of this family.
Arguments: By theorem 6 of [4] the eigenvalues are a complete system of \
invariants for pairs of circles not contained in a 2-sphere.  From the case \
discussion in this paper, cases 1 and 4, there exists at most one eigenvalue \
\[GreaterEqual] 1, and then it has multiplicity 1. Since the eigenvalues are \
all non-negative, in the isogonal case  the unique eigenvalue \[Alpha] \
fulfils 0 \[LessEqual] \[Alpha] < 1. For  \[Alpha] = 0 the circles are \
orthogonal, and this is a special case of isogonal. We shall see that in the \
family we are going to construct exists for any  \[Alpha], 0 < \[Alpha]  < 1, \
 exactly one member. The value \[Alpha]  = 1 corresponds to a pair of tangent \
circles (case 2.c); all these pairs are M\[ODoubleDot]bius equivalent, see \
also section 5.6.\
\>", "Text"],

Cell[BoxData[{
    \(\(iso1[s_]\  := \ {1, \ 0, \ 1/Cosh[s]\ , \ 1/Sinh[s], 0}/
          Sqrt[1\  + \ 1/Cosh[s]^2 + 1/Sinh[s]^2];\)\), "\n", 
    \(\t\t\(iso2[s_]\  := \ {0, 1, \ 1/Sinh[s]\ , \ \(-1\)/Cosh[s], 0}/
          Sqrt[1\  + \ 1/Cosh[s]^2 + 1/Sinh[s]^2];\)\), "\n", 
    \(grisofam[s_]\  := \ {iso1[s], iso2[s]}\)}], "Input",
  CellLabel->"In[172]:="],

Cell[BoxData[
    \(\(\(grisofam[s]\)\(\ \)\)\)], "Input",
  CellLabel->"In[175]:="],

Cell[BoxData[
    \(iso2[s]\)], "Input",
  CellLabel->"In[176]:="],

Cell[BoxData[
    \(Simplify[pssp[iso2[s], iso2[s]]]\)], "Input",
  CellLabel->"In[177]:="],

Cell[BoxData[
    \(pp[stb[3], \ stb[4], \ iso1[s], \ iso2[s]]\)], "Input",
  CellLabel->"In[178]:="],

Cell[BoxData[
    \(Eigenvalues[%]\)], "Input",
  CellLabel->"In[179]:="],

Cell[BoxData[
    \(Together[%]\)], "Input",
  CellLabel->"In[180]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[181]:="],

Cell["\<\
f[s_] := Csch[s]^2/(1 + Csch[s]^2 + Sech[s]^2) + 
   Sech[s]^2/(1 + Csch[s]^2 + Sech[s]^2)\
\>", "Input",
  CellLabel->"In[182]:="],

Cell[BoxData[
    \(f[s]\  - \ \((Cosh[s]^2\  + \ Sinh[s]^2)\)/\((Cosh[s]^2\  + \ 
            Sinh[s]^2\  + Cosh[s]^2\ *Sinh[s]^2)\)\)], "Input",
  CellLabel->"In[183]:="],

Cell[BoxData[
    \(Together[%]\)], "Input",
  CellLabel->"In[184]:="],

Cell["\<\
Therefore avoiding the poles will give a better definition for f:\
\>", "Text"],

Cell[BoxData[
    \(f[s_]\  := \ \((Cosh[s]^2\  + \ Sinh[s]^2)\)/\((Cosh[s]^2\  + \ 
            Sinh[s]^2\  + Cosh[s]^2\ *Sinh[s]^2)\)\)], "Input",
  CellLabel->"In[185]:="],

Cell[BoxData[
    \(Plot[f[s], \ {s, 0, 5}]\)], "Input",
  CellLabel->"In[186]:="],

Cell["\<\
The case s= 0 , f(0) = 1, does not give a circle since s = 0 is a singularity \
of grisofam:\
\>", "Text"],

Cell[BoxData[
    \(grisofam[0]\)], "Input",
  CellLabel->"In[187]:="],

Cell["\<\
Sometimes it is useful to create a circle family instead of a circle surface. \
In mcirc.m is defined\
\>", "Text"],

Cell[BoxData[
    \(circlefamily[v_, w_, t_, a_, b_, \ d_:  20, \ opt___]\  := \ 
      ParametricPlot3D[\n\t\tEvaluate[\n\t\tTable[\(circle2spv[v[t], \ 
                w[t]]\)[u], \ {t, \ a, \ b, \ \((b - a)\)/d}]], {u, 0, 
          2\ Pi}, \ opt]\)], "Input",
  CellLabel->"In[37]:=",
  Evaluatable->False],

Cell[BoxData[
    \(circlefamily[iso1, iso2, t, 0.1, 2.1, 10, \ 
      PlotPoints \[Rule] 80]\)], "Input",
  CellLabel->"In[188]:="],

Cell[BoxData[
    \(isofam\  = \ {grc1, %}\)], "Input",
  CellLabel->"In[189]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[190]:="],

Cell[BoxData[
    \(isofam = \ %\)], "Input",
  CellLabel->"In[191]:="],

Cell[BoxData[
    \(ThreeScript["\<isofam.ts\>", isofam]\)], "Input",
  CellLabel->"In[102]:="],

Cell["The circle family isofam is contained in the circle surface ", "Text"],

Cell[BoxData[
    \(ParametricPlot3D[\(circle2spv[\(grisofam[s]\)[\([1]\)], \ \(grisofam[
              s]\)[\([2]\)]]\)[t], \n\t{s, \ 0.1, 2.1}, \ {t, 0, \ 
        2\ \ Pi}, \ PlotPoints\  -> \ 50]\)], "Input",
  CellLabel->"In[192]:="],

Cell[BoxData[
    \(isosf\  = \ %\)], "Input",
  CellLabel->"In[193]:="],

Cell[BoxData[
    \(Show[{grc1, isosf}]\)], "Input",
  CellLabel->"In[194]:="],

Cell[BoxData[
    \(isosf = \ %\)], "Input",
  CellLabel->"In[195]:="],

Cell["\<\
Constructing smaller parts of the circle surface, e.g.  for 0.5 < s < 1.5, is \
recommended.\
\>", "Text"],

Cell[BoxData[
    \(ParametricPlot3D[\(circle2spv[\(grisofam[s]\)[\([1]\)], \ \(grisofam[
              s]\)[\([2]\)]]\)[t], \n\t{s, \ 0.4, 1.7}, \ {t, 0, \ 
        2\ \ Pi}, \ PlotPoints\  -> \ {30, 50}]\)], "Input",
  CellLabel->"In[196]:="],

Cell[BoxData[
    \(isosfb\  = \ %\)], "Input",
  CellLabel->"In[197]:="],

Cell[BoxData[
    \(Show[{grc1, isosfb}]\)], "Input",
  CellLabel->"In[198]:="],

Cell[BoxData[
    \(isosfb = %\)], "Input",
  CellLabel->"In[199]:="],

Cell[BoxData[
    \(ThreeScript["\<isosfb.ts\>", isosfb]\)], "Input",
  CellLabel->"In[123]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["The meaning of  'isogonal'", "Subsubsection"],

Cell["Spheres, containing the circles:", "Text",
  FontSize->18],

Cell[BoxData[
    \(Clear[fct, s, b, a, \ sph2]\)], "Input",
  CellLabel->"In[200]:="],

Cell[BoxData[
    \(sph1[a_]\  := \ {0, 0, Cos[a], \ Sin[a], 0}\)], "Input",
  CellLabel->"In[201]:="],

Cell[BoxData[
    \(sph2[s_, b_]\  := \ iso1[s]*Cos[b]\  + \ iso2[s]*Sin[b]\)], "Input",
  CellLabel->"In[202]:="],

Cell["\<\
The cosinus of the intersection angle of two of such spheres is:\
\>", "Text"],

Cell[BoxData[
    \(\(fct[s_]\)[a_, b_] := \ pssp[sph1[a], \ sph2[s, b]]\)], "Input",
  CellLabel->"In[203]:="],

Cell[BoxData[
    \(\(fct[s]\)[a, b]\)], "Input",
  CellLabel->"In[204]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[205]:="],

Cell["\<\
Clearly, for fixed a (or b) the resulting functions of  b (resp. a) are \
cos--functions with the same amplitude, depending on s only, and distinct \
phases.  For non isogonal circle pairs the amplitude is not constant (see the \
next subsection).\
\>", "Text"],

Cell[BoxData[
    \(Plot3D[\(fct[1. ]\)[a, b], {a, 0, 2*Pi}, {b, 0, 2*Pi}]\)], "Input",
  CellLabel->"In[206]:="],

Cell[BoxData[
    \(tgri = \ %\)], "Input",
  CellLabel->"In[207]:="],

Cell[BoxData[
    \(MVShow3D[tgri]\)], "Input",
  CellLabel->"In[208]:="],

Cell[BoxData[
    \(Clear[a, b]; \ a\  = 2*Pi*\ Random[]; \ 
    Plot[\(fct[1. ]\)[a, b], {b, 0, 2*Pi}]; a\)], "Input",
  CellLabel->"In[212]:="],

Cell[BoxData[
    \(Table[\ \((a\  = 2*Pi*Random[]; \ 
        Plot[\(fct[1. ]\)[a, b], {b, 0, 2*Pi}])\), {5}]\)], "Input",
  CellLabel->"In[213]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[214]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Intersection angle for spheres containing non isogonal circles\
\>", "Subsubsection"],

Cell[BoxData[
    \(Clear[a, b, br1, br2, sph1, \ sph2]\)], "Input",
  CellLabel->"In[215]:="],

Cell[BoxData[
    \(sph1[a_]\  := \ {0, 0, Cos[a], \ Sin[a], 0}\)], "Input",
  CellLabel->"In[216]:="],

Cell["Here are two orthonormal vectors defined \"at random\":", "Text"],

Cell[BoxData[
    \(br1\  = \ {\(-0.72439280526\), 0, 0.6893874554, 0, 0}\)], "Input",
  CellLabel->"In[217]:="],

Cell[BoxData[
    \(br2 = \ {0, \(-0.33744663644\), 0, 0.9413446593, 0}\)], "Input",
  CellLabel->"In[218]:="],

Cell[BoxData[
    \(pssp[br1, br2]\)], "Input",
  CellLabel->"In[219]:="],

Cell[BoxData[
    \(pssp[br1, br1]\)], "Input",
  CellLabel->"In[220]:="],

Cell[BoxData[
    \(pssp[br2, br2]\)], "Input",
  CellLabel->"In[221]:="],

Cell[BoxData[
    \(sph2[b_]\  := br1*Cos[b]\  + br2*Sin[b]\)], "Input",
  CellLabel->"In[222]:="],

Cell[BoxData[
    \(fct2[a_, b_] := \ pssp[sph1[a], \ sph2[b]]\)], "Input",
  CellLabel->"In[223]:="],

Cell[BoxData[
    \(fct2[a, b]\)], "Input",
  CellLabel->"In[224]:="],

Cell[BoxData[
    \(Plot3D[fct2[a, b], {a, 0, 2*Pi}, {b, 0, 2*Pi}]\)], "Input",
  CellLabel->"In[225]:="],

Cell[BoxData[
    \(ptgr\  = \ %\)], "Input",
  CellLabel->"In[226]:="],

Cell[BoxData[
    \(MVShow3D[ptgr]\)], "Input",
  CellLabel->"In[228]:="],

Cell["\<\
For fixed a (or b) we again have cos-functions; now the amplitude depends on \
a:\
\>", "Text"],

Cell[BoxData[
    \(Clear[a, b]; \ a\  = \ Random[]; \ Plot[fct2[a, b], {b, 0, 2*Pi}]; 
    a\)], "Input",
  CellLabel->"In[229]:="],

Cell[BoxData[
    \(Clear[a]\)], "Input",
  CellLabel->"In[230]:="],

Cell["The amplitude of fct2[.,b] as function of a is", "Text"],

Cell[BoxData[
    \(amplitude[a_]\  := 
      Sqrt[\((0.689387455\ Cos[a]\ )\)^2 + \((0.9413446593\ *
                Sin[a])\)^2]\)], "Input",
  CellLabel->"In[231]:="],

Cell[BoxData[
    \(Plot[amplitude[a], \ {a, 0, 2*Pi}]\)], "Input",
  CellLabel->"In[232]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Interlaced, non isogonal circles", "Subsubsection"],

Cell["\<\
Of course, the random circle pair, case 1.b, in section 4.2 are non-isogonal \
with probability 1:\
\>", "Text"],

Cell[BoxData[
    \(\((vb1[t_]\  := \ stb[3]*Cos[t]\  + \ stb[4]*Sin[t]; \ 
      vb2\ [s_] := \ \(-stb[1]\)*\ Sin[s]\  + \ stb[2]*\ Cos[s]; \n\t
      wb1[s_]\  := \ stb[1]*Cos[s]\  + \ stb[2]*Sin[s]; \ 
      wb2[t_]\  := \ \(-stb[3]\)*Sin[t]\  + \ stb[4]*\ Cos[t];)\)\)], "Input",\

  CellLabel->"In[233]:="],

Cell[BoxData[
    \(Eigenvalues[
      pp[vb1[Random[]*Pi], \ vb2[Random[]*Pi], \nwb1[Random[]*Pi], \ 
        wb2[Random[]*Pi]]]\)], "Input",
  CellLabel->"In[234]:="],

Cell[TextData[{
  "With probability 1 one gets two distinct eigenvalues, 0 < ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\_1\)]],
  "<",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\_2\)]],
  "<1.\nWe consider the following example:"
}], "Text"],

Cell[BoxData[
    \(\((v1\  = \ stb[3]\ ; \ v2\  = \ stb[4]\ ; \n\t
      w1\  = \ {\(-\(3\/5\)\), 0, 0, 4\/5, 0}; \ w2\  = \ stb[2]\ ; \n
      C0xC1\  = \ pscross[{v1, v2, w1, w2}])\)\)], "Input",
  CellLabel->"In[235]:="],

Cell[BoxData[
    \(ch[%]\)], "Input",
  CellLabel->"In[236]:="],

Cell["It follows: case 1.b; the circles are interlaced.", "Text"],

Cell[BoxData[
    \(plotcircle2spv[w1, w2]\)], "Input",
  CellLabel->"In[237]:="],

Cell[BoxData[
    \(Show[{grc1, \ %}]\)], "Input",
  CellLabel->"In[238]:="],

Cell[BoxData[
    \(pp[stb[3], \ stb[4], w1, \ w2]\)], "Input",
  CellLabel->"In[239]:="],

Cell[BoxData[
    \(Eigenvalues[%]\)], "Input",
  CellLabel->"In[240]:="],

Cell["\<\
The spheres containing the circles C0, C1 are the one-parameter families \
S0[t]\[Superset]C0, S1[s]\[Superset] C1 defined by the spacelike unit vectors\
\
\>", "Text"],

Cell[BoxData[{
    \(\(uv0[t_] := \ stb[3]\ Cos[t]\  + \ \ stb[4]\ Sin[t];\)\ \), "\n", 
    \(\(uv1[s_]\  := \ w1\ Cos[s]\  + \ w2\ Sin[s];\)\)}], "Input",
  CellLabel->"In[241]:="],

Cell["\<\
Any pair (S0[t], S1[s]) intersects in a certain circle, since (C0,C1) are \
interlaced. For the cosinus of the intersection angle (see the notebook \
mspheres.nb) we calculate\
\>", "Text"],

Cell[BoxData[
    \(pssp[uv0[t], \ uv1[s]]\)], "Input",
  CellLabel->"In[243]:="],

Cell[BoxData[
    \(fctr[s_, t_]\  = \ %\)], "Input",
  CellLabel->"In[244]:="],

Cell[BoxData[
    \(Plot3D[fctr[s, t], {s, 0, \ 2\ Pi}, {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[245]:="],

Cell["\<\
We have the eigenvector stb[3] of pp[v1,v2,w1,w2], which correspond to t=0 or \
Pi, where the spheres uv1[s] intersect the sphere stb[3] (that is the \
x,y-plane) orthogonally:\
\>", "Text"],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[uv1[s\ *2\ Pi], \(-1\), 1, \(-1\), 1\ , 
        PlotPoints\  -> \ 30], \ {s, \ 0.3, \ 0.5, \ 0.1}]\)], "Input",
  CellLabel->"In[246]:="],

Cell[BoxData[
    \(testgr1 = \ %\)], "Input",
  CellLabel->"In[247]:="],

Cell[BoxData[
    \(euklidsphereplot3D[stb[3]]\)], "Input",
  CellLabel->"In[248]:="],

Cell[BoxData[
    \(testgr\  = \ Flatten[{testgr1, \ %}]\)], "Input",
  CellLabel->"In[249]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[250]:="],

Cell[BoxData[
    \(testgr\  = \ %\)], "Input",
  CellLabel->"In[251]:="],

Cell["\<\
The other eigenvector of pp[v1,v2,w1,w2]  is stb[4], which correspond to \
t=Pi/2 or 3 Pi/2, where the spheres uv1[s] intersect the sphere stb[4] (that \
is the unit sphere with center o)  in different angles, whose cosinus are \
defined by:\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ Pi/2], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[252]:="],

Cell["\<\
Maximum and Minimum are defined by the square root of the eigenvalue, here \
4/5.  This is the unit sphere\
\>", "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[stb[4]]\)], "Input",
  CellLabel->"In[253]:="],

Cell[BoxData[
    \(grsph1 = %\)], "Input",
  CellLabel->"In[254]:="],

Cell["There are many possibilities for experimental graphics...", "Text"],

Cell[BoxData[
    \(testgr\  = \ Flatten[{testgr1, \ grsph1}]\)], "Input",
  CellLabel->"In[255]:="],

Cell[BoxData[
    \(Table[Show[{testgr[\([i]\)], \ \ testgr[\([4]\)]}], {i, 3}]\)], "Input",\

  CellLabel->"In[256]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[257]:="],

Cell[BoxData[
    \(testgr2\  = \ %\)], "Input",
  CellLabel->"In[258]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.5 Case 1c: Circles intersecting in exactly one point", "Subsection"],

Cell[CellGroupData[{

Cell["First example", "Subsubsection"],

Cell["\<\
See also the random pair, case 1.c, in section 4.2. Circles in orthogonal \
planes intersecting at the origin:\
\>", "Text"],

Cell[BoxData[
    \(circlespacevectors[0, 1, 0, 0, Pi/2, \ 1]\)], "Input",
  CellLabel->"In[123]:="],

Cell[BoxData[
    \(v1\  = \ %[\([1]\)]\)], "Input",
  CellLabel->"In[124]:="],

Cell[BoxData[
    \(v2\  = \ %%[\([2]\)]\)], "Input",
  CellLabel->"In[125]:="],

Cell[BoxData[
    \(plotcircle2spv[v1, \ v2]\)], "Input",
  CellLabel->"In[126]:="],

Cell[BoxData[
    \(circlespacevectors[0, 0, 1, 0, 0, \ 1]\)], "Input",
  CellLabel->"In[127]:="],

Cell[BoxData[
    \(w1\  = \ %[\([1]\)]\)], "Input",
  CellLabel->"In[128]:="],

Cell[BoxData[
    \(w2\  = \ %%[\([2]\)]\)], "Input",
  CellLabel->"In[129]:="],

Cell[BoxData[
    \(plotcircle2spv[w1, w2]\)], "Input",
  CellLabel->"In[130]:="],

Cell[BoxData[
    \(pscross[{v1, v2, w1, w2}]\)], "Input",
  CellLabel->"In[131]:="],

Cell[BoxData[
    \(ch[%]\)], "Input",
  CellLabel->"In[132]:="],

Cell["\<\
It follows: case 1.c; the circles are intersecting in one point and are in \
\"general position\".\
\>", "Text"],

Cell[BoxData[
    \({plotcircle2spv[v1, v2], plotcircle2spv[w1, w2]}\)], "Input",
  CellLabel->"In[133]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[134]:="],

Cell[BoxData[
    \(testgr\  = \ %\)], "Input",
  CellLabel->"In[135]:="],

Cell[BoxData[
    \(pp[v1, v2, w1, \ w2]\)], "Input",
  CellLabel->"In[136]:="],

Cell[BoxData[
    \(Eigenvectors[pp[v1, v2, w1, \ w2]]\)], "Input",
  CellLabel->"In[137]:="],

Cell["\<\
Therefore the \"eigenspheres\" (spheres corresponding to the eigenvectors) \
containing the circle C0 are\
\>", "Text"],

Cell[BoxData[
    \(eigenv1\  = v1\)], "Input",
  CellLabel->"In[138]:="],

Cell[BoxData[
    \(eigenv2\  = \ v2\)], "Input",
  CellLabel->"In[139]:="],

Cell["The two  eigenspheres are", "Text"],

Cell[BoxData[
    \({euklidsphereplot3D[eigenv1], euklidsphereplot3D[eigenv2]}\)], "Input",
  CellLabel->"In[140]:="],

Cell[BoxData[
    \(Show[%, \ PlotRange\  -> \ {\(-1\), 1}]\)], "Input",
  CellLabel->"In[141]:="],

Cell[BoxData[
    \(eigensph\  = \ %\)], "Input",
  CellLabel->"In[142]:="],

Cell[TextData[
"They intersect orthogonally in the circle C0. The spheres containing the \
circles C0, C1 are the one-parameter families S0[t]\[Superset]C0, S1[s]\
\[Superset] C1 defined by the spacelike unit vectors"], "Text"],

Cell[BoxData[{
    \(\(uv0[t_] := \ eigenv1\ Cos[t]\  + \ \ eigenv2\ Sin[t];\)\ \), "\n", 
    \(\(uv1[s_]\  := \ w1\ Cos[s]\  + \ w2\ Sin[s];\)\)}], "Input",
  CellLabel->"In[143]:="],

Cell["\<\
 For the generalized cosinus of the intersection angle (see the notebook \
mspheres.nb) we calculate\
\>", "Text"],

Cell[BoxData[
    \(pssp[uv0[t], \ uv1[s]]\)], "Input",
  CellLabel->"In[145]:="],

Cell[BoxData[
    \(fctr[s_, t_]\  = \ %\)], "Input",
  CellLabel->"In[146]:="],

Cell[BoxData[
    \(Plot3D[fctr[s, t], {s, 0, \ 2\ Pi}, {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[147]:="],

Cell["\<\
The first eigenvector  of pp[v1,v2,w1,w2] corresponds to t=0 or Pi, where the \
spheres uv1[s] have the generalized cosinus with the sphere eigenv1 .\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ 0], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[148]:="],

Cell["\<\
For t = Pi/2, 3Pi/2 we get correspondingly for the second eigenvector:\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ Pi/2], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[149]:="],

Cell["\<\
Clearly, that means zero; we have only numerical errors,  remember:\
\>", "Text"],

Cell[BoxData[
    \(fctr[s, t]\)], "Input",
  CellLabel->"In[150]:="],

Cell[BoxData[
    \(fctr[s, Pi/2]\)], "Input",
  CellLabel->"In[151]:="],

Cell["\<\
It is a surprise by plotting zero to get a  -sin curve. Try better\
\>", "Text"],

Cell[BoxData[
    \(Plot[Evaluate[fctr[s, \ Pi/2]], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[152]:="],

Cell[BoxData[
    \(pssp[eigenv1, uv1[s]]\)], "Input",
  CellLabel->"In[153]:="],

Cell[BoxData[
    \(uv1[s]\)], "Input",
  CellLabel->"In[154]:="],

Cell[BoxData[
    \(eigenv1\)], "Input",
  CellLabel->"In[155]:="],

Cell["\<\
The amplitudes correspond to the square roots of the corresponding \
eigenvalues.\
\>", "Text"],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[uv1[s\ *2\ Pi], \(-1\), 1, \(-1\), 1, \ 
        PlotPoints\  -> \ 40], \ {s, \ 0.4, \ 0.5, \ 0.1}]\)], "Input",
  CellLabel->"In[156]:="],

Cell[BoxData[
    \(testgr1 = \ %\)], "Input",
  CellLabel->"In[157]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[158]:="],

Cell[BoxData[
    \(euklidsphereplot3D[eigenv1]\)], "Input",
  CellLabel->"In[159]:="],

Cell[BoxData[
    \(testgr2 = \ Flatten[{testgr1, %}]\)], "Input",
  CellLabel->"In[160]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[161]:="],

Cell[BoxData[
    \(testgr2\  = \ %\)], "Input",
  CellLabel->"In[162]:="],

Cell["There are many possibilities for experimental graphics...", "Text"],

Cell[BoxData[
    \(Remove[testgr2, v1, v2, w1, w2, fctr, testgr1, testgr, \ grsph1, uv1, 
      uv0]\)], "Input",
  CellLabel->"In[163]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Another example", "Subsubsection"],

Cell[BoxData[
    \(vspace[{1, 0, 0}, {2, 0.5, 1.5}, {2.8, 0, 0}]\)], "Input",
  CellLabel->"In[164]:="],

Cell[BoxData[
    \(bsp\  = \ smoothing[%]\)], "Input",
  CellLabel->"In[165]:="],

Cell[BoxData[
    \(v1\  = stb[3]\)], "Input",
  CellLabel->"In[166]:="],

Cell[BoxData[
    \(v2\  = \ stb[4]\)], "Input",
  CellLabel->"In[167]:="],

Cell[BoxData[
    \(w1\  = \ bsp[\([1]\)]\)], "Input",
  CellLabel->"In[168]:="],

Cell[BoxData[
    \(w2\  = \ bsp[\([2]\)]\)], "Input",
  CellLabel->"In[169]:="],

Cell[BoxData[
    \({plotcircle2spv[v1, \ v2], plotcircle2spv[w1, w2]}\)], "Input",
  CellLabel->"In[170]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[171]:="],

Cell[BoxData[
    \(exp1c2\  = \ %\)], "Input",
  CellLabel->"In[172]:="],

Cell[BoxData[
    \(pscross[{v1, v2, w1, w2}]\)], "Input",
  CellLabel->"In[173]:="],

Cell[BoxData[
    \(ch[%]\)], "Input",
  CellLabel->"In[174]:="],

Cell["\<\
This corresponds to case 1.c. The unique intersection point is {1,0,0}.\
\>", "Text"],

Cell[BoxData[
    \(pp[v1, v2, w1, \ w2]\)], "Input",
  CellLabel->"In[175]:="],

Cell[BoxData[
    \(Eigenvalues[%]\)], "Input",
  CellLabel->"In[176]:="],

Cell[BoxData[
    \(Eigenvectors[pp[v1, v2, w1, \ w2]]\)], "Input",
  CellLabel->"In[177]:="],

Cell["\<\
Therefore the \"eigenspheres\" (spheres corresponding to the eigenvectors) \
containing the circle C0 are\
\>", "Text"],

Cell[BoxData[
    \(eigenv1\  = 
      normalize[\(-v1\)*0.532813269686138823`\  - \ 
          v2*0.846232840089750837`]\)], "Input",
  CellLabel->"In[178]:="],

Cell[BoxData[
    \(eigenv2\  = 
      normalize[
        v2*0.532813269686138823`\  - \ v1*0.846232840089750837`]\)], "Input",
  CellLabel->"In[179]:="],

Cell[BoxData[
    \({euklidsphereplot3D[eigenv1], euklidsphereplot3D[eigenv2]}\)], "Input",
  CellLabel->"In[180]:="],

Cell[BoxData[
    \(Show[%, \ PlotRange\  -> All]\)], "Input",
  CellLabel->"In[181]:="],

Cell[BoxData[
    \(eigensph\  = \ %\)], "Input",
  CellLabel->"In[182]:="],

Cell[TextData[
"They intersect orthogonally in the circle C0. The spheres containing the \
circles C0, C1 are the one-parameter families S0[t]\[Superset]C0, S1[s]\
\[Superset] C1 defined by the spacelike unit vectors"], "Text"],

Cell[BoxData[{
    \(\(uv0[t_] := \ eigenv1\ Cos[t]\  + \ \ eigenv2\ Sin[t];\)\ \), "\n", 
    \(\(uv1[s_]\  := \ w1\ Cos[s]\  + \ w2\ Sin[s];\)\)}], "Input",
  CellLabel->"In[183]:="],

Cell["\<\
 For the generalized cosinus of the intersection angle (see the notebook \
mspheres.nb) we calculate\
\>", "Text"],

Cell[BoxData[
    \(pssp[uv0[t], \ uv1[s]]\)], "Input",
  CellLabel->"In[185]:="],

Cell[BoxData[
    \(fctr[s_, t_]\  = \ %\)], "Input",
  CellLabel->"In[186]:="],

Cell[BoxData[
    \(Plot3D[fctr[s, t], {s, 0, \ 2\ Pi}, {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[187]:="],

Cell["\<\
The first eigenvector  of pp[v1,v2,w1,w2] corresponds to t=0 or Pi, where the \
spheres uv1[s] have the followwing generalized cosinus with the sphere \
eigenv1 :\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ 0], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[188]:="],

Cell["\<\
For t = Pi/2, 3Pi/2 we get correspondingly for the second eigenvector:\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ Pi/2], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[189]:="],

Cell[BoxData[
    \(pssp[eigenv1, uv1[s]]\)], "Input",
  CellLabel->"In[190]:="],

Cell[BoxData[
    \(uv1[s]\)], "Input",
  CellLabel->"In[191]:="],

Cell[BoxData[
    \(eigenv1\)], "Input",
  CellLabel->"In[192]:="],

Cell["\<\
The amplitudes correspond to the square roots of the corresponding \
eigenvalues: 1., .\
\>", "Text"],

Cell[BoxData[
    \(Sqrt[0.0737033666969973921`]\)], "Input",
  CellLabel->"In[193]:="],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[uv1[s\ *2\ Pi], \(-1\), 1, \(-1\), 1, \ 
        PlotPoints\  -> \ 40], \ {s, \ 0.1, \ 0.3, \ 0.1}]\)], "Input",
  CellLabel->"In[194]:="],

Cell[BoxData[
    \(testgr1 = \ %\)], "Input",
  CellLabel->"In[195]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[196]:="],

Cell[BoxData[
    \(euklidsphereplot3D[eigenv1]\)], "Input",
  CellLabel->"In[197]:="],

Cell[BoxData[
    \(testgr2 = \ Flatten[{testgr1, %}]\)], "Input",
  CellLabel->"In[198]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[199]:="],

Cell[BoxData[
    \(testgr2\  = \ %\)], "Input",
  CellLabel->"In[200]:="],

Cell["There are many possibilities for experimental graphics...", "Text"],

Cell[BoxData[
    \(Remove[testgr2, v1, v2, w1, w2, fctr, testgr1, testgr, \ grsph1, uv1, 
      uv0]\)], "Input",
  CellLabel->"In[201]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.6 The cases 2", "Subsection"],

Cell[CellGroupData[{

Cell["Definition of vspace 2d", "Subsubsection"],

Cell["\<\
 A general circle in the x,y-plane with center {x, y} and radius r has the \
vspace spanne by\
\>", "Text"],

Cell[BoxData[
    \(circlespacevectors[x, y, 0, 0, Pi/2, r]\)], "Input",
  CellLabel->"In[267]:="],

Cell["\<\
This formula leads to the definition (we consider the circles in the \
x,y-plane as special 3D-objects):\
\>", "Text"],

Cell["\<\
vspace2d[x_, y_, r_] := 
  {{x/r, y/r, 0, 
    -(1 + r^2 - x^2 - y^2)/(2*r), (1 - r^2 + x^2 + y^2)/(2*r)}, 
   stb[3]}\
\>", "Input",
  CellLabel->"In[9]:=",
  InitializationCell->True],

Cell[BoxData[
    \(vspace2d[x, y, r]\)], "Input",
  CellLabel->"In[268]:="],

Cell[BoxData[
    \(pssp[%[\([1]\)], %[\([1]\)]]\)], "Input",
  CellLabel->"In[269]:="],

Cell[BoxData[
    \(Together[%]\)], "Input",
  CellLabel->"In[270]:="],

Cell[BoxData[
    \(\((w1\  = \(vspace2d[0, 1, 1]\)[\([1]\)]; \ 
      w2\  = \(vspace2d[0, 1, 1]\)[\([2]\)]; \n
      plotcircle2spv[w1, w2])\)\)], "Input",
  CellLabel->"In[271]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["a) Two disjoint circles:", "Subsubsection"],

Cell[BoxData[
    \(\((v1\  = \ stb[3]; \ v2\  = \ stb[4]\ ; \n\t
      w1\  = \(vspace2d[0, 1/2, \ 3]\)[\([1]\)]; 
      w2\  = \(vspace2d[0, 1/2, \ 3]\)[\([2]\)]; \n
      C0xC1\  = \ pscross[{v1, v2, w1, w2}])\)\)], "Input",
  CellLabel->"In[272]:="],

Cell["\<\
Indeed, since dimension(U0 + U1) = 3 the vectors must be linearily dependent.\
\
\>", "Text"],

Cell[BoxData[
    \(ch[%]\)], "Input",
  CellLabel->"In[273]:="],

Cell[BoxData[
    \({w1, w2}\)], "Input",
  CellLabel->"In[274]:="],

Cell[BoxData[
    \(ma = {v1, v2, w1}\)], "Input",
  CellLabel->"In[275]:="],

Cell[BoxData[
    \(Det[Table[pssp[ma[\([i]\)], \ ma[\([j]\)]], {i, 3}, {j, 3}]]\)], "Input",\

  CellLabel->"In[276]:="],

Cell["\<\
It follows: case 2.a; the circles are disjoint and separated; the sum U0+U1 = \
span[ma] is pseudo-euclidean.\
\>", "Text"],

Cell[BoxData[
    \({plotcircle2spv[v1, v2], plotcircle2spv[w1, w2]}\)], "Input",
  CellLabel->"In[277]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[278]:="],

Cell[BoxData[
    \(pp[v1, v2, w1, \ w2]\)], "Input",
  CellLabel->"In[279]:="],

Cell[BoxData[
    \(Eigenvalues[%]\)], "Input",
  CellLabel->"In[280]:="],

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[281]:="],

Cell[BoxData[
    \(Eigenvectors[pp[v1, v2, w1, \ w2]]\)], "Input",
  CellLabel->"In[282]:="],

Cell["\<\
Therefore the \"eigenspheres\" (spheres corresponding to the eigenvectors) \
containing the circle C0 are v1,v2\
\>", "Text"],

Cell[BoxData[
    \({euklidsphereplot3D[v1], euklidsphereplot3D[v2]}\)], "Input",
  CellLabel->"In[283]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[284]:="],

Cell[TextData[
"They intersect orthogonally in the circle C0. The spheres containing the \
circles C0, C1 are the one-parameter families S0[t]\[Superset]C0, S1[s]\
\[Superset] C1 defined by the spacelike unit vectors"], "Text"],

Cell[BoxData[{
    \(\(uv0[t_] := v1\ Cos[t]\  + \ \ v2\ Sin[t];\)\ \), "\n", 
    \(\(uv1[s_]\  := \ w1\ Cos[s]\  + \ w2\ Sin[s];\)\)}], "Input",
  CellLabel->"In[285]:="],

Cell["\<\
 For the generalized cosinus of the intersection angle (see the notebook \
mspheres.nb) we calculate\
\>", "Text"],

Cell[BoxData[
    \(pssp[uv0[t], \ uv1[s]]\)], "Input",
  CellLabel->"In[287]:="],

Cell[BoxData[
    \(fctr[s_, t_]\  = \ %\)], "Input",
  CellLabel->"In[288]:="],

Cell[BoxData[
    \(Plot3D[fctr[s, t], {s, 0, \ 2\ Pi}, {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[289]:="],

Cell["\<\
The first eigenvector  of pp[v1,v2,w1,w2] corresponds to t=0 or Pi, where the \
spheres uv1[s] have the following generalized cosinus with the sphere v1:\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ 0], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[290]:="],

Cell["\<\
We remark that v1 = stb[3]  = w2 spans U0\[Intersection]U1 and defines the \
\"sphere\" containing both circles: the x,y-plane.  uv1[0] defines the sphere \
intersecting the x,y-plane orthogonally in the circle C1.\
\>", "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[uv1[0]]\)], "Input",
  CellLabel->"In[291]:="],

Cell["Of course s= Pi/2 yields the x,y-plane:", "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[uv1[Pi/2]]\)], "Input",
  CellLabel->"In[292]:="],

Cell["\<\

For t = Pi/2, 3Pi/2 we get correspondingly for the second eigenvector v2 = \
stb[4] the unit sphere through o; :\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ Pi/2], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[293]:="],

Cell["\<\
For s = Pi/2 - the x,y-plane uv1[Pi/2] and the unitsphere intersect \
orthogonally. At the points with Abs[fctr[s, Pi/2]] = 1 the sphere uv1[s] and \
the unit sphere are tangent to each other, and for Abs[fctr[s, Pi/2]] > 1 \
both spheres are disjoint. The amplitudes correspond to the square roots of \
the corresponding eigenvalues.\
\>", "Text"],

Cell[BoxData[
    \(Sqrt[2.640625`]\)], "Input",
  CellLabel->"In[294]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["b) Two intersecting circles:", "Subsubsection"],

Cell[BoxData[
    \(\((v1\  = \ stb[3]; \ v2\  = \ stb[4]\ ; \n\t
      w1\  = \(vspace2d[0, 1/2, \ 0.7]\)[\([1]\)]; 
      w2\  = \(vspace2d[0, 1/2, \ 0.7]\)[\([2]\)]; \n
      C0xC1\  = \ pscross[{v1, v2, w1, w2}])\)\)], "Input",
  CellLabel->"In[295]:="],

Cell["\<\
Indeed, since dimension(U0 + U1) = 3 the vectors must be linearily dependant.\
\
\>", "Text"],

Cell[BoxData[
    \({w1, w2}\)], "Input",
  CellLabel->"In[296]:="],

Cell[BoxData[
    \(ma = {v1, v2, w1}\)], "Input",
  CellLabel->"In[297]:="],

Cell[BoxData[
    \(Det[Table[pssp[ma[\([i]\)], \ ma[\([j]\)]], {i, 3}, {j, 3}]]\)], "Input",\

  CellLabel->"In[298]:="],

Cell["\<\
It follows: case 2.b; the circles intersect; the sum U0+U1 = span[ma] is \
euclidean.\
\>", "Text"],

Cell[BoxData[
    \({plotcircle2spv[v1, v2], plotcircle2spv[w1, w2]}\)], "Input",
  CellLabel->"In[299]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[300]:="],

Cell[BoxData[
    \(pp[v1, v2, w1, \ w2]\)], "Input",
  CellLabel->"In[301]:="],

Cell[BoxData[
    \(Eigenvectors[pp[v1, v2, w1, \ w2]]\)], "Input",
  CellLabel->"In[302]:="],

Cell["\<\
Therefore the \"eigenspheres\" (spheres corresponding to the eigenvectors) \
containing the circle C0 are v1,v2\
\>", "Text"],

Cell[BoxData[
    \({euklidsphereplot3D[v1], euklidsphereplot3D[v2]}\)], "Input",
  CellLabel->"In[303]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[304]:="],

Cell[TextData[
"They intersect orthogonally in the circle C0. The spheres containing the \
circles C0, C1 are the one-parameter families S0[t]\[Superset]C0, S1[s]\
\[Superset] C1 defined by the spacelike unit vectors"], "Text"],

Cell[BoxData[{
    \(\(uv0[t_] := v1\ Cos[t]\  + \ \ v2\ Sin[t];\)\ \), "\n", 
    \(\(uv1[s_]\  := \ w1\ Cos[s]\  + \ w2\ Sin[s];\)\)}], "Input",
  CellLabel->"In[305]:="],

Cell["\<\
 For the generalized cosinus of the intersection angle (see the notebook \
mspheres.nb) we calculate\
\>", "Text"],

Cell[BoxData[
    \(pssp[uv0[t], \ uv1[s]]\)], "Input",
  CellLabel->"In[307]:="],

Cell[BoxData[
    \(fctr[s_, t_]\  = \ %\)], "Input",
  CellLabel->"In[308]:="],

Cell[BoxData[
    \(Plot3D[fctr[s, t], {s, 0, \ 2\ Pi}, {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[309]:="],

Cell["\<\
The first eigenvector  of pp[v1,v2,w1,w2] corresponds to t=0 or Pi, where the \
spheres uv1[s] have the following generalized cosinus with the sphere v1:\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ 0], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[310]:="],

Cell["\<\
We remark that v1 = stb[3]  = w2 spans U0\[Intersection]U1 and defines the \
\"sphere\" containing both circles: the x,y-plane.  uv1[0] defines the sphere \
intersecting the x,y-plane orthogonally in the circle C1.\
\>", "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[uv1[0]]\)], "Input",
  CellLabel->"In[311]:="],

Cell["Of course s= Pi/2 yields the x,y-plane:", "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[uv1[Pi/2]]\)], "Input",
  CellLabel->"In[312]:="],

Cell["\<\

For t = Pi/2, 3Pi/2 we get correspondingly for the second eigenvector v2 = \
stb[4] the unit sphere through o :\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ Pi/2], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[313]:="],

Cell["\<\
Of course, any two spheres S0[t], S1[s] intersect: the \"generalized\" \
cosinus is the usual cosinus of the intersection angle.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["c) Two tangent circles:", "Subsubsection"],

Cell[BoxData[
    \(\((v1\  = \ stb[3]; \ v2\  = \ stb[4]\ ; \n\t
      w1\  = \(vspace2d[0, 1/2, \ 1/2]\)[\([1]\)]; 
      w2\  = \(vspace2d[0, 1/2, \ 1/2]\)[\([2]\)]; \n
      C0xC1\  = \ pscross[{v1, v2, w1, w2}])\)\)], "Input",
  CellLabel->"In[314]:="],

Cell["\<\
Indeed, since dimension(U0 + U1) = 3 the vectors must be linearily dependant.\
\
\>", "Text"],

Cell[BoxData[
    \({w1, w2}\)], "Input",
  CellLabel->"In[315]:="],

Cell[BoxData[
    \(ma = {v1, v2, w1}\)], "Input",
  CellLabel->"In[316]:="],

Cell[BoxData[
    \(Det[Table[pssp[ma[\([i]\)], \ ma[\([j]\)]], {i, 3}, {j, 3}]]\)], "Input",\

  CellLabel->"In[317]:="],

Cell["\<\
It follows: case 2.c; the circles are tangent to each other; the sum U0+U1 = \
span[ma] is isotropic: the vector v2 + w1 is orthogonal to U0+U1.\
\>", "Text"],

Cell[BoxData[
    \({plotcircle2spv[v1, v2], plotcircle2spv[w1, w2]}\)], "Input",
  CellLabel->"In[318]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[319]:="],

Cell[BoxData[
    \(pp[v1, v2, w1, \ w2]\)], "Input",
  CellLabel->"In[320]:="],

Cell[BoxData[
    \(Eigenvectors[pp[v1, v2, w1, \ w2]]\)], "Input",
  CellLabel->"In[321]:="],

Cell["\<\
Therefore the \"eigenspheres\" (spheres corresponding to the eigenvectors) \
containing the circle C0 are v1,v2\
\>", "Text"],

Cell[BoxData[
    \({euklidsphereplot3D[v1], euklidsphereplot3D[v2]}\)], "Input",
  CellLabel->"In[322]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[323]:="],

Cell[TextData[
"They intersect orthogonally in the circle C0. The spheres containing the \
circles C0, C1 are the one-parameter families S0[t]\[Superset]C0, S1[s]\
\[Superset] C1 defined by the spacelike unit vectors"], "Text"],

Cell[BoxData[{
    \(\(uv0[t_] := v1\ Cos[t]\  + \ \ v2\ Sin[t];\)\ \), "\n", 
    \(\(uv1[s_]\  := \ w1\ Cos[s]\  + \ w2\ Sin[s];\)\)}], "Input",
  CellLabel->"In[324]:="],

Cell[BoxData[
    \(pssp[uv0[t], \ uv1[s]]\)], "Input",
  CellLabel->"In[326]:="],

Cell[BoxData[
    \(fctr[s_, t_]\  = \ %\)], "Input",
  CellLabel->"In[327]:="],

Cell[BoxData[
    \(Plot3D[fctr[s, t], {s, 0, \ 2\ Pi}, {t, 0, 2\ Pi}]\)], "Input",
  CellLabel->"In[328]:="],

Cell["\<\
The first eigenvector  of pp[v1,v2,w1,w2] corresponds to t=0 or Pi, where the \
spheres uv1[s] have the following generalized cosinus with the sphere v1:\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ 0], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[329]:="],

Cell["\<\
We remark that v1 = stb[3]  = w2 spans U0\[Intersection]U1 and defines the \
\"sphere\" containing both circles: the x,y-plane.  uv1[0] defines the sphere \
intersecting the x,y-plane orthogonally in the circle C1.\
\>", "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[uv1[0]]\)], "Input",
  CellLabel->"In[330]:="],

Cell["Of course s= Pi/2 yields the x,y-plane:", "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[uv1[Pi/2]]\)], "Input",
  CellLabel->"In[331]:="],

Cell["\<\

For t = Pi/2, 3Pi/2 we get correspondingly for the second eigenvector v2 = \
stb[4] the unit sphere through o:\
\>", "Text"],

Cell[BoxData[
    \(Plot[fctr[s, \ Pi/2], \ {s, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[332]:="],

Cell["\<\
Of course, any two spheres S0[t], S1[s] intersect: the \"generalized\" \
cosinus is the usual cosinus of the intersection angle. Only for s = k Pi  we \
have S1[s] tangent to the unitsphere:\
\>", "Text"],

Cell[BoxData[
    \({euklidsphereplot3D[v2], \ euklidsphereplot3D[uv1[0]]}\)], "Input",
  CellLabel->"In[333]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[334]:="],

Cell[BoxData[
    \(testgr\  = \ %\)], "Input",
  CellLabel->"In[335]:="],

Cell["Call MathGL3d (section 7.2) with testgr and use wire frame!", "Text"],

Cell[BoxData[
    \(MVShow3D[testgr]\)], "Input",
  CellLabel->"In[336]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.7 Further examples", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
The x-axis and the standard unit circle compared with  circle families\
\>", "Subsubsection"],

Cell["The x-axis", "Text"],

Cell[BoxData[
    \(plotcircle2spv[stb[2], \ stb[3]]\)], "Input",
  CellLabel->"In[413]:="],

Cell[BoxData[
    \(gro\  = \ %\)], "Input",
  CellLabel->"In[414]:="],

Cell[BoxData[
    \(cv[t_] := circlespacevectors[0, 0, t, 0, Pi/2, 1]\)], "Input",
  CellLabel->"In[415]:="],

Cell[BoxData[
    \(cv[t]\)], "Input",
  CellLabel->"In[416]:="],

Cell[BoxData[
    \(cv1[t_]\  := \(cv[t]\)[\([1]\)]; 
    cv2[t_] := \ \(cv[t]\)[\([2]\)]\)], "Input",
  CellLabel->"In[417]:="],

Cell[BoxData[
    \(circlefamily[cv1, cv2, t, \(-1\), 1]\)], "Input",
  CellLabel->"In[418]:="],

Cell[BoxData[
    \(Show[{gro, %}]\)], "Input",
  CellLabel->"In[419]:="],

Cell[BoxData[
    \(cvpp[t_]\  := \ pp[stb[2], stb[3], \ cv1[t], \ cv2[t]]\)], "Input",
  CellLabel->"In[420]:="],

Cell[BoxData[
    \(cvpp[t]\)], "Input",
  CellLabel->"In[421]:="],

Cell[TextData[{
  "The circles for t \[NotEqual]0 are disjoint to the x-axis; the maximal \
eigenvalue is 1+",
  Cell[BoxData[
      \(t\^2\)]],
  ">1corresponding to case 1a."
}], "Text"],

Cell[BoxData[
    \(pscross[{stb[2], stb[3], \ cv1[t], \ cv2[t]}]\)], "Input",
  CellLabel->"In[422]:="],

Cell["Replacing the x-axis by the standard unit circle we obtain", "Text"],

Cell[BoxData[
    \(Clear[cvpp]\)], "Input",
  CellLabel->"In[423]:="],

Cell[BoxData[
    \(cvpp[t_]\  := \ pp[stb[3], stb[4], \ cv1[t], \ cv2[t]]\)], "Input",
  CellLabel->"In[424]:="],

Cell[BoxData[
    \(cvpp[t]\)], "Input",
  CellLabel->"In[425]:="],

Cell[BoxData[
    \(Eigenvalues[%]\)], "Input",
  CellLabel->"In[426]:="],

Cell["\<\
R e m a r k. In this example the circle pair is not in general position:\
\>", "Text"],

Cell[BoxData[
    \(pscross[{stb[3], stb[4], \ cv1[t], \ cv2[t]}]\)], "Input",
  CellLabel->"In[459]:="],

Cell["The sphere", "Text"],

Cell[BoxData[
    \(vn[t_]\  := spacevec[0, 0, t/2, \ Sqrt[1\  + \ \((t/2)\)^2]]\)], "Input",\

  CellLabel->"In[460]:="],

Cell[BoxData[
    \(vn[t]\)], "Input",
  CellLabel->"In[461]:="],

Cell["\<\
contains both circles of the pair belonging to the parameter t. For the \
standard unit circle this is obvious. On the other hand, considering\
\>", "Text"],

Cell[BoxData[
    \(compl[t_] := \ pscomplement[cv1[t], cv2[t]]\)], "Input",
  CellLabel->"In[463]:="],

Cell[BoxData[
    \(compl[t]\)], "Input",
  CellLabel->"In[464]:="],

Cell["we obtain", "Text"],

Cell[BoxData[
    \(Table[
      Simplify[pssp[vn[t], \ \(compl[t]\)[\([i]\)]]], \ {i, 3}]\)], "Input",
  CellLabel->"In[465]:="],

Cell["\<\
what proves the statement. Therefore, the fact that there exist two \
eigenvalues greater or equal 1 does not contradict to the statement after \
formula (50) of the paper [4], since assumption B is not fulfilled, which is \
essential for the proof of the eigenvalue properties.\
\>", "Text"],

Cell[BoxData[
    \(Eigenvectors[cvpp[t]]\)], "Input",
  CellLabel->"In[427]:="],

Cell[BoxData[
    \(Clear[eigenv1]\)], "Input",
  CellLabel->"In[428]:="],

Cell[BoxData[
    \(eigenv1[t_]\  := \ normalize[{0, 0, \(-t\)/2, \ 1, 0}]\)], "Input",
  CellLabel->"In[429]:="],

Cell[BoxData[
    \(eigenv1[t]\)], "Input",
  CellLabel->"In[430]:="],

Cell["\<\
This again is the sphere containing both circles of the pair (up to \
orientation): \
\>", "Text"],

Cell[BoxData[
    \(Simplify[vn[t]\  + \ eigenv1[t], \ t > \ 0]\)], "Input",
  CellLabel->"In[466]:="],

Cell[BoxData[
    \(Clear[eigenv2]\)], "Input",
  CellLabel->"In[431]:="],

Cell[BoxData[
    \(eigenv2[t_]\  := \ normalize[{0, 0, 2/t, 1, 0}]\)], "Input",
  CellLabel->"In[432]:="],

Cell[BoxData[
    \(eigenv2[t]\)], "Input",
  CellLabel->"In[433]:="],

Cell["For t= 1 we get the corresponding eigenspheres", "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[eigenv1[1. ]]\)], "Input",
  CellLabel->"In[434]:="],

Cell[BoxData[
    \(es1\  = \ %\)], "Input",
  CellLabel->"In[435]:="],

Cell[BoxData[
    \(euklidsphereplot3D[eigenv2[1. ]]\)], "Input",
  CellLabel->"In[436]:="],

Cell[BoxData[
    \(es2\  = \ %\)], "Input",
  CellLabel->"In[437]:="],

Cell[BoxData[
    \(Show[{es1, es2}]\)], "Input",
  CellLabel->"In[438]:="],

Cell[BoxData[
    \(vradius[eigenv1[t]]\)], "Input",
  CellLabel->"In[439]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[440]:="],

Cell[BoxData[
    \(Simplify[vcenter[eigenv1[t]]]\)], "Input",
  CellLabel->"In[441]:="],

Cell[BoxData[
    \(Table[euklidsphereplot3D[eigenv1[t]], {t, \ 0, 5}]\)], "Input",
  CellLabel->"In[442]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[443]:="],

Cell[BoxData[
    \(Simplify[vradius[eigenv2[t]]]\)], "Input",
  CellLabel->"In[444]:="],

Cell[BoxData[
    \(Simplify[vcenter[eigenv2[t]]]\)], "Input",
  CellLabel->"In[445]:="],

Cell["\<\
The normed projections of eigenv1,eigenv2 on the vspace spanned by cv1,cv2 \
are\
\>", "Text"],

Cell[BoxData[
    \(projeigenv1[t_]\  := \ 
      normalize[
        cv1[t]*pssp[eigenv1[t], cv1[t]] + 
          cv2[t]*pssp[eigenv1[t], cv2[t]]]\)], "Input",
  CellLabel->"In[446]:="],

Cell[BoxData[
    \(projeigenv2[t_]\  := \ 
      normalize[
        cv1[t]*pssp[eigenv2[t], cv1[t]] + 
          cv2[t]*pssp[eigenv2[t], cv2[t]]]\)], "Input",
  CellLabel->"In[447]:="],

Cell[BoxData[
    \(Simplify[pssp[projeigenv1[t], projeigenv2[t]]]\)], "Input",
  CellLabel->"In[448]:="],

Cell[BoxData[
    \(euklidsphereplot3D[projeigenv1[1. ]]\)], "Input",
  CellLabel->"In[449]:="],

Cell[BoxData[
    \(pes1\  = \ %\)], "Input",
  CellLabel->"In[450]:="],

Cell[BoxData[
    \(euklidsphereplot3D[projeigenv2[1. ]]\)], "Input",
  CellLabel->"In[451]:="],

Cell[BoxData[
    \(pes2\  = \ %\)], "Input",
  CellLabel->"In[452]:="],

Cell[BoxData[
    \(Show[{pes1, pes2}]\)], "Input",
  CellLabel->"In[453]:="],

Cell[BoxData[
    \(vradius[projeigenv1[t]]\)], "Input",
  CellLabel->"In[454]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[455]:="],

Cell[BoxData[
    \(Simplify[vcenter[projeigenv1[t]]]\)], "Input",
  CellLabel->"In[456]:="],

Cell[BoxData[
    \(Simplify[vcenter[projeigenv2[t]]]\)], "Input",
  CellLabel->"In[457]:="],

Cell[BoxData[
    \(Simplify[vradius[projeigenv2[t]]]\)], "Input",
  CellLabel->"In[458]:="],

Cell[BoxData[
    \(Clear[cv, cvpp, eigenv1, vn, eigenv2]\)], "Input",
  CellLabel->"In[467]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Standard unit circle compared with a circle family with increasing radius\
\>", "Subsubsection"],

Cell[BoxData[
    \(cv[t_]\  := \ circlespacevectors[1, 0, 0, Pi/2, 0, t]\)], "Input",
  CellLabel->"In[468]:="],

Cell[BoxData[
    \(cv1[t_]\  := \(cv[t]\)[\([1]\)]; 
    cv2[t_] := \ \(cv[t]\)[\([2]\)]\)], "Input",
  CellLabel->"In[469]:="],

Cell[BoxData[
    \(circlefamily[cv1, cv2, t, 0.1, 3]\)], "Input",
  CellLabel->"In[470]:="],

Cell[BoxData[
    \(Show[{grc1, %}]\)], "Input",
  CellLabel->"In[471]:="],

Cell[BoxData[
    \(cvpp[t_]\  := \ pp[stb[3], stb[4], \ cv1[t], \ cv2[t]]\)], "Input",
  CellLabel->"In[472]:="],

Cell[BoxData[
    \(cvpp[t]\)], "Input",
  CellLabel->"In[473]:="],

Cell["For all t >0 we have case 2:", "Text"],

Cell[BoxData[
    \(pscross[{stb[3], stb[4], \ cv1[t], \ cv2[t]}]\)], "Input",
  CellLabel->"In[488]:="],

Cell[BoxData[
    \(fct[t_]\  = pssp[%, %]\)], "Input",
  CellLabel->"In[489]:="],

Cell[BoxData[
    \(Together[Simplify[fct[t]]]\)], "Input",
  CellLabel->"In[490]:="],

Cell[BoxData[
    \(Plot[fct[t], {t, 0, 3}]\)], "Input",
  CellLabel->"In[492]:="],

Cell[TextData[{
  "Since the maximal eigenvalue is ",
  Cell[BoxData[
      \(t\^2\/4\)]],
  ", we have for 0<t<2 circles interlaced with grc1, for t = 2 a circle \
intersecting grc1 in one point, and for t>2 two disjoint circles which may be \
separated by spheres."
}], "Text"],

Cell[BoxData[
    \(circlefamily[cv1, cv2, t, 0.2, 3.2, 10]\)], "Input",
  CellLabel->"In[493]:="],

Cell[BoxData[
    \(Show[{%, grc1}]\)], "Input",
  CellLabel->"In[494]:="],

Cell[BoxData[
    \(cfv1\  = \ %\)], "Input",
  CellLabel->"In[495]:="],

Cell[BoxData[
    \(MVShow3D[cfv1]\)], "Input",
  CellLabel->"In[496]:="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["5 Normal forms of pairs of circles ",
  FontFamily->"Helvetica"]], "Section",
  FontFamily->"Courier"],

Cell["\<\
In the general classification of pairs {k-sphere, l-sphere} contained in an \
n-sphere  ( see [4]) we obtained normal forms for the bases of the generating \
vector spaces, where the coefficients of one base with respect to the other \
are the invariant  eigenvalues of the matrix pp. In this section we describe \
these normal forms for each of the cases mentioned in the subsection \"Case \
discussion\" above.  The advantage of the normal form representation is that \
the relations between the invariants and the geometry of the circle pair can \
be studied directly. For the first circle we always take the unit circle grc1 \
= plotcircle [stb[3],stb[4]] with center o in the x,y-plane, defined with the \
initialization; the other circle is given by the corrresponding normal form \
of the base. We remark that the case numbering, the numbering of the basis \
vectors, and the notations differ from those used in [4]. 
ATTENTION: For getting the normal forms the definitions in subsection 5.1 are \
needed. Run 5.1 first!\
\>", "Text"],

Cell[BoxData[
    \(Show\ [grc1]\)], "Input",
  CellLabel->"In[66]:="],

Cell[CellGroupData[{

Cell["\<\
5.1 Some vector functions needed for the definition of the normal forms\
\>", "Subsection"],

Cell[BoxData[
    \(v3[a_]\  := \ {0, Sin[a], 0, Cos[a], 0}\)], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[
    \(v4[a_]\  := \ {0, 0, Cosh[a], 0, Sinh[a]}\)], "Input",
  CellLabel->"In[68]:="],

Cell[BoxData[
    \(v5[a_]\  := \ {Sin[a], 0, Cos[a], 0, 0}\)], "Input",
  CellLabel->"In[69]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["5.2 Case 1a: disjoint, non-interlaced circles", "Subsection"],

Cell["This corresponds to case 1, n =5, m = l =1, in [4].", "Text"],

Cell[BoxData[
    \(pscross[{stb[3], \ stb[4], v4[a], v3[b]}]\)], "Input",
  CellLabel->"In[71]:="],

Cell[TextData[
"For b = k\[Pi] , or a =0 we have case 2; for other values of a, b this \
vector is spacelike; therefore U0 + U1 is pseudo-euclidean:case 1.a."], "Text"],

Cell[BoxData[
    \(MatrixForm[Evaluate[pp[stb[3], \ stb[4], \ v4[a], \ v3[b]]]]\)], "Input",\

  CellLabel->"In[72]:="],

Cell[BoxData[
    \(circ1a[a_, \ b_]\  := \ plotcircle2spv[v4[a], \ v3[b]]\)], "Input",
  CellLabel->"In[73]:="],

Cell["For a = b = 0 we get ste standard circle grc1 again.", "Text"],

Cell[BoxData[
    \(circ1a[0, 0]\)], "Input",
  CellLabel->"In[74]:="],

Cell[CellGroupData[{

Cell["A first series", "Subsubsection"],

Cell["We compare a series of circles with grc1", "Text"],

Cell[BoxData[
    \(Table[
      Show[{grc1, circ1a[a, \ a + 1]}], {a, \ 0.2, 3.2, \ 0.5}]\)], "Input",
  CellLabel->"In[75]:="],

Cell[BoxData[
    \(Show[%, \ PlotRange\  -> All]\)], "Input",
  CellLabel->"In[76]:="],

Cell[BoxData[
    \(nf1a\  = %\)], "Input",
  CellLabel->"In[77]:="],

Cell[BoxData[
    \(MVShow3D[nf1a, \ MVLineTubeSize\  -> \ 0.05]\)], "Input",
  CellLabel->"In[83]:="],

Cell["A surface generated by the circles circ1a[a,a+1]:", "Text"],

Cell[BoxData[
    \(ParametricPlot3D[
      Evaluate[\(circle2spv[v4[a], \ v3[a + 1]]\)[t]], {a, \ 0, \ 5}, \ {t, \ 
        0, \ 2\ Pi}, PlotPoints\  -> \ 90, \ 
      PlotRange\  \[Rule] \ All]\)], "Input",
  CellLabel->"In[87]:="],

Cell[BoxData[
    \(ngr1a\  = \ %\)], "Input",
  CellLabel->"In[88]:="],

Cell[BoxData[
    \(MVShow3D[ngr1a]\)], "Input",
  CellLabel->"In[89]:="],

Cell[BoxData[
    \(Limit[\(circle2spv[v4[a], \ v3[a + 1]]\)[0], \ 
      a\  \[Rule] \ Infinity]\)], "Input",
  CellLabel->"In[90]:="],

Cell[BoxData[
    \(Limit[\(circle2spv[v4[\(-a\)], \ v3[\(-a\) + 1]]\)[1. ], \ 
      a\  \[Rule] \ Infinity]\)], "Input",
  CellLabel->"In[91]:="],

Cell["ThreeScript[\"nf1a.ts\", nf1a]", "Input",
  CellLabel->"In[286]:="],

Cell["ThreeScript[\"ngr1a.ts\", ngr1a]", "Input",
  CellLabel->"In[287]:="],

Cell[BoxData[
    \(Show[{grc1, \ ngr1a}, \ PlotRange\  -> All]\)], "Input",
  CellLabel->"In[92]:="],

Cell[BoxData[
    \(ngr1a1\  = \ %\)], "Input",
  CellLabel->"In[93]:="],

Cell[BoxData[
    \(MVShow3D[ngr1a1, \ MVLineTubeSize\  -> \ 0.03]\)], "Input",
  CellLabel->"In[94]:="],

Cell["ThreeScript[\"ngr1a1.ts\", ngr1a1]", "Input",
  CellLabel->"In[290]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Another series", "Subsubsection"],

Cell[BoxData[
    \(circ1a[0, 1]\)], "Input",
  CellLabel->"In[95]:="],

Cell[BoxData[
    \(\(?center\)\)], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \(center[v4[a], \ v3[b]]\)], "Input",
  CellLabel->"In[99]:="],

Cell[BoxData[
    \(Simplify[%, {\ a \[Element] Reals, b \[Element] Reals}]\)], "Input",
  CellLabel->"In[100]:="],

Cell[BoxData[
    \(centfct[a_, 
        b_]\  := \ {0, \(-\(Sin[2\ b]\/\(Cos[2\ b] + Cosh[2\ a]\)\)\), 
        Sinh[2\ a]\/\(Cos[2\ b] + Cosh[2\ a]\)}\)], "Input",
  CellLabel->"In[102]:="],

Cell[BoxData[
    \(\(?radius\)\)], "Input",
  CellLabel->"In[103]:="],

Cell[BoxData[
    \(radius[v4[a], \ v3[b]]\)], "Input",
  CellLabel->"In[104]:="],

Cell[BoxData[
    \(Simplify[%, {\ a \[Element] Reals, b \[Element] Reals}]\)], "Input",
  CellLabel->"In[105]:="],

Cell[BoxData[
    \(radfct[a_, b_]\  := \ \@2\/\@\(Cos[2\ b] + Cosh[2\ a]\)\)], "Input",
  CellLabel->"In[106]:="],

Cell[BoxData[
    \(centfct[0, 1. ]\)], "Input",
  CellLabel->"In[107]:="],

Cell[BoxData[
    \(radfct[0, 1. ]\)], "Input",
  CellLabel->"In[108]:="],

Cell["We compare a series of circles with grc1", "Text"],

Cell[BoxData[
    \(Table[
      Show[{grc1, circ1a[a, \ a + 1]}], {a, \ 0.2, 3.2, \ 0.5}]\)], "Input",
  CellLabel->"In[109]:="],

Cell[BoxData[
    \(Show[%, \ PlotRange\  -> All]\)], "Input",
  CellLabel->"In[110]:="],

Cell[BoxData[
    \(nf1a\  = %\)], "Input",
  CellLabel->"In[25]:="],

Cell[BoxData[
    \(MVShow3D[nf1a, \ MVLineTubeSize\  -> \ 0.03]\)], "Input",
  CellLabel->"In[32]:="],

Cell["A surface generated by the circles circ1a[a,a+1]:", "Text"],

Cell[BoxData[
    \(ParametricPlot3D[
      Evaluate[\(circle2spv[v4[a], \ v3[a + 1]]\)[t]], {a, \ 0, \ 5}, \ {t, \ 
        0, \ 2\ Pi}, PlotPoints\  -> \ 30, \ 
      PlotRange\  \[Rule] \ All]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(ngr1a\  = \ %\)], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(MVShow3D[ngr1a]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(Limit[\(circle2spv[v4[a], \ v3[a + 1]]\)[0], \ 
      a\  \[Rule] \ Infinity]\)], "Input",
  CellLabel->"In[38]:="],

Cell[BoxData[
    \(Limit[\(circle2spv[v4[\(-a\)], \ v3[\(-a\) + 1]]\)[1. ], \ 
      a\  \[Rule] \ Infinity]\)], "Input",
  CellLabel->"In[39]:="],

Cell["ThreeScript[\"nf1a.ts\", nf1a]", "Input",
  CellLabel->"In[286]:="],

Cell["ThreeScript[\"ngr1a.ts\", ngr1a]", "Input",
  CellLabel->"In[287]:="],

Cell[BoxData[
    \(Show[{grc1, \ ngr1a}, \ PlotRange\  -> All]\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(ngr1a1\  = \ %\)], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(MVShow3D[ngr1a1, \ MVLineTubeSize\  -> \ 0.03]\)], "Input",
  CellLabel->"In[40]:="],

Cell["ThreeScript[\"ngr1a1.ts\", ngr1a1]", "Input",
  CellLabel->"In[290]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["5.3 Case 1b: interlaced circles", "Subsection"],

Cell["This corresponds to case 3, n =5, m = l =1, in [4].", "Text"],

Cell[BoxData[
    \(pscross[{stb[3], \ stb[4], v5[a], v3[b]}]\)], "Input",
  CellLabel->"In[111]:="],

Cell[TextData[
"For b = k\[Pi] , or a = k\[Pi]  we have case 2; for other values of a, b \
this vector is timelike; therefore U0 + U1 is euclidean:case 1.b."], "Text"],

Cell[BoxData[
    \(MatrixForm[Evaluate[pp[stb[3], \ stb[4], \ v5[a], \ v3[b]]]]\)], "Input",\

  CellLabel->"In[112]:="],

Cell[BoxData[
    \(circ1b[a_, \ b_]\  := \ 
      plotcircle2spv[Evaluate[v5[a]], Evaluate[\ v3[b]]]\)], "Input",
  CellLabel->"In[113]:="],

Cell["\<\
For a = b = 0 we get the standard circle grc1 again. For a = b > 0 the \
pairing with grc1 yields the family of isogonal circles.\
\>", "Text"],

Cell[BoxData[
    \(circ1b[0, 0]\)], "Input",
  CellLabel->"In[114]:="],

Cell["\<\
A family of to grc1  non-isogonal, with grc1 interlaced circles:\
\>", "Text"],

Cell[BoxData[
    \(Table[circ1b[a, \ 2*a], {a, \ 1, 2, \ 0.25}]\)], "Input",
  CellLabel->"In[115]:="],

Cell[BoxData[
    \(Show[{%\ , grc1}, \ PlotRange\  -> All]\)], "Input",
  CellLabel->"In[116]:="],

Cell[BoxData[
    \(nf1b\  = %\)], "Input",
  CellLabel->"In[117]:="],

Cell["MVShow3D[nf1b]", "Input",
  CellLabel->"In[118]:="],

Cell["A corresponding surface:", "Text"],

Cell[BoxData[
    \(ParametricPlot3D[
      Evaluate[\(circle2spv[v5[a], \ v3[2*a]]\)[t]], {a, \ 1, \ 2}, \ {t, \ 
        0, \ 2\ Pi}, PlotPoints\  \[Rule] \ 70]\)], "Input",
  CellLabel->"In[120]:="],

Cell[BoxData[
    \(ngr1b\  = \ %\)], "Input",
  CellLabel->"In[121]:="],

Cell[TextData[{
  "Remark: If MathGL3d is already  loaded appears here a \"MeshGraphics3D\" \
object. This package changes the definition of Parametricplot3D. Of course, I \
could not find neither in the Help nor in the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " book a description of this type of objects. MeshGraphics3D cannot be \
joined with a Graphics3D object:"
}], "Text"],

Cell[BoxData[
    \(Show[{grc1, \ ngr1b}]\)], "Input",
  CellLabel->"In[122]:="],

Cell[BoxData[
    \(ngr1b1\  = \ %\)], "Input",
  CellLabel->"In[123]:="],

Cell["\<\
MVShow3D[ngr1b1, MVLineTubeSize \[Rule] 0.03]\
\>", "Input",
  CellLabel->"In[124]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["5.4 Case 2a", "Subsection"],

Cell[BoxData[
    \(b4\  = \ stb[4]\)], "Input",
  CellLabel->"In[125]:="],

Cell["The eigenvalues are obtained from the normal form", "Text"],

Cell[BoxData[
    \(pp[stb[3], \ stb[4], \ v4[a], \ stb[4]]\)], "Input",
  CellLabel->"In[126]:="],

Cell["\<\
 Both circles lie on the standard unit sphere. U0 +U1 is spanned by stb[3], \
stb[4], v4[a]. We set\
\>", "Text"],

Cell[BoxData[
    \(mat[a_]\  := \ {stb[3], \ stb[4], v4[a]}\)], "Input",
  CellLabel->"In[127]:="],

Cell[" and calculate", "Text"],

Cell[BoxData[
    \(Det[
      Table[pssp[\(mat[a]\)[\([i]\)], \(mat[a]\)[\([j]\)]], \ {i, 3}, \ {j, 
          3}]]\)], "Input",
  CellLabel->"In[128]:="],

Cell[TextData[
"It follows that  for a \[NotEqual] 0 the subspace U0 + U1 is \
pseudo-euclidean: case 2.a"], "Text"],

Cell[BoxData[
    \(Table[plotcircle2spv[v4[a], stb[4]], {a, \ 0.2, 3.2, \ 0.5}]\)], "Input",\

  CellLabel->"In[129]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[130]:="],

Cell[BoxData[
    \(nf2a = \ %\)], "Input",
  CellLabel->"In[131]:="],

Cell[BoxData[
    \(ParametricPlot3D[
      Evaluate[\(circle2spv[v4[a], \ stb[4]]\)[t]], {a, \ 0, \ Pi}, \ {t, \ 
        0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[132]:="],

Cell[BoxData[
    \(ngr2a\  = \ %\)], "Input",
  CellLabel->"In[133]:="],

Cell["ThreeScript[\"nf2a.ts\", nf2a]", "Input",
  CellLabel->"In[128]:="],

Cell["ThreeScript[\"ngr2a.ts\", ngr2a]", "Input",
  CellLabel->"In[129]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["5.5 Case 2b: 2-point intersecting circles in the x,y-plane", 
  "Subsection"],

Cell["\<\
This case one gets specialising case 1a to a = 0. Then the eigenvalues are 1, \
cos^2[b]:\
\>", "Text"],

Cell[BoxData[
    \(pp[stb[3], \ stb[4], \ stb[3], \ v3[b]]\)], "Input",
  CellLabel->"In[134]:="],

Cell[BoxData[
    \(mat[b_]\  := \ {stb[3], \ stb[4], v3[b]}\)], "Input",
  CellLabel->"In[135]:="],

Cell[" and calculate", "Text"],

Cell[BoxData[
    \(Det[
      Table[pssp[\(mat[b]\)[\([i]\)], \(mat[b]\)[\([j]\)]], \ {i, 3}, \ {j, 
          3}]]\)], "Input",
  CellLabel->"In[136]:="],

Cell[TextData[
"It follows that  for a \[NotEqual] 0 the subspace U0 + U1 is euclidean: case \
2.b"], "Text"],

Cell[BoxData[
    \(Table[plotcircle2spv[stb[3], v3[b]], {b, \ 0.2, 3.2, \ 0.5}]\)], "Input",\

  CellLabel->"In[137]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[138]:="],

Cell[BoxData[
    \(Show[{%, \ grc1}]\)], "Input",
  CellLabel->"In[139]:="],

Cell[BoxData[
    \(nf2a = \ %\)], "Input",
  CellLabel->"In[140]:="],

Cell[TextData[
"All the circles belong to the x,y-plane and intersect the standard unit \
circle grc1. The common intersection points are the unit points x = \
\[PlusMinus]1, y = 0."], "Text"],

Cell[BoxData[
    \(vec[x]\)], "Input",
  CellLabel->"In[141]:="],

Cell[BoxData[
    \(Solve[{pssp[\(mat[b]\)[\([1]\)], vec[\ x]] == 0, \ 
        pssp[\(mat[b]\)[\([2]\)], vec[\ x]] == 
          0, \n\t\tpssp[\(mat[b]\)[\([3]\)], vec[\ x]] == 0\ , 
        pssp[vec[x], vec[\ x]] == 0}, vec[x]]\)], "Input",
  CellLabel->"In[142]:="],

Cell["The solutions are, as stated:", "Text"],

Cell[BoxData[
    \({stepro[{\(-1\), 0, 0, 0, 1}], \ 
      stepro[{1, \ 0, \ 0, \ 0, 1}]}\)], "Input",
  CellLabel->"In[143]:="],

Cell[TextData[
"Obviously, we have the case 2b), and the argument b is the intersection \
angle of the two circles.  For b = k \[Pi]  we obtain the trivial pair \
{grc1,grc1}."], "Text"],

Cell[BoxData[
    \(plotcircle2spv[stb[3], v3[Pi/2]]\)], "Input",
  CellLabel->"In[144]:="],

Cell[BoxData[
    \(Show[%, \ grc1]\)], "Input",
  CellLabel->"In[145]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["5.6 Cases 1c and 2c: Circles intersecting in one point", "Subsection"],

Cell["\<\
For the normal form in this case we use v5[b] and the vector \
\>", "Text"],

Cell[BoxData[
    \(b4\  := \ {0, 1, 0, \(-1\), 1}\)], "Input",
  CellLabel->"In[146]:="],

Cell["Therefore all the considered circles lie on the sphere", "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[b4]\)], "Input",
  CellLabel->"In[147]:="],

Cell[BoxData[
    \(MatrixForm[Evaluate[pp[stb[3], \ stb[4], \ v5[b], \ b4]]]\)], "Input",
  CellLabel->"In[148]:="],

Cell[BoxData[
    \(pscross[{stb[3], \ stb[4], v5[b], \ b4}]\)], "Input",
  CellLabel->"In[149]:="],

Cell[TextData[
"For all values of b \[NotEqual] k\[Pi]  this is an isotropic vector; \
therefore we have case 1.c: all the corresponding circles intersect the unit \
circle in a uniquely defined point."], "Text"],

Cell[BoxData[
    \(circ1c[\ b_]\  := \ plotcircle2spv[v5[b], \ b4\ ]\)], "Input",
  CellLabel->"In[150]:="],

Cell[BoxData[
    \(Table[\ 
      plotcircle2spv[v5[b*Pi], \ b4], \ {b, \ 0.1, \ 1, \ 0.3}]\)], "Input",
  CellLabel->"In[151]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[152]:="],

Cell[BoxData[
    \(sertest1\  = %\)], "Input",
  CellLabel->"In[153]:="],

Cell[BoxData[
    \(Show[{sertest1, grc1}]\)], "Input",
  CellLabel->"In[154]:="],

Cell[BoxData[
    \(test1c = %\)], "Input",
  CellLabel->"In[155]:="],

Cell[BoxData[
    \(matc[b_]\  := \ {stb[3], \ stb[4], v5[b], \ b4}\)], "Input",
  CellLabel->"In[156]:="],

Cell[BoxData[
    \(Solve[{pssp[\(matc[b]\)[\([1]\)], vec[x]\ ] == \ 0, 
        pssp[\(matc[b]\)[\([2]\)], vec[x]\ ] == \ 0, 
        pssp[\(matc[b]\)[\([3]\)], vec[x]\ ] == \ 0, 
        pssp[\(matc[b]\)[\([4]\)], vec[x]\ ] == \ 0}, \ vec[x]]\)], "Input",
  CellLabel->"In[157]:="],

Cell["\<\
All the circles intersect the standard unit circle in the uniquely defined \
point\
\>", "Text"],

Cell[BoxData[
    \(stepro[{0, \ 1, 0, 0, 1}]\)], "Input",
  CellLabel->"In[158]:="],

Cell["\<\
All the circles of the family defined by {b4, v5[b]} intersect in two points:\
\
\>", "Text"],

Cell[BoxData[
    \(Solve[{pssp[\(matc[b]\)[\([3]\)], vec[x]\ ] == \ 0, 
        pssp[\(matc[b]\)[\([4]\)], vec[x]\ ] == \ 0, 
        pssp[\(matc[0]\)[\([3]\)], vec[x]] == \ 0, \ 
        pssp[vec[x], \ vec[x]] == \ 0}, \ vec[x]]\)], "Input",
  CellLabel->"In[159]:="],

Cell[BoxData[
    \({stepro[{0, 0, 0, \(-1\), 1}], \ stepro[{0, 1, 0, 0, 1}]}\)], "Input",
  CellLabel->"In[160]:="],

Cell["\<\
Thus we have a family of circles through the point {0,1,0}. Especially we get\
\
\>", "Text"],

Cell[BoxData[
    \(plotcircle2spv[v5[Pi], \ b4]\)], "Input",
  CellLabel->"In[161]:="],

Cell[BoxData[
    \(Show[{grc1, \ %}]\)], "Input",
  CellLabel->"In[162]:="],

Cell["\<\
This is a tangent circle to grc1 in the point {0,1,0}: case 2.c. The matrix \
pp has the diagonal form\
\>", "Text"],

Cell[BoxData[
    \(MatrixForm[Evaluate[pp[stb[3], \ stb[4], \ v5[Pi], \ b4]]]\)], "Input",
  CellLabel->"In[163]:="],

Cell[BoxData[
    \(pscross[{stb[3], \ stb[4], v5[Pi], \ b4}]\)], "Input",
  CellLabel->"In[164]:="],

Cell["\<\
If b4 is replaced by the vector {0,1,0,1,1} one gets  a family of lines lying \
in the plane y = 1 through the point {0,1,0}.\
\>", "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[{0, 1, 0, 1, 1}]\)], "Input",
  CellLabel->"In[165]:="],

Cell[BoxData[
    \(v5[t]\)], "Input",
  CellLabel->"In[166]:="],

Cell[BoxData[
    \(radius[v5[t], \ {0, 1, 0, 1, 1}]\)], "Input",
  CellLabel->"In[167]:="],

Cell[BoxData[
    \(center[v5[t], \ {0, 1, 0, 1, 1}]\)], "Input",
  CellLabel->"In[168]:="],

Cell[BoxData[
    \(Table[
      plotcircle2spv[v5[s], \ {0, 1, 0, 1, 1}], {s, 0, 5, 0.5}]\)], "Input",
  CellLabel->"In[169]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[170]:="],

Cell["Especially we get the tangent:", "Text"],

Cell[BoxData[
    \(plotcircle2spv[v5[Pi], \ {0, 1, 0, 1, 1}]\)], "Input",
  CellLabel->"In[171]:="],

Cell[BoxData[
    \(Show[{grc1, \ %}]\)], "Input",
  CellLabel->"In[172]:="],

Cell["Of course, we have the same invariants:", "Text"],

Cell[BoxData[
    \(MatrixForm[
      Evaluate[pp[stb[3], \ stb[4], \ v5[Pi], \ {0, 1, 0, 1, 1}]]]\)], "Input",\

  CellLabel->"In[177]:="],

Cell[BoxData[
    \(pscross[{stb[3], \ stb[4], v5[Pi], \ {0, 1, 0, 1, 1}}]\)], "Input",
  CellLabel->"In[174]:="],

Cell[BoxData[
    \(Clear[x, y, z, u, v, r, ma]\)], "Input",
  CellLabel->"In[175]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["6 Circle surfaces, circle orbits, geodesics in the \
circle space",
  FontFamily->"Helvetica"]], "Section",
  Evaluatable->False,
  FontFamily->"Courier"],

Cell[CellGroupData[{

Cell["6.1 Introduction", "Subsection"],

Cell["\<\
A circle surface is a surface in the 3-space through any point of which \
exists a circle contained in the surface. With other words: A circle surface \
can be described  as a curve in the circle space. The spheres (and planes) \
are the simplest circle surfaces. The tubes considered in section 2.4 are \
circle surfaces. More general, envelopes of one-parameter sphere families: \
the canal surfaces, are circle surfaces.  A special case are the Dupin \
cyclides, which are canal surfaces in a twofold way. That means, that there \
exists two different sphere curves whose envelopes is the same canal surface: \
tori, circular coni,  circular cylinders, and their images under Moebius \
transformations. Other special circle surfaces are the orbits of  1-parameter \
subgroups of the M\[ODoubleDot]bius group in the circle space.\
\>", "Text"],

Cell[TextData[{
  "The circle space is pseudo-Riemannian symmetric of rank two. The geodesics \
of the circle space are orbits of certain 1-parameter subgroups of the M\
\[ODoubleDot]bius group.   As in all reductive spaces, the geodesics through \
an element C are the orbits of  one-parameter subgroups exp(At) C, t \
\[Element] R, where in our case C is the standard unit circle grc1 and A an \
element of the orthogonal complement of the isotropy algebra of C in the Lie \
algebra o(5,1); here orthogonality is defined by the Killing form  of the Lie \
algebra o(5,1), see my notebook liealgeb.nb, in ",
  StyleBox["MathSource",
    FontSlant->"Italic"],
  " [5]",
  StyleBox[". ", "MathCaption"],
  " It would be interesting to investigate the relation between the \
pseudo-Riemannian geometry of the circle space and the circle manifolds more \
basically. But this may be the subject of another notebook. Some results \
about the M\[ODoubleDot]bius geometry of circle surfaces are contained in the \
diploma paper ",
  " [3]",
  " of Uwe May; especially the geodesics defined in the next subsections can \
be found there. We emphasize that we do not give here a classification of all \
the geodesics with respect to the M\[ODoubleDot]bius group. We only treat \
three simple cases of geodesics lying in a 2-sphere."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["6.2 Spacelike geodesics", "Subsection",
  Evaluatable->False],

Cell["\<\
The following matrix function defines the 1-parameter subgroup exp[Uh*t], \
where Uh is an element of the Lie algebra o(n,1) orthogonal to the isotropy \
algebra of grc1 and spacelike with respect to the Killing form of o(n,1) (see \
 the notebook liealgeb.nb mentioned above):\
\>", "Text"],

Cell[BoxData[
    \(Uh\  = \ {{0, 0, 0, 0, 0}, {0, 0, 0, 0, 0}, {0, 0, 0, 0, 0}, {0, 0, 0, 
          0, 1}, {0, 0, 0, 1, 0}}\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(gUh[t_]\ \  := \ MatrixExp[Uh\ t]\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(MatrixForm[gUh[t]]\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \(vh1[t_] := \ gUh[t] . stb[4]\)], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[
    \(vh1[t]\)], "Input",
  CellLabel->"In[16]:="],

Cell["\<\
Obviously, stb[3] is fixed under gUh[t]. Therefore the orbit of {stb[3], \
stb[4]} is given by\
\>", "Text"],

Cell[BoxData[
    \(hypgeodesic[t_, u_] := \(circle2spv[vh1[t], stb[3]]\)[u]\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(ParametricPlot3D[
      hypgeodesic[t, u], {t, \(-0.5\), 1}, {u, 0, 2\ \[Pi]}]\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(hypgeo\  = \ %\)], "Input",
  CellLabel->"In[19]:="],

Cell["\<\
This is the family of concentric circles in the x,y-plane containing the \
standard unit circle grc1 (for t = 0).\
\>", "Text"],

Cell[BoxData[
    \(ThreeScript["\<hypgeo.ts\>", hypgeo]\)], "Input",
  CellLabel->"In[40]:="],

Cell[TextData[{
  "The group is the group of all dilatations in the euclidean picture, \
leaving the \"north pole\" of ",
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)]],
  " fixed."
}], "Text"],

Cell[CellGroupData[{

Cell["The invariants along the hyperbolic geodesic", "Subsubsection"],

Cell[BoxData[
    \(invpp[s_, t_] := \ pp[vh1[t], stb[3], vh1[s], stb[3]]\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(invpp[s, t]\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[22]:="],

Cell["\<\
The Invariants are the elements of the diagonal; the variable one is (for s \
\[NotEqual]t) always greater than 1, we show this: \
\>", "Text"],

Cell[BoxData[
    \(ftt[s_, t_]\  := \ 
      1\/4\ \[ExponentialE]\^\(\(-2\)\ \((s + t)\)\)\ \((\[ExponentialE]\^\(2\
\ s\) + \[ExponentialE]\^\(2\ t\))\)\^2\)], "Input",
  CellLabel->"In[23]:="],

Cell[BoxData[
    \(Plot3D[ftt[s, t], {s, \(-1\), 1}, \ {t, \(-1\), 1}]\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(ftt1[s_, t_] := \ D[ftt[s1, t], s1] /. s1 \[Rule] s\)], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \(ftt2[s_, t_] := \ D[ftt[s, t1], t1] /. t1 \[Rule] t\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(Simplify[ftt1[s, t]]\)], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(Simplify[ftt2[s, t]]\)], "Input",
  CellLabel->"In[30]:="],

Cell["\<\
The derivatives are zero for s = t only; obviously, for any fixed t, ftt[s,t] \
reaches its minimum for s = t, and this minimum is\
\>", "Text"],

Cell[BoxData[
    \(ftt[s, s]\)], "Input",
  CellLabel->"In[31]:="],

Cell["The radius of the circle as function of t is", "Text"],

Cell[BoxData[
    \(radius[vh1[t], stb[3]]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[37]:="],

Cell["\<\
For the varying invariant expressed as a function of the radius we obtain\
\>", "Text"],

Cell[BoxData[
    \(\(invpp[t, \ 0]\)[\([1, 1]\)] /. \[ExponentialE]\^t \[Rule] \ 
        r\)], "Input",
  CellLabel->"In[38]:="],

Cell[BoxData[
    \(% /. \[ExponentialE]\^\(-t\) \[Rule] \ r^\((\(-1\))\)\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(Plot[\((1\/\(2\ r\) + r\/2)\)\^2, {r, \ 0.2, 5}]\)], "Input",
  CellLabel->"In[40]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Equivalent description on the 2-sphere", "Subsubsection"],

Cell[BoxData[
    \(U3h\  = \ {{0, 0, 0, 0, 0}, {0, 0, 0, 0, 0}, {0, 0, 0, 0, 1}, {0, 0, 0, 
          0, 0}, {0, 0, 1, 0, 0}}\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(MatrixForm[U3h]\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(gU3h[t_]\ \  := \ MatrixExp[U3h\ t]\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(MatrixForm[gU3h[t]]\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(v3h1[t_] := gU3h[t] . stb[3]\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \(v3h1[t]\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(hyp3geodesic[t_, u_] := \(circle2spv[v3h1[t], stb[4]]\)[u]\)], "Input",
  CellLabel->"In[47]:="],

Cell[BoxData[
    \(ParametricPlot3D[
      hyp3geodesic[t, u], {t, \(-1. \), 1}, {u, 0, 2\ \[Pi]}]\)], "Input",
  CellLabel->"In[48]:="],

Cell[BoxData[
    \(hyp3geo\  = %\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(\(?circlefamily\)\)], "Input",
  CellLabel->"In[50]:="],

Cell[BoxData[
    \(xy[t_] := \ stb[4]\)], "Input",
  CellLabel->"In[51]:="],

Cell[BoxData[
    \(circlefamily[v3h1, xy, t, 0, \ 1. , 10, \ 
      PlotPoints\  \[Rule] \ 100]\)], "Input",
  CellLabel->"In[52]:="],

Cell[BoxData[
    \(hgeofam\  = \ %\)], "Input",
  CellLabel->"In[53]:="],

Cell[BoxData[
    \(ThreeScript["\<hgeofam.ts\>", hgeofam]\)], "Input",
  CellLabel->"In[234]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["6.3 Timelike geodesics", "Subsection",
  Evaluatable->False],

Cell["\<\
The following matrix function defines the 1-parameter subgroup exp[Ue*t], \
where Ue is an element of the Lie algebra o(n,1) orthogonal to the isotropy \
algebra of grc1 and timelike with respect to the Killing form of o(n,1) (see \
liealgeb.nb ):\
\>", "Text"],

Cell[BoxData[
    \(Ue\  = \ {{0, 0, 1, 0, 0}, {0, 0, 0, 0, 0}, {\(-1\), 0, 0, 0, 0}, {0, 
          0, 0, 0, 0}, {0, 0, 0, 0, 0}}\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(gUe[t_] := MatrixExp[Ue\ t]\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(gUe[t] // MatrixForm\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \(ve1[t_]\  := \ gUe[t] . stb[3]\)], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[
    \(ve1[t]\)], "Input",
  CellLabel->"In[16]:="],

Cell[BoxData[
    \(ve2[t_]\  := \ stb[4]\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(circlefamily[ve1, ve2, t, \(-N[Pi]\), N[Pi], \ 30]\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(ellgeo\  = \ %\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(ThreeScript["\<ellgeo.ts\>", ellgeo]\)], "Input",
  CellLabel->"In[244]:="],

Cell["\<\
The standard circle rotates around the y-axis. The surface creation  yields \
the unit sphere, with two omitted sections:\
\>", "Text"],

Cell[BoxData[
    \(ellgeodesic[t_, u_] := \(circle2spv[ve1[t], stb[4]]\)[u]\)], "Input",
  CellLabel->"In[31]:="],

Cell[BoxData[
    \(ParametricPlot3D[
      ellgeodesic[t, u], {t, \(-Pi\)/2, Pi/2}, {u, 0, 2\ Pi}, \ 
      PlotPoints\  \[Rule] \ 30]\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(MVShow3D[%]\)], "Input",
  CellLabel->"In[24]:="],

Cell[TextData[{
  "The strange behaviour of the picture at the borders t = Pi/2, -Pi/2 comes \
from the careful calculation of the expression ellgeodesic[t,u] by ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " :"
}], "Text"],

Cell[BoxData[
    \(ellgeodesic[t, u]\)], "Input",
  CellLabel->"In[32]:="],

Cell["\<\
Simplification with the condition  t>-Pi/2&&t<Pi/2 yields a usual parameter \
representation of the unit sphere:\
\>", "Text"],

Cell[BoxData[
    \(Simplify[%, \ t > \(-Pi\)/2 && t < Pi/2]\)], "Input",
  CellLabel->"In[33]:="],

Cell[BoxData[
    \(Clear[ellgeodesic]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(ellgeodesic[t_, u_] := \ {Cos[t]\ Sin[u], \ 
        Cos[u], \(-\ Sin[t]\)\ Sin[u]}\)], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(ParametricPlot3D[
      ellgeodesic[t, u], {t, \(-Pi\)/2, Pi/2}, {u, 0, 2\ Pi}, \ 
      PlotPoints\  \[Rule] \ 30]\)], "Input",
  CellLabel->"In[36]:="],

Cell[CellGroupData[{

Cell["The invariants along the elliptic geodesic", "Subsubsection"],

Cell[BoxData[
    \(invellpp[s_, t_]\  := \ pp[ve1[s], stb[4], ve1[t], stb[4]]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(Simplify[invellpp[s, t]]\)], "Input",
  CellLabel->"In[38]:="],

Cell["\<\
As expected, the only essential invariant is the cosinus square of the \
intersection angle.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["6.4 Isotropic geodesics", "Subsection",
  Evaluatable->False],

Cell["\<\
The following matrix function defines the 1-parameter subgroup exp[U*t], \
where U is an element of the Lie algebra o(n,1) orthogonal to the isotropy \
algebra of grc1 and isotropic with respect to the Killing form of o(n,1) (see \
[5], liealgeb.nb ):\
\>", "Text"],

Cell[BoxData[
    \(U\  = {{0, 0, 1, 0, 0}, {0, 0, 0, 0, 0}, {\(-1\), 0, 0, 0, 1}, {0, 0, 
          0, 0, 0}, {0, 0, 1, 0, 0}}\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(MatrixForm[U]\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(isotropugr[t_] := \ MatrixExp[U*t]\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(isotropugr[t]\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(gh[t_] := \ isotropugr[t]\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(Simplify[
      Table[pssp[\(gh[t]\)[\([i]\)], \ \(gh[t]\)[\([j]\)]], {i, 5}, {j, 
          5}]]\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(v1c[t_]\  := \ gh[t] . stb[3]\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \(v2c[t_]\  := \ gh[t] . stb[4]\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(v1c[t]\)], "Input",
  CellLabel->"In[47]:="],

Cell[BoxData[
    \(v2c[t]\)], "Input",
  CellLabel->"In[48]:="],

Cell[BoxData[
    \(plotcircle2spv[v1c[1. ], v2c[1. ]]\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(circlefamily[v1c, v2c, t, 0, 10, \ 20, PlotRange\  -> \ All]\)], "Input",\

  CellLabel->"In[50]:="],

Cell[BoxData[
    \(circlefamily[v1c, v2c, t, \(-10\), 0, \ 20, 
      PlotRange\  -> \ All]\)], "Input",
  CellLabel->"In[51]:="],

Cell[BoxData[
    \(circlefamily[v1c, v2c, t, \(-3\), 3, \ 30, 
      PlotRange\  -> \ All]\)], "Input",
  CellLabel->"In[52]:="],

Cell[BoxData[
    \(isotrop\  = \ %\)], "Input",
  CellLabel->"In[53]:="],

Cell[BoxData[
    \(Show[Graphics3D[{PointSize[0.05], \ Point[{1, 0, 0}]}]]\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \(isoge\  = {%, \ isotrop}\)], "Input",
  CellLabel->"In[55]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[56]:="],

Cell[BoxData[
    \(isoge\  = \ %\)], "Input",
  CellLabel->"In[57]:="],

Cell[BoxData[
    \(ThreeScript["\<isoge.ts\>", isoge]\)], "Input",
  CellLabel->"In[275]:="],

Cell[BoxData[
    \(MVShow3D[isoge, MVPointSphereSize \[Rule] 0.5, \[IndentingNewLine]\ 
      MVLineTubeSize\  -> \ 0.01]\)], "Input",
  CellLabel->"In[59]:="],

Cell["\<\
 This is a 1-parameter circle family on the unit sphere consisting of all \
circles tangent to the equator at the point {1,0,0}. We check this using \
Solve:\
\>", "Text"],

Cell[BoxData[
    \(Solve[{pssp[stb[4], vec[x]] == \ 0, \ pssp[stb[3], vec[x]] == \ 0, \ 
        pssp[v1c[1. ], vec[x]] == \ 0, \ pssp[v1c[ .5], vec[x]] == \ 0}, 
      vec[x]]\)], "Input",
  CellLabel->"In[60]:="],

Cell["The solution is", "Text"],

Cell[BoxData[
    \(pz\  = \ {t, s, 0, 0, t}\)], "Input",
  CellLabel->"In[62]:="],

Cell[BoxData[
    \(pssp[pz, pz]\)], "Input",
  CellLabel->"In[63]:="],

Cell["\<\
Since the points correspond to isotropic vectors, the uniquely defined \
solution is given by s = 0:\
\>", "Text"],

Cell[BoxData[
    \(pzz\  = \ {t, 0, 0, 0, t}; stepro[pzz]\)], "Input",
  CellLabel->"In[64]:="],

Cell["The representation as circle surface can be obtained with", "Text"],

Cell[BoxData[
    \(ParametricPlot3D[\(circle2spv[v1c[t], v2c[t]]\)[u], \ {t, \(-Pi\), 
        Pi}, {u, \ 0, \ 2\ Pi}]\)], "Input",
  CellLabel->"In[65]:="],

Cell[BoxData[
    \(MVShow3D[%]\)], "Input",
  CellLabel->"In[66]:="],

Cell[CellGroupData[{

Cell["The invariants along the isotropic geodesic", "Subsubsection"],

Cell[BoxData[
    \(invisopp[s_, t_]\  := \ pp[v1c[s], v2c[s], v1c[t], v2c[t]]\)], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[
    \(Simplify[invisopp[s, t]]\)], "Input",
  CellLabel->"In[68]:="],

Cell["\<\
As can be proved easily by elementary considerations, any tw pairs of two \
tangent circles are M\[ODoubleDot]bius equivalent.\
\>", "Text"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["7 ThreeScript, MathGL3d", "Section"],

Cell[TextData[{
  "If you want to generate *.ts files for animations outside of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", activate the next section. To use the animation program MathGl3d in this \
notebook, activate section 7.2."
}], "Text"],

Cell[CellGroupData[{

Cell["7.1 Threescript ", "Subsection"],

Cell["\<\
 The ThreeScript commands generate *.ts files appropriate  for certain \
animation programs. The good freeware program  MathGL3d  created by Jens-Peer \
Kuska (for Windows95, Linux and other platforms) can be downloaded from \
[2].\
\>", "Text",
  Evaluatable->False],

Cell["\<\
Under Windows the commercial 3D-Animation programs MathLive and \
DynamicVisualizer are available. To produce .ts-files one \
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(Needs["\<Graphics`ThreeScript`\>"]\)], "Input",
  CellLabel->"In[257]:="],

Cell[BoxData[
    \(\(?ThreeScript\)\)], "Input",
  CellLabel->"In[258]:="],

Cell["\<\
The standard extension of these files is ts: \"filename.ts\" or \
\"Path/filename.ts\"\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(ThreeScript["\<isofam.ts\>", \ isofam]\)], "Input",
  CellLabel->"In[82]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["7.2 MathGL3d", "Subsection"],

Cell[TextData[{
  "The free animation program MathGL3d by Jens-Peer Kuska [2] can be used \
from a command line and within a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session. A new version of this program is in preparation. Jens-Peer Kuska \
has sent me a new startup file OpenGLViewer.m, and friendly permitted to add \
it to the item \"Spheres\". Replace the old file OpenGLViewer.m by this new \
OpenGLViewer.m. If you did this, changes of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s ParametricPlot3D, especially the sometimes disturbing generating of \
MeshGraphics3D objects, will not take place.\nEstablish the link to the \
current ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session with the command:"
}], "Text"],

Cell[BoxData[
    \(Get["\<MathGL3d`OpenGLViewer`\>"]\)], "Input",
  CellLabel->"In[23]:="],

Cell["If you want to close the link, enter", "Text"],

Cell["MVClose[]", "Text",
  CellLabel->"In[44]:="],

Cell["\<\
The following command clears the Viewer window; this and other MathGL3d \
commands you may enter also with  palettes.\
\>", "Text"],

Cell[BoxData[{
    \(\(MVClear[];\)\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(MVShow3D[\[Placeholder], MVPolygonShading \[Rule] MVSmooth, 
      MVGrayBackground \[Rule] False, MVTexture \[Rule] None, 
      MVTextureMapType \[Rule] MVMeshUVMapping, 
      MVScaleTexture \[Rule] {1, 1}, 
      MVTranslateTextureTarget \[Rule] Automatic, 
      MVRotateTextureTarget \[Rule] Automatic, 
      MVScaleTextureTarget \[Rule] Automatic]\)}], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(MVShow3D[gre1\[IndentingNewLine], MVPointSphereSize \[Rule] 0.8, \ 
      MVLineTubeSize\  -> \ 0.03]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(MVShow3D[isoge, MVPointSphereSize \[Rule] 0.5, \[IndentingNewLine]\ 
      MVLineTubeSize\  -> \ 0.01]\)], "Input",
  CellLabel->"In[274]:="],

Cell[BoxData[
    \(MVShow3D[tgr3, \ MVPointSphereSize\  \[Rule] \ 1.8]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(MVShow3D[ex123, MVPointSphereSize \[Rule] 1.8]\)], "Input",
  CellLabel->"In[69]:="],

Cell[BoxData[
    \(MVShow3D[hyp3geo]\)], "Input",
  CellLabel->"In[225]:="],

Cell[BoxData[
    \(Options[MVShow3D]\)], "Input",
  CellLabel->"In[56]:="],

Cell[BoxData[
    \(MVClose[]\)], "Input",
  CellLabel->"In[77]:="]
}, Closed]]
}, Closed]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 695}},
AutoGeneratedPackage->Automatic,
WindowToolbars->{},
WindowSize->{896, 668},
WindowMargins->{{32, Automatic}, {Automatic, 0}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{597.563, 842.375},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{"home", "rolf", "mymath", \
"spheres.v3"}, "mcircles.nb.ps", CharacterEncoding -> "WindowsANSI"],
"Magnification"->1},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
SpellingDictionaries->{"CorrectWords"->{
  "Erhard", "orthonormalization", "semisimple", "Activite", 
    "Orthonormalization", "Cartan", "spacelike", "timelike", "Lorentz", 
    "Inspiron", "symplectic", "nullmatrices", "conformality", "subalgebra", 
    "Jacoby", "ch"}},
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
StyleDefinitions -> "TutorialBook.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 118, 3, 73, "Title",
  Evaluatable->False],
Cell[1860, 56, 232, 8, 49, "Subsubtitle",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[2117, 68, 113, 3, 33, "Subtitle",
  Evaluatable->False],
Cell[2233, 73, 151, 3, 32, "Subsubtitle",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[2409, 80, 55, 1, 37, "Subsubsection"],
Cell[2467, 83, 2662, 38, 424, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[5166, 126, 35, 0, 31, "Subsubsection"],
Cell[5204, 128, 1015, 21, 208, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[6256, 154, 33, 0, 31, "Subsubsection"],
Cell[6292, 156, 299, 5, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[6628, 166, 32, 0, 31, "Subsubsection"],
Cell[6663, 168, 47, 0, 28, "Text"],
Cell[6713, 170, 90, 2, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[6840, 177, 33, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[6898, 181, 41, 0, 44, "Subsection"],
Cell[6942, 183, 305, 9, 220, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[7284, 197, 45, 0, 38, "Subsection"],
Cell[7332, 199, 334, 7, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[7703, 211, 33, 0, 38, "Subsection"],
Cell[7739, 213, 205, 5, 61, "Text"],
Cell[7947, 220, 56, 0, 28, "Text"],
Cell[8006, 222, 68, 2, 31, "Input"],
Cell[8077, 226, 282, 5, 48, "Text"],
Cell[8362, 233, 114, 2, 31, "Input"],
Cell[8479, 237, 117, 3, 28, "Text"],
Cell[8599, 242, 77, 2, 31, "Input"],
Cell[8679, 246, 130, 3, 28, "Text"],
Cell[8812, 251, 74, 2, 31, "Input"],
Cell[8889, 255, 156, 3, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9082, 263, 65, 0, 44, "Subsection"],
Cell[9150, 265, 152, 3, 32, "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9351, 274, 34, 0, 92, "Section"],

Cell[CellGroupData[{
Cell[9410, 278, 117, 2, 43, "Subsection"],
Cell[9530, 282, 567, 9, 88, "Text"],
Cell[10100, 293, 220, 4, 48, "Text"],
Cell[10323, 299, 112, 3, 31, "Input",
  InitializationCell->True],
Cell[10438, 304, 569, 10, 88, "Text",
  Evaluatable->False],
Cell[11010, 316, 124, 3, 28, "Text"],
Cell[11137, 321, 142, 3, 31, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[11316, 329, 36, 0, 38, "Subsection"],
Cell[11355, 331, 186, 5, 28, "Text",
  Evaluatable->False],
Cell[11544, 338, 72, 2, 31, "Input"],
Cell[11619, 342, 76, 2, 31, "Input"],
Cell[11698, 346, 85, 2, 28, "Text"],
Cell[11786, 350, 64, 2, 31, "Input"],
Cell[11853, 354, 318, 7, 48, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[12196, 365, 43, 0, 37, "Subsubsection"],
Cell[12242, 367, 103, 3, 28, "Text"],
Cell[12348, 372, 549, 9, 123, "Input"],
Cell[12900, 383, 57, 0, 28, "Text"],
Cell[12960, 385, 713, 11, 146, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[13710, 401, 43, 0, 37, "Subsubsection"],
Cell[13756, 403, 561, 9, 169, "Input"],
Cell[14320, 414, 57, 0, 28, "Text"],
Cell[14380, 416, 680, 10, 169, "Input"],
Cell[15063, 428, 72, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[15172, 435, 45, 0, 31, "Subsubsection"],
Cell[15220, 437, 239, 4, 54, "Input"],
Cell[15462, 443, 57, 0, 28, "Text"],
Cell[15522, 445, 276, 5, 77, "Input"],
Cell[15801, 452, 71, 2, 31, "Input"],
Cell[15875, 456, 68, 2, 31, "Input"],
Cell[15946, 460, 68, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[16051, 467, 44, 0, 31, "Subsubsection"],
Cell[16098, 469, 358, 6, 123, "Input"],
Cell[16459, 477, 57, 0, 28, "Text"],
Cell[16519, 479, 413, 6, 123, "Input"],
Cell[16935, 487, 67, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[17051, 495, 36, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[17112, 499, 57, 0, 37, "Subsubsection"],
Cell[17172, 501, 146, 4, 30, "Input",
  Evaluatable->False],
Cell[17321, 507, 66, 2, 31, "Input"],
Cell[17390, 511, 62, 2, 31, "Input"],
Cell[17455, 515, 130, 3, 31, "Input",
  Evaluatable->False],
Cell[17588, 520, 67, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[17692, 527, 49, 0, 31, "Subsubsection"],
Cell[17744, 529, 64, 0, 28, "Text"],
Cell[17811, 531, 72, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[17920, 538, 33, 0, 31, "Subsubsection"],
Cell[17956, 540, 88, 2, 31, "Input"],
Cell[18047, 544, 88, 2, 31, "Input"],
Cell[18138, 548, 64, 2, 31, "Input"],
Cell[18205, 552, 111, 2, 31, "Input"],
Cell[18319, 556, 75, 2, 31, "Input"],
Cell[18397, 560, 75, 2, 31, "Input"],
Cell[18475, 564, 76, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[18588, 571, 71, 0, 31, "Subsubsection"],
Cell[18662, 573, 71, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[18770, 580, 50, 0, 31, "Subsubsection"],
Cell[18823, 582, 89, 1, 28, "Text",
  Evaluatable->False],
Cell[18915, 585, 70, 2, 31, "Input"],
Cell[18988, 589, 215, 5, 54, "Input",
  Evaluatable->False],
Cell[19206, 596, 163, 4, 28, "Text",
  Evaluatable->False],
Cell[19372, 602, 24, 0, 28, "Text"],
Cell[19399, 604, 105, 2, 31, "Input"],
Cell[19507, 608, 96, 2, 31, "Input"],
Cell[19606, 612, 83, 2, 31, "Input"],
Cell[19692, 616, 227, 5, 77, "Input"],
Cell[19922, 623, 74, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[20033, 630, 34, 0, 31, "Subsubsection"],
Cell[20070, 632, 308, 7, 48, "Text"],
Cell[20381, 641, 186, 4, 54, "Input",
  Evaluatable->False],
Cell[20570, 647, 79, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[20710, 656, 82, 1, 61, "Section"],

Cell[CellGroupData[{
Cell[20817, 661, 83, 1, 44, "Subsection"],

Cell[CellGroupData[{
Cell[20925, 666, 94, 2, 37, "Subsubsection"],
Cell[21022, 670, 190, 4, 48, "Text"],
Cell[21215, 676, 77, 2, 31, "Input"],
Cell[21295, 680, 145, 3, 54, "Input"],
Cell[21443, 685, 72, 0, 28, "Text"],
Cell[21518, 687, 70, 2, 31, "Input"],
Cell[21591, 691, 74, 2, 31, "Input"],
Cell[21668, 695, 78, 2, 31, "Input"],
Cell[21749, 699, 101, 2, 31, "Input"],
Cell[21853, 703, 69, 2, 31, "Input"],
Cell[21925, 707, 64, 0, 28, "Text"],
Cell[21992, 709, 100, 2, 31, "Input"],
Cell[22095, 713, 69, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[22201, 720, 33, 0, 37, "Subsubsection"],
Cell[22237, 722, 201, 4, 48, "Text"],
Cell[22441, 728, 145, 3, 31, "Input"],
Cell[22589, 733, 114, 2, 31, "Input"],
Cell[22706, 737, 119, 3, 31, "Input"],
Cell[22828, 742, 71, 2, 31, "Input"],
Cell[22902, 746, 114, 2, 31, "Input"],
Cell[23019, 750, 119, 3, 31, "Input"],
Cell[23141, 755, 129, 3, 31, "Input"],
Cell[23273, 760, 71, 2, 31, "Input"],
Cell[23347, 764, 69, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[23453, 771, 47, 0, 31, "Subsubsection"],
Cell[23503, 773, 71, 2, 31, "Input"],
Cell[23577, 777, 173, 4, 54, "Input",
  Evaluatable->False],
Cell[23753, 783, 91, 2, 31, "Input"],
Cell[23847, 787, 380, 8, 68, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[24264, 800, 32, 0, 37, "Subsubsection"],
Cell[24299, 802, 143, 3, 54, "Input"],
Cell[24445, 807, 209, 4, 54, "Input"],
Cell[24657, 813, 229, 4, 77, "Input"],
Cell[24889, 819, 65, 2, 31, "Input"],
Cell[24957, 823, 71, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[25065, 830, 54, 0, 31, "Subsubsection"],
Cell[25122, 832, 89, 2, 28, "Text"],
Cell[25214, 836, 73, 2, 31, "Input"],
Cell[25290, 840, 668, 13, 215, "Input",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[25995, 858, 33, 0, 37, "Subsubsection"],
Cell[26031, 860, 156, 3, 77, "Input"],
Cell[26190, 865, 68, 2, 31, "Input"],
Cell[26261, 869, 246, 5, 77, "Input"],
Cell[26510, 876, 71, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[26630, 884, 46, 0, 38, "Subsection"],
Cell[26679, 886, 71, 2, 31, "Input"],
Cell[26753, 890, 249, 5, 77, "Input",
  Evaluatable->False],
Cell[27005, 897, 73, 2, 31, "Input"],
Cell[27081, 901, 229, 5, 54, "Input",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[27335, 910, 33, 0, 37, "Subsubsection"],
Cell[27371, 912, 92, 2, 31, "Input"],
Cell[27466, 916, 97, 2, 31, "Input"],
Cell[27566, 920, 103, 2, 31, "Input"],
Cell[27672, 924, 93, 2, 31, "Input"],
Cell[27768, 928, 114, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[27931, 936, 64, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[28020, 940, 83, 1, 37, "Subsubsection"],
Cell[28106, 943, 219, 4, 48, "Text"],
Cell[28328, 949, 329, 6, 123, "Input",
  Evaluatable->False],
Cell[28660, 957, 124, 3, 28, "Text"],
Cell[28787, 962, 126, 3, 31, "Input",
  Evaluatable->False],
Cell[28916, 967, 41, 0, 28, "Text"],
Cell[28960, 969, 263, 5, 54, "Input",
  Evaluatable->False],
Cell[29226, 976, 367, 6, 68, "Text"],
Cell[29596, 984, 714, 14, 169, "Input",
  Evaluatable->False],
Cell[30313, 1000, 70, 0, 28, "Text"],
Cell[30386, 1002, 388, 8, 169, "Input",
  Evaluatable->False],
Cell[30777, 1012, 217, 5, 48, "Text"],
Cell[30997, 1019, 101, 2, 31, "Input"],
Cell[31101, 1023, 101, 2, 31, "Input"],
Cell[31205, 1027, 153, 3, 28, "Text"],
Cell[31361, 1032, 73, 2, 31, "Input"],
Cell[31437, 1036, 1223, 22, 422, "Input",
  Evaluatable->False],
Cell[32663, 1060, 55, 0, 28, "Text"],
Cell[32721, 1062, 244, 6, 77, "Input",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[33002, 1073, 33, 0, 37, "Subsubsection"],
Cell[33038, 1075, 134, 3, 31, "Input",
  Evaluatable->True],
Cell[33175, 1080, 110, 2, 31, "Input"],
Cell[33288, 1084, 134, 3, 31, "Input",
  Evaluatable->True],
Cell[33425, 1089, 110, 2, 31, "Input"],
Cell[33538, 1093, 431, 10, 169, "Input",
  Evaluatable->True],
Cell[33972, 1105, 80, 2, 31, "Input"],
Cell[34055, 1109, 80, 2, 31, "Input"],
Cell[34138, 1113, 80, 2, 31, "Input"],
Cell[34221, 1117, 63, 2, 31, "Input"],
Cell[34287, 1121, 434, 11, 169, "Input",
  Evaluatable->True],
Cell[34724, 1134, 70, 2, 31, "Input"],
Cell[34797, 1138, 75, 0, 28, "Text"],
Cell[34875, 1140, 108, 2, 31, "Input"],
Cell[34986, 1144, 105, 2, 31, "Input"],
Cell[35094, 1148, 89, 2, 28, "Text"],
Cell[35186, 1152, 108, 2, 31, "Input"],
Cell[35297, 1156, 105, 2, 31, "Input"],
Cell[35405, 1160, 105, 2, 31, "Input"],
Cell[35513, 1164, 108, 2, 31, "Input"],
Cell[35624, 1168, 108, 2, 31, "Input"],
Cell[35735, 1172, 108, 2, 31, "Input"],
Cell[35846, 1176, 127, 3, 31, "Input",
  Evaluatable->True],
Cell[35976, 1181, 128, 3, 31, "Input",
  Evaluatable->True],
Cell[36107, 1186, 128, 3, 31, "Input",
  Evaluatable->True],
Cell[36238, 1191, 128, 3, 31, "Input",
  Evaluatable->True],
Cell[36369, 1196, 128, 3, 31, "Input",
  Evaluatable->True],
Cell[36500, 1201, 128, 3, 31, "Input",
  Evaluatable->True],
Cell[36631, 1206, 768, 15, 284, "Input",
  Evaluatable->True],
Cell[37402, 1223, 538, 13, 192, "Input",
  Evaluatable->True],
Cell[37943, 1238, 452, 11, 146, "Input",
  Evaluatable->True],
Cell[38398, 1251, 109, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[38556, 1259, 64, 0, 38, "Subsection"],
Cell[38623, 1261, 48, 0, 28, "Text"],
Cell[38674, 1263, 72, 2, 31, "Input"],
Cell[38749, 1267, 88, 2, 31, "Input"],
Cell[38840, 1271, 214, 4, 54, "Input"],
Cell[39057, 1277, 72, 2, 31, "Input"],
Cell[39132, 1281, 111, 3, 28, "Text"],
Cell[39246, 1286, 99, 2, 31, "Input"],
Cell[39348, 1290, 76, 2, 31, "Input"],
Cell[39427, 1294, 89, 2, 28, "Text"],
Cell[39519, 1298, 238, 4, 100, "Input"],
Cell[39760, 1304, 71, 2, 31, "Input"],
Cell[39834, 1308, 101, 2, 31, "Input"],
Cell[39938, 1312, 77, 2, 31, "Input"],
Cell[40018, 1316, 282, 5, 48, "Text"],
Cell[40303, 1323, 68, 2, 31, "Input"],
Cell[40374, 1327, 538, 12, 146, "Input",
  Evaluatable->False],
Cell[40915, 1341, 104, 2, 31, "Input"],
Cell[41022, 1345, 112, 2, 31, "Input"],
Cell[41137, 1349, 246, 6, 48, "Text"],

Cell[CellGroupData[{
Cell[41408, 1359, 39, 0, 37, "Subsubsection"],
Cell[41450, 1361, 104, 3, 28, "Text"],
Cell[41557, 1366, 114, 2, 32, "Input"],
Cell[41674, 1370, 98, 2, 31, "Input"],
Cell[41775, 1374, 111, 2, 31, "Input"],
Cell[41889, 1378, 71, 2, 31, "Input"],
Cell[41963, 1382, 90, 2, 31, "Input"],
Cell[42056, 1386, 73, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[42166, 1393, 52, 0, 31, "Subsubsection"],
Cell[42221, 1395, 87, 2, 28, "Text"],
Cell[42311, 1399, 112, 2, 38, "Input"],
Cell[42426, 1403, 98, 2, 31, "Input"],
Cell[42527, 1407, 111, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[42675, 1414, 45, 0, 31, "Subsubsection"],
Cell[42723, 1416, 50, 0, 28, "Text"],
Cell[42776, 1418, 188, 4, 48, "Input"],
Cell[42967, 1424, 95, 2, 31, "Input"],
Cell[43065, 1428, 69, 2, 31, "Input"],
Cell[43137, 1432, 134, 3, 31, "Input"],
Cell[43274, 1437, 103, 2, 31, "Input"],
Cell[43380, 1441, 152, 3, 31, "Input"],
Cell[43535, 1446, 108, 3, 28, "Text"],
Cell[43646, 1451, 129, 3, 31, "Input"],
Cell[43778, 1456, 76, 2, 31, "Input"],
Cell[43857, 1460, 139, 3, 31, "Input"],
Cell[43999, 1465, 161, 3, 54, "Input"],
Cell[44163, 1470, 75, 2, 31, "Input"],
Cell[44241, 1474, 77, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[44355, 1481, 45, 0, 31, "Subsubsection"],
Cell[44403, 1483, 46, 0, 28, "Text"],
Cell[44452, 1485, 187, 5, 54, "Input"],
Cell[44642, 1492, 103, 2, 31, "Input"],
Cell[44748, 1496, 72, 2, 31, "Input"],
Cell[44823, 1500, 149, 3, 54, "Input"],
Cell[44975, 1505, 69, 2, 31, "Input"],
Cell[45047, 1509, 90, 2, 31, "Input"],
Cell[45140, 1513, 111, 3, 28, "Text"],
Cell[45254, 1518, 73, 2, 31, "Input"],
Cell[45330, 1522, 105, 2, 31, "Input"],
Cell[45438, 1526, 70, 2, 31, "Input"],
Cell[45511, 1530, 71, 0, 28, "Text"],
Cell[45585, 1532, 200, 5, 54, "Input",
  InitializationCell->True],
Cell[45788, 1539, 145, 3, 54, "Input"],
Cell[45936, 1544, 94, 2, 28, "Text"],
Cell[46033, 1548, 218, 5, 77, "Input",
  InitializationCell->True],
Cell[46254, 1555, 67, 2, 31, "Input"],
Cell[46324, 1559, 67, 2, 31, "Input"],
Cell[46394, 1563, 131, 3, 28, "Text"],
Cell[46528, 1568, 160, 3, 54, "Input"],
Cell[46691, 1573, 67, 2, 31, "Input"],
Cell[46761, 1577, 90, 2, 31, "Input"],
Cell[46854, 1581, 108, 2, 31, "Input"],
Cell[46965, 1585, 103, 2, 31, "Input"],
Cell[47071, 1589, 115, 2, 31, "Input"],
Cell[47189, 1593, 74, 2, 31, "Input"],
Cell[47266, 1597, 76, 2, 31, "Input"],
Cell[47345, 1601, 96, 2, 31, "Input"],
Cell[47444, 1605, 157, 3, 54, "Input"],
Cell[47604, 1610, 70, 2, 31, "Input"],
Cell[47677, 1614, 90, 2, 31, "Input"],
Cell[47770, 1618, 69, 2, 31, "Input"],
Cell[47842, 1622, 100, 2, 31, "Input"],
Cell[47945, 1626, 114, 2, 31, "Input"],
Cell[48062, 1630, 69, 2, 31, "Input"],
Cell[48134, 1634, 96, 2, 31, "Input"],
Cell[48233, 1638, 158, 3, 54, "Input"],
Cell[48394, 1643, 71, 2, 31, "Input"],
Cell[48468, 1647, 90, 2, 31, "Input"],
Cell[48561, 1651, 70, 2, 31, "Input"],
Cell[48634, 1655, 101, 2, 31, "Input"],
Cell[48738, 1659, 115, 2, 31, "Input"],
Cell[48856, 1663, 70, 2, 31, "Input"],
Cell[48929, 1667, 76, 2, 31, "Input"],
Cell[49008, 1671, 96, 2, 31, "Input"],
Cell[49107, 1675, 70, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[49238, 1684, 125, 4, 61, "Section"],
Cell[49366, 1690, 824, 12, 128, "Text"],

Cell[CellGroupData[{
Cell[50215, 1706, 57, 0, 44, "Subsection"],
Cell[50275, 1708, 295, 5, 48, "Text"],

Cell[CellGroupData[{
Cell[50595, 1717, 102, 2, 37, "Subsubsection"],
Cell[50700, 1721, 226, 4, 48, "Text"],
Cell[50929, 1727, 76, 2, 31, "Input"],
Cell[51008, 1731, 909, 17, 238, "Input",
  Evaluatable->False],
Cell[51920, 1750, 237, 4, 48, "Text"],
Cell[52160, 1756, 378, 7, 100, "Input",
  Evaluatable->False],
Cell[52541, 1765, 78, 2, 31, "Input"],
Cell[52622, 1769, 179, 4, 54, "Input",
  Evaluatable->False],
Cell[52804, 1775, 112, 3, 28, "Text"],
Cell[52919, 1780, 380, 9, 77, "Input",
  Evaluatable->False],
Cell[53302, 1791, 171, 3, 28, "Text"],
Cell[53476, 1796, 73, 2, 31, "Input"],
Cell[53552, 1800, 24, 0, 28, "Text"],
Cell[53579, 1802, 392, 7, 169, "Input"],
Cell[53974, 1811, 78, 2, 31, "Input"],
Cell[54055, 1815, 397, 8, 169, "Input"],
Cell[54455, 1825, 67, 0, 28, "Text"],
Cell[54525, 1827, 83, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[54657, 1835, 41, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[54723, 1839, 39, 0, 37, "Subsubsection"],
Cell[54765, 1841, 89, 2, 31, "Input"],
Cell[54857, 1845, 89, 2, 31, "Input"],
Cell[54949, 1849, 89, 2, 31, "Input"],
Cell[55041, 1853, 98, 2, 31, "Input"],
Cell[55142, 1857, 102, 2, 31, "Input"],
Cell[55247, 1861, 91, 2, 31, "Input"],
Cell[55341, 1865, 102, 2, 31, "Input"],
Cell[55446, 1869, 99, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[55582, 1876, 64, 1, 37, "Subsubsection",
  Evaluatable->False],
Cell[55649, 1879, 64, 0, 28, "Text"],
Cell[55716, 1881, 124, 3, 31, "Input",
  InitializationCell->True],
Cell[55843, 1886, 121, 3, 28, "Text",
  Evaluatable->False],
Cell[55967, 1891, 96, 2, 31, "Input"],
Cell[56066, 1895, 121, 3, 28, "Text",
  Evaluatable->False],
Cell[56190, 1900, 96, 2, 31, "Input"],
Cell[56289, 1904, 57, 1, 28, "Text",
  Evaluatable->False],
Cell[56349, 1907, 96, 2, 31, "Input"],
Cell[56448, 1911, 57, 1, 28, "Text",
  Evaluatable->False],
Cell[56508, 1914, 96, 2, 31, "Input"],
Cell[56607, 1918, 57, 1, 28, "Text",
  Evaluatable->False],
Cell[56667, 1921, 96, 2, 31, "Input"],
Cell[56766, 1925, 256, 5, 54, "Input"],
Cell[57025, 1932, 75, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[57137, 1939, 41, 0, 31, "Subsubsection"],
Cell[57181, 1941, 195, 4, 28, "Text"],
Cell[57379, 1947, 102, 2, 31, "Input"],
Cell[57484, 1951, 95, 1, 35, "Input"],
Cell[57582, 1954, 77, 1, 35, "Input"],
Cell[57662, 1957, 84, 1, 35, "Input"],
Cell[57749, 1960, 99, 2, 28, "Text"],
Cell[57851, 1964, 88, 1, 35, "Input"],
Cell[57942, 1967, 51, 0, 28, "Text"],
Cell[57996, 1969, 84, 1, 35, "Input"],
Cell[58083, 1972, 264, 5, 48, "Text"],
Cell[58350, 1979, 88, 2, 31, "Input"],
Cell[58441, 1983, 101, 2, 31, "Input"],
Cell[58545, 1987, 88, 2, 31, "Input"],
Cell[58636, 1991, 99, 2, 31, "Input"],
Cell[58738, 1995, 457, 8, 146, "Input"],
Cell[59198, 2005, 89, 2, 31, "Input"],
Cell[59290, 2009, 107, 2, 31, "Input"],
Cell[59400, 2013, 89, 2, 31, "Input"],
Cell[59492, 2017, 102, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[59643, 2025, 57, 0, 38, "Subsection"],
Cell[59703, 2027, 1067, 15, 148, "Text"],
Cell[60773, 2044, 94, 2, 31, "Input"],
Cell[60870, 2048, 92, 2, 31, "Input"],
Cell[60965, 2052, 258, 4, 48, "Text"],
Cell[61226, 2058, 70, 2, 31, "Input"],
Cell[61299, 2062, 77, 2, 31, "Input"],
Cell[61379, 2066, 83, 2, 31, "Input"],
Cell[61465, 2070, 86, 2, 31, "Input"],
Cell[61554, 2074, 93, 2, 31, "Input"],
Cell[61650, 2078, 104, 2, 31, "Input"],
Cell[61757, 2082, 330, 6, 48, "Text"],
Cell[62090, 2090, 139, 3, 31, "Input"],
Cell[62232, 2095, 120, 3, 28, "Text"],
Cell[62355, 2100, 95, 2, 31, "Input"],
Cell[62453, 2104, 56, 0, 28, "Text"],
Cell[62512, 2106, 95, 2, 31, "Input"],
Cell[62610, 2110, 143, 3, 28, "Text"],
Cell[62756, 2115, 130, 3, 31, "Input"],
Cell[62889, 2120, 29, 0, 28, "Text"],
Cell[62921, 2122, 94, 2, 31, "Input"],
Cell[63018, 2126, 98, 2, 31, "Input"],
Cell[63119, 2130, 44, 0, 28, "Text"],
Cell[63166, 2132, 277, 5, 77, "Input",
  Evaluatable->False],
Cell[63446, 2139, 122, 3, 28, "Text"],
Cell[63571, 2144, 72, 2, 31, "Input"],
Cell[63646, 2148, 168, 3, 28, "Text"],
Cell[63817, 2153, 79, 2, 31, "Input"],
Cell[63899, 2157, 196, 5, 54, "Input",
  Evaluatable->False],
Cell[64098, 2164, 76, 2, 31, "Input"],
Cell[64177, 2168, 898, 19, 108, "Text"],

Cell[CellGroupData[{
Cell[65100, 2191, 30, 0, 37, "Subsubsection"],
Cell[65133, 2193, 48, 0, 28, "Text"],
Cell[65184, 2195, 82, 2, 31, "Input"],
Cell[65269, 2199, 109, 2, 31, "Input"],
Cell[65381, 2203, 143, 3, 31, "Input"],
Cell[65527, 2208, 148, 3, 28, "Text"],
Cell[65678, 2213, 78, 2, 31, "Input"],
Cell[65759, 2217, 196, 4, 31, "Input"],
Cell[65958, 2223, 172, 3, 31, "Input"],
Cell[66133, 2228, 101, 2, 31, "Input"],
Cell[66237, 2232, 73, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[66347, 2239, 53, 0, 37, "Subsubsection"],
Cell[66403, 2241, 102, 3, 28, "Text"],
Cell[66508, 2246, 110, 2, 31, "Input"],
Cell[66621, 2250, 64, 2, 31, "Input"],
Cell[66688, 2254, 148, 4, 31, "Input"],
Cell[66839, 2260, 70, 2, 31, "Input"],
Cell[66912, 2264, 116, 2, 31, "Input"],
Cell[67031, 2268, 79, 2, 31, "Input"],
Cell[67113, 2272, 164, 4, 31, "Input"],
Cell[67280, 2278, 75, 2, 31, "Input"],
Cell[67358, 2282, 39, 0, 28, "Text"],
Cell[67400, 2284, 258, 6, 54, "Input"],
Cell[67661, 2292, 69, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[67779, 2300, 44, 0, 38, "Subsection"],
Cell[67826, 2302, 518, 8, 68, "Text"],
Cell[68347, 2312, 2083, 38, 491, "Input",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[70455, 2354, 30, 0, 37, "Subsubsection"],
Cell[70488, 2356, 90, 2, 31, "Input"],
Cell[70581, 2360, 90, 2, 31, "Input"],
Cell[70674, 2364, 90, 2, 31, "Input"],
Cell[70767, 2368, 141, 3, 31, "Input"],
Cell[70911, 2373, 130, 3, 31, "Input"],
Cell[71044, 2378, 107, 2, 31, "Input"],
Cell[71154, 2382, 107, 2, 31, "Input"],
Cell[71264, 2386, 137, 3, 31, "Input"],
Cell[71404, 2391, 88, 2, 31, "Input"],
Cell[71495, 2395, 90, 2, 28, "Text"],
Cell[71588, 2399, 91, 2, 31, "Input"],
Cell[71682, 2403, 137, 3, 31, "Input"],
Cell[71822, 2408, 154, 3, 31, "Input"],
Cell[71979, 2413, 116, 2, 31, "Input"],
Cell[72098, 2417, 81, 2, 31, "Input"],
Cell[72182, 2421, 99, 2, 31, "Input"],
Cell[72284, 2425, 85, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[72418, 2433, 99, 2, 38, "Subsection"],
Cell[72520, 2437, 680, 10, 88, "Text"],

Cell[CellGroupData[{
Cell[73225, 2451, 70, 0, 37, "Subsubsection"],
Cell[73298, 2453, 81, 2, 31, "Input"],
Cell[73382, 2457, 94, 2, 31, "Input"],
Cell[73479, 2461, 353, 8, 100, "Input",
  Evaluatable->False],
Cell[73835, 2471, 464, 11, 100, "Input",
  Evaluatable->False],
Cell[74302, 2484, 397, 9, 100, "Input",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[74736, 2498, 66, 0, 37, "Subsubsection"],
Cell[74805, 2500, 209, 4, 48, "Text"],
Cell[75017, 2506, 74, 2, 31, "Input"],
Cell[75094, 2510, 2359, 49, 261, "Input",
  InitializationCell->True],
Cell[77456, 2561, 2349, 50, 261, "Input",
  InitializationCell->True],
Cell[79808, 2613, 2323, 45, 192, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[82168, 2663, 33, 0, 37, "Subsubsection"],
Cell[82204, 2665, 81, 2, 31, "Input"],
Cell[82288, 2669, 81, 2, 31, "Input"],
Cell[82372, 2673, 81, 2, 31, "Input"],
Cell[82456, 2677, 81, 2, 31, "Input"],
Cell[82540, 2681, 81, 2, 31, "Input"],
Cell[82624, 2685, 86, 2, 31, "Input"],
Cell[82713, 2689, 81, 2, 31, "Input"],
Cell[82797, 2693, 214, 5, 54, "Input"],
Cell[83014, 2700, 79, 2, 31, "Input"],
Cell[83096, 2704, 80, 2, 31, "Input"],
Cell[83179, 2708, 79, 2, 31, "Input"],
Cell[83261, 2712, 80, 2, 31, "Input"],
Cell[83344, 2716, 79, 2, 31, "Input"],
Cell[83426, 2720, 80, 2, 31, "Input"],
Cell[83509, 2724, 71, 2, 31, "Input"],
Cell[83583, 2728, 791, 15, 261, "Input"],
Cell[84377, 2745, 585, 12, 192, "Input"],
Cell[84965, 2759, 115, 2, 31, "Input"],
Cell[85083, 2763, 788, 15, 261, "Input"],
Cell[85874, 2780, 791, 15, 261, "Input"],
Cell[86668, 2797, 115, 2, 31, "Input"],
Cell[86786, 2801, 814, 17, 192, "Input"],
Cell[87603, 2820, 72, 2, 31, "Input"],
Cell[87678, 2824, 74, 2, 31, "Input"],
Cell[87755, 2828, 77, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[87881, 2836, 96, 2, 38, "Subsection"],

Cell[CellGroupData[{
Cell[88002, 2842, 36, 0, 37, "Subsubsection"],
Cell[88041, 2844, 345, 6, 48, "Text"],
Cell[88389, 2852, 69, 2, 31, "Input"],
Cell[88461, 2856, 995, 18, 284, "Input",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[89493, 2879, 33, 0, 37, "Subsubsection"],
Cell[89529, 2881, 102, 2, 31, "Input"],
Cell[89634, 2885, 102, 2, 31, "Input"],
Cell[89739, 2889, 99, 2, 31, "Input"],
Cell[89841, 2893, 128, 3, 31, "Input"],
Cell[89972, 2898, 105, 2, 31, "Input"],
Cell[90080, 2902, 65, 2, 31, "Input"],
Cell[90148, 2906, 107, 2, 31, "Input"],
Cell[90258, 2910, 91, 2, 31, "Input"],
Cell[90352, 2914, 63, 2, 31, "Input"],
Cell[90418, 2918, 94, 2, 31, "Input"],
Cell[90515, 2922, 78, 2, 31, "Input"],
Cell[90596, 2926, 70, 2, 31, "Input"],
Cell[90669, 2930, 94, 2, 31, "Input"],
Cell[90766, 2934, 105, 2, 31, "Input"],
Cell[90874, 2938, 69, 2, 31, "Input"],
Cell[90946, 2942, 99, 2, 31, "Input"],
Cell[91048, 2946, 91, 2, 31, "Input"],
Cell[91142, 2950, 91, 2, 31, "Input"],
Cell[91236, 2954, 113, 2, 31, "Input"],
Cell[91352, 2958, 91, 2, 31, "Input"],
Cell[91446, 2962, 251, 5, 123, "Input"],
Cell[91700, 2969, 230, 4, 123, "Input"],
Cell[91933, 2975, 92, 2, 31, "Input"],
Cell[92028, 2979, 107, 2, 31, "Input"],
Cell[92138, 2983, 139, 3, 31, "Input"],
Cell[92280, 2988, 78, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[92419, 2997, 147, 3, 61, "Section"],
Cell[92569, 3002, 496, 12, 68, "Text"],

Cell[CellGroupData[{
Cell[93090, 3018, 41, 0, 44, "Subsection"],
Cell[93134, 3020, 1614, 27, 464, "Text"],
Cell[94751, 3049, 419, 7, 80, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[95207, 3061, 71, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[95303, 3065, 39, 0, 37, "Subsubsection"],
Cell[95345, 3067, 248, 4, 48, "Text"],
Cell[95596, 3073, 76, 1, 28, "Text"],
Cell[95675, 3076, 308, 9, 208, "Text"],
Cell[95986, 3087, 65, 2, 31, "Input"],
Cell[96054, 3091, 31, 0, 28, "Text"],
Cell[96088, 3093, 67, 2, 31, "Input"],
Cell[96158, 3097, 333, 6, 80, "Text"],
Cell[96494, 3105, 238, 5, 77, "Input"],
Cell[96735, 3112, 74, 2, 31, "Input"],
Cell[96812, 3116, 71, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[96920, 3123, 55, 0, 31, "Subsubsection"],
Cell[96978, 3125, 152, 3, 28, "Text"],
Cell[97133, 3130, 365, 6, 123, "Input"],
Cell[97501, 3138, 69, 2, 31, "Input"],
Cell[97573, 3142, 73, 2, 31, "Input"],
Cell[97649, 3146, 160, 3, 28, "Text"],
Cell[97812, 3151, 261, 5, 77, "Input"],
Cell[98076, 3158, 72, 2, 31, "Input"],
Cell[98151, 3162, 64, 0, 28, "Text"],
Cell[98218, 3164, 74, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[98329, 3171, 55, 0, 31, "Subsubsection"],
Cell[98387, 3173, 157, 3, 28, "Text"],
Cell[98547, 3178, 371, 6, 123, "Input"],
Cell[98921, 3186, 69, 2, 31, "Input"],
Cell[98993, 3190, 63, 2, 31, "Input"],
Cell[99059, 3194, 376, 6, 68, "Text"],
Cell[99438, 3202, 261, 5, 77, "Input"],
Cell[99702, 3209, 72, 2, 31, "Input"],
Cell[99777, 3213, 74, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[99888, 3220, 55, 0, 31, "Subsubsection"],
Cell[99946, 3222, 74, 0, 28, "Text"],
Cell[100023, 3224, 614, 11, 146, "Input"],
Cell[100640, 3237, 71, 2, 31, "Input"],
Cell[100714, 3241, 130, 3, 31, "Input"],
Cell[100847, 3246, 74, 2, 31, "Input"],
Cell[100924, 3250, 72, 2, 31, "Input"],
Cell[100999, 3254, 109, 2, 31, "Input"],
Cell[101111, 3258, 104, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[101252, 3265, 42, 0, 31, "Subsubsection"],
Cell[101297, 3267, 413, 6, 68, "Text"],
Cell[101713, 3275, 234, 3, 92, "Text"],
Cell[101950, 3280, 450, 9, 123, "Input",
  Evaluatable->False],
Cell[102403, 3291, 71, 0, 28, "Text"],
Cell[102477, 3293, 154, 4, 31, "Input",
  Evaluatable->False],
Cell[102634, 3299, 202, 6, 31, "Input",
  Evaluatable->False],
Cell[102839, 3307, 76, 2, 31, "Input"],
Cell[102918, 3311, 79, 2, 31, "Input"],
Cell[103000, 3315, 69, 2, 31, "Input"],
Cell[103072, 3319, 78, 2, 31, "Input"],
Cell[103153, 3323, 164, 3, 28, "Text"],
Cell[103320, 3328, 192, 5, 31, "Input"],
Cell[103515, 3335, 69, 2, 31, "Input"],
Cell[103587, 3339, 181, 5, 28, "Text"],
Cell[103771, 3346, 97, 2, 31, "Input"],
Cell[103871, 3350, 107, 2, 31, "Input"],
Cell[103981, 3354, 95, 2, 31, "Input"],
Cell[104079, 3358, 39, 0, 28, "Text"],
Cell[104121, 3360, 65, 2, 31, "Input"],
Cell[104189, 3364, 173, 3, 28, "Text"],
Cell[104365, 3369, 111, 2, 31, "Input"],
Cell[104479, 3373, 69, 2, 31, "Input"],
Cell[104551, 3377, 183, 4, 48, "Text"],
Cell[104737, 3383, 180, 4, 31, "Input"],
Cell[104920, 3389, 82, 2, 31, "Input"],
Cell[105005, 3393, 284, 7, 48, "Text"],
Cell[105292, 3402, 106, 2, 28, "Text"],
Cell[105401, 3406, 243, 4, 36, "Input"],
Cell[105647, 3412, 625, 13, 68, "Text"],
Cell[106275, 3427, 165, 3, 54, "Input"],
Cell[106443, 3432, 106, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[106598, 3440, 69, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[106692, 3444, 32, 0, 37, "Subsubsection"],
Cell[106727, 3446, 99, 2, 28, "Text"],
Cell[106829, 3450, 263, 5, 149, "Input"],
Cell[107095, 3457, 63, 2, 31, "Input"],
Cell[107161, 3461, 85, 2, 28, "Text"],
Cell[107249, 3465, 106, 2, 31, "Input"],
Cell[107358, 3469, 90, 2, 31, "Input"],
Cell[107451, 3473, 72, 2, 31, "Input"],
Cell[107526, 3477, 78, 2, 31, "Input"],
Cell[107607, 3481, 69, 2, 31, "Input"],
Cell[107679, 3485, 91, 2, 31, "Input"],
Cell[107773, 3489, 62, 2, 31, "Input"],
Cell[107838, 3493, 65, 2, 31, "Input"],
Cell[107906, 3497, 237, 4, 48, "Text"],
Cell[108146, 3503, 92, 2, 31, "Input"],
Cell[108241, 3507, 129, 3, 28, "Text"],
Cell[108373, 3512, 136, 3, 59, "Input"],
Cell[108512, 3517, 128, 3, 59, "Input"],
Cell[108643, 3522, 239, 4, 48, "Text"],
Cell[108885, 3528, 183, 3, 54, "Input"],
Cell[109071, 3533, 124, 3, 28, "Text"],
Cell[109198, 3538, 80, 2, 31, "Input"],
Cell[109281, 3542, 79, 2, 31, "Input"],
Cell[109363, 3546, 78, 2, 31, "Input"],
Cell[109444, 3550, 108, 2, 31, "Input"],
Cell[109555, 3554, 185, 4, 48, "Text"],
Cell[109743, 3560, 97, 2, 31, "Input"],
Cell[109843, 3564, 94, 2, 28, "Text"],
Cell[109940, 3568, 100, 2, 31, "Input"],
Cell[110043, 3572, 105, 3, 28, "Text"],
Cell[110151, 3577, 49, 0, 28, "Text"],
Cell[110203, 3579, 116, 2, 31, "Input"],
Cell[110322, 3583, 65, 2, 31, "Input"],
Cell[110390, 3587, 74, 2, 31, "Input"],
Cell[110467, 3591, 77, 2, 31, "Input"],
Cell[110547, 3595, 80, 2, 31, "Input"],
Cell[110630, 3599, 79, 2, 31, "Input"],
Cell[110712, 3603, 50, 0, 28, "Text"],
Cell[110765, 3605, 80, 2, 31, "Input"],
Cell[110848, 3609, 79, 2, 31, "Input"],
Cell[110930, 3613, 73, 2, 31, "Input"],
Cell[111006, 3617, 111, 2, 31, "Input"],
Cell[111120, 3621, 189, 4, 54, "Input"],
Cell[111312, 3627, 71, 2, 31, "Input"],
Cell[111386, 3631, 71, 2, 31, "Input"],
Cell[111460, 3635, 73, 2, 31, "Input"],
Cell[111536, 3639, 85, 2, 31, "Input"],
Cell[111624, 3643, 91, 2, 31, "Input"],
Cell[111718, 3647, 65, 2, 31, "Input"],
Cell[111786, 3651, 73, 2, 31, "Input"],
Cell[111862, 3655, 72, 2, 31, "Input"],
Cell[111937, 3659, 62, 2, 31, "Input"],
Cell[112002, 3663, 76, 2, 31, "Input"],
Cell[112081, 3667, 76, 2, 31, "Input"],
Cell[112160, 3671, 73, 0, 28, "Text"],
Cell[112236, 3673, 87, 2, 28, "Text"],
Cell[112326, 3677, 129, 3, 31, "Input"],
Cell[112458, 3682, 70, 2, 31, "Input"],
Cell[112531, 3686, 96, 2, 31, "Input"],
Cell[112630, 3690, 70, 2, 31, "Input"],
Cell[112703, 3694, 87, 2, 31, "Input"],
Cell[112793, 3698, 166, 3, 28, "Text"],
Cell[112962, 3703, 81, 2, 31, "Input"],
Cell[113046, 3707, 69, 2, 31, "Input"],
Cell[113118, 3711, 86, 2, 31, "Input"],
Cell[113207, 3715, 69, 2, 31, "Input"],
Cell[113279, 3719, 73, 2, 31, "Input"],
Cell[113355, 3723, 70, 2, 31, "Input"],
Cell[113428, 3727, 81, 2, 31, "Input"],
Cell[113512, 3731, 69, 2, 31, "Input"],
Cell[113584, 3735, 89, 2, 31, "Input"],
Cell[113676, 3739, 69, 2, 31, "Input"],
Cell[113748, 3743, 73, 2, 31, "Input"],
Cell[113824, 3747, 70, 2, 31, "Input"],
Cell[113897, 3751, 75, 2, 31, "Input"],
Cell[113975, 3755, 71, 2, 31, "Input"],
Cell[114049, 3759, 58, 0, 28, "Text"],
Cell[114110, 3761, 101, 2, 31, "Input"],
Cell[114214, 3765, 281, 5, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[114532, 3775, 40, 0, 31, "Subsubsection"],
Cell[114575, 3777, 111, 2, 31, "Input"],
Cell[114689, 3781, 208, 4, 77, "Input"],
Cell[114900, 3787, 64, 2, 31, "Input"],
Cell[114967, 3791, 38, 0, 28, "Text"],
Cell[115008, 3793, 107, 2, 31, "Input"],
Cell[115118, 3797, 68, 2, 31, "Input"],
Cell[115189, 3801, 81, 2, 31, "Input"],
Cell[115273, 3805, 73, 2, 31, "Input"],
Cell[115349, 3809, 95, 2, 31, "Input"],
Cell[115447, 3813, 71, 2, 31, "Input"],
Cell[115521, 3817, 157, 4, 31, "Input"],
Cell[115681, 3823, 150, 4, 31, "Input"],
Cell[115834, 3829, 181, 3, 28, "Text"],
Cell[116018, 3834, 184, 3, 54, "Input"],
Cell[116205, 3839, 123, 3, 28, "Text"],
Cell[116331, 3844, 81, 2, 31, "Input"],
Cell[116415, 3848, 85, 2, 31, "Input"],
Cell[116503, 3852, 109, 2, 31, "Input"],
Cell[116615, 3856, 98, 2, 31, "Input"],
Cell[116716, 3860, 93, 2, 28, "Text"],
Cell[116812, 3864, 84, 2, 31, "Input"],
Cell[116899, 3868, 190, 4, 54, "Input"],
Cell[117092, 3874, 72, 2, 31, "Input"],
Cell[117167, 3878, 86, 2, 31, "Input"],
Cell[117256, 3882, 95, 2, 31, "Input"],
Cell[117354, 3886, 66, 2, 31, "Input"],
Cell[117423, 3890, 73, 2, 31, "Input"],
Cell[117499, 3894, 207, 4, 48, "Text"],
Cell[117709, 3900, 101, 2, 31, "Input"],
Cell[117813, 3904, 93, 2, 28, "Text"],
Cell[117909, 3908, 85, 2, 31, "Input"],
Cell[117997, 3912, 86, 2, 31, "Input"],
Cell[118086, 3916, 73, 0, 28, "Text"],
Cell[118162, 3918, 95, 2, 31, "Input"],
Cell[118260, 3922, 120, 3, 31, "Input"],
Cell[118383, 3927, 66, 2, 31, "Input"],
Cell[118452, 3931, 74, 2, 31, "Input"],
Cell[118529, 3935, 115, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[118693, 3943, 54, 0, 38, "Subsection"],
Cell[118750, 3945, 234, 4, 48, "Text"],

Cell[CellGroupData[{
Cell[119009, 3953, 50, 0, 37, "Subsubsection"],
Cell[119062, 3955, 54, 0, 28, "Text"],
Cell[119119, 3957, 188, 4, 54, "Input"],
Cell[119310, 3963, 78, 2, 31, "Input"],
Cell[119391, 3967, 69, 2, 31, "Input"],
Cell[119463, 3971, 148, 3, 28, "Text"],
Cell[119614, 3976, 89, 2, 31, "Input"],
Cell[119706, 3980, 73, 2, 31, "Input"],
Cell[119782, 3984, 181, 3, 28, "Text"],
Cell[119966, 3989, 182, 3, 54, "Input"],
Cell[120151, 3994, 200, 4, 48, "Text"],
Cell[120354, 4000, 81, 2, 31, "Input"],
Cell[120438, 4004, 70, 2, 31, "Input"],
Cell[120511, 4008, 79, 2, 31, "Input"],
Cell[120593, 4012, 111, 2, 31, "Input"],
Cell[120707, 4016, 380, 6, 68, "Text"],
Cell[121090, 4024, 74, 2, 31, "Input"],
Cell[121167, 4028, 63, 2, 31, "Input"],
Cell[121233, 4032, 77, 2, 31, "Input"],
Cell[121313, 4036, 195, 4, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[121545, 4045, 77, 0, 37, "Subsubsection"],
Cell[121625, 4047, 1065, 16, 160, "Text"],
Cell[122693, 4065, 362, 6, 100, "Input"],
Cell[123058, 4073, 84, 2, 31, "Input"],
Cell[123145, 4077, 66, 2, 31, "Input"],
Cell[123214, 4081, 91, 2, 31, "Input"],
Cell[123308, 4085, 101, 2, 31, "Input"],
Cell[123412, 4089, 73, 2, 31, "Input"],
Cell[123488, 4093, 70, 2, 31, "Input"],
Cell[123561, 4097, 70, 2, 31, "Input"],
Cell[123634, 4101, 141, 4, 59, "Input"],
Cell[123778, 4107, 172, 3, 31, "Input"],
Cell[123953, 4112, 70, 2, 31, "Input"],
Cell[124026, 4116, 89, 2, 28, "Text"],
Cell[124118, 4120, 174, 3, 31, "Input"],
Cell[124295, 4125, 82, 2, 31, "Input"],
Cell[124380, 4129, 116, 3, 28, "Text"],
Cell[124499, 4134, 70, 2, 31, "Input"],
Cell[124572, 4138, 125, 3, 28, "Text"],
Cell[124700, 4143, 310, 6, 77, "Input",
  Evaluatable->False],
Cell[125013, 4151, 132, 3, 31, "Input"],
Cell[125148, 4156, 81, 2, 31, "Input"],
Cell[125232, 4160, 66, 2, 31, "Input"],
Cell[125301, 4164, 71, 2, 31, "Input"],
Cell[125375, 4168, 95, 2, 31, "Input"],
Cell[125473, 4172, 76, 0, 28, "Text"],
Cell[125552, 4174, 238, 4, 54, "Input"],
Cell[125793, 4180, 72, 2, 31, "Input"],
Cell[125868, 4184, 78, 2, 31, "Input"],
Cell[125949, 4188, 70, 2, 31, "Input"],
Cell[126022, 4192, 116, 3, 28, "Text"],
Cell[126141, 4197, 244, 4, 54, "Input"],
Cell[126388, 4203, 73, 2, 31, "Input"],
Cell[126464, 4207, 79, 2, 31, "Input"],
Cell[126546, 4211, 69, 2, 31, "Input"],
Cell[126618, 4215, 95, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[126750, 4222, 51, 0, 37, "Subsubsection"],
Cell[126804, 4224, 64, 1, 32, "Text"],
Cell[126871, 4227, 86, 2, 31, "Input"],
Cell[126960, 4231, 102, 2, 31, "Input"],
Cell[127065, 4235, 114, 2, 31, "Input"],
Cell[127182, 4239, 88, 2, 28, "Text"],
Cell[127273, 4243, 111, 2, 31, "Input"],
Cell[127387, 4247, 75, 2, 31, "Input"],
Cell[127465, 4251, 70, 2, 31, "Input"],
Cell[127538, 4255, 271, 5, 48, "Text"],
Cell[127812, 4262, 113, 2, 31, "Input"],
Cell[127928, 4266, 69, 2, 31, "Input"],
Cell[128000, 4270, 73, 2, 31, "Input"],
Cell[128076, 4274, 145, 3, 31, "Input"],
Cell[128224, 4279, 149, 3, 31, "Input"],
Cell[128376, 4284, 66, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[128479, 4291, 95, 2, 37, "Subsubsection"],
Cell[128577, 4295, 94, 2, 31, "Input"],
Cell[128674, 4299, 102, 2, 31, "Input"],
Cell[128779, 4303, 71, 0, 28, "Text"],
Cell[128853, 4305, 112, 2, 31, "Input"],
Cell[128968, 4309, 110, 2, 31, "Input"],
Cell[129081, 4313, 73, 2, 31, "Input"],
Cell[129157, 4317, 73, 2, 31, "Input"],
Cell[129233, 4321, 73, 2, 31, "Input"],
Cell[129309, 4325, 98, 2, 31, "Input"],
Cell[129410, 4329, 101, 2, 31, "Input"],
Cell[129514, 4333, 69, 2, 31, "Input"],
Cell[129586, 4337, 105, 2, 31, "Input"],
Cell[129694, 4341, 71, 2, 31, "Input"],
Cell[129768, 4345, 73, 2, 31, "Input"],
Cell[129844, 4349, 105, 3, 28, "Text"],
Cell[129952, 4354, 132, 3, 31, "Input"],
Cell[130087, 4359, 67, 2, 31, "Input"],
Cell[130157, 4363, 62, 0, 28, "Text"],
Cell[130222, 4365, 169, 4, 31, "Input"],
Cell[130394, 4371, 93, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[130524, 4378, 57, 0, 37, "Subsubsection"],
Cell[130584, 4380, 122, 3, 28, "Text"],
Cell[130709, 4385, 311, 6, 100, "Input"],
Cell[131023, 4393, 168, 4, 54, "Input"],
Cell[131194, 4399, 258, 8, 60, "Text"],
Cell[131455, 4409, 224, 4, 112, "Input"],
Cell[131682, 4415, 64, 2, 31, "Input"],
Cell[131749, 4419, 65, 0, 28, "Text"],
Cell[131817, 4421, 81, 2, 31, "Input"],
Cell[131901, 4425, 76, 2, 31, "Input"],
Cell[131980, 4429, 89, 2, 31, "Input"],
Cell[132072, 4433, 73, 2, 31, "Input"],
Cell[132148, 4437, 178, 4, 28, "Text"],
Cell[132329, 4443, 182, 3, 54, "Input"],
Cell[132514, 4448, 199, 4, 48, "Text"],
Cell[132716, 4454, 81, 2, 31, "Input"],
Cell[132800, 4458, 79, 2, 31, "Input"],
Cell[132882, 4462, 109, 2, 31, "Input"],
Cell[132994, 4466, 200, 4, 48, "Text"],
Cell[133197, 4472, 190, 4, 54, "Input"],
Cell[133390, 4478, 72, 2, 31, "Input"],
Cell[133465, 4482, 85, 2, 31, "Input"],
Cell[133553, 4486, 95, 2, 31, "Input"],
Cell[133651, 4490, 66, 2, 31, "Input"],
Cell[133720, 4494, 73, 2, 31, "Input"],
Cell[133796, 4498, 265, 5, 48, "Text"],
Cell[134064, 4505, 101, 2, 31, "Input"],
Cell[134168, 4509, 130, 3, 28, "Text"],
Cell[134301, 4514, 85, 2, 31, "Input"],
Cell[134389, 4518, 69, 2, 31, "Input"],
Cell[134461, 4522, 73, 0, 28, "Text"],
Cell[134537, 4524, 100, 2, 31, "Input"],
Cell[134640, 4528, 120, 3, 31, "Input"],
Cell[134763, 4533, 66, 2, 31, "Input"],
Cell[134832, 4537, 74, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[134955, 4545, 76, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[135056, 4549, 38, 0, 36, "Subsubsection"],
Cell[135097, 4551, 134, 3, 27, "Text"],
Cell[135234, 4556, 100, 2, 32, "Input"],
Cell[135337, 4560, 78, 2, 32, "Input"],
Cell[135418, 4564, 79, 2, 32, "Input"],
Cell[135500, 4568, 83, 2, 32, "Input"],
Cell[135586, 4572, 97, 2, 32, "Input"],
Cell[135686, 4576, 78, 2, 32, "Input"],
Cell[135767, 4580, 79, 2, 32, "Input"],
Cell[135849, 4584, 81, 2, 32, "Input"],
Cell[135933, 4588, 84, 2, 32, "Input"],
Cell[136020, 4592, 64, 2, 32, "Input"],
Cell[136087, 4596, 122, 3, 27, "Text"],
Cell[136212, 4601, 107, 2, 32, "Input"],
Cell[136322, 4605, 66, 2, 32, "Input"],
Cell[136391, 4609, 73, 2, 32, "Input"],
Cell[136467, 4613, 79, 2, 32, "Input"],
Cell[136549, 4617, 93, 2, 32, "Input"],
Cell[136645, 4621, 129, 3, 27, "Text"],
Cell[136777, 4626, 73, 2, 32, "Input"],
Cell[136853, 4630, 75, 2, 32, "Input"],
Cell[136931, 4634, 41, 0, 27, "Text"],
Cell[136975, 4636, 117, 2, 32, "Input"],
Cell[137095, 4640, 98, 2, 32, "Input"],
Cell[137196, 4644, 75, 2, 32, "Input"],
Cell[137274, 4648, 227, 3, 46, "Text"],
Cell[137504, 4653, 184, 3, 56, "Input"],
Cell[137691, 4658, 124, 3, 27, "Text"],
Cell[137818, 4663, 81, 2, 32, "Input"],
Cell[137902, 4667, 79, 2, 32, "Input"],
Cell[137984, 4671, 109, 2, 32, "Input"],
Cell[138096, 4675, 173, 3, 46, "Text"],
Cell[138272, 4680, 98, 2, 32, "Input"],
Cell[138373, 4684, 94, 2, 27, "Text"],
Cell[138470, 4688, 101, 2, 32, "Input"],
Cell[138574, 4692, 91, 2, 27, "Text"],
Cell[138668, 4696, 69, 2, 32, "Input"],
Cell[138740, 4700, 72, 2, 32, "Input"],
Cell[138815, 4704, 90, 2, 27, "Text"],
Cell[138908, 4708, 111, 2, 32, "Input"],
Cell[139022, 4712, 80, 2, 32, "Input"],
Cell[139105, 4716, 65, 2, 32, "Input"],
Cell[139173, 4720, 66, 2, 32, "Input"],
Cell[139242, 4724, 105, 3, 27, "Text"],
Cell[139350, 4729, 190, 4, 56, "Input"],
Cell[139543, 4735, 72, 2, 32, "Input"],
Cell[139618, 4739, 66, 2, 32, "Input"],
Cell[139687, 4743, 86, 2, 32, "Input"],
Cell[139776, 4747, 92, 2, 32, "Input"],
Cell[139871, 4751, 66, 2, 32, "Input"],
Cell[139940, 4755, 74, 2, 32, "Input"],
Cell[140017, 4759, 73, 0, 27, "Text"],
Cell[140093, 4761, 140, 3, 56, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[140270, 4769, 40, 0, 36, "Subsubsection"],
Cell[140313, 4771, 104, 2, 32, "Input"],
Cell[140420, 4775, 81, 2, 32, "Input"],
Cell[140504, 4779, 72, 2, 32, "Input"],
Cell[140579, 4783, 74, 2, 32, "Input"],
Cell[140656, 4787, 80, 2, 32, "Input"],
Cell[140739, 4791, 80, 2, 32, "Input"],
Cell[140822, 4795, 109, 2, 32, "Input"],
Cell[140934, 4799, 66, 2, 32, "Input"],
Cell[141003, 4803, 73, 2, 32, "Input"],
Cell[141079, 4807, 84, 2, 32, "Input"],
Cell[141166, 4811, 64, 2, 32, "Input"],
Cell[141233, 4815, 95, 2, 27, "Text"],
Cell[141331, 4819, 79, 2, 32, "Input"],
Cell[141413, 4823, 73, 2, 32, "Input"],
Cell[141489, 4827, 93, 2, 32, "Input"],
Cell[141585, 4831, 129, 3, 27, "Text"],
Cell[141717, 4836, 160, 4, 32, "Input"],
Cell[141880, 4842, 153, 4, 32, "Input"],
Cell[142036, 4848, 117, 2, 32, "Input"],
Cell[142156, 4852, 88, 2, 32, "Input"],
Cell[142247, 4856, 75, 2, 32, "Input"],
Cell[142325, 4860, 227, 3, 46, "Text"],
Cell[142555, 4865, 184, 3, 56, "Input"],
Cell[142742, 4870, 124, 3, 27, "Text"],
Cell[142869, 4875, 81, 2, 32, "Input"],
Cell[142953, 4879, 79, 2, 32, "Input"],
Cell[143035, 4883, 109, 2, 32, "Input"],
Cell[143147, 4887, 186, 4, 46, "Text"],
Cell[143336, 4893, 98, 2, 32, "Input"],
Cell[143437, 4897, 94, 2, 27, "Text"],
Cell[143534, 4901, 101, 2, 32, "Input"],
Cell[143638, 4905, 80, 2, 32, "Input"],
Cell[143721, 4909, 65, 2, 32, "Input"],
Cell[143789, 4913, 66, 2, 32, "Input"],
Cell[143858, 4917, 111, 3, 27, "Text"],
Cell[143972, 4922, 87, 2, 32, "Input"],
Cell[144062, 4926, 190, 4, 56, "Input"],
Cell[144255, 4932, 72, 2, 32, "Input"],
Cell[144330, 4936, 66, 2, 32, "Input"],
Cell[144399, 4940, 86, 2, 32, "Input"],
Cell[144488, 4944, 92, 2, 32, "Input"],
Cell[144583, 4948, 66, 2, 32, "Input"],
Cell[144652, 4952, 74, 2, 32, "Input"],
Cell[144729, 4956, 73, 0, 27, "Text"],
Cell[144805, 4958, 140, 3, 56, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[144994, 4967, 37, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[145056, 4971, 48, 0, 37, "Subsubsection"],
Cell[145107, 4973, 117, 3, 28, "Text"],
Cell[145227, 4978, 98, 2, 31, "Input"],
Cell[145328, 4982, 128, 3, 28, "Text"],
Cell[145459, 4987, 195, 7, 107, "Input",
  InitializationCell->True],
Cell[145657, 4996, 76, 2, 31, "Input"],
Cell[145736, 5000, 87, 2, 31, "Input"],
Cell[145826, 5004, 70, 2, 31, "Input"],
Cell[145899, 5008, 183, 4, 54, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[146119, 5017, 49, 0, 37, "Subsubsection"],
Cell[146171, 5019, 253, 5, 77, "Input"],
Cell[146427, 5026, 103, 3, 28, "Text"],
Cell[146533, 5031, 64, 2, 31, "Input"],
Cell[146600, 5035, 67, 2, 31, "Input"],
Cell[146670, 5039, 76, 2, 31, "Input"],
Cell[146749, 5043, 121, 3, 31, "Input"],
Cell[146873, 5048, 133, 3, 28, "Text"],
Cell[147009, 5053, 107, 2, 31, "Input"],
Cell[147119, 5057, 66, 2, 31, "Input"],
Cell[147188, 5061, 79, 2, 31, "Input"],
Cell[147270, 5065, 73, 2, 31, "Input"],
Cell[147346, 5069, 63, 2, 31, "Input"],
Cell[147412, 5073, 93, 2, 31, "Input"],
Cell[147508, 5077, 135, 3, 28, "Text"],
Cell[147646, 5082, 107, 2, 31, "Input"],
Cell[147756, 5086, 66, 2, 31, "Input"],
Cell[147825, 5090, 227, 3, 48, "Text"],
Cell[148055, 5095, 172, 3, 54, "Input"],
Cell[148230, 5100, 124, 3, 28, "Text"],
Cell[148357, 5105, 81, 2, 31, "Input"],
Cell[148441, 5109, 79, 2, 31, "Input"],
Cell[148523, 5113, 109, 2, 31, "Input"],
Cell[148635, 5117, 177, 3, 28, "Text"],
Cell[148815, 5122, 98, 2, 31, "Input"],
Cell[148916, 5126, 238, 4, 48, "Text"],
Cell[149157, 5132, 85, 2, 31, "Input"],
Cell[149245, 5136, 55, 0, 28, "Text"],
Cell[149303, 5138, 88, 2, 31, "Input"],
Cell[149394, 5142, 137, 4, 60, "Text"],
Cell[149534, 5148, 101, 2, 31, "Input"],
Cell[149638, 5152, 358, 6, 68, "Text"],
Cell[149999, 5160, 74, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[150110, 5167, 53, 0, 37, "Subsubsection"],
Cell[150166, 5169, 257, 5, 77, "Input"],
Cell[150426, 5176, 103, 3, 28, "Text"],
Cell[150532, 5181, 67, 2, 31, "Input"],
Cell[150602, 5185, 76, 2, 31, "Input"],
Cell[150681, 5189, 121, 3, 31, "Input"],
Cell[150805, 5194, 109, 3, 28, "Text"],
Cell[150917, 5199, 107, 2, 31, "Input"],
Cell[151027, 5203, 66, 2, 31, "Input"],
Cell[151096, 5207, 79, 2, 31, "Input"],
Cell[151178, 5211, 93, 2, 31, "Input"],
Cell[151274, 5215, 135, 3, 28, "Text"],
Cell[151412, 5220, 107, 2, 31, "Input"],
Cell[151522, 5224, 66, 2, 31, "Input"],
Cell[151591, 5228, 227, 3, 48, "Text"],
Cell[151821, 5233, 172, 3, 54, "Input"],
Cell[151996, 5238, 124, 3, 28, "Text"],
Cell[152123, 5243, 81, 2, 31, "Input"],
Cell[152207, 5247, 79, 2, 31, "Input"],
Cell[152289, 5251, 109, 2, 31, "Input"],
Cell[152401, 5255, 177, 3, 28, "Text"],
Cell[152581, 5260, 98, 2, 31, "Input"],
Cell[152682, 5264, 238, 4, 48, "Text"],
Cell[152923, 5270, 85, 2, 31, "Input"],
Cell[153011, 5274, 55, 0, 28, "Text"],
Cell[153069, 5276, 88, 2, 31, "Input"],
Cell[153160, 5280, 136, 4, 60, "Text"],
Cell[153299, 5286, 101, 2, 31, "Input"],
Cell[153403, 5290, 152, 3, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[153592, 5298, 48, 0, 37, "Subsubsection"],
Cell[153643, 5300, 257, 5, 77, "Input"],
Cell[153903, 5307, 103, 3, 28, "Text"],
Cell[154009, 5312, 67, 2, 31, "Input"],
Cell[154079, 5316, 76, 2, 31, "Input"],
Cell[154158, 5320, 121, 3, 31, "Input"],
Cell[154282, 5325, 168, 3, 28, "Text"],
Cell[154453, 5330, 107, 2, 31, "Input"],
Cell[154563, 5334, 66, 2, 31, "Input"],
Cell[154632, 5338, 79, 2, 31, "Input"],
Cell[154714, 5342, 93, 2, 31, "Input"],
Cell[154810, 5346, 135, 3, 28, "Text"],
Cell[154948, 5351, 107, 2, 31, "Input"],
Cell[155058, 5355, 66, 2, 31, "Input"],
Cell[155127, 5359, 227, 3, 48, "Text"],
Cell[155357, 5364, 172, 3, 54, "Input"],
Cell[155532, 5369, 81, 2, 31, "Input"],
Cell[155616, 5373, 79, 2, 31, "Input"],
Cell[155698, 5377, 109, 2, 31, "Input"],
Cell[155810, 5381, 177, 3, 28, "Text"],
Cell[155990, 5386, 98, 2, 31, "Input"],
Cell[156091, 5390, 238, 4, 48, "Text"],
Cell[156332, 5396, 85, 2, 31, "Input"],
Cell[156420, 5400, 55, 0, 28, "Text"],
Cell[156478, 5402, 88, 2, 31, "Input"],
Cell[156569, 5406, 135, 4, 60, "Text"],
Cell[156707, 5412, 101, 2, 31, "Input"],
Cell[156811, 5416, 214, 4, 48, "Text"],
Cell[157028, 5422, 113, 2, 31, "Input"],
Cell[157144, 5426, 66, 2, 31, "Input"],
Cell[157213, 5430, 73, 2, 31, "Input"],
Cell[157289, 5434, 75, 0, 28, "Text"],
Cell[157367, 5436, 75, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[157491, 5444, 42, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[157558, 5448, 103, 2, 37, "Subsubsection"],
Cell[157664, 5452, 26, 0, 28, "Text"],
Cell[157693, 5454, 91, 2, 31, "Input"],
Cell[157787, 5458, 70, 2, 31, "Input"],
Cell[157860, 5462, 108, 2, 31, "Input"],
Cell[157971, 5466, 64, 2, 31, "Input"],
Cell[158038, 5470, 128, 3, 31, "Input"],
Cell[158169, 5475, 95, 2, 31, "Input"],
Cell[158267, 5479, 73, 2, 31, "Input"],
Cell[158343, 5483, 113, 2, 31, "Input"],
Cell[158459, 5487, 66, 2, 31, "Input"],
Cell[158528, 5491, 188, 6, 28, "Text"],
Cell[158719, 5499, 104, 2, 31, "Input"],
Cell[158826, 5503, 74, 0, 28, "Text"],
Cell[158903, 5505, 70, 2, 31, "Input"],
Cell[158976, 5509, 113, 2, 31, "Input"],
Cell[159092, 5513, 66, 2, 31, "Input"],
Cell[159161, 5517, 73, 2, 31, "Input"],
Cell[159237, 5521, 96, 2, 28, "Text"],
Cell[159336, 5525, 104, 2, 31, "Input"],
Cell[159443, 5529, 26, 0, 28, "Text"],
Cell[159472, 5531, 121, 3, 31, "Input"],
Cell[159596, 5536, 64, 2, 31, "Input"],
Cell[159663, 5540, 166, 3, 28, "Text"],
Cell[159832, 5545, 102, 2, 31, "Input"],
Cell[159937, 5549, 67, 2, 31, "Input"],
Cell[160007, 5553, 25, 0, 28, "Text"],
Cell[160035, 5555, 129, 3, 31, "Input"],
Cell[160167, 5560, 302, 5, 48, "Text"],
Cell[160472, 5567, 80, 2, 31, "Input"],
Cell[160555, 5571, 73, 2, 31, "Input"],
Cell[160631, 5575, 113, 2, 31, "Input"],
Cell[160747, 5579, 69, 2, 31, "Input"],
Cell[160819, 5583, 108, 3, 28, "Text"],
Cell[160930, 5588, 102, 2, 31, "Input"],
Cell[161035, 5592, 73, 2, 31, "Input"],
Cell[161111, 5596, 106, 2, 31, "Input"],
Cell[161220, 5600, 69, 2, 31, "Input"],
Cell[161292, 5604, 62, 0, 28, "Text"],
Cell[161357, 5606, 91, 2, 31, "Input"],
Cell[161451, 5610, 70, 2, 31, "Input"],
Cell[161524, 5614, 91, 2, 31, "Input"],
Cell[161618, 5618, 70, 2, 31, "Input"],
Cell[161691, 5622, 75, 2, 31, "Input"],
Cell[161769, 5626, 78, 2, 31, "Input"],
Cell[161850, 5630, 70, 2, 31, "Input"],
Cell[161923, 5634, 88, 2, 31, "Input"],
Cell[162014, 5638, 109, 2, 31, "Input"],
Cell[162126, 5642, 66, 2, 31, "Input"],
Cell[162195, 5646, 88, 2, 31, "Input"],
Cell[162286, 5650, 88, 2, 31, "Input"],
Cell[162377, 5654, 104, 3, 28, "Text"],
Cell[162484, 5659, 185, 5, 54, "Input"],
Cell[162672, 5666, 185, 5, 54, "Input"],
Cell[162860, 5673, 105, 2, 31, "Input"],
Cell[162968, 5677, 95, 2, 31, "Input"],
Cell[163066, 5681, 71, 2, 31, "Input"],
Cell[163140, 5685, 95, 2, 31, "Input"],
Cell[163238, 5689, 71, 2, 31, "Input"],
Cell[163312, 5693, 77, 2, 31, "Input"],
Cell[163392, 5697, 82, 2, 31, "Input"],
Cell[163477, 5701, 70, 2, 31, "Input"],
Cell[163550, 5705, 92, 2, 31, "Input"],
Cell[163645, 5709, 92, 2, 31, "Input"],
Cell[163740, 5713, 92, 2, 31, "Input"],
Cell[163835, 5717, 96, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[163968, 5724, 106, 2, 37, "Subsubsection"],
Cell[164077, 5728, 112, 2, 31, "Input"],
Cell[164192, 5732, 128, 3, 31, "Input"],
Cell[164323, 5737, 92, 2, 31, "Input"],
Cell[164418, 5741, 74, 2, 31, "Input"],
Cell[164495, 5745, 113, 2, 31, "Input"],
Cell[164611, 5749, 66, 2, 31, "Input"],
Cell[164680, 5753, 44, 0, 28, "Text"],
Cell[164727, 5755, 104, 2, 31, "Input"],
Cell[164834, 5759, 81, 2, 31, "Input"],
Cell[164918, 5763, 85, 2, 31, "Input"],
Cell[165006, 5767, 82, 2, 31, "Input"],
Cell[165091, 5771, 279, 7, 51, "Text"],
Cell[165373, 5780, 98, 2, 31, "Input"],
Cell[165474, 5784, 74, 2, 31, "Input"],
Cell[165551, 5788, 71, 2, 31, "Input"],
Cell[165625, 5792, 73, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[165759, 5801, 126, 2, 60, "Section"],
Cell[165888, 5805, 1051, 15, 160, "Text"],
Cell[166942, 5822, 70, 2, 31, "Input"],

Cell[CellGroupData[{
Cell[167037, 5828, 101, 2, 44, "Subsection"],
Cell[167141, 5832, 97, 2, 31, "Input"],
Cell[167241, 5836, 99, 2, 31, "Input"],
Cell[167343, 5840, 97, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[167477, 5847, 67, 0, 44, "Subsection"],
Cell[167547, 5849, 67, 0, 28, "Text"],
Cell[167617, 5851, 99, 2, 31, "Input"],
Cell[167719, 5855, 168, 2, 28, "Text"],
Cell[167890, 5859, 120, 3, 31, "Input"],
Cell[168013, 5864, 112, 2, 31, "Input"],
Cell[168128, 5868, 68, 0, 28, "Text"],
Cell[168199, 5870, 70, 2, 31, "Input"],

Cell[CellGroupData[{
Cell[168294, 5876, 39, 0, 31, "Subsubsection"],
Cell[168336, 5878, 56, 0, 28, "Text"],
Cell[168395, 5880, 128, 3, 31, "Input"],
Cell[168526, 5885, 87, 2, 31, "Input"],
Cell[168616, 5889, 68, 2, 31, "Input"],
Cell[168687, 5893, 102, 2, 31, "Input"],
Cell[168792, 5897, 65, 0, 28, "Text"],
Cell[168860, 5899, 233, 5, 54, "Input"],
Cell[169096, 5906, 71, 2, 31, "Input"],
Cell[169170, 5910, 73, 2, 31, "Input"],
Cell[169246, 5914, 135, 3, 31, "Input"],
Cell[169384, 5919, 147, 3, 31, "Input"],
Cell[169534, 5924, 73, 1, 35, "Input"],
Cell[169610, 5927, 75, 1, 35, "Input"],
Cell[169688, 5930, 101, 2, 31, "Input"],
Cell[169792, 5934, 72, 2, 31, "Input"],
Cell[169867, 5938, 104, 2, 31, "Input"],
Cell[169974, 5942, 77, 1, 35, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[170088, 5948, 39, 0, 31, "Subsubsection"],
Cell[170130, 5950, 70, 2, 31, "Input"],
Cell[170203, 5954, 69, 2, 31, "Input"],
Cell[170275, 5958, 80, 2, 31, "Input"],
Cell[170358, 5962, 114, 2, 31, "Input"],
Cell[170475, 5966, 191, 4, 49, "Input"],
Cell[170669, 5972, 70, 2, 31, "Input"],
Cell[170742, 5976, 81, 2, 31, "Input"],
Cell[170826, 5980, 114, 2, 31, "Input"],
Cell[170943, 5984, 114, 2, 60, "Input"],
Cell[171060, 5988, 74, 2, 31, "Input"],
Cell[171137, 5992, 73, 2, 31, "Input"],
Cell[171213, 5996, 56, 0, 28, "Text"],
Cell[171272, 5998, 129, 3, 31, "Input"],
Cell[171404, 6003, 88, 2, 31, "Input"],
Cell[171495, 6007, 68, 2, 31, "Input"],
Cell[171566, 6011, 102, 2, 31, "Input"],
Cell[171671, 6015, 65, 0, 28, "Text"],
Cell[171739, 6017, 233, 5, 54, "Input"],
Cell[171975, 6024, 71, 2, 31, "Input"],
Cell[172049, 6028, 73, 2, 31, "Input"],
Cell[172125, 6032, 135, 3, 31, "Input"],
Cell[172263, 6037, 147, 3, 31, "Input"],
Cell[172413, 6042, 73, 1, 35, "Input"],
Cell[172489, 6045, 75, 1, 35, "Input"],
Cell[172567, 6048, 101, 2, 31, "Input"],
Cell[172671, 6052, 72, 2, 31, "Input"],
Cell[172746, 6056, 104, 2, 31, "Input"],
Cell[172853, 6060, 77, 1, 35, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[172979, 6067, 53, 0, 38, "Subsection"],
Cell[173035, 6069, 67, 0, 28, "Text"],
Cell[173105, 6071, 100, 2, 31, "Input"],
Cell[173208, 6075, 167, 2, 28, "Text"],
Cell[173378, 6079, 121, 3, 31, "Input"],
Cell[173502, 6084, 140, 3, 31, "Input"],
Cell[173645, 6089, 153, 3, 28, "Text"],
Cell[173801, 6094, 71, 2, 31, "Input"],
Cell[173875, 6098, 88, 2, 28, "Text"],
Cell[173966, 6102, 103, 2, 31, "Input"],
Cell[174072, 6106, 98, 2, 31, "Input"],
Cell[174173, 6110, 69, 2, 31, "Input"],
Cell[174245, 6114, 57, 1, 35, "Input"],
Cell[174305, 6117, 40, 0, 28, "Text"],
Cell[174348, 6119, 201, 4, 54, "Input"],
Cell[174552, 6125, 72, 2, 31, "Input"],
Cell[174627, 6129, 395, 8, 48, "Text"],
Cell[175025, 6139, 80, 2, 31, "Input"],
Cell[175108, 6143, 73, 2, 31, "Input"],
Cell[175184, 6147, 96, 3, 35, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[175317, 6155, 33, 0, 38, "Subsection"],
Cell[175353, 6157, 74, 2, 31, "Input"],
Cell[175430, 6161, 65, 0, 28, "Text"],
Cell[175498, 6163, 98, 2, 31, "Input"],
Cell[175599, 6167, 123, 3, 28, "Text"],
Cell[175725, 6172, 99, 2, 31, "Input"],
Cell[175827, 6176, 30, 0, 28, "Text"],
Cell[175860, 6178, 155, 4, 31, "Input"],
Cell[176018, 6184, 116, 2, 28, "Text"],
Cell[176137, 6188, 121, 3, 31, "Input"],
Cell[176261, 6193, 66, 2, 31, "Input"],
Cell[176330, 6197, 69, 2, 31, "Input"],
Cell[176402, 6201, 174, 4, 54, "Input"],
Cell[176579, 6207, 72, 2, 31, "Input"],
Cell[176654, 6211, 73, 1, 35, "Input"],
Cell[176730, 6214, 75, 1, 35, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[176842, 6220, 83, 1, 38, "Subsection"],
Cell[176928, 6223, 113, 3, 28, "Text"],
Cell[177044, 6228, 98, 2, 31, "Input"],
Cell[177145, 6232, 99, 2, 31, "Input"],
Cell[177247, 6236, 30, 0, 28, "Text"],
Cell[177280, 6238, 155, 4, 31, "Input"],
Cell[177438, 6244, 109, 2, 28, "Text"],
Cell[177550, 6248, 121, 3, 31, "Input"],
Cell[177674, 6253, 66, 2, 31, "Input"],
Cell[177743, 6257, 76, 2, 31, "Input"],
Cell[177822, 6261, 69, 2, 31, "Input"],
Cell[177894, 6265, 192, 3, 28, "Text"],
Cell[178089, 6270, 65, 2, 31, "Input"],
Cell[178157, 6274, 268, 5, 54, "Input"],
Cell[178428, 6281, 45, 0, 28, "Text"],
Cell[178476, 6283, 129, 3, 31, "Input"],
Cell[178608, 6288, 186, 3, 28, "Text"],
Cell[178797, 6293, 91, 2, 31, "Input"],
Cell[178891, 6297, 74, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[179002, 6304, 76, 0, 44, "Subsection"],
Cell[179081, 6306, 85, 2, 28, "Text"],
Cell[179169, 6310, 89, 2, 31, "Input"],
Cell[179261, 6314, 70, 0, 28, "Text"],
Cell[179334, 6316, 81, 2, 31, "Input"],
Cell[179418, 6320, 116, 2, 31, "Input"],
Cell[179537, 6324, 99, 2, 31, "Input"],
Cell[179639, 6328, 212, 3, 28, "Text"],
Cell[179854, 6333, 108, 2, 31, "Input"],
Cell[179965, 6337, 131, 3, 31, "Input"],
Cell[180099, 6342, 66, 2, 31, "Input"],
Cell[180168, 6346, 73, 2, 31, "Input"],
Cell[180244, 6350, 81, 2, 31, "Input"],
Cell[180328, 6354, 69, 2, 31, "Input"],
Cell[180400, 6358, 106, 2, 31, "Input"],
Cell[180509, 6362, 283, 5, 54, "Input"],
Cell[180795, 6369, 106, 3, 28, "Text"],
Cell[180904, 6374, 84, 2, 31, "Input"],
Cell[180991, 6378, 103, 3, 28, "Text"],
Cell[181097, 6383, 269, 5, 54, "Input"],
Cell[181369, 6390, 116, 2, 31, "Input"],
Cell[181488, 6394, 103, 3, 28, "Text"],
Cell[181594, 6399, 87, 2, 31, "Input"],
Cell[181684, 6403, 76, 2, 31, "Input"],
Cell[181763, 6407, 126, 3, 28, "Text"],
Cell[181892, 6412, 117, 2, 31, "Input"],
Cell[182012, 6416, 100, 2, 31, "Input"],
Cell[182115, 6420, 149, 3, 28, "Text"],
Cell[182267, 6425, 94, 2, 31, "Input"],
Cell[182364, 6429, 64, 2, 31, "Input"],
Cell[182431, 6433, 91, 2, 31, "Input"],
Cell[182525, 6437, 91, 2, 31, "Input"],
Cell[182619, 6441, 129, 3, 31, "Input"],
Cell[182751, 6446, 66, 2, 31, "Input"],
Cell[182820, 6450, 46, 0, 28, "Text"],
Cell[182869, 6452, 100, 2, 31, "Input"],
Cell[182972, 6456, 76, 2, 31, "Input"],
Cell[183051, 6460, 55, 0, 28, "Text"],
Cell[183109, 6462, 139, 4, 31, "Input"],
Cell[183251, 6468, 113, 2, 31, "Input"],
Cell[183367, 6472, 86, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[183502, 6480, 178, 4, 60, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[183705, 6488, 38, 0, 44, "Subsection"],
Cell[183746, 6490, 855, 12, 128, "Text"],
Cell[184604, 6504, 1333, 23, 168, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[185974, 6532, 67, 1, 38, "Subsection",
  Evaluatable->False],
Cell[186044, 6535, 301, 5, 48, "Text"],
Cell[186348, 6542, 163, 3, 54, "Input"],
Cell[186514, 6547, 91, 2, 31, "Input"],
Cell[186608, 6551, 76, 2, 31, "Input"],
Cell[186687, 6555, 86, 2, 31, "Input"],
Cell[186776, 6559, 64, 2, 31, "Input"],
Cell[186843, 6563, 118, 3, 28, "Text"],
Cell[186964, 6568, 114, 2, 31, "Input"],
Cell[187081, 6572, 136, 3, 31, "Input"],
Cell[187220, 6577, 72, 2, 31, "Input"],
Cell[187295, 6581, 137, 3, 28, "Text"],
Cell[187435, 6586, 94, 2, 31, "Input"],
Cell[187532, 6590, 196, 6, 28, "Text"],

Cell[CellGroupData[{
Cell[187753, 6600, 69, 0, 37, "Subsubsection"],
Cell[187825, 6602, 111, 2, 31, "Input"],
Cell[187939, 6606, 69, 2, 31, "Input"],
Cell[188011, 6610, 69, 2, 31, "Input"],
Cell[188083, 6614, 153, 3, 28, "Text"],
Cell[188239, 6619, 196, 4, 46, "Input"],
Cell[188438, 6625, 109, 2, 31, "Input"],
Cell[188550, 6629, 109, 2, 31, "Input"],
Cell[188662, 6633, 109, 2, 31, "Input"],
Cell[188774, 6637, 78, 2, 31, "Input"],
Cell[188855, 6641, 78, 2, 31, "Input"],
Cell[188936, 6645, 154, 3, 28, "Text"],
Cell[189093, 6650, 67, 2, 31, "Input"],
Cell[189163, 6654, 60, 0, 28, "Text"],
Cell[189226, 6656, 80, 2, 31, "Input"],
Cell[189309, 6660, 69, 2, 31, "Input"],
Cell[189381, 6664, 97, 2, 28, "Text"],
Cell[189481, 6668, 130, 3, 32, "Input"],
Cell[189614, 6673, 111, 2, 32, "Input"],
Cell[189728, 6677, 106, 2, 48, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[189871, 6684, 63, 0, 37, "Subsubsection"],
Cell[189937, 6686, 164, 3, 54, "Input"],
Cell[190104, 6691, 73, 2, 31, "Input"],
Cell[190180, 6695, 93, 2, 31, "Input"],
Cell[190276, 6699, 77, 2, 31, "Input"],
Cell[190356, 6703, 86, 2, 31, "Input"],
Cell[190445, 6707, 65, 2, 31, "Input"],
Cell[190513, 6711, 116, 2, 31, "Input"],
Cell[190632, 6715, 137, 3, 31, "Input"],
Cell[190772, 6720, 71, 2, 31, "Input"],
Cell[190846, 6724, 75, 2, 31, "Input"],
Cell[190924, 6728, 76, 2, 31, "Input"],
Cell[191003, 6732, 134, 3, 31, "Input"],
Cell[191140, 6737, 73, 2, 31, "Input"],
Cell[191216, 6741, 97, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[191362, 6749, 66, 1, 38, "Subsection",
  Evaluatable->False],
Cell[191431, 6752, 271, 5, 48, "Text"],
Cell[191705, 6759, 168, 3, 54, "Input"],
Cell[191876, 6764, 85, 2, 31, "Input"],
Cell[191964, 6768, 78, 2, 31, "Input"],
Cell[192045, 6772, 88, 2, 31, "Input"],
Cell[192136, 6776, 64, 2, 31, "Input"],
Cell[192203, 6780, 79, 2, 31, "Input"],
Cell[192285, 6784, 108, 2, 31, "Input"],
Cell[192396, 6788, 72, 2, 31, "Input"],
Cell[192471, 6792, 95, 2, 31, "Input"],
Cell[192569, 6796, 145, 3, 28, "Text"],
Cell[192717, 6801, 114, 2, 31, "Input"],
Cell[192834, 6805, 173, 4, 54, "Input"],
Cell[193010, 6811, 69, 2, 31, "Input"],
Cell[193082, 6815, 235, 6, 48, "Text"],
Cell[193320, 6823, 75, 2, 31, "Input"],
Cell[193398, 6827, 136, 3, 28, "Text"],
Cell[193537, 6832, 98, 2, 31, "Input"],
Cell[193638, 6836, 76, 2, 31, "Input"],
Cell[193717, 6840, 141, 3, 31, "Input"],
Cell[193861, 6845, 173, 4, 54, "Input"],

Cell[CellGroupData[{
Cell[194059, 6853, 67, 0, 37, "Subsubsection"],
Cell[194129, 6855, 116, 2, 31, "Input"],
Cell[194248, 6859, 82, 2, 31, "Input"],
Cell[194333, 6863, 116, 3, 28, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[194498, 6872, 67, 1, 38, "Subsection",
  Evaluatable->False],
Cell[194568, 6875, 275, 5, 48, "Text"],
Cell[194846, 6882, 165, 3, 54, "Input"],
Cell[195014, 6887, 71, 2, 31, "Input"],
Cell[195088, 6891, 92, 2, 31, "Input"],
Cell[195183, 6895, 71, 2, 31, "Input"],
Cell[195257, 6899, 83, 2, 31, "Input"],
Cell[195343, 6903, 155, 4, 31, "Input"],
Cell[195501, 6909, 87, 2, 31, "Input"],
Cell[195591, 6913, 87, 2, 31, "Input"],
Cell[195681, 6917, 64, 2, 31, "Input"],
Cell[195748, 6921, 64, 2, 31, "Input"],
Cell[195815, 6925, 92, 2, 31, "Input"],
Cell[195910, 6929, 120, 3, 31, "Input"],
Cell[196033, 6934, 130, 3, 31, "Input"],
Cell[196166, 6939, 129, 3, 31, "Input"],
Cell[196298, 6944, 73, 2, 31, "Input"],
Cell[196374, 6948, 113, 2, 31, "Input"],
Cell[196490, 6952, 82, 2, 31, "Input"],
Cell[196575, 6956, 65, 2, 31, "Input"],
Cell[196643, 6960, 71, 2, 31, "Input"],
Cell[196717, 6964, 93, 2, 31, "Input"],
Cell[196813, 6968, 160, 3, 54, "Input"],
Cell[196976, 6973, 181, 4, 48, "Text"],
Cell[197160, 6979, 215, 4, 54, "Input"],
Cell[197378, 6985, 31, 0, 28, "Text"],
Cell[197412, 6987, 82, 2, 31, "Input"],
Cell[197497, 6991, 70, 2, 31, "Input"],
Cell[197570, 6995, 124, 3, 28, "Text"],
Cell[197697, 7000, 96, 2, 31, "Input"],
Cell[197796, 7004, 73, 0, 28, "Text"],
Cell[197872, 7006, 157, 3, 54, "Input"],
Cell[198032, 7011, 69, 2, 31, "Input"],

Cell[CellGroupData[{
Cell[198126, 7017, 68, 0, 37, "Subsubsection"],
Cell[198197, 7019, 116, 2, 31, "Input"],
Cell[198316, 7023, 82, 2, 31, "Input"],
Cell[198401, 7027, 150, 3, 28, "Text"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[198612, 7037, 42, 0, 60, "Section"],
Cell[198657, 7039, 259, 6, 48, "Text"],

Cell[CellGroupData[{
Cell[198941, 7049, 38, 0, 44, "Subsection"],
Cell[198982, 7051, 277, 6, 48, "Text",
  Evaluatable->False],
Cell[199262, 7059, 170, 4, 28, "Text",
  Evaluatable->False],
Cell[199435, 7065, 93, 2, 31, "Input"],
Cell[199531, 7069, 75, 2, 31, "Input"],
Cell[199609, 7073, 132, 4, 28, "Text",
  Evaluatable->False],
Cell[199744, 7079, 96, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[199877, 7086, 34, 0, 38, "Subsection"],
Cell[199914, 7088, 771, 17, 120, "Text"],
Cell[200688, 7107, 91, 2, 31, "Input"],
Cell[200782, 7111, 52, 0, 28, "Text"],
Cell[200837, 7113, 50, 1, 28, "Text"],
Cell[200890, 7116, 141, 3, 28, "Text"],
Cell[201034, 7121, 482, 9, 146, "Input"],
Cell[201519, 7132, 159, 3, 54, "Input"],
Cell[201681, 7137, 161, 3, 54, "Input"],
Cell[201845, 7142, 109, 2, 31, "Input"],
Cell[201957, 7146, 104, 2, 31, "Input"],
Cell[202064, 7150, 76, 2, 31, "Input"],
Cell[202143, 7154, 75, 2, 31, "Input"],
Cell[202221, 7158, 67, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

