(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     98920,       3542]*)
(*NotebookOutlinePosition[    100266,       3583]*)
(*  CellTagsIndexPosition[    100222,       3579]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["M\[ODoubleDot]bius Geometry of Spheres", "Title",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center],

Cell["by", "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontFamily->"Times New Roman",
  FontSize->14,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell["Rolf Sulanke", "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontSize->24],

Cell["March 1, 1999, last revision November 3, 2000", "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontSlant->"Italic"],

Cell[CellGroupData[{

Cell["Summary", "Subsubsection"],

Cell["\<\
This notebook treates 2-spheres within the 3-sphere,or the euclidean 3-space, \
as objects of M\[ODoubleDot]bius Geometry. First we introduce some basic \
objects of n-dimensional M\[ODoubleDot]bius geometry, which is the conformal \
geometry of the n-sphere. We emphasize the case n=3, in which \
M\[ODoubleDot]bius geometry can be visualized in the Euclidean 3-space by \
stereographical projection from the north pole. For this purpose we construct \
a version of the stereographical projection,and its inversion,which relates \
isotropic vectors of the pseudo-euclidean 5-space and points of the \
3-space.The bijectivity between subspheres of the 3-sphere and \
one-dimensional euclidean subspaces of the 5-dimensional pseudo-euclidean \
vector space is established. Functions describing this bijective relation are \
constructed. Finally we find and visualize the geodesics of the sphere space.
The concepts developed in this notebook are collected in the package \
mspher.m.
This notebook continues the notebook eusphere.nb, which contains the metric \
geometry of spheres. It uses tools of pseudo-euclidean linear algebra, \
developed in the notebook pseuklid.nb.
For the concepts of M\[ODoubleDot]bius geometry applied here see the preprint \
[3] and my homepage [4].\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["References", "Subsubsection"],

Cell["\<\
[1] Coxeter, H. S. M.  Inversive distance. Ann. Mat. Pura Apli. 4, 71 (1966), \
73 - 83. 
[2] Kuska, Jens-Peer. http://phong.informatik.uni-leipzig.de/ \
~kuska/mview3d.html.\
\>", "Text"],

Cell[TextData[{
  "[3] Sulanke,  R.  M\[ODoubleDot]bius invariants for pairs of spheres (",
  Cell[BoxData[
      \(TraditionalForm\`S\_1\%m\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`S\_2\%l\)]],
  ") in the M\[ODoubleDot]bius space ",
  Cell[BoxData[
      \(TraditionalForm\`S\^n\)]],
  ", Beitr\[ADoubleDot]ge zur Algebra und Geometrie, 41 (2000), No.1, \
233-246. Preprint No. 373, SFB 288, Berlin 1999.  \
http://xxx.lanl.gov/abs/math.MG/9902131. "
}], "Text",
  Evaluatable->False],

Cell["\<\
[4] Sulanke,  R.  Homepage:  http://www-irm.mathematik.hu-berlin.de/~sulanke/\
\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Keywords", "Subsubsection"],

Cell["\<\
Generalized angles between spheres. Conformal invariant. Inversive distance.  \
Spheres defined by spacelike vectors. Random spacelike unit vectors. Random \
spheres. Spacelike vectors corresponding to spheres and planes.  Spheres \
through four points. Stereographical projection of isotropic vectors.  Space \
of all spheres. Geodesics in the sphere space; spacelike, timelike, and \
isotropic geodesics. \
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Address", "Subsubsection"],

Cell["sulanke@mathematik.hu-berlin.de", "Text"],

Cell["\<\
The author will be very thankful for hints, corrections, comments.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[CellGroupData[{

Cell["The needed packages", "Subsection"],

Cell["\<\
If you want tu use all the notebooks of the item \"Spheres\", ensure that the \
packages
euvec.m
pseuvec.m
mspher.m
mcirc.m
spiralsf.m
and the master package init.m are in your working directory. These packages \
are contained in the item \"Spheres\".
 For the present notebook we only need the packages euvec.m, pseuvec.m, \
mspher.m. \
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["3D graphics, animations", "Subsection"],

Cell[TextData[{
  "In the notebook command lines using ThreeScript, or MVShow3D appear. They \
may used for 3D-representations and animations of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Graphics3D objects. To apply them you must have the necessary software on \
your computer. Look in the last section. "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["For Windows", "Subsection"],

Cell[TextData[{
  "For the commands in this subsection:\n",
  StyleBox["DO NOT CONFIRM THE EVALUATION OF THE INITIALIZATION CELLS !!!",
    FontWeight->"Bold",
    FontColor->RGBColor[1, 0, 0]]
}], "Text"],

Cell[" If the result of the following command ", "Text"],

Cell[BoxData[
    \(Directory[]\)], "Input",
  CellLabel->"In[4]:="],

Cell["\<\
does not coincide with the working directory, e.g. \"C:\\Eigene \
Dateien\\mymath\\spheres.v4\", containing the needed packages, insert the \
Path of your working directory in the next command line and activate it \
first, without calling the initialization:\
\>", "Text"],

Cell[BoxData[
    \(SetDirectory["\<C:\\Eigene Dateien\\mymath\\spheres\>"]\)], "Input",
  CellLabel->"In[34]:="],

Cell["\<\
If you want to use the package euvec.mpseuvec.m,mspher.m only,  carry out the \
next command.\
\>", "Text"],

Cell[BoxData[
    \(Needs["\<mspher`\>"]\)], "Input",
  CellLabel->"In[1]:="],

Cell["\<\
If you want  that all commands contained in the packages of the item \
\"Spheres\" are available, activate\
\>", "Text"],

Cell[BoxData[
    \(Get["\<init.m\>"]\)], "Input",
  CellLabel->"In[2]:="],

Cell[TextData[StyleBox["Now: Activite Kernel/Evaluation/Evaluate \
Initialization from the menu!",
  FontSize->18,
  FontColor->RGBColor[1, 0, 0]]], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["For Linux (and other  systems: not tested!)", "Subsection"],

Cell[TextData[{
  "For the next command:\n",
  StyleBox["DO NOT CONFIRM THE EVALUATION OF THE INITIALIZATION CELLS !!!",
    FontWeight->"Bold",
    FontColor->RGBColor[1, 0, 0]]
}], "Text"],

Cell["\<\
If you want to use the package euvec.mpseuvec.m,mspher.m only, remove the \
package init.m from the working directory and carry out the next command.\
\>", "Text"],

Cell[BoxData[
    \(Needs["\<mspher`\>"]\)], "Input",
  CellLabel->"In[1]:="],

Cell[TextData[StyleBox["Now: Activite Kernel/Evaluation/Evaluate \
Initialization from the menu!",
  FontSize->18,
  FontColor->RGBColor[1, 0, 0]]], "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1  Introduction", "Section"],

Cell[CellGroupData[{

Cell["1.1 Pseudo-euclidean vector algebra and other prerequisites", 
  "Subsection",
  FontFamily->"Times New Roman"],

Cell["\<\
We consider the n-dimensional sphere, shortly n-sphere, as the standard \
hyperellipsoid in the (n+1)-dimensional projective space. In homogeneous \
coordinates x \[RightTeeArrow] (x[1], ..., x[n+2])  its equation is\
\>", "Text"],

Cell[BoxData[
    \(\[Sum]\+\(i = 1\)\%\(n + 1\)\((x[i])\)\^2 - \((x[n + 2])\)\^2 = 
      0\)], "Text",
  CellLabel->"In[5]:=",
  TextAlignment->Center],

Cell["\<\
By Liouville's theorem (and some elementary consideration about functions of \
one complexe variable in the case n=2) the conformal group of the n-sphere \
(n>1) coincides with the group of  all projective transformations preserving \
this hyperellipsoid. Elementary linear algebra can be used to show, that this \
group can be represented as the pseudo-orthogonal group  (factored by \
\[MinusPlus] identity) of the (n+2)-dimensional real vector space underlying \
the (n+1)-dimensional projective space. Therefore M\[ODoubleDot]bius geometry \
is treated by tools of pseudo-euclidean linear algebra. The basic concepts of \
pseudo-euclidean linear algebra are developed in the notebook pseuklid.nb and \
contained in the package pseuvec.m. They are introduced automatically as part \
of the master package init.m of this directory.  Before importing and using \
them one has to fix the dimension dim of the underlying vector space and the \
index  ind of the (non-degenerate, symmetric) scalar product. These basic \
constants are now defined by\
\>", "Text"],

Cell[BoxData[
    \(dim\  = \ 5; \ ind\  = \ 1;\)], "Input",
  CellLabel->"In[3]:=",
  InitializationCell->True],

Cell["\<\
because in this notebook we are mainly interested in the 3-dimensional \
sphere. You can change these constants only before starting the evaluation of \
the initialization cells  by changing the last command line. But remember \
that these constants enter in almost all of the treated concepts; therefore \
they should be defined before using the packages. \
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"],

Cell["\<\
The basic invariant of pseudo-eulidean linear algebra is the scalar \
product:\
\>", "Text"],

Cell[BoxData[
    \(\(?pssp\)\)], "Input",
  CellLabel->"In[9]:="],

Cell["\<\
The matrix of the scalar products in any orthonormed basis is\
\>", "Text"],

Cell[BoxData[
    \(MatrixForm[Table[pssp[stb[i], \ stb[j]], {i, 5}, {j, 5}]]\)], "Input",
  CellLabel->"In[4]:=",
  InitializationCell->True],

Cell[TextData[
"Terminology: Spheres are usual 2-dimenional spheres if we consider the \
3-dimensional spherical geometry; more generally one may consider m-spheres \
in the n-sphere, where m, n with 0\[LessEqual]m\[LessEqual]n denote the \
dimensions. For  m = n-1 the (n-1)-spheres are called  hyperspheres.  "], 
  "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"],

Cell["\<\
In 1.2 you can find the new symbols and evoke the corresponding usages. \
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["1.2 The usages", "Subsection"],

Cell["\<\
After a package has been loaded you can get information about the  listed \
concepts in the usual way, e.g.\
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(\(?orthonorm\)\)], "Input",
  CellLabel->"In[10]:="],

Cell["\<\
For getting the usage and the complete definition, enter e.g.\
\>", "Text"],

Cell[BoxData[
    \(?? spt\)], "Input",
  CellLabel->"In[11]:="],

Cell["\<\
Since the names are prepended by the ContextNames the definition obtained in \
this way are hard to read. Therefore you will find some of them in the \
original form in unevaluable cells.  It is recommended to look in the \
packages.\
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"],

Cell["\<\
Here you can get information about the commands which are defined in \
mspher.m. For the functions and mofules defined in the packages euvec.m, \
pseuvec.m  see the notebooks euspher.nb, pseuklid.nb.\
\>", "Text"],

Cell[BoxData[
    \(\(?spt\)\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(\(?stepro\)\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(\(?invstepro\)\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \(\(?confinv\)\)], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[
    \(\(?gencos\)\)], "Input",
  CellLabel->"In[16]:="],

Cell[BoxData[
    \(\(?plotsphere\)\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(\(?euklidsphere\)\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(\(?euklidsphereplot3D\)\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(\(?spunit\)\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(\(?pstg5frame\)\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(\(?vradius\)\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(\(?vcenter\)\)], "Input",
  CellLabel->"In[23]:="],

Cell[BoxData[
    \(\(?spacevec\)\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(\(?sphvec\)\)], "Input",
  CellLabel->"In[25]:="],

Cell[BoxData[
    \(\(?planev\)\)], "Input",
  CellLabel->"In[26]:="]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2  Points: Adaption of the stereographical projection", "Section"],

Cell[CellGroupData[{

Cell["2.1 Stereographical projection of isotropic vectors", "Subsection"],

Cell["\<\
In the projective context, the points of the 3-sphere are the generating \
lines of the isotropic cone\
\>", "Text"],

Cell[TextData[{
  "pssp[x,x]  = ",
  Cell[BoxData[
      \(x[1]\^2 + x[2]\^2 + x[3]\^2 + x[4]\^2 - x[5]\^2\)]],
  "= 0,"
}], "Text",
  TextAlignment->Center],

Cell["\<\
analogously in higher dimensions. Our aim is to describe the point of the \
euclidean 3-space obtained by stereographical projection sterproj (defined in \
pseuvec.m, see also eusphere.nb) of the 3-sphere onto its equatorial \
hyperplane as a function of the isotropic vector x. There exists a natural \
bijection between the  lines intersecting the hyperplane \
\>", "Text"],

Cell["- pssp[stb[5], x] = x[5]  = 1 \t\t\t\t\t\t(H)", "Text",
  TextAlignment->0.5],

Cell["\<\
of the 5-space and the points of the euclidean 4-space  defined by\
\>", "Text"],

Cell[BoxData[
    \(\(?spt\)\)], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \(spt[v_List]\  := \ 
      Module[{w, u, d = Length[v]}, 
        w\  = v[\([d]\)]; \n\t\t\tIf[
          w === 0, \ \ Return[
            Print[v\ , "\< has last component \>", w]], \n\t\t\t\tu = 
            Take[v/w, d - 1]; Return[u]]]\)], "Input",
  CellLabel->"In[20]:=",
  Evaluatable->False],

Cell["\<\
Now remember that the  3-sphere appears as  intersection of the hyperplane \
(H) with the isotropic cone. If we restrict spt onto the isotropic cone and \
combine it with the general stereographical projection\
\>", "Text"],

Cell[BoxData[
    \(\(?sterproj\)\)], "Input",
  CellLabel->"In[43]:="],

Cell["we obtain the wanted map", "Text"],

Cell[BoxData[
    \(\(?stepro\)\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(stepro[v_List]\  := \ sterproj[spt[v]]\)], "Input",
  Evaluatable->False],

Cell["Its inverse is generated by the vector", "Text"],

Cell[BoxData[
    \(invstepro[v_List]\  := \ Append[invsterproj[v], 1]\)], "Input",
  Evaluatable->False],

Cell[BoxData[
    \(\(?invstepro\)\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(\(?invsterproj\)\)], "Input",
  CellLabel->"In[29]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["2.2 Examples", "Subsection"],

Cell[BoxData[
    \(spt[{2, 2, 2, 2, 2, 2, 2, 2, 4}]\)], "Input",
  CellLabel->"In[30]:="],

Cell["vec[x] ", "Input",
  CellLabel->"In[5]:=",
  InitializationCell->True],

Cell["spt[vec[x]]", "Input",
  CellLabel->"In[32]:="],

Cell["spt[stb[1]+stb[2]]", "Input",
  CellLabel->"In[33]:="],

Cell["spt[stb[5]]", "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(spt[stb[5] + stb[2] Cos[t] + stb[3] Sin[t]]\)], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(stepro[stb[5] + stb[2] Cos[t] + stb[3] Sin[t]]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(invstepro[%]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[38]:="],

Cell["\<\
If one applies stepro on non-isotropic vectors one obtains incorrect results \
without any warning! If there are any doubts one should control the character \
ch of the arguments! Here is such an example:\
\>", "Text"],

Cell[BoxData[
    \(vf[s_, t_]\  := 
      stb[1] + \((stb[2] Cos[t] + stb[3] Sin[t])\) Cos[s]\  + 
        stb[5] Sin[s]\)], "Input",
  CellLabel->"In[53]:="],

Cell[BoxData[
    \(vf[s, t]\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \(ch[vf[s, t]]\)], "Input",
  CellLabel->"In[55]:="],

Cell["since ch[vf[s,t]]depends on s,t!", "Text"],

Cell[BoxData[
    \(pssp[vf[s, t], vf[s, t]]\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(fct[s_]\  = \ Simplify[%]\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(Plot[fct[s], \ {s, 0, \ Pi}]\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(spt[vf[s, t]]\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(stepro[vf[s, t]]\)], "Input",
  CellLabel->"In[45]:="],

Cell["\<\
Thus, we obtained a result for all values of t, but it corresponds to  a \
point of a the sphere only for solutions of Cos[s] == 0.\
\>", "Text"],

Cell[BoxData[
    \(vf[Pi/2, t]\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(sterproj[spt[vf[Pi/2, \ t]]]\)], "Input",
  CellLabel->"In[47]:="],

Cell[BoxData[
    \(invstepro[%]\)], "Input",
  CellLabel->"In[48]:="],

Cell["spt[nul[]]", "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(invstepro[nul[3]]\)], "Input",
  CellLabel->"In[50]:="],

Cell[BoxData[
    \(Remove[vf, \ fct]\)], "Input",
  CellLabel->"In[56]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["2.3 A formula for the scalar products", "Subsection"],

Cell["\<\
As an example for symbolic calculation we derive a formula, which expresses \
the pseudo-euclidean scalar products of the images by investepro of two \
points in terms of the euclidean scalar products of their radius vectors. The \
 points are\
\>", "Text"],

Cell[BoxData[
    \(vx\  = vec[x, 3]; \ vy\  = \ vec[y, 3];\)], "Input",
  CellLabel->"In[57]:="],

Cell[BoxData[
    \({vx, vy} // MatrixForm\)], "Input",
  CellLabel->"In[58]:="],

Cell["The preimages are the rows of the matrix", "Text"],

Cell[BoxData[
    \({invstepro[vx], \ invstepro[vy]}\)], "Input",
  CellLabel->"In[61]:="],

Cell["The preimages  are isotropic:", "Text"],

Cell[BoxData[
    \(Simplify[Together[pssp[%[\([1]\)], %[\([1]\)]]]]\)], "Input",
  CellLabel->"In[62]:="],

Cell["Their pseudo-euclidean scalar product is", "Text"],

Cell[BoxData[
    \(pssp[invstepro[vx], \ invstepro[vy]]\)], "Input",
  CellLabel->"In[63]:="],

Cell[BoxData[
    \(Simplify[Together[%]]\)], "Input",
  CellLabel->"In[64]:="],

Cell[BoxData[
    \(Simplify[% /. {x[1]\^2 + x[2]\^2 + x[3]\^2 \[Rule] \ nvx^2, 
          y[1]\^2 + y[2]\^2 + y[3]\^2 \[Rule] nvy^2}]\)], "Input",
  CellLabel->"In[65]:="],

Cell["Therefore, the following formula is true:", "Text"],

Cell[BoxData[
    \(Simplify[
      pssp[invstepro[vx], \ 
          invstepro[
            vy]] \[Equal] \ \(-2\)*\((vx - vy)\) . \((vx - 
                  vy)\)/\((\((1 + vx . vx)\)*\((1 + vy . vy)\))\)]\)], "Input",\

  CellLabel->"In[69]:="],

Cell["\<\
Thus we obtained the wanted formula, which symbolically can be written as\
\>", "Text"],

Cell[BoxData[
    \(\(\(pssp[invstepro[x], invstepro[y]]\)\(\[IndentingNewLine]\)\(==\)\) | 
      x - y | \(\(^\)\(2\)\)/\((\((\(\(1\)\(+\)\) | 
                x | \(\(^\)\(2\)\))\)*\((\(\(1\)\(+\)\) | 
                y | \(\(^\)\(2\)\))\))\)\)], "Input",
  CellLabel->"In[73]:=",
  Evaluatable->False],

Cell[TextData[{
  StyleBox["Remark.",
    FontColor->RGBColor[1, 0, 0]],
  "  The isotropic vector "
}], "Text"],

Cell[BoxData[
    \(invstepro[{x1, x2, x3}]\)], "Input",
  CellLabel->"In[70]:="],

Cell["is a special representant of the corresponding point", "Text"],

Cell[BoxData[
    \(stepro[invstepro[{x1, x2, x3}]]\)], "Input",
  CellLabel->"In[71]:="],

Cell["defined by the condition", "Text"],

Cell[BoxData[
    \(pssp[invstepro[{x1, x2, x3}], \ stb[5]] \[Equal] \ \(-1\)\)], "Input",
  CellLabel->"In[72]:="],

Cell["\<\
Thus, the calculated scalar product does not have any invariant meaning in M\
\[ODoubleDot]bius geometry. In M\[ODoubleDot]bius geometry all pairs of \
(distinct) points are equivalent.\
\>", "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3 Spheres", "Section"],

Cell["\<\
In this section we shall establish the basic bijection between spheres \
(including planes as spheres of infinite radius) of the euclidean 3-space and \
one-dimensional euclidean subspaces of the 5-dimensional pseudo-euclidean \
vector space, see also subsection 3.5.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
3.1 Spheres through four points and corresponding spacelike vectors: vradius, \
vcenter, sphereplot3D\
\>", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
The definitions of the spacelike vector functions sphvec, spacevec\
\>", "Subsubsection"],

Cell["\<\
The following Module yields an unnormed spacelike vector which defines the \
sphere through four points in general position (i.e. do not belong to a \
circle or a line):\
\>", "Text"],

Cell[BoxData[
    \(\(?sphvec\)\)], "Input",
  CellLabel->"In[129]:="],

Cell[BoxData[
    \(\(\(sphvec[p1_List, p2_List, p3_List, 
        p4_List]\)\(:=\)\(Module[{v = nul[5]}, 
        If[Length[p1] \[NotEqual] 3 || Length[p2] \[NotEqual] 3 || 
            Length[p3] \[NotEqual] 3 || Length[p4] \[NotEqual] 3, 
          Print["\<An argument does not have the correct
dimension!\>"] Return[]]; \ \tv = 
          Simplify[
            pscross[{invstepro[p1], invstepro[p2], invstepro[p3], 
                invstepro[p4]}]]; \[IndentingNewLine]If[v === nul[5], 
          Print["\<The points are not in general
position!\>"]; Return[], Return[v], Return[v]]]\)\(\t\)\)\)], "Input",
  CellLabel->"In[14]:=",
  Evaluatable->False],

Cell["\<\
Applying this procedure to four points lying on the sphere with center{x,y,z} \
and radius r, we obtain after some simplifications (see below) the formula\
\>", "Text"],

Cell[BoxData[
    \(\(?spacevec\)\)], "Input",
  CellLabel->"In[74]:="],

Cell[BoxData[
    \(vn\  = spacevec[x, y, z, r]\)], "Input",
  CellLabel->"In[75]:="],

Cell[BoxData[
    \(pssp[vn, vn]\)], "Input",
  CellLabel->"In[76]:="],

Cell[BoxData[
    \(Simplify[Together[%]]\)], "Input",
  CellLabel->"In[77]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples, applications", "Subsubsection"],

Cell["\<\
We use euklidsphereplot3D to plot the corresponding sphere or plane. For a \
detailed discussion see subsection 3.3. 
Here we get the standard unit sphere:\
\>", "Text"],

Cell[BoxData[
    \(sphvec[stb[1, 3], \ stb[2, 3], \ 
      stb[3, 3], \ \(-stb[3, 3]\)]\)], "Input",
  CellLabel->"In[84]:="],

Cell[BoxData[
    \(euklidsphereplot3D[%]\)], "Input",
  CellLabel->"In[85]:="],

Cell["\<\
In M\[ODoubleDot]bius geometry, planes appear as stereographical projections \
of spheres through the pole; sphvec includes this case:\
\>", "Text"],

Cell[BoxData[
    \(sphvec[nul[3], \ stb[2, 3], \ \(-stb[2, 3]\), \ stb[3, 3]]\)], "Input",
  CellLabel->"In[82]:="],

Cell[BoxData[
    \(euklidsphereplot3D[%]\)], "Input",
  CellLabel->"In[83]:="],

Cell[BoxData[
    \(sphvec[stb[1, 3], \ \(-stb[1, 3]\), \ 
      stb[2, 3], \ \(-2\) stb[2, 3]]\)], "Input",
  CellLabel->"In[86]:="],

Cell[BoxData[
    \(euklidsphereplot3D[%]\)], "Input",
  CellLabel->"In[87]:="],

Cell["\<\
If the four points ly on a circle, one gets a corresponding message:\
\>", "Text"],

Cell[BoxData[
    \(sphvec[stb[1, 3], \ \(-stb[1, 3]\), \ 
      stb[2, 3], \ \(-stb[2, 3]\)]\)], "Input",
  CellLabel->"In[88]:="],

Cell[BoxData[
    \({invstepro[stb[1, 3]], \ invstepro[\(-stb[1, 3]\)], 
      invstepro[\ stb[2, 3]], \ invstepro[\(-stb[2, 3]\)]}\)], "Input",
  CellLabel->"In[89]:="],

Cell[BoxData[
    \(pscross[%]\)], "Input",
  CellLabel->"In[90]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
The spacelike vector  as a funtion of center and radius of the sphere: \
spacvec\
\>", "Subsubsection"],

Cell["\<\
We demonstrate the developing of the procedure spacevec mentioned above. This \
is the spacelike vector corresponding to the sphere with center {x,y,z}and \
radius r:\
\>", "Text"],

Cell[BoxData[
    \(sphvec[{x, y, z} + r*stb[1, 3], {x, y, z} - 
        r*stb[1, 3], \n\t\t{x, y, z} + r*stb[2, 3], \ {x, y, z} + 
        r*stb[3, 3]]\)], "Input",
  CellLabel->"In[112]:="],

Cell[BoxData[
    \(%*\((1 + r\^2 - 2\ r\ x + x\^2 + y\^2 + z\^2)\)\ \((1 + r\^2 + 
          2\ r\ x + x\^2 + y\^2 + z\^2)\)\ \((1 + r\^2 + x\^2 + 2\ r\ y + 
          y\^2 + z\^2)\)\ \((1 + r\^2 + x\^2 + y\^2 + 2\ r\ z + 
          z\^2)\)\)], "Input",
  CellLabel->"In[113]:="],

Cell[BoxData[
    \(%/\((16  r^3)\)\)], "Input",
  CellLabel->"In[114]:="],

Cell["\<\
Since the norming is unessential, we  defined in the package mspher.m\
\>", "Text"],

Cell[BoxData[
    \(spacevec[x_, y_, z_, 
        r_]\  := \ {2\ x, 2\ y, 2\ z, \(-1\) - r\^2 + x\^2 + y\^2 + z\^2, \ 
          1 - r\^2 + x\^2 + y\^2 + z\^2}/\((2*r)\)\)], "Input",
  CellLabel->"In[15]:=",
  Evaluatable->False],

Cell[BoxData[
    \(Simplify[
      pssp[{2\ x, 2\ y, 2\ z, \(-1\) - r\^2 + x\^2 + y\^2 + z\^2, 
          1 - r\^2 + x\^2 + y\^2 + z\^2}, \ \n\t\t{2\ x, 2\ y, 
          2\ z, \(-1\) - r\^2 + x\^2 + y\^2 + z\^2, 
          1 - r\^2 + x\^2 + y\^2 + z\^2}]]\)], "Input",
  CellLabel->"In[115]:="],

Cell[BoxData[
    \(spacevec[x, y, z, r]\)], "Input",
  CellLabel->"In[116]:="],

Cell[BoxData[
    \(Simplify[pssp[spacevec[x, y, z, r], \ spacevec[x, y, z, r]]]\)], "Input",\

  CellLabel->"In[117]:="],

Cell[BoxData[
    \(euklidsphereplot3D[spacevec[0, 0, 10, 2]]\)], "Input",
  CellLabel->"In[118]:="],

Cell["Here one gets error messages:", "Text"],

Cell[BoxData[
    \(spacevec[0, 0, 1, 0]\)], "Input",
  CellLabel->"In[119]:="],

Cell["\<\
The following yields a spacelike unit vector of the sphere with radius r and \
center {x,0,0}:\
\>", "Text"],

Cell[BoxData[
    \(spherevec[x_, r_] := \ spacevec[x, 0, 0, r]\)], "Input",
  CellLabel->"In[6]:=",
  InitializationCell->True],

Cell[BoxData[
    \(spherevec[x, \ r]\)], "Input",
  CellLabel->"In[121]:="],

Cell[BoxData[
    \(\(euklidsphere[spherevec[x, r]]\)[u, v]\)], "Input",
  CellLabel->"In[122]:="],

Cell[BoxData[
    \(% /. {x\  \[Rule] \ 3, r \[Rule] \ 2}\)], "Input",
  CellLabel->"In[123]:="],

Cell[BoxData[
    \(ParametricPlot3D[%, \ {u, \ \(-1\), \ 1}, {v, \ \(-1\), \ 
        1}]\)], "Input",
  CellLabel->"In[124]:="],

Cell[BoxData[
    \(spacevec[0, 0, 0, 1]\)], "Input",
  CellLabel->"In[125]:="],

Cell[BoxData[
    \(spacevec[0, 0, 0, \(-r\)]\)], "Input",
  CellLabel->"In[126]:="],

Cell[BoxData[
    \(pssp[%, \ %]\)], "Input",
  CellLabel->"In[127]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[128]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Inversion of spacevec, and an euclidean definition of the sphere \
corresponding to a spacelike vector\
\>", "Subsubsection"],

Cell["From the formula", "Text"],

Cell[BoxData[
    \(vn\  = spacevec[x, y, z, r]\)], "Input",
  CellLabel->"In[130]:="],

Cell["we conclude", "Text"],

Cell[BoxData[
    \(Simplify[Together[vn[\([5]\)] - \ vn[\([4]\)]]]\)], "Input",
  CellLabel->"In[131]:="],

Cell["\<\
LEMMA. The sphere corresponding to the spacelike vector vn is a plane if and \
only if one has vn[[4]] = vn[[5]].
Proof: The relation between spheres (including planes) and one-dimensional \
euclidean subspaces of the 5-dimensional pseudo-euclidean space is bijective. \
Since the north pole of the 3-sphere corresponds to the isotropic vector \
stb[4]+stb[5], our condition is equivalent to\
\>", "Text"],

Cell[BoxData[
    \(pssp[stb[4]\  + \ stb[5], \ vec[v]] \[Equal] \ 0\)], "Input",
  CellLabel->"In[132]:="],

Cell["\<\
COROLLARY. The euclidean sphere which corresponds to the spacelike unit  \
vector \
\>", "Text"],

Cell[BoxData[
    \(vec[v]\)], "Input",
  CellLabel->"In[133]:="],

Cell["\<\
with v[4] \[NotEqual] v[5]  is the sphere with center {x, y, z}\
\>", "Text"],

Cell[BoxData[
    \(x\  = \ v[1]/\((v[5] - v[4])\); y\  = \ v[2]/\((v[5] - v[4])\); 
    z\  = \ v[3]/\((v[5] - v[4])\);\)], "Input",
  CellLabel->"In[134]:="],

Cell["and radius", "Text"],

Cell[BoxData[
    \(\(r\  = \ 1/\((v[5] - v[4])\);\)\)], "Input",
  CellLabel->"In[135]:="],

Cell["\<\
Remember that vn is normed: pssp[vn,vn] ==1. Thus we conclude for radius and \
center of the sphere corresponding to the spacelike unit vector v:\
\>", "Text"],

Cell[BoxData[
    \(vradius[wx_List]\  := \ 
      Module[{s, q, \ vx}, \[IndentingNewLine]q\  = \ 
            pssp[wx, wx]; \[IndentingNewLine]If[q \[LessEqual] \ 0, 
            Print[wx, \ "\< is not spacelike!\>"]; 
            Return[]]; \[IndentingNewLine]vx\  = \ 
            wx/Sqrt[q]; \[IndentingNewLine]s = 
            vx[\([5]\)] - \ vx[\([4]\)]; \[IndentingNewLine]If[
            s \[Equal] 0, \ Return[Infinity], Return[1/s], Return[1/s]]] /; 
        Length[wx]\  \[Equal] \ 5\)], "Input",
  CellLabel->"In[44]:=",
  Evaluatable->False],

Cell[BoxData[
    \(vcenter[vx_List]\  := \ 
      Module[{s}, \[IndentingNewLine]If[pssp[vx, vx] \[LessEqual] \ 0, 
            Print[vx, \ "\< is not spacelike!\>"]; 
            Return[]]; \[IndentingNewLine]s = 
            vx[\([5]\)] - \ vx[\([4]\)]; \[IndentingNewLine]If[
            s \[Equal] 0, \ Return[{Infinity, Infinity, Infinity}]]; 
          Return[{vx[\([1]\)]/s, vx[\([2]\)]/s, vx[\([3]\)]/s}]] /; 
        Length[vx] \[Equal] 5\)], "Input",
  CellLabel->"In[28]:=",
  Evaluatable->False],

Cell["\<\
WARNINGS!
1. These definitions make sense for spacelike vectors only!\
\>", "Text"],

Cell[BoxData[
    \(vcenter[{1, 0, 0, 1, 3}]\)], "Input",
  CellLabel->"In[136]:="],

Cell[BoxData[
    \(vradius[nul[]]\)], "Input",
  CellLabel->"In[137]:="],

Cell["\<\
2. If the corresponding sphere is a plane  center and radius will become \
infinite,  e.g.\
\>", "Text"],

Cell[BoxData[
    \(vcenter[stb[1]]\)], "Input",
  CellLabel->"In[138]:="],

Cell[BoxData[
    \(vcenter[{1, 0, 0, 1, 1}]\)], "Input",
  CellLabel->"In[139]:="],

Cell["This case is treated in the next subsection. ", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Orientation", "Subsubsection"],

Cell["A negative radius yields the negatively oriented sphere.", "Text"],

Cell[BoxData[
    \(spacevec[x, y, z, r]\)], "Input",
  CellLabel->"In[140]:="],

Cell[BoxData[
    \(spacevec[x, y, z, \(-r\)]\)], "Input",
  CellLabel->"In[141]:="],

Cell[TextData[{
  "For v[[4]] > v[[5]] the radius of the sphere becomes negative. Changing \
the sign of v one may always reach r > 0; the center is independent of the \
sign. The sign of the radius can be interpreted as the orientation of the \
sphere. If one considers the sphere corresponding to v as subsphere of ",
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)]],
  " (that means: not carry out the stereographic projection), the sphere \
divides the 3-sphere into two disks: the inner and the outer disk. Let us \
define the inner disks as the set of all points (= isotropic 1-spaces) w of \
the 3-sphere  ",
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)]],
  " with pssp[v,w] < 0. Then changing the sign of v means permuting inner and \
outer disk. It follows that the spacelike unit vectors correspond bijectively \
to the oriented subspheres of  ",
  Cell[BoxData[
      \(TraditionalForm\`S\^3\)]],
  ". "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["The sphere corresponding to a spacelike unit vector", "Subsubsection"],

Cell["\<\
Applying the standard euclidean parameter representation of a sphere ( see \
the notebook eusphere.nb) we get immediately the sphere corresponding to the \
spacelike unit vector  v:\
\>", "Text"],

Cell[BoxData[
    \(\(?sphere\)\)], "Input",
  CellLabel->"In[142]:="],

Cell[BoxData[
    \(\(sphere[vcenter[vn], vradius[vn]]\)[u, v]\)], "Input",
  CellLabel->"In[143]:="],

Cell[BoxData[
    \(Simplify[Together[%]]\)], "Input",
  CellLabel->"In[144]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Some examples", "Subsubsection"],

Cell[BoxData[
    \(vradius[{1, 0, 0, 2, 1}]\)], "Input",
  CellLabel->"In[145]:="],

Cell[BoxData[
    \(vcenter[{1, 0, 0, 1, 3}]\)], "Input",
  CellLabel->"In[146]:="],

Cell[BoxData[
    \(vradius[stb[4]*5]\)], "Input",
  CellLabel->"In[147]:="],

Cell[BoxData[
    \(vcenter[stb[4]*5]\)], "Input",
  CellLabel->"In[148]:="],

Cell[BoxData[
    \(\(?sphereplot3D\)\)], "Input",
  CellLabel->"In[149]:="],

Cell["This is the standard unit sphere with center in the origin:", "Text"],

Cell[BoxData[
    \(sphereplot3D[vcenter[stb[4]], \ vradius[stb[4]]]\)], "Input",
  CellLabel->"In[150]:="],

Cell[BoxData[
    \(Table[
      sphereplot3D[vcenter[{0, 0, 0, Sqrt[1 + j^2], \ j}], \ 
        vradius[{0, 0, 0, Sqrt[1 + j^2], \ j}], \ Pi, \ 2*Pi, \(-Pi\)/2, 
        Pi/2], {j, 0, 4}]\)], "Input",
  CellLabel->"In[151]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[152]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.2 Planes", "Subsection"],

Cell["\<\
Before constructing  modules which yield the sphere or plane corresponding to \
a spacelike vector v we find the spacelike vector corresponding to a plane of \
the euclidean 3-space.\
\>", "Text"],

Cell[CellGroupData[{

Cell["The definition planev", "Subsubsection"],

Cell[BoxData[
    \(\(?planev\)\)], "Input",
  CellLabel->"In[10]:="],

Cell["\<\
The following Module yields an unnormed spacelike vector which defines the \
plane, fulfilling the equation w.x==p:\
\>", "Text"],

Cell[BoxData[
    \(planevec[w_List, \ p_]\  := \ 
      Module[{v1, v2, v3, u4\  = stb[4] + stb[5], \n\t\tu1, u2, u3}, \n\t\tIf[
          Length[w] != 3 || \ w\  === nul[3], \n\t\t\tPrint[
            w, \ "\< is not a normal vector!\>"]; Return[]]; \n\t\tv1 = 
          w*p/w . w; \n\t\tIf[w[\([1]\)] === 0, 
          v2\  = v1 + stb[1, 3]; \n\t\t\tIf[w[\([2]\)] === \ 0, \ 
            v3\  = \ v1\  + \ stb[2, 3], 
            v3\  = {0, \ p/w[\([2]\)], 
                0}], \n\t\t\tv2\  = \ {p/w[\([1]\)]\  - \ 
                w[\([2]\)]/w[\([1]\)], 1, 
              0}; \n\t\t\tv3\  = \ {p/w[\([1]\)]\  - \ w[\([3]\)]/w[\([1]\)], 
              0, 1}]; \n\t\tu1 = \ invstepro[v1]; \n\t\tu2 = \ 
          invstepro[v2]; \n\t\tu3 = \ invstepro[v3]; \n
        pscross[{u1, u2, u3, u4}]]\)], "Input",
  CellLabel->"In[11]:="],

Cell["The general evaluation with succeeding simplification", "Text"],

Cell[BoxData[
    \(Simplify[planevec[{x, y, z}, h]]\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(%*\((\((h^2\  + \ 2*x^2\  - \ 2*h*y\  + \ y^2)\)*\((h^2\  + \ x^2\  + \ 
              y^2\  + \ z^2)\)*\n\ \ \ \ \ \ \ \((h^2\  + \ 2*x^2\  - \ 
              2*h*z\  + \ z^2)\))\)\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(%/\((\((8*x^3)\)*\((\(-x^2\) - y^2 - z^2 + 
              h*\((y + z)\))\))\)\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[15]:="],

Cell["\<\
finally leads to the following definition, contained in the package mspher.m:\
\
\>", "Text"],

Cell[BoxData[
    \(planev[a_, b_, c_, \ p_]\  := \ 
      If[Not[a^2 + b^2 + c^2 === 0], \n\tReturn[{a, \ b, \ c, \ p, \ p}], 
        Print["\<Normal vector null is not admissible!\>"]; 
        Return[]]\)], "Input",
  CellLabel->"In[84]:=",
  Evaluatable->False],

Cell[BoxData[
    \(planev[a, b, c, p]\)], "Input",
  CellLabel->"In[16]:="],

Cell[BoxData[
    \(planev[0, 0, 0, p]\)], "Input",
  CellLabel->"In[17]:="],

Cell["Clearly, if we norm planev[a,b,c,p] by ", "Text"],

Cell[BoxData[
    \(dotnorme[{a, b, c}]\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \({a, b, c} . {a, b, c} \[Equal] \ 1\)], "Input",
  CellLabel->"In[19]:="],

Cell["or, what means the same,", "Text"],

Cell[BoxData[
    \(normalize[{a, b, c, p, p}]\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(pssp[{a, b, c, p, p}, {a, b, c, p, p}]\  \[Equal] 1\)], "Input",
  CellLabel->"In[21]:="],

Cell["\<\
we have a bijection between the manifold of all oriented planes of the \
euclidean 3-space and the manifold of all spacelike unit vectors v with the \
property v[[4]]==v[[5]]. Usually one names an equation n.x == p with normed \
3-vector n the Hesse normal form of the plane equation. The following module \
yields a parameter representation of the plane with this normal form, which \
we translate into the 5-vector vc = {a,b,c,p,p}:\
\>", "Text"],

Cell[BoxData[
    \(\(plane[vc_List]\)[u_, v_]\  := \ 
      Module[{q, p, w, ob}, \[IndentingNewLine]q = 
          pssp[vc, vc]; \[IndentingNewLine]If[
          q \[LessEqual] 0 || \ vc[\([4]\)] \[NotEqual] \ vc[\([5]\)], 
          Print[vec, \ "\< incorrect argument!\>"]; 
          Return[]]; \ \[IndentingNewLine]p\  = \ 
          vc[\([5]\)]/Sqrt[q]; \[IndentingNewLine]w\  = \ 
          Take[vc, 3]; \[IndentingNewLine]ob\  = \ 
          orthobase[w]; \[IndentingNewLine]Return[
          ob[\([3]\)]*p\  + \ ob[\([1]\)]*u\  + \ ob[\([2]\)]*v]]\)], "Input",\

  CellLabel->"In[7]:=",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    \(\(\(\ \)\(nx = planev[1, 1, 1, 1]\)\)\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(pssp[nx, nx]\)], "Input",
  CellLabel->"In[23]:="],

Cell[BoxData[
    \(\(plane[nx]\)[u, v]\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(ParametricPlot3D[%, \ {u, \ \(-3\), 3}, \ {v, \ \(-3\), 3}]\)], "Input",\

  CellLabel->"In[25]:="],

Cell[BoxData[
    \(nx\  = planev[0, 0, 1, 0]\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(\(plane[%]\)[u, v]\)], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \(ParametricPlot3D[%, \ {u, \ \(-3\), 3}, \ {v, \ \(-3\), 3}]\)], "Input",\

  CellLabel->"In[28]:="],

Cell[BoxData[
    \(Clear[nx]\)], "Input",
  CellLabel->"In[29]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["The plane through three points", "Subsubsection"],

Cell["\<\
Through three points in general position there exists exactly one plan in the \
euclidean 3-space. The following Module gives a spacelike 5-vector \
corresponding to this plane:\
\>", "Text"],

Cell[BoxData[
    \(spvplane3pts[p1_List, p2_List, p3_List] := \ 
      Module[{q1, q2, q3, vq, \ wrong}, \[IndentingNewLine]If[
          Not[Length[p1] \[Equal] 3 && Length[p2] \[Equal] 3 && 
              Length[p3] \[Equal] 3], \ 
          Goto[wrong]]; \[IndentingNewLine]q1\  = \ invstepro[p1]; 
        q2 = \ invstepro[p2]; 
        q3\  = \ invstepro[p3]; \[IndentingNewLine]vq\  = \ 
          pscross[{q1, q2, q3, stb[4] + stb[5]}]; \[IndentingNewLine]If[
          vq\  \[Equal] \ nul[5], \ Goto[wrong], Return[vq], 
          Return[vq]]; \[IndentingNewLine]Label[
          wrong]; \[IndentingNewLine]Print["\<Incorrect arguments, or points \
not in general position!.\>"]; \[IndentingNewLine]Return[]]\)], "Input",
  CellLabel->"In[30]:="],

Cell[BoxData[
    \(spvplane3pts[nul[3], \ stb[1, 3], \ \(-stb[1, 3]\)]\)], "Input",
  CellLabel->"In[31]:="],

Cell[BoxData[
    \(spvplane3pts[nul[3], \ stb[1, 3], \ stb[2, 3]]\)], "Input",
  CellLabel->"In[32]:="],

Cell[BoxData[
    \(\(plane[%]\)[u, v]\)], "Input",
  CellLabel->"In[33]:="],

Cell[BoxData[
    \(ParametricPlot3D[%, \ {u, \ \(-3\), 3}, \ {v, \ \(-3\), 3}]\)], "Input",\

  CellLabel->"In[34]:="],

Cell[BoxData[
    \(spvplane3pts[stb[3, 3], \ stb[1, 3], \ stb[2, 3]]\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \(\(plane[%]\)[u, v]\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(ParametricPlot3D[%, \ {u, \ \(-3\), 3}, \ {v, \ \(-3\), 3}]\)], "Input",\

  CellLabel->"In[47]:="],

Cell["\<\
This is a random point in the unit cube of the euclidean 3-space:\
\>", "Text"],

Cell[BoxData[
    \(randomv[3]\)], "Input",
  CellLabel->"In[38]:="],

Cell["A random plane as the plane through three random points:", "Text"],

Cell[BoxData[
    \(spvplane3pts[randomv[3], randomv[3], randomv[3]]\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(\(plane[%]\)[u, v]\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(ParametricPlot3D[%, \ {u, \ \(-3\), 3}, \ {v, \ \(-3\), 3}]\)], "Input",\

  CellLabel->"In[41]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.3 The sphere or plane corresponding to a spacelike vector", \
"Subsection"],

Cell["\<\
In this section we join the results of the two foregoing sections: We \
construct a procedure, which gives the parameter representation of the sphere \
or plane corresponding to a spacelike vector, and apply it in a plot command.\
\
\>", "Text"],

Cell[CellGroupData[{

Cell["The definition of the parameter representation euklidsphere", \
"Subsubsection"],

Cell[BoxData[
    \(\(euklidsphere[va_List]\)[u_, v_]\  := \ 
      Module[{\(q\)\(,\)}, \[IndentingNewLine]q\  = \ 
            pssp[va, va]; \[IndentingNewLine]If[q \[LessEqual] \ 0, \ 
            Print[va, \ "\< is not spacelike!\>"]; \ 
            Return[]]; \[IndentingNewLine]If[
            va[\([4]\)]\  == \ va[\([5]\)], \ Goto[pla]]; 
          Return[\(sphere[vcenter[va], vradius[va]]\)[u*Pi, 
              v*Pi/2]]; \[IndentingNewLine]Label[pla]; \[IndentingNewLine]\ 
          p\  = \ va[\([5]\)]/Sqrt[q]; \[IndentingNewLine]w\  = \ 
            Take[va, 3]; \[IndentingNewLine]ob\  = \ 
            orthobase[w]; \[IndentingNewLine]Return[
            ob[\([3]\)]*p\  + \ ob[\([1]\)]*u\  + \ 
              ob[\([2]\)]*v]\[IndentingNewLine]] /; 
        Length[va] \[Equal] \ 5\)], "Input",
  CellLabel->"In[57]:=",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[BoxData[
    \(\(euklidsphere[stb[5]]\)[u, v]\)], "Input",
  CellLabel->"In[48]:="],

Cell[BoxData[
    \(\(euklidsphere[stb[4]]\)[u, v]\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(\(euklidsphere[stb[1]]\)[u, v]\)], "Input",
  CellLabel->"In[50]:="],

Cell[BoxData[
    \(\(euklidsphere[stb[2]]\)[u, v]\)], "Input",
  CellLabel->"In[51]:="],

Cell[BoxData[
    \(\(euklidsphere[stb[3]]\)[u, v]\)], "Input",
  CellLabel->"In[52]:="],

Cell["\<\
Evaluate repeated the following cell; \"almost surely\" you will get a random \
halfsphere.\
\>", "Text"],

Cell[BoxData[
    \(vrand\  = \ randomv[5]; 
    ParametricPlot3D[\(euklidsphere[vrand]\)[u, v], \ {u, \(-1\), 1}, {v, \ 
        0, \ 1}]\)], "Input",
  CellLabel->"In[62]:="],

Cell["\<\
Clearly, the probability for vrand[4] = vrand[5] is zero, therefore planes do \
happen with probability zero. Furthermore remember, that the probability for \
a random vector in the 5-dimensional unit cube to be not spacelike, is small; \
therefore the probability of error messages is small. Indeed, as already \
shown in the notebook pseuklid.nb, we have for the empirical probability \
nearly 6.3 %.\
\>", "Text"],

Cell[BoxData[
    \(\((\ nr\  = \ 0; \ 
      For[i\  = \ 1, \ i \[LessEqual] \ 10000, \(i++\), 
        If[ch[randomv[5]] < \ 1, \ nr\  = \ nr + 1]]; \ 
      nr/10000)\)\)], "Input",
  CellLabel->"In[63]:="],

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[64]:="],

Cell[BoxData[
    \(Remove[nr, vrand]\)], "Input",
  CellLabel->"In[65]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["euklidsphereplot3D and plotsphere", "Subsubsection"],

Cell["Now it is easy to build the plot command for euklidsphere:", "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[vn_List, u0_: - 1, u1_:  1, \ v0_: - 1, v1_:  1, 
        opts___] := 
      ParametricPlot3D[
        Evaluate[\(euklidsphere[vn]\)[u, v]], {u, u0, u1}, \ {v, v0, v1}, 
        opts]\)], "Input",
  CellLabel->"In[69]:=",
  Evaluatable->False],

Cell["\<\
The package mspher.m contains a similar procedure plotsphere:\
\>", "Text"],

Cell[BoxData[
    \(\(?plotsphere\)\)], "Input",
  CellLabel->"In[52]:="],

Cell["\<\
Now we test euklidsphereplot3D  and compare it with plotsphere.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.4 Examples, comparision", "Subsection"],

Cell[CellGroupData[{

Cell["Example: equidistant unit spheres", "Subsubsection"],

Cell[BoxData[
    \(Timing[
      Table[plotsphere[spacevec[x, 0, 0, 1]], \ {x, \ 0, \ 12, \ 
          3}]]\)], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[68]:="],

Cell[BoxData[
    \(Timing[
      Table[euklidsphereplot3D[spacevec[x, 0, 0, 1]], \ {x, \ 0, \ 12, \ 
          3}]]\)], "Input",
  CellLabel->"In[69]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[70]:="],

Cell[BoxData[
    \(grser\  = %\)], "Input",
  CellLabel->"In[71]:="],

Cell[BoxData[
    \(ThreeScript["\<spherser.ts\>", grser]\)], "Input",
  CellLabel->"In[115]:=",
  PageWidth->Infinity],

Cell["\<\
With plotsphere 3.41 s, and with euklidsphereplot3D 1.15 s have been needed. \
There is yet another difference: with euklidsphereplot3D the geographical \
coordinates are defined with respect to the fixed orthonormed standard frame, \
and with plotsphere the coordinates depend on the considered sphere. \
Furthermore, since in the definition of plotsphere we used the \
stereographical projection, and in euklidsphereplot3D not, the coordinates \
and the representations generated by plotsphere are often distorted.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Example: The spacelike frame vectors", "Subsubsection"],

Cell[BoxData[
    \(Timing[Table[plotsphere[stb[i]], \ {i, 4}]]\)], "Input",
  CellLabel->"In[72]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[73]:="],

Cell[BoxData[
    \(Timing[Table[euklidsphereplot3D[stb[i]], \ {i, 4}]]\)], "Input",
  CellLabel->"In[74]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[75]:="],

Cell[BoxData[
    \(\(?spherevec\)\)], "Input",
  CellLabel->"In[76]:="],

Cell[BoxData[
    \(Timing[
      Table[plotsphere[spherevec[2  t, t/2]], \ {t, \ 0.5, \ 9.5, \ 
          3}]]\)], "Input",
  CellLabel->"In[77]:="],

Cell[BoxData[
    \(Show[%[\([2]\)], \ PlotRange\  -> \ All]\)], "Input",
  CellLabel->"In[78]:="],

Cell[BoxData[
    \(grser2 = %\)], "Input",
  CellLabel->"In[79]:="],

Cell[BoxData[
    \(ThreeScript["\<spherse2.ts\>", grser2]\)], "Input",
  CellLabel->"In[80]:=",
  PageWidth->Infinity],

Cell[BoxData[
    \(Timing[
      Table[euklidsphereplot3D[spherevec[2  t, t/2]], \ {t, \ 0.5, \ 9.5, \ 
          3}]]\)], "Input",
  CellLabel->"In[81]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[82]:="],

Cell["\<\
We define a function which constructs a certain monotone sequence from a \
given sequence of positive numbers:\
\>", "Text"],

Cell[BoxData[
    \(ms[rs_List]\  := \ 
      Table[2*Sum[rs[\([j]\)], {j, 1, \ i - 1}] + \ rs[\([i]\)], \ {i, 1, 
          Length[rs]}]\)], "Input",
  CellLabel->"In[83]:="],

Cell["Here is a random  sequence of positive real numbers:", "Text"],

Cell[BoxData[
    \(rd\  = \ randomv[]\)], "Input",
  CellLabel->"In[84]:="],

Cell["\<\
Now we obtain a sequence of spheres with random radii rd less than 1 and \
centers on the x-axis in such a way that to consecutive spheres are tangent \
to each other; the sequence ms[rd] defines the centers of the spheres.\
\>", "Text"],

Cell[BoxData[
    \(ms[rd]\)], "Input",
  CellLabel->"In[85]:="],

Cell[BoxData[
    \(Timing[
      Table[plotsphere[spherevec[\(ms[rd]\)[\([i]\)], rd[\([i]\)]]], \ {i, 1, 
          5}]]\)], "Input",
  CellLabel->"In[86]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[87]:="],

Cell[BoxData[
    \(Timing[
      Table[euklidsphereplot3D[
          spherevec[\(ms[rd]\)[\([i]\)], rd[\([i]\)]]], \ {i, 1, 
          5}]]\)], "Input",
  CellLabel->"In[88]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[89]:="],

Cell[BoxData[
    \(Timing[plotsphere[stb[1]*Cos[Pi/5]\  + \ stb[4]*Sin[Pi/5]]]\)], "Input",\

  CellLabel->"In[90]:="],

Cell[BoxData[
    \(Timing[
      euklidsphereplot3D[stb[1]*Cos[Pi/5]\  + \ stb[4]*Sin[Pi/5]]]\)], "Input",\

  CellLabel->"In[91]:="],

Cell[BoxData[
    \(Timing[
      Table[plotsphere[
          stb[1]*Cos[i*Pi/5]\  + \ stb[4]*Sin[i*Pi/5]], \n\t{i, \ 1, \ 
          5}]]\)], "Input",
  CellLabel->"In[92]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[93]:="],

Cell[BoxData[
    \(Timing[
      Table[euklidsphereplot3D[
          stb[1]*Cos[i*Pi/5]\  + \ stb[4]*Sin[i*Pi/5]], \n\t{i, \ 1, \ 
          5}]]\)], "Input",
  CellLabel->"In[94]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[95]:="],

Cell["\<\
Here we obtain some exemplars of the bundle of planes containing the \
y-axis:\
\>", "Text"],

Cell[BoxData[
    \(Timing[
      Table[plotsphere[
          stb[1]*Cos[i*Pi/5]\  + \ stb[3]*Sin[i*Pi/5]], \n\t{i, \ 1, \ 
          5}]]\)], "Input",
  CellLabel->"In[96]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[97]:="],

Cell[BoxData[
    \(Timing[
      Table[euklidsphereplot3D[
          stb[1]*Cos[i*Pi/5]\  + \ stb[3]*Sin[i*Pi/5]], \n\t{i, \ 1, \ 
          5}]]\)], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \(Show[%[\([2]\)]]\)], "Input",
  CellLabel->"In[99]:="],

Cell["\<\
euklidsphereplot3D is essentially faster than plotsphere, and gives nicer \
pictures!\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Evaluation problems: Floating point numbers", "Subsubsection"],

Cell["\<\
WARNING!
By numerical reasons, the results may be distorted, or completely false. E.g. \
we met the following situation\
\>", "Text"],

Cell[BoxData[
    \(vd[t_]\  := \ stb[1]*Cosh[t]\  + stb[5]*Sinh[t]\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(\(euklidsphere[vd[t]]\)[u, v]\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(Timing[
      Table[euklidsphereplot3D[vd[t*\((2*Pi)\)]], \ {t, 0, 4, 
          0.5}]]\)], "Input",
  CellLabel->"In[24]:="],

Cell[TextData[{
  "On the other hand, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " may calculate and Plot the last two spheres of the sequence, if we \
calculate with higher precision::"
}], "Text"],

Cell[BoxData[
    \(euklidsphereplot3D[N[{Cosh[7*Pi], 0, 0, 0, Sinh[7*Pi]}, 20]]\)], "Input",\

  CellLabel->"In[28]:="],

Cell[BoxData[
    \(N[{Cosh[8*Pi], 0, 0, 0, Sinh[8*Pi]}, 30]\)], "Input",
  CellLabel->"In[31]:="],

Cell[BoxData[
    \(euklidsphereplot3D[N[{Cosh[8*Pi], 0, 0, 0, Sinh[8*Pi]}, 30]]\)], "Input",\

  CellLabel->"In[32]:="],

Cell[TextData[{
  "The reason is that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", by the 0.5 as the running index step, applied floating point numbers in \
the calculations of the sphere sequence! With lower precision, the vectors \
seem to be isotropic; they are very near to the asymptotic (isotreopic) cone. \
With  .5 replaced by 1/2 the same happens:"
}], "Text"],

Cell[BoxData[
    \(Timing[
      Table[euklidsphereplot3D[vd[t*\((2*Pi)\)]], \ {t, 0, 4, 
          1/2}]]\)], "Input",
  CellLabel->"In[33]:="],

Cell[TextData[{
  "Besides of this, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " does not know the simplest trigonometric rules:"
}], "Text"],

Cell["Cosh[6*Pi]^2 - Sinh[6*Pi]^2", "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[35]:="],

Cell["TrueQ[Cosh[6*Pi]^2 - Sinh[6*Pi]^2 == 1]", "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(TrueQ[Cosh[x]\^2 - Sinh[x]\^2 \[Equal] 1]\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(TrueQ[Cos[x]\^2 + \ Sin[x]\^2 \[Equal] 1]\)], "Input",
  CellLabel->"In[38]:="],

Cell["Using Simplify the equations become True:", "Text"],

Cell[BoxData[
    \(TrueQ[Simplify[Cosh[x]\^2 - Sinh[x]\^2] \[Equal] 1]\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(TrueQ[Simplify[Cos[x]\^2 + \ Sin[x]\^2] \[Equal] 1]\)], "Input",
  CellLabel->"In[40]:="],

Cell["\<\
Conclusion: It is often better introducing more Simplify's in the procedures.
But then it may happen that the evaluation needs much more time.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[" 3.5 The manifold of all spheres, random spheres", "Subsection"],

Cell[CellGroupData[{

Cell["The sphere manifold", "Subsubsection"],

Cell["\<\
Any (non oriented) sphere is represented by a pseudo-euclidean subspace of \
dimension 4; the manifold of all spheres is the open orbit of \
pseudo-euclidean 4-spaces under the action of the M\[ODoubleDot]bius group.  \
Going to the orthogonal complements one finds that the spacelike \
unit-hypersphere, the hyper-hyperboloid\
\>", "Text"],

Cell[TextData[{
  "pssp[x,x]=",
  Cell[BoxData[
      \(x[1]\^2 + x[2]\^2 + x[3]\^2 + x[4]\^2 - x[5]\^2\)]],
  "= 1\t\t(H)"
}], "Text",
  TextAlignment->Center],

Cell["\<\
is a twofold covering of the manifold of all non-oriented spheres. We \
immediately can write down a parameter representation of this hypersurface \
calculating\
\>", "Text"],

Cell[BoxData[
    \(Append[\(unit[3]\)[{s, \ t, \ u}], \ 0]*Cosh[v]\  + \ 
      stb[5]*Sinh[v]\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(\(?unit\)\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(Context[unit]\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(\(?spunit\)\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(\(\(spunit[s, t, u, \ v]\)\(\ \)\)\)], "Input",
  CellLabel->"In[45]:="],

Cell["\<\
Here we used unit[3][{s,t,u}], the parameter representation of the unit \
3-sphere in the 4-dimensional euclidean subspace orthogonal to stb[5], see \
euvec.m, or eusphere.nb. Obviously, spunit can be generalized easily to \
arbitrary dimension.\
\>", "Text"],

Cell["\<\
Now we may apply  the euklidsphere functions of  section 3.3 on this vector \
and obtain a parameter representation or plot of the corresponding sphere. In \
an old version of this notebook we started with the now deleted function \
psphere:\
\>", "Text"],

Cell[BoxData[
    \(\(?vsphere\)\)], "Input",
  CellLabel->"In[51]:="],

Cell[BoxData[
    \(\(\(\t\)\(Timing[\(psphere[spunit[s, t, u, v]]\)[x, y]]\)\)\)], "Input",\

  CellLabel->"In[14]:=",
  Evaluatable->False],

Cell["The needed time was 1483.02 s (on a very fast PC). ", "Text"],

Cell["\<\
Mathematica were working a long time calculating an odd looking formula.  \
Also Simplify did not help very much. FullSimplify I aborted after more than \
one hour.  With psphere replaced by euklidsphere  we get in less than 5 s a \
very nice formula:\
\>", "Text"],

Cell[BoxData[
    \(\(\(\t\)\(Timing[
      Simplify[\(euklidsphere[spunit[s, t, u, v]]\)[x, 
          y], \[IndentingNewLine]s \[Element] Reals && t \[Element] Reals && 
          u \[Element] Reals && v \[Element] Reals && x \[Element] Reals && 
          y \[Element] Reals]]\)\)\)], "Input",
  CellLabel->"In[9]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["A bad definition", "Subsubsection"],

Cell["\<\
Therefore, and by certain evaluation advantages, we would prefer the direct \
definition\
\>", "Text"],

Cell[BoxData[
    \(\(euklidspherefct[s_, t_, u_, v_]\)[x_, 
        y_] := \n{\(Cos[\[Pi]\ x]\ Cos[\(\[Pi]\ y\)\/2] + Cos[s]\ Cos[t]\ \
Cos[u]\ Cosh[v]\)\/\(\(-Cosh[v]\)\ Sin[u] + Sinh[v]\), \(Cos[t]\ Cos[u]\ \
Cosh[v]\ Sin[s] + Cos[\(\[Pi]\ y\)\/2]\ Sin[\[Pi]\ x]\)\/\(\(-Cosh[v]\)\ \
Sin[u] + Sinh[v]\), \(Cos[u]\ Cosh[v]\ Sin[t] + Sin[\(\[Pi]\ \
y\)\/2]\)\/\(\(-Cosh[v]\)\ Sin[u] + Sinh[v]\)}\)], "Input",
  CellLabel->"In[10]:="],

Cell["Obviously, we then get infinite vectors for u==v==0:", "Text"],

Cell[BoxData[
    \(\(euklidspherefct[s, 0, 0, 0]\)[x, y]\)], "Input",
  CellLabel->"In[18]:="],

Cell["On the other hand, we get the correct result", "Text"],

Cell[BoxData[
    \(\(euklidsphere[spunit[s, 0, 0, 0]]\)[x, y]\)], "Input",
  CellLabel->"In[19]:="],

Cell["Something must be wrong!", "Text"],

Cell[BoxData[
    \(\(euklidsphere[spunit[s, t, u, v]]\)[x, y]\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(% /. u \[Rule] 0\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(% /. v \[Rule] 0\)], "Input",
  CellLabel->"In[19]:="],

Cell["\<\
The reason must be that under the general calculation of \
euklidsphere[spunit[s,t,u,v]][x,y] an error appears.
Indeed, looking at the definition of  euklidsphere in mspher.m one sees that \
for \
\>", "Text"],

Cell[BoxData[
    \(spunit[s, t, u, v]\)], "Input",
  CellLabel->"In[28]:="],

Cell["the procedure, of course, en general would refuse", "Text"],

Cell[BoxData[
    \(%[\([4]\)]\  \[Equal] %[\([5]\)]\)], "Input",
  CellLabel->"In[29]:="],

Cell["Therefore vectors as ", "Text"],

Cell[BoxData[
    \(spunit[s, t, 0, 0]\)], "Input",
  CellLabel->"In[31]:="],

Cell["\<\
are not recognized as those which  generate planes. The attempt to evaluate \
the command line Return[xtest] \
\>", "Text"],

Cell[BoxData[
    RowBox[{
      StyleBox[\(va\  = spunit[s, t, 0, 0]\),
        FontSize->14], 
      StyleBox[";",
        FontSize->14], 
      StyleBox["  ",
        FontSize->14], 
      StyleBox[\(xtest\  = \ \(sphere[vcenter[va], vradius[va]]\)[u*Pi, 
            v*Pi/2]\),
        FontFamily->"Courier Neu",
        FontSize->14]}]], "Input",
  CellLabel->"In[32]:="],

Cell["\<\
leads to infinite vectors what is reflected in the calculation of the general \
formula. Thus, it does not make sense to evaluate for values with u = v = 0, \
which correspond to planes, the function\
\>", "Text"],

Cell[BoxData[
    \(\(euklidspherefct[s, t, u, v]\)[x, y]\)], "Input",
  CellLabel->"In[33]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["A better definition", "Subsubsection"],

Cell["Since the derived formula is not general enough, we set", "Text"],

Cell[BoxData[
    \(Clear[euklidspherefct]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(\(euklidspherefct[s_, t_, u_, v_]\)[x_, 
        y_] := \ \(euklidsphere[spunit[s, t, u, v]]\)[x, y]\)], "Input",
  CellLabel->"In[37]:="],

Cell["and obtain the correct result:", "Text"],

Cell[BoxData[
    \(\(euklidspherefct[s, 0, 0, 0]\)[x, y]\)], "Input",
  CellLabel->"In[38]:="],

Cell[BoxData[
    \(\(euklidsphere[spunit[s, 0, 0, 0]]\)[x, y]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(Timing[\(euklidspherefct[s, 0, 0, 0]\)[x, y]]\)], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(spunit[Pi/4, Pi, \ Pi/4, 0]\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(euklidsphereplot3D[spunit[Pi/4, Pi, \ Pi/4, 0]]\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(\(euklidspherefct[Pi/4, Pi, \ Pi/4, 0]\)[x, y]\)], "Input",
  CellLabel->"In[41]:="],

Cell["Since", "Text"],

Cell[BoxData[
    \(pssp[spunit[2*Pi*Random[], \ Pi*Random[]/2, \ 0, 0], 
        stb[4] + stb[5]] \[Equal] \ 0\)], "Input",
  CellLabel->"In[48]:="],

Cell["\<\
Commands like the following always create a set of planes through the origin:\
\
\>", "Text"],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[
        spunit[2*Pi*Random[], \ Pi*Random[]/2, \ 0, 0]], {20}]\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[47]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["A random spacelike unit vector", "Subsubsection"],

Cell[BoxData[
    \(sprandomv\  := \ 
      Module[{re, \ rs, \ rt, rv, \ i}, \[IndentingNewLine]re[i_]\  := \ 
          Sign[0.5\  - \ Random[]]; \[IndentingNewLine]rs[]\  := \ 
          Random[]*2*Pi; \[IndentingNewLine]rt[i_]\  := \ 
          Random[]*Pi/2; \[IndentingNewLine]rs[]\  := \ 
          re[3]*Random[]*10; \[IndentingNewLine]spunit[rs[], re[1]*rt[1], \ 
          re[2]*rt[2], \ rs[]]\ ]\)], "Input",
  CellLabel->"In[53]:="],

Cell["\<\
WARNING: The distribution of the random spacelike unit vector and the \
corresponding random sphere is not invariant under the action of the M\
\[ODoubleDot]bius group! The corresponding random sphere is:\
\>", "Text"],

Cell[BoxData[
    \(vte\  = \ sprandomv\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \(pssp[vte, vte]\)], "Input",
  CellLabel->"In[55]:="],

Cell[BoxData[
    \(Timing[\(euklidsphere[vte]\)[x, \ y]]\)], "Input",
  CellLabel->"In[56]:="],

Cell[BoxData[
    \(Timing[euklidsphereplot3D[vte]]\)], "Input",
  CellLabel->"In[57]:="],

Cell[BoxData[
    \(vte\  = \ Table[sprandomv, {j, 10}]\)], "Input",
  CellLabel->"In[58]:="],

Cell[BoxData[
    \(Table[pssp[%[\([j]\)], %[\([j]\)]], \ {j, 10}]\)], "Input",
  CellLabel->"In[59]:="],

Cell[BoxData[
    \(Timing[
      Table[\(euklidsphere[vte[\([j]\)]]\)[x, \ y], {j, 10}]]\)], "Input",
  CellLabel->"In[63]:="],

Cell[BoxData[
    \(vte\  = \ Table[sprandomv, {j, 8}]\)], "Input",
  CellLabel->"In[92]:="],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[vte[\([j]\)], \ 0, 1, \ \(-1\), 1], {j, 
        8}]\)], "Input",
  CellLabel->"In[93]:="],

Cell[BoxData[
    \(Show[%, \ PlotRange\  \[Rule] \ All]\)], "Input",
  CellLabel->"In[94]:="],

Cell["\<\
Since the random radii and centers are wide spread  the union of the \
halfspheres give not a nice picture\
\>", "Text"],

Cell[BoxData[
    \(Table[{vradius[vte[\([j]\)]], \ vcenter[vte[\([j]\)]]}, {j, 
        8}]\)], "Input",
  CellLabel->"In[96]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Oriented spheres", "Subsubsection"],

Cell["\<\
Already in section 3.1 we mentioned oriented spheres. Let v be a spacelike \
vector. The sphere S(v) defined by v corresponds to the isotropic vectors x \
solving pssp[v,x] == 0. This defines a linear pseudo-euclidean 4-space \
dividing  the isotropic cone into two parts. On the 3-sphere these parts are \
the \"inner ball\", defined by pssp[v, y] < 0, and the outer ball, defined by \
pssp[v,y] > 0. Clearly, if we replace v by -v, the corresponding sphere does \
not change, but inner and outer ball exchange there roles. Therefore the \
hyper-hyperboloid (H) represents the manifold of all oriented spheres \
contained in  the 3-sphere bijectively.\
\>", "Text"],

Cell[BoxData[
    \(vte\  = \ sprandomv\)], "Input",
  CellLabel->"In[97]:="],

Cell[BoxData[
    \(pssp[vte, vte]\)], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \(Timing[\(euklidsphere[vte]\)[x, \ y]]\)], "Input",
  CellLabel->"In[99]:="],

Cell[BoxData[
    \(Timing[euklidsphereplot3D[vte]]\)], "Input",
  CellLabel->"In[100]:="],

Cell[BoxData[
    \(Timing[\(euklidsphere[\(-vte\)]\)[x, \ y]]\)], "Input",
  CellLabel->"In[101]:="],

Cell[BoxData[
    \(Timing[euklidsphereplot3D[\(-vte\)]]\)], "Input",
  CellLabel->"In[102]:="],

Cell[BoxData[
    \(vte\)], "Input",
  CellLabel->"In[103]:="],

Cell[BoxData[
    \(vradius[vte] \[Equal] \(-vradius[\(-vte\)]\)\)], "Input",
  CellLabel->"In[108]:="],

Cell[BoxData[
    \(vcenter[vte]\  \[Equal] \ vcenter[\(-vte\)]\)], "Input",
  CellLabel->"In[109]:="],

Cell[BoxData[
    \(vec[n]\)], "Input",
  CellLabel->"In[104]:="],

Cell[BoxData[
    \(\(euklidsphere[vec[n]]\)[x, y] + \(euklidsphere[\(-vec[n]\)]\)[x, 
        y]\)], "Input",
  CellLabel->"In[105]:="],

Cell[BoxData[
    \(Simplify[Together[%]]\)], "Input",
  CellLabel->"In[106]:="],

Cell["\<\
It follows that the spheres are identical as sets; the radius vectors with \
the same parameter values with respect to the centers are opposite: The sum \
of the radius vectors is twice the center.\
\>", "Text"]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["4 The generalized angle", "Section"],

Cell[CellGroupData[{

Cell["4.1 Definition. Cases", "Subsection"],

Cell["\<\
The consideration of this section are valid for hyperspheres in any dimension \
(with exception of plotting commands, of course). We consider two spheres, \
represented by the spacelike vectors v, w. Since the map v -> sphere[v] is \
equivariant, the generalized cosinus gencos of these vectors already yields a \
complete invariant system  in the space of oriented sphere pairs, as can be \
proved easily.\
\>", "Text"],

Cell[BoxData[
    \(\(?gencos\)\)], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \(gencos[v_List, \ w_List]\  := \ 
      Simplify[pssp[v, \ w]/Sqrt[pssp[v, v]\ pssp[w, w]]]\)], "Input",
  CellLabel->"In[17]:=",
  Evaluatable->False],

Cell["\<\
We have to distinguish three cases: The linear hull of {v, w} can be 1) \
euclidean, 2) pseudo-euclidean, 3) isotropic. Here and in the following we \
assume that the spheres are distinct, i.e. the vectors v, w are linearely \
independent.\
\>", "Text"],

Cell[BoxData[
    \(\(?spacevec\)\)], "Input",
  CellLabel->"In[9]:="],

Cell[BoxData[
    \(spacevec[x, y, z, r]\)], "Input",
  CellLabel->"In[10]:="],

Cell[BoxData[
    \(pssp[spacevec[x, y, z, r], spacevec[X, Y, Z, R]]\)], "Input",
  CellLabel->"In[11]:="],

Cell[BoxData[
    \(Together[%]\)], "Input",
  CellLabel->"In[12]:="],

Cell["\<\
Since the distance square of the centers of the spheres in the euclidean \
space is\
\>", "Text"],

Cell[BoxData[
    \(d^2 = {\ X - x, Y - y, Z - z} . {\ X - x, Y - y, Z - z}\)], "Input",
  CellLabel->"In[13]:="],

Cell["\<\
we obtain the conformal invariant gencos of two spheres in terms of their \
euclidean invariants:\
\>", "Text"],

Cell[BoxData[
    \(\(?confinv\)\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \(confinv[r, R, d]\)], "Input",
  CellLabel->"In[15]:="],

Cell["\<\
If the spheres intersect this invariant is the cosinus of the intersection \
angle; If they do not intersect the invariant\
\>", "Text"],

Cell[BoxData[
    \(ArcCosh[confinv[r, R, d]]\)], "Input",
  CellLabel->"In[49]:="],

Cell[TextData[{
  "has been considered by H. S. M. Coxeter in [1]; he called it the ",
  StyleBox["inversive distance ",
    FontColor->RGBColor[1, 0, 0]],
  "of the spheres. "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2 Intersecting spheres", "Subsection"],

Cell["\<\
The spheres intersect in a circle (resp. (n-2)-sphere) iff v, w span an \
euclidean subspace. Then gencos[v,w] obviously is the cosinus of the  usual \
angle between the vectors v, w, and this is valid iff -1 < gencos[v,w] < 1:\
\>", "Text"],

Cell[BoxData[
    \(\(\(\ \)\(phi[v_, \ w_]\  := \ ArcCos[gencos[v, w]]\)\)\)], "Input",
  CellLabel->"In[16]:="],

Cell["\<\
is the angle between the normal vectors (= radii vectors) of the spheres \
along the intersection. This follows easily from the cos-theorem. \
\>", "Text"],

Cell[CellGroupData[{

Cell["Example", "Subsubsection"],

Cell[BoxData[
    \(vx\  = \ spherevec[0, \ 1]\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(euklidsphereplot3D[vx]\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(sph1\  = \ %\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(euklidsphereplot3D[spherevec[0.7, \ 0.8]]\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(sph2\  = \ %\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(Show[{sph1, sph2}]\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(gencos[vx, \ spherevec[0.7, \ 0.8]]\)], "Input",
  CellLabel->"In[23]:="],

Cell[BoxData[
    \(wx = \ spherevec[x, \ r]\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(pssp[%, \ %]\)], "Input",
  CellLabel->"In[25]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(gencos[vx, \ wx]\)], "Input",
  CellLabel->"In[27]:="],

Cell["\<\
The absolute value of this invariant equals 1 in the cases x = \
\[PlusMinus](1-r) or x = \[PlusMinus](1+r), and this means that the spheres \
are tangent to each other, see also section 4.4 .\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Intersecting planes", "Subsubsection"],

Cell["\<\
A simple example of symbolic calculations: let us take two general planes:\
\>", "Text"],

Cell[BoxData[
    \(na = \ planev[x, y, z, h]\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(pssp[na, na]\)], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(nb = \ planev[X, Y, Z, H]\)], "Input",
  CellLabel->"In[30]:="],

Cell[BoxData[
    \(Simplify[\((pssp[na, nb])\)^2/\((pssp[na, na]*
            pssp[nb, nb])\)]\)], "Input",
  CellLabel->"In[31]:="],

Cell["\<\
And this is the cos^2 of the intersection angle, if the planes are not \
parallel, and 1 else, as expected!If the planes are parallel we have\
\>", "Text"],

Cell[BoxData[
    \(nc\  = \ \ planev[x, y, z, k]\)], "Input",
  CellLabel->"In[32]:="],

Cell[BoxData[
    \(Simplify[\((pssp[na, nc])\)^2/\((pssp[na, na]*
            pssp[nc, nc])\)]\)], "Input",
  CellLabel->"In[33]:="],

Cell["Parallel planes are tangent to each other at infinity. ", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "4.3 ",
  StyleBox["Non-intersecting spheres ",
    FontSize->14]
}], "Subsection",
  FontSize->16],

Cell["\<\
In the pseudo-euclidean case the othogonal complement of the linear hull[v, \
w] is euclidean, thus, it does not contain isotropic vectors, and the \
intersection of the spheres is empty, and vice versa. We have \
\>", "Text"],

Cell[BoxData[
    \(fct[x1_, \ r1_, x2_, \ r2_]\  := \ 
      gencos[spherevec[x1, \ r1], \ spherevec[x2, \ r2]]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(fct[x1, r1, x2, r2]\)], "Input",
  CellLabel->"In[35]:="],

Cell["\<\
Since d = x1 - x2 is the euclidean distance between the centers of the \
spheres, we obtain the conformal invariant of two spheres with radii r1, r2 \
and distance d of its centers as \
\>", "Text"],

Cell[BoxData[
    \(\(?confinv\)\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(confinv[r1_, \ r2_, \ d_]\  := \ 
      \((r1^2\  + \ r2^2\  - d^2)\)/\((2\ r1\ r2)\)\)], "Input",
  CellLabel->"In[18]:=",
  Evaluatable->False],

Cell["More generally we derived this expression already in 4.1.", "Text"],

Cell["\<\
In the non-intersecting case one has Abs[confinv[r1, r2, d]]>1, and vice \
versa.\
\>", "Text"],

Cell[BoxData[
    \(confinv[r1, \ r2, \ d]\)], "Input",
  CellLabel->"In[36]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["4.4 Tangential spheres", "Subsection"],

Cell["\<\
The spheres are tangent to each other at the common point, which correspond \
to the isotropic vector of the linear hull[v,w], iff one has Abs[confinv[r1, \
r2, d]]=1.
We have two subcases:\
\>", "Text"],

Cell[CellGroupData[{

Cell["a)  d = |r1 + r2|", "Subsubsection"],

Cell[BoxData[
    \(confinv[r1, \ r2, \ r1\  + \ r2]\)], "Input",
  CellLabel->"In[73]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[74]:="],

Cell[BoxData[
    \(spherevec[1 + r, \ r]\)], "Input",
  CellLabel->"In[75]:="],

Cell[BoxData[
    \(Simplify[2\ r\ %]\)], "Input",
  CellLabel->"In[76]:="],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[spherevec[1 + r, \ r]], \ {r, \ 0.2, \ 1, \ 
        0.2}]\)], "Input",
  CellLabel->"In[77]:="],

Cell[BoxData[
    \(tangser\  = \ %\)], "Input",
  CellLabel->"In[78]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[79]:="],

Cell[BoxData[
    \(euklidsphereplot3D[stb[4]]\)], "Input",
  CellLabel->"In[80]:="],

Cell[BoxData[
    \(spst = %\)], "Input",
  CellLabel->"In[81]:="],

Cell[BoxData[
    \(Append[tangser, spst]\)], "Input",
  CellLabel->"In[82]:="],

Cell[BoxData[
    \(taser\  = \ %\)], "Input",
  CellLabel->"In[83]:="],

Cell[BoxData[
    \(Show[taser]\)], "Input",
  CellLabel->"In[84]:="],

Cell[BoxData[
    \(tanser\  = %\)], "Input",
  CellLabel->"In[85]:="],

Cell[BoxData[
    \(MVShow3D[tanser]\)], "Input",
  CellLabel->"In[87]:="],

Cell[BoxData[
    \(ThreeScript["\<tangsph.ts\>", tanser]\)], "Input",
  CellLabel->"In[117]:="],

Cell["The common point of all these spheres is", "Text"],

Cell[BoxData[
    \(stepro[{1, 0, 0, 0, 1}]\)], "Input",
  CellLabel->"In[88]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["b)  d = |r1  - r2|", "Subsubsection"],

Cell[BoxData[
    \(confinv[r1, \ r2, \ r1\  - \ r2]\)], "Input",
  CellLabel->"In[89]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[90]:="],

Cell[BoxData[
    \(spherevec[1 - r, \ r]\)], "Input",
  CellLabel->"In[91]:="],

Cell[BoxData[
    \(Simplify[2\ r\ %]\)], "Input",
  CellLabel->"In[92]:="],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[spherevec[1 - r, \ r]], \ {r, \ 0.2, \ 1, \ 
        0.2}]\)], "Input",
  CellLabel->"In[93]:="],

Cell[BoxData[
    \(tangser2\  = \ %\)], "Input",
  CellLabel->"In[94]:="],

Cell[BoxData[
    \(Show[tangser2]\)], "Input",
  CellLabel->"In[95]:="],

Cell[BoxData[
    \(tanser2\  = %\)], "Input",
  CellLabel->"In[96]:="],

Cell[BoxData[
    \(MVShow3D[tanser2]\)], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \(ThreeScript["\<tangsph2.ts\>", tanser2]\)], "Input",
  CellLabel->"In[97]:="],

Cell["The common point of all these spheres is again ", "Text"],

Cell[BoxData[
    \(stepro[{1, 0, 0, 0, \ 1}]\)], "Input",
  CellLabel->"In[99]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.5 Intersection of a sphere and a plane", "Subsection"],

Cell["\<\
Now we consider the intersection of a plane with a sphere. For the conformal \
invariant of a sphere and a plane we calculate\
\>", "Text"],

Cell[BoxData[
    \(pssp[spacevec[x, y, z, r], \ planev[X, Y, Z, p]]\)], "Input",
  CellLabel->"In[100]:="],

Cell[BoxData[
    \(Together[%]\)], "Input",
  CellLabel->"In[101]:="],

Cell[BoxData[
    \(%^2/\((pssp[spacevec[x, y, z, r], spacevec[x, y, z, r]]*\n\t\t\tpssp[\ 
            planev[X, Y, Z, p], \ planev[X, Y, Z, p]])\)\)], "Input",
  CellLabel->"In[102]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[103]:="],

Cell["\<\
If plane and sphere really intersect, this is the cosinus square of the \
intersection angle. In the general case it corresponds to the conformal \
invariant confinv of two spheres. The plane is tangent to the sphere iff this \
expression equals one.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Example", "Subsubsection"],

Cell["\<\
For the intersection of a unit sphere with center {x,0,0} and the y,z-plane \
through the origin we obtain\
\>", "Text"],

Cell[BoxData[
    \(sph[x_]\  := \ spherevec[x, \ 1]\)], "Input",
  CellLabel->"In[104]:="],

Cell[BoxData[
    \(sph[x]\)], "Input",
  CellLabel->"In[105]:="],

Cell[BoxData[
    \(pl\  = \ planev[1, 0, 0, 0]\)], "Input",
  CellLabel->"In[106]:="],

Cell[BoxData[
    \(gencos[pl, \ sph[x]]\)], "Input",
  CellLabel->"In[107]:="],

Cell[BoxData[
    \(pssp[sph[x], \ sph[x]]\)], "Input",
  CellLabel->"In[108]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[109]:="],

Cell[BoxData[
    \({euklidsphereplot3D[stb[1]], 
      euklidsphereplot3D[{1, 0, 0, \(-1\)/2, 1/2}]}\)], "Input",
  CellLabel->"In[110]:="],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[111]:="],

Cell[BoxData[
    \(MVShow3D[%]\)], "Input",
  CellLabel->"In[112]:="]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["5 Geodesics of the sphere space", "Section"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["5.1 Introduction",
  FontSize->16]], "Subsection"],

Cell["\<\
In subsection 3.5 we identified the space of oriented spheres with the \
hypersurface  H: pssp[x,x] = 1 of spacelike unit vectors in the \
pseudo-euclidean vector space: a \"pseudo-hypersphere\".  It is a \
pseudo-Riemannian symmetric space of constant curvature. As in Riemannian \
sphere geometry, the geodesics of  H are the intersections of the \
pseudo-hypersphere H with 2-planes through the center o. But now we  have \
three kinds of such planes, and the corresponding geodesics. We take the \
equatorial sphere stb[4] as starting point of  the geodesics. To find the \
types of geodesic we may restrict to the linear hull[stb[5],stb[1]], as a \
subspace of the tangent space of this hyper-hyperboloid at stb[4]: look at \
the isotropy action, the action of the subgroup of the M\[ODoubleDot]bius \
group preserving stb[4] on the tangential space of H at stb[4]. Obviously, \
this is the usual standard action of the pseudo-orthogonal group O(3,1) on \
the 4-dimensional pseudo-euclidean vector space of index 1. 
We remember the following situation:\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(se1 = euklidsphereplot3D[stb[3]]\)], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \(se2\  = euklidsphereplot3D[stb[4]]\)], "Input",
  CellLabel->"In[9]:="],

Cell[BoxData[
    \(Show[{se1, se2}, PlotRange\ \  -> {\(-1\), 1}]\)], "Input",
  CellLabel->"In[10]:="],

Cell["\<\
The intersection of the spheres defined by stb[4] and stb[3] is the unit \
circle in the x,y-plane around o. The spheres containing this circle are \
exactly represented by the unit vectors of the linear hull[stb[4], stb[3]]. \
All the spheres corresponding to spacelike vectors of the linear hull[stb[1], \
stb[2], stb[5]] intersect all the spheres of this family orthogonally.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["5.2 Timelike geodesics",
  FontSize->16]], "Subsection",
  Evaluatable->False],

Cell["\<\
A timelike geodesics is the intersection of H with a pseudo-Euclidean plane, \
say, spanned by stb[4], stb[5]. This 2-plane contains two isotropic \
directions, i.e. points of the 3-sphere. The geodesic consists of all spheres \
which intersect all the spheres through these two points  in an angle Pi/2. \
If we take one of this point as the north pole (corresponding to stb[4,4]) \
and the other as the origin o , and the \"spheres\" through these points are \
the planes through the origin (under stereographical projection).Therefore  \
the geodesic is the set of all spheres with center o. The tangential vector \
of this geodesic at t=0 is stb[5]. Therefore the geodesic is timelike.\
\>", "Text",
  Evaluatable->False],

Cell["\<\
Five spheres on this timelike geodesic (for better visibility we only plot \
the halfspheres):\
\>", "Text",
  Evaluatable->False],

Cell["\<\
Table[euklidsphereplot3D[{0, 0, 0, Cosh[t], Sinh[t]},-1,1,0,1],
{t, -1, 1, 1/2}]\
\>", "Input",
  CellLabel->"In[11]:=",
  PageWidth->Infinity],

Cell[BoxData[
    \(grr\  = %\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(Show[grr]\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(grr1\  = %\)], "Input",
  CellLabel->"In[16]:="],

Cell[BoxData[
    \(MVShow3D[grr1]\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(ThreeScript["\<spheres1.ts\>", grr1]\)], "Input",
  CellLabel->"In[133]:=",
  PageWidth->Infinity],

Cell["\<\
Replacing stb[4] by stb[1] gives a M\[ODoubleDot]bius equivalent, also again \
timelike geodesic. In the euclidean picture, this geodesic starts at the \
\"sphere\" defined by stb[1], that is the y,z-plane. It is defined as the \
one-parameter sphere family corresponding to the spacelike vectors {Cosh[t], \
0, 0, 0, Sinh[t]}:\
\>", "Text"],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[{Cosh[t], 0, 0, 0, Sinh[t]}, \(-1\), 0, \(-1\), 
        1], {t, \(-5\), 5}]\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(Show[%, \ PlotRange\  \[Rule] \ All]\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(tgr\  = \ %\)], "Input",
  CellLabel->"In[23]:="],

Cell["\<\
One sees that we obtained a one parameter family of pairwise non intersecting \
spheres,  containing the y,z-plane and contracting to two points, which \
correspond to the origin and the infinite point in our first example.These \
points are\
\>", "Text"],

Cell[BoxData[
    \({stepro[stb[1]\  + stb[5]], stepro[stb[1]\  - stb[5]]}\)], "Input",
  CellLabel->"In[24]:="],

Cell["\<\
The points are not arbitrary; they are in involution with respect to the \
starting sphere (here: the y,z-plane) of the geodesic. Consider all circles \
through the two points: they cut the y,z-plane orthogonally. Since the \
geodesic is generated moving the starting sphere by a 1-parameter group of M\
\[ODoubleDot]bius transformations  preserving these two points (the \
pseudo-orthogonal group of the subspace spanned by stb[1], stb[5]):\
\>", "Text"],

Cell[BoxData[
    \({{Cosh[t], 0, \ 0, \ 0, \ Sinh[t]}, \[IndentingNewLine]{0, \ 1, \ 0, \ 
          0, \ 0}, \[IndentingNewLine]{0, \ 0, \ 1, \ 0, \ 
          0}, \[IndentingNewLine]{0, \ 0, \ 0, \ 1, \ 
          0}, \[IndentingNewLine]{Sinh[t], 0, \ 0, \ 0, \ Cosh[t]}} . {1, 0, 
        0, 0, 0}\)], "Input",
  CellLabel->"In[25]:="],

Cell[BoxData[
    \({{Cosh[t], 0, \ 0, \ 0, \ Sinh[t]}, \[IndentingNewLine]{0, \ 1, \ 0, \ 
          0, \ 0}, \[IndentingNewLine]{0, \ 0, \ 1, \ 0, \ 
          0}, \[IndentingNewLine]{0, \ 0, \ 0, \ 1, \ 
          0}, \[IndentingNewLine]{Sinh[t], 0, \ 0, \ 0, \ Cosh[t]}} . {1, 0, 
        0, 0, 1}\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \({{Cosh[t], 0, \ 0, \ 0, \ Sinh[t]}, \[IndentingNewLine]{0, \ 1, \ 0, \ 
          0, \ 0}, \[IndentingNewLine]{0, \ 0, \ 1, \ 0, \ 
          0}, \[IndentingNewLine]{0, \ 0, \ 0, \ 1, \ 
          0}, \[IndentingNewLine]{Sinh[t], 0, \ 0, \ 0, \ Cosh[t]}} . {1, 0, 
        0, 0, \(-1\)}\)], "Input",
  CellLabel->"In[27]:="],

Cell["\<\
all the spheres of the geodesic intersect the circle bundle through the two \
points orthogonally, and therefore the geodesic is the family of orthogonal \
trajectories of the bundle. Thus, the recipe to conctruct a timelike geodesic \
through a given sphere is the following: Take a point outside the sphere, \
reflect it at the sphere, consider the bundle of all circles through the \
point pair in involution obtained this way: The geodesic is the family of \
orthogonal trajectories of this bundle, which are spheres. All these \
geodesics are M\[ODoubleDot]bius equivalent. The set of these geodesics is a \
six-dimensional homogeneous space under the action of the M\[ODoubleDot]bius \
group.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["5.3 Spacelike geodesics", "Subsection",
  Evaluatable->False,
  FontSize->16],

Cell["\<\
The spacelike geodesic corresponding to the intersection with the linear \
hull[stb[1], stb[4]] is the \"circle\"
{Sin[t], 0, 0, Cos[t],0}.
Five spheres on this spacelike geodesic (again we plot halfspheres only):\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[\[IndentingNewLine]{Sin[t], 0, 0, Cos[t], 
          0}, \(-1\), 0, \(-1\), 1], \[IndentingNewLine]{t, \(-1\)/3, 1/3, 
        1/6}]\)], "Input",
  CellLabel->"In[28]:=",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[29]:=",
  PageWidth->Infinity],

Cell[BoxData[
    \(spacege\  = \ %\)], "Input",
  CellLabel->"In[30]:="],

Cell[BoxData[
    \(MVShow3D[spacege]\)], "Input",
  CellLabel->"In[31]:="],

Cell[BoxData[
    \(ThreeScript["\<spacege4.ts\>", spacege]\)], "Input",
  CellLabel->"In[132]:=",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell["\<\
These spheres intersect in the unit circle centered in o in the y,z-plane :\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(ParametricPlot3D[{0, Cos[t], \ Sin[t]}, {t, 0, 2\ \[Pi]}]\)], "Input",
  CellLabel->"In[32]:=",
  PageWidth->Infinity],

Cell[BoxData[
    \(circ1yz = %\)], "Input",
  CellLabel->"In[33]:="],

Cell[BoxData[
    \(Show[{spacege, circ1yz}]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(spacegec\  = %\)], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(MVShow3D[spacegec, \ MVLineTubeSize\  \[Rule]  .01]\)], "Input",
  CellLabel->"In[37]:="],

Cell["\<\
The whole geodesic consists of all the spheres through the fixed circle \
defined by the orthogonal complement of
linearhull{{Cos[t], 0, 0, Sin[t], 0}, { Cos[t + Pi/2], 0, 0, Sin[t + \
Pi/2],0}} = linearhull{stb[1], stb[4]}. 
As a special case of a spacelike geodesic we get the bundle of planes through \
a line. Since the space of all circles is six-dimensional, we can state: The \
space of all spacelike geodesics in the sphere space is a six-dimensional \
homogeneous space under the action of the M\[ODoubleDot]bius group, \
equivariant isomorphic to the space of all circles. \
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman",
  FontWeight->"Plain",
  FontSlant->"Plain",
  FontTracking->"Plain",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1],
  FontVariations->{"Underline"->False,
  "Outline"->False,
  "Shadow"->False}],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[\[IndentingNewLine]{Sin[t], 0, 0, Cos[t], 
          0}, \(-1\), 0, \(-1\), 1], \[IndentingNewLine]{t, 0, Pi, 
        Pi/8}]\)], "Input",
  CellLabel->"In[38]:=",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(spacegeo1\  = \ %\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(MVShow3D[spacegeo1]\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(Show[{spacegeo1, circ1yz}]\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(spacegeo2\  = \ %\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(MVShow3D[spacegeo2, \ MVLineTubeSize\  \[Rule]  .05]\)], "Input",
  CellLabel->"In[46]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["5.4 Isotropic geodesics", "Subsection",
  Evaluatable->False,
  FontSize->16],

Cell["\<\
An isotropic geodesic through stb[4] is the intersection of H with the \
isotropic plane through the origin, spanned by the vectors stb[4] and stb[1] \
+ stb[5]. This is a generating line contained in the hyperboloid H, with \
parameter representation
 {t,0,0,1, t}.
The starting point: stb[4], the unit sphere with centre o, and any other \
sphere of this geodesic have the isotropic span:
linearhull[{1,0,0,0,1}, {0,0,0,1,0}], and therefore they intersect in the \
point\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[{
    \(\(cpt\  = {1, 0, 0, 0, 1};\)\), "\n", 
    \(stepro[cpt]\)}], "Input",
  CellLabel->"In[47]:="],

Cell["\<\
Because spheres which intersect in exactly one point must have a common \
tangential plane at this point,  the isotropic geodesic under consideration \
is the 1-parameter family of spheres through the point {1,0,0} having there \
the same tangent plane as the unitsphere with centre o, and this is the plane \
cutting the x-axis orthogonally. This tangent plane corresponds to the \
spacelike vector\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(planev[1, 0, 0, 1]\)], "Input",
  CellLabel->"In[49]:="],

Cell["\<\
and belongs to the geodesic. If vec[n] is a spacelike unit vector the \
corresponding sphere of which is tangent to the unit sphere at the point \
{1,0,0}, it must be a solution of the following system\
\>", "Text"],

Cell[BoxData[
    \(Solve[{pssp[cpt, {x1, x2, x3, x4, x5}] == 
          0, \n\t\tpssp[stb[4], {x1, x2, x3, x4, x5}] == 1}, \n\t{x1, x2, x3, 
        x4, x5}]\)], "Input",
  CellLabel->"In[50]:=",
  PageWidth->Infinity],

Cell["The general solution is", "Text"],

Cell[BoxData[
    \(vs = \ {t, a, b, 1, t}\)], "Input",
  CellLabel->"In[78]:="],

Cell[BoxData[
    \(pssp[vs, vs]\)], "Input",
  CellLabel->"In[79]:="],

Cell["\<\
Since vs should be a spacelike unit vector it follows a=b=0: The isotropic \
geodesic consists exactly of all spheres tangent to the unit sphere at the \
point {1,0,0}.\
\>", "Text"],

Cell["Here are 5 spheres on this isotropic geodesic:", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[{t, 0, 0, 1, t}, 0, 1, \ \(-1\), 
        1, \[IndentingNewLine]PlotPoints -> \ 20], {t, \(-3\), 3}]\)], "Input",\

  CellLabel->"In[51]:=",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[52]:=",
  PageWidth->Infinity],

Cell[BoxData[
    \(isogeod1 = %\)], "Input",
  CellLabel->"In[53]:="],

Cell[BoxData[
    \(MVShow3D[isogeod1]\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \(ThreeScript["\<isoge1.ts\>", isogeod1]\)], "Input",
  CellLabel->"In[12]:=",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell["\<\
Another isotropic geodesic through the unit sphere in the linear hull under \
consideration is given by the spacelike vectors\
\>", "Text"],

Cell[BoxData[
    \({t, 0, 0, 1, \(-t\)}\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(Table[
      euklidsphereplot3D[{t, 0, 0, 1, \(-t\)}, \(-1\), 0, \ \(-1\), 
        1, \[IndentingNewLine]PlotPoints -> \ 20], {t, \(-3\), 3}]\)], "Input",\

  CellLabel->"In[55]:=",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell[BoxData[
    \(Show[%]\)], "Input",
  CellLabel->"In[56]:="],

Cell["\<\
The geodesic consists of all spheres having at the common point \
\>", "Text"],

Cell[BoxData[
    \(stepro[{1, 0, \ 0, \ 0, \ \(-1\)}]\)], "Input",
  CellLabel->"In[57]:="],

Cell["\<\
 the plane parallel to the y,z-plane as its common tangential plane. \
Obviously, there exists a 2-parameter family of isotropic geodesics through \
stb[4], corresponding to the points of the sphere, i.e. to the generators of \
the isotropic cone in the 4-dimensional tangential hyperplane of the \
hyper-hyperboloid of spacelike unit vectors at stb[4]. The set of all \
isotropic geodesics is a five-dimensional homogeneous manifold under the \
action of the M\[ODoubleDot]bius group.\
\>", "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["6 ThreeScript, MathGL3d", "Section"],

Cell[TextData[{
  "If you want to generate *.ts files for animations outside of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", activate the next section. To use the animation program MathGL3d in this \
notebook, activate section 6.2."
}], "Text"],

Cell[CellGroupData[{

Cell["6.1 Threescript ", "Subsection"],

Cell["\<\
 The ThreeScript commands generate *.ts files appropriate  for certain \
animation programs, e.g. the  freeware program  MathGL3d  created by \
Jens-Peer Kuska [2] (for Windows95, Linux and other platforms).\
\>", "Text",
  Evaluatable->False],

Cell["\<\
Under Windows the commercial 3D-Animation programs MathLive and \
DynamicVisualizer are available. To produce .ts-files one \
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(Needs["\<Graphics`ThreeScript`\>"]\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(\(?ThreeScript\)\)], "Input",
  CellLabel->"In[32]:="],

Cell["\<\
The standard extension of these files is ts: \"filename.ts\" or \
\"Path/filename.ts\", e.g.\
\>", "Text",
  Evaluatable->False],

Cell[BoxData[
    \(ThreeScript["\<tangr.ts\>", \ tangr3]\)], "Input",
  CellLabel->"In[18]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["6.2 MathGL3d", "Subsection"],

Cell[TextData[{
  "The free animation program MathGL3d by Jens-Peer Kuska [2] can be used \
from a command line and within a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session. A new version of this program is in preparation. Jens-Peer Kuska \
has sent me a new startup file OpenGLViewer.m, and friendly permitted to add \
it to the item \"Spheres\". Replace the old file OpenGLViewer.m by this new \
OpenGLViewer.m. If you did this, changes of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s ParametricPlot3D, especially the sometimes disturbing generating of \
MeshGraphics3D objects, will not take place.\nEstablish the link to the \
current ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session with the command:"
}], "Text"],

Cell[BoxData[
    \(Get["\<MathGL3d`OpenGLViewer`\>"]\)], "Input",
  CellLabel->"In[14]:="],

Cell["If you want to close the link, enter", "Text"],

Cell["MVClose[]", "Text",
  CellLabel->"In[44]:="],

Cell["\<\
The following command clears the Viewer window; this and other MathGL3d \
commands you may enter also with  palettes.\
\>", "Text"],

Cell[BoxData[{
    \(\(MVClear[];\)\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(MVShow3D[\[Placeholder], MVPolygonShading \[Rule] MVSmooth, 
      MVGrayBackground \[Rule] False, MVTexture \[Rule] None, 
      MVTextureMapType \[Rule] MVMeshUVMapping, 
      MVScaleTexture \[Rule] {1, 1}, 
      MVTranslateTextureTarget \[Rule] Automatic, 
      MVRotateTextureTarget \[Rule] Automatic, 
      MVScaleTextureTarget \[Rule] Automatic]\)}], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(MVShow3D[gre1\[IndentingNewLine], MVPointSphereSize \[Rule] 0.8, \ 
      MVLineTubeSize\  -> \ 0.03]\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(MVShow3D[tgr3, \ MVPointSphereSize\  \[Rule] \ 1.8]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(MVShow3D[ex123, MVPointSphereSize \[Rule] 1.8]\)], "Input",
  CellLabel->"In[69]:="],

Cell[BoxData[
    \(MVShow3D[isogeod1]\)], "Input",
  CellLabel->"In[11]:="],

Cell[BoxData[
    \(Options[MVShow3D]\)], "Input",
  CellLabel->"In[56]:="],

Cell[BoxData[
    \(MVClose[]\)], "Input",
  CellLabel->"In[77]:="]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Appendix", "Section"],

Cell[CellGroupData[{

Cell["Obsolete: plotsphere", "Subsection"],

Cell["\<\
A procedure which directly plots the sphere corresponding to the spacelike \
vector nv is defined in mspher.m as follows:\
\>", "Text"],

Cell[BoxData[
    \(\(?plotsphere\)\)], "Input",
  CellLabel->"In[75]:="],

Cell[BoxData[
    \(plotsphere[nv_List, \ opts___]\  := \ 
      Module[{a, \ p, \ r, \ m4\  = \ nul[4], \ z\  = \ nul[4], \ \n\tvx, \ 
          tt\  = \ IdentityMatrix[4], \ u, \ v}, \n\t
        a\  = \ pssp[nv, \ nv]; \n\t
        If[Length[nv]\  != \ 5\  || \ a\  <= \ 0, \ 
          Return[Print[\n\t\t\t\t"\<Vector \>", \ 
              nv, \ "\<: incorrect argument!\>"]]]; \n\t
        m4\  = \ Take[nv, \ 4]; \ a\  = \ m4\  . \ m4; \ 
        m4\  = \ m4/Sqrt[a]; \n\tp\  = \ nv[\([5]\)]/Sqrt[a]; \ 
        r\  = \ Sqrt[1\  - \ p^2]; \n\tz\  = \ p\ m4; \ \n\t
        tt\  = \ orthoframe[m4]; \n\t
        vx[u_, \ v_]\  := \ 
          z\  + \ \((\((tt[\([1]\)]\ Cos[u]\  + \ tt[\([2]\)]\ Sin[u])\)\ Cos[
                      v]\  + \ \n\t\ttt[\([3]\)]\ Sin[v])\)\ r; \n\t
        ParametricPlot3D[
          sterproj[vx[u, \ v]], \ \ {u, \ 0, \ 2\ Pi}, \ {v, \ \(-Pi\)/2, \ 
            Pi/2}, \ opts]]\)], "Input",
  CellLabel->"In[68]:=",
  Evaluatable->False],

Cell[BoxData[
    \(\(?orthoframe\)\)], "Input",
  CellLabel->"In[76]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Moving frame on the sphere manifold:pstg5frame", "Subsection"],

Cell["\<\
We construct a pseudo-orthogonal moving 5-frame of the hypersurface (H), \
using its parameter representation spunit. We follow the procedure which us \
led to the definition of tg4frame, see the notebook euspher.nb:\
\>", "Text"],

Cell[BoxData[
    \(pstg1[s_, t_, u_, v_]\  := \ 
      normalize[D[spunit[x, \ t, u, v], x] /. x\  -> \ s]\)], "Input",
  CellLabel->"In[58]:="],

Cell[BoxData[
    \(\(\(pstg1[s, t, u, v]\)\(\ \)\)\)], "Input",
  CellLabel->"In[59]:="],

Cell[BoxData[
    \(pstg2[s_, t_, u_, v_]\  := \ 
      normalize[D[spunit[s, \ x, u, v], x] /. x\  -> \ t]\)], "Input",
  CellLabel->"In[60]:="],

Cell[BoxData[
    \(\(\(pstg2[s, t, u, v]\)\(\ \)\)\)], "Input",
  CellLabel->"In[61]:="],

Cell[BoxData[
    \(pstg3[s_, t_, u_, v_]\  := \ 
      normalize[D[spunit[s, \ t, x, v], x] /. x\  -> \ u]\)], "Input",
  CellLabel->"In[62]:="],

Cell[BoxData[
    \(\(\(pstg3[s, t, u, v]\)\(\ \)\)\)], "Input",
  CellLabel->"In[63]:="],

Cell[BoxData[
    \(pstg4[s_, t_, u_, v_]\  := \ 
      normalize[D[spunit[s, \ t, u, x], x] /. x\  -> \ v]\)], "Input",
  CellLabel->"In[64]:="],

Cell[BoxData[
    \(\(\(pstg4[s, t, u, v]\)\(\ \)\)\)], "Input",
  CellLabel->"In[65]:="],

Cell[BoxData[
    \(spunit[s, t, u, v]\)], "Input",
  CellLabel->"In[66]:="],

Cell["\<\
Simplifying these vectors we defined in mspher.m the moving orthonormal \
tangentially adapted  5-frame of the spacelike unitsphere in the \
pseudo-euclidean 5-space,  analogous to tg4frame:\
\>", "Text"],

Cell[BoxData[
    \(\(?pstg5frame\)\)], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[
    \(pstg5frame[s_, \ t_, \ u_, \ 
        v_]\  := \n\t{{\(-\ Sin[s]\), Cos[s], 0, 0, 
          0}, \n\t\t{\(-Cos[s]\)\ Sin[t], \(-\ Sin[s]\)\ Sin[t], Cos[t], 0, 
          0}, \n\t\t{\(-Cos[s]\)\ Cos[t]\ \ Sin[u], \(-Cos[t]\)\ \ Sin[
              s]\ Sin[u], \(-\ Sin[t]\)\ Sin[u], Cos[u], 
          0}, \n\t\t{Cos[s]\ Cos[t]\ Cos[u]\ Sinh[v], 
          Cos[t]\ Cos[u]\ Sin[s]\ Sinh[v], Cos[u]\ Sin[t]\ Sinh[v], 
          Sin[u]\ Sinh[v], Cosh[v]}, \n\t\t{Cos[s]\ Cos[t]\ Cos[u]\ Cosh[v], 
          Cos[t]\ Cos[u]\ Cosh[v]\ Sin[s], Cos[u]\ Cosh[v]\ Sin[t], 
          Cosh[v]\ Sin[u], Sinh[v]}}\)], "Input",
  CellLabel->"In[43]:=",
  Evaluatable->False],

Cell[BoxData[
    \(\(pst5\  = pstg5frame[s, \ t, \ u, \ v];\)\)], "Input",
  CellLabel->"In[68]:="],

Cell[BoxData[
    \(Table[
      Simplify[pssp[pst5[\([i]\)], \ pst5[\([j]\)]]], \ {i, 5}, {j, 
        5}]\)], "Input",
  CellLabel->"In[69]:="],

Cell[BoxData[
    \(Simplify[Det[pst5]]\)], "Input",
  CellLabel->"In[70]:="],

Cell[BoxData[
    \(spunit[s, t, u, v]\  + \ 
      spunit[s + Pi, \(-t\), \(-u\), \(-v\)]\)], "Input",
  CellLabel->"In[71]:="],

Cell["\<\
We conclude that changing the sign of v = spunit[s,t,u,v] changes the \
orientation of the orthogonal pseudo-euclidean subspace, if we consider the \
pseudo-euclidean 5-space as oriented. \
\>", "Text"],

Cell["\<\
The isotropic vectors in the tangential space spanned by (pstg5frame[[i]], i \
= 1,...,4) are:\
\>", "Text"],

Cell[BoxData[
    \(\(isotr[s_, t_, u_, v_]\)[x_, 
        y_]\  := \n\(\(\(pstg5frame[s, \ t, \ u, \ v]\)[\([1]\)] 
          Cos[x]\ Cos[
            y]\  + \ \n\(pstg5frame[s, \ t, \ u, \ v]\)[\([2]\)]\ Sin[x]\ Cos[
            y]\  + \n\(pstg5frame[s, \ t, \ u, \ v]\)[\([3]\)]\ Sin[
            y]\  + \n\(pstg5frame[s, \ t, \ u, \ 
            v]\)[\([4]\)]\)\(\n\)\(\t\)\)\)], "Input",
  CellLabel->"In[72]:="],

Cell[BoxData[
    \(iso\  = \ Simplify[\(isotr[s, t, u, v]\)[x, y]]\)], "Input",
  CellLabel->"In[73]:="],

Cell[BoxData[
    \(Simplify[pssp[iso, iso]]\)], "Input",
  CellLabel->"In[74]:="]
}, Closed]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 695}},
AutoGeneratedPackage->Automatic,
WindowToolbars->{},
WindowSize->{827, 656},
WindowMargins->{{58, Automatic}, {Automatic, -1}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PaperSize"->{597.563, 842.375},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{"home", "rolf", "mymath", \
"spheres.v3"}, "msphere.nb.ps", CharacterEncoding -> "WindowsANSI"],
"Magnification"->1},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
StyleDefinitions -> "TutorialBook.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 136, 3, 73, "Title",
  Evaluatable->False],
Cell[1878, 56, 225, 8, 45, "Text",
  Evaluatable->False],
Cell[2106, 66, 125, 4, 39, "Text",
  Evaluatable->False],
Cell[2234, 72, 165, 4, 28, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[2424, 80, 32, 0, 37, "Subsubsection"],
Cell[2459, 82, 1300, 20, 264, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[3796, 107, 35, 0, 31, "Subsubsection"],
Cell[3834, 109, 198, 5, 60, "Text"],
Cell[4035, 116, 499, 14, 48, "Text",
  Evaluatable->False],
Cell[4537, 132, 103, 3, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[4677, 140, 33, 0, 31, "Subsubsection"],
Cell[4713, 142, 431, 7, 68, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[5181, 154, 32, 0, 31, "Subsubsection"],
Cell[5216, 156, 47, 0, 28, "Text"],
Cell[5266, 158, 90, 2, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[5393, 165, 33, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[5451, 169, 41, 0, 44, "Subsection"],
Cell[5495, 171, 360, 12, 252, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[5892, 188, 45, 0, 38, "Subsection"],
Cell[5940, 190, 334, 7, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[6311, 202, 33, 0, 38, "Subsection"],
Cell[6347, 204, 205, 5, 61, "Text"],
Cell[6555, 211, 56, 0, 28, "Text"],
Cell[6614, 213, 68, 2, 31, "Input"],
Cell[6685, 217, 282, 5, 48, "Text"],
Cell[6970, 224, 113, 2, 31, "Input"],
Cell[7086, 228, 117, 3, 28, "Text"],
Cell[7206, 233, 77, 2, 31, "Input"],
Cell[7286, 237, 130, 3, 28, "Text"],
Cell[7419, 242, 74, 2, 31, "Input"],
Cell[7496, 246, 156, 3, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7689, 254, 65, 0, 44, "Subsection"],
Cell[7757, 256, 190, 5, 61, "Text"],
Cell[7950, 263, 173, 3, 48, "Text"],
Cell[8126, 268, 77, 2, 31, "Input"],
Cell[8206, 272, 156, 3, 32, "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8411, 281, 34, 0, 92, "Section"],

Cell[CellGroupData[{
Cell[8470, 285, 117, 2, 43, "Subsection"],
Cell[8590, 289, 240, 4, 48, "Text"],
Cell[8833, 295, 153, 4, 52, "Text"],
Cell[8989, 301, 1071, 15, 168, "Text"],
Cell[10063, 318, 112, 3, 31, "Input",
  InitializationCell->True],
Cell[10178, 323, 436, 8, 68, "Text",
  Evaluatable->False],
Cell[10617, 333, 102, 3, 28, "Text"],
Cell[10722, 338, 66, 2, 31, "Input"],
Cell[10791, 342, 85, 2, 28, "Text"],
Cell[10879, 346, 142, 3, 31, "Input",
  InitializationCell->True],
Cell[11024, 351, 380, 7, 68, "Text",
  Evaluatable->False],
Cell[11407, 360, 96, 2, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[11540, 367, 36, 0, 38, "Subsection"],
Cell[11579, 369, 186, 5, 28, "Text",
  Evaluatable->False],
Cell[11768, 376, 72, 2, 31, "Input"],
Cell[11843, 380, 85, 2, 28, "Text"],
Cell[11931, 384, 64, 2, 31, "Input"],
Cell[11998, 388, 312, 7, 48, "Text",
  Evaluatable->False],
Cell[12313, 397, 223, 4, 48, "Text"],
Cell[12539, 403, 66, 2, 31, "Input"],
Cell[12608, 407, 69, 2, 31, "Input"],
Cell[12680, 411, 72, 2, 31, "Input"],
Cell[12755, 415, 70, 2, 31, "Input"],
Cell[12828, 419, 69, 2, 31, "Input"],
Cell[12900, 423, 73, 2, 31, "Input"],
Cell[12976, 427, 75, 2, 31, "Input"],
Cell[13054, 431, 81, 2, 31, "Input"],
Cell[13138, 435, 69, 2, 31, "Input"],
Cell[13210, 439, 73, 2, 31, "Input"],
Cell[13286, 443, 70, 2, 31, "Input"],
Cell[13359, 447, 70, 2, 31, "Input"],
Cell[13432, 451, 71, 2, 31, "Input"],
Cell[13506, 455, 69, 2, 31, "Input"],
Cell[13578, 459, 69, 2, 31, "Input"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[13696, 467, 72, 0, 92, "Section"],

Cell[CellGroupData[{
Cell[13793, 471, 73, 0, 44, "Subsection"],
Cell[13869, 473, 126, 3, 28, "Text"],
Cell[13998, 478, 157, 6, 28, "Text"],
Cell[14158, 486, 385, 6, 68, "Text"],
Cell[14546, 494, 83, 1, 28, "Text"],
Cell[14632, 497, 90, 2, 28, "Text"],
Cell[14725, 501, 66, 2, 31, "Input"],
Cell[14794, 505, 318, 8, 77, "Input",
  Evaluatable->False],
Cell[15115, 515, 233, 4, 48, "Text"],
Cell[15351, 521, 71, 2, 31, "Input"],
Cell[15425, 525, 40, 0, 28, "Text"],
Cell[15468, 527, 69, 2, 31, "Input"],
Cell[15540, 531, 93, 2, 31, "Input",
  Evaluatable->False],
Cell[15636, 535, 54, 0, 28, "Text"],
Cell[15693, 537, 105, 2, 31, "Input",
  Evaluatable->False],
Cell[15801, 541, 72, 2, 31, "Input"],
Cell[15876, 545, 74, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[15987, 552, 34, 0, 38, "Subsection"],
Cell[16024, 554, 90, 2, 31, "Input"],
Cell[16117, 558, 76, 2, 35, "Input",
  InitializationCell->True],
Cell[16196, 562, 53, 1, 35, "Input"],
Cell[16252, 565, 60, 1, 35, "Input"],
Cell[16315, 568, 53, 1, 35, "Input"],
Cell[16371, 571, 101, 2, 31, "Input"],
Cell[16475, 575, 104, 2, 31, "Input"],
Cell[16582, 579, 70, 2, 31, "Input"],
Cell[16655, 583, 69, 2, 31, "Input"],
Cell[16727, 587, 228, 4, 48, "Text"],
Cell[16958, 593, 159, 4, 31, "Input"],
Cell[17120, 599, 66, 2, 31, "Input"],
Cell[17189, 603, 70, 2, 31, "Input"],
Cell[17262, 607, 48, 0, 28, "Text"],
Cell[17313, 609, 82, 2, 31, "Input"],
Cell[17398, 613, 83, 2, 31, "Input"],
Cell[17484, 617, 86, 2, 31, "Input"],
Cell[17573, 621, 71, 2, 31, "Input"],
Cell[17647, 625, 74, 2, 31, "Input"],
Cell[17724, 629, 155, 3, 28, "Text"],
Cell[17882, 634, 69, 2, 31, "Input"],
Cell[17954, 638, 86, 2, 31, "Input"],
Cell[18043, 642, 70, 2, 31, "Input"],
Cell[18116, 646, 52, 1, 35, "Input"],
Cell[18171, 649, 75, 2, 31, "Input"],
Cell[18249, 653, 75, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[18361, 660, 59, 0, 38, "Subsection"],
Cell[18423, 662, 267, 5, 48, "Text"],
Cell[18693, 669, 97, 2, 31, "Input"],
Cell[18793, 673, 80, 2, 31, "Input"],
Cell[18876, 677, 56, 0, 28, "Text"],
Cell[18935, 679, 90, 2, 31, "Input"],
Cell[19028, 683, 45, 0, 28, "Text"],
Cell[19076, 685, 106, 2, 31, "Input"],
Cell[19185, 689, 56, 0, 28, "Text"],
Cell[19244, 691, 94, 2, 31, "Input"],
Cell[19341, 695, 79, 2, 31, "Input"],
Cell[19423, 699, 172, 3, 32, "Input"],
Cell[19598, 704, 57, 0, 28, "Text"],
Cell[19658, 706, 246, 7, 54, "Input"],
Cell[19907, 715, 97, 2, 28, "Text"],
Cell[20007, 719, 305, 6, 54, "Input",
  Evaluatable->False],
Cell[20315, 727, 112, 4, 28, "Text"],
Cell[20430, 733, 81, 2, 31, "Input"],
Cell[20514, 737, 68, 0, 28, "Text"],
Cell[20585, 739, 89, 2, 31, "Input"],
Cell[20677, 743, 40, 0, 28, "Text"],
Cell[20720, 745, 115, 2, 31, "Input"],
Cell[20838, 749, 209, 4, 48, "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[21096, 759, 28, 0, 92, "Section"],
Cell[21127, 761, 291, 5, 48, "Text"],

Cell[CellGroupData[{
Cell[21443, 770, 131, 3, 44, "Subsection"],

Cell[CellGroupData[{
Cell[21599, 777, 99, 2, 37, "Subsubsection"],
Cell[21701, 781, 193, 4, 48, "Text"],
Cell[21897, 787, 70, 2, 31, "Input"],
Cell[21970, 791, 658, 13, 238, "Input",
  Evaluatable->False],
Cell[22631, 806, 178, 3, 48, "Text"],
Cell[22812, 811, 71, 2, 31, "Input"],
Cell[22886, 815, 85, 2, 31, "Input"],
Cell[22974, 819, 70, 2, 31, "Input"],
Cell[23047, 823, 79, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[23163, 830, 47, 0, 31, "Subsubsection"],
Cell[23213, 832, 179, 4, 60, "Text"],
Cell[23395, 838, 126, 3, 31, "Input"],
Cell[23524, 843, 79, 2, 31, "Input"],
Cell[23606, 847, 158, 3, 28, "Text"],
Cell[23767, 852, 116, 2, 31, "Input"],
Cell[23886, 856, 79, 2, 31, "Input"],
Cell[23968, 860, 133, 3, 31, "Input"],
Cell[24104, 865, 79, 2, 31, "Input"],
Cell[24186, 869, 92, 2, 28, "Text"],
Cell[24281, 873, 131, 3, 31, "Input"],
Cell[24415, 878, 169, 3, 54, "Input"],
Cell[24587, 883, 68, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[24692, 890, 113, 3, 31, "Subsubsection"],
Cell[24808, 895, 190, 4, 48, "Text"],
Cell[25001, 901, 191, 4, 54, "Input"],
Cell[25195, 907, 280, 5, 55, "Input"],
Cell[25478, 914, 74, 2, 31, "Input"],
Cell[25555, 918, 93, 2, 28, "Text"],
Cell[25651, 922, 229, 5, 54, "Input",
  Evaluatable->False],
Cell[25883, 929, 295, 6, 55, "Input"],
Cell[26181, 937, 79, 2, 31, "Input"],
Cell[26263, 941, 121, 3, 31, "Input"],
Cell[26387, 946, 100, 2, 31, "Input"],
Cell[26490, 950, 45, 0, 28, "Text"],
Cell[26538, 952, 79, 2, 31, "Input"],
Cell[26620, 956, 118, 3, 28, "Text"],
Cell[26741, 961, 128, 3, 31, "Input",
  InitializationCell->True],
Cell[26872, 966, 76, 2, 31, "Input"],
Cell[26951, 970, 98, 2, 31, "Input"],
Cell[27052, 974, 96, 2, 31, "Input"],
Cell[27151, 978, 129, 3, 31, "Input"],
Cell[27283, 983, 79, 2, 31, "Input"],
Cell[27365, 987, 84, 2, 31, "Input"],
Cell[27452, 991, 71, 2, 31, "Input"],
Cell[27526, 995, 70, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[27633, 1002, 135, 3, 31, "Subsubsection"],
Cell[27771, 1007, 32, 0, 28, "Text"],
Cell[27806, 1009, 86, 2, 31, "Input"],
Cell[27895, 1013, 27, 0, 28, "Text"],
Cell[27925, 1015, 106, 2, 31, "Input"],
Cell[28034, 1019, 415, 7, 100, "Text"],
Cell[28452, 1028, 107, 2, 31, "Input"],
Cell[28562, 1032, 106, 3, 28, "Text"],
Cell[28671, 1037, 65, 2, 31, "Input"],
Cell[28739, 1041, 87, 2, 28, "Text"],
Cell[28829, 1045, 159, 3, 54, "Input"],
Cell[28991, 1050, 26, 0, 28, "Text"],
Cell[29020, 1052, 91, 2, 31, "Input"],
Cell[29114, 1056, 169, 3, 48, "Text"],
Cell[29286, 1061, 555, 11, 169, "Input",
  Evaluatable->False],
Cell[29844, 1074, 508, 10, 123, "Input",
  Evaluatable->False],
Cell[30355, 1086, 93, 3, 60, "Text"],
Cell[30451, 1091, 83, 2, 31, "Input"],
Cell[30537, 1095, 73, 2, 31, "Input"],
Cell[30613, 1099, 114, 3, 28, "Text"],
Cell[30730, 1104, 74, 2, 31, "Input"],
Cell[30807, 1108, 83, 2, 31, "Input"],
Cell[30893, 1112, 61, 0, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[30991, 1117, 36, 0, 37, "Subsubsection"],
Cell[31030, 1119, 72, 0, 28, "Text"],
Cell[31105, 1121, 79, 2, 31, "Input"],
Cell[31187, 1125, 84, 2, 31, "Input"],
Cell[31274, 1129, 931, 19, 128, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[32242, 1153, 76, 0, 37, "Subsubsection"],
Cell[32321, 1155, 205, 4, 48, "Text"],
Cell[32529, 1161, 70, 2, 31, "Input"],
Cell[32602, 1165, 101, 2, 31, "Input"],
Cell[32706, 1169, 80, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[32823, 1176, 38, 0, 37, "Subsubsection"],
Cell[32864, 1178, 83, 2, 31, "Input"],
Cell[32950, 1182, 83, 2, 31, "Input"],
Cell[33036, 1186, 76, 2, 31, "Input"],
Cell[33115, 1190, 76, 2, 31, "Input"],
Cell[33194, 1194, 76, 2, 31, "Input"],
Cell[33273, 1198, 75, 0, 28, "Text"],
Cell[33351, 1200, 107, 2, 31, "Input"],
Cell[33461, 1204, 227, 5, 77, "Input"],
Cell[33691, 1211, 66, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[33806, 1219, 32, 0, 38, "Subsection"],
Cell[33841, 1221, 206, 4, 48, "Text"],

Cell[CellGroupData[{
Cell[34072, 1229, 46, 0, 37, "Subsubsection"],
Cell[34121, 1231, 69, 2, 31, "Input"],
Cell[34193, 1235, 139, 3, 28, "Text"],
Cell[34335, 1240, 835, 15, 330, "Input"],
Cell[35173, 1257, 69, 0, 28, "Text"],
Cell[35245, 1259, 90, 2, 31, "Input"],
Cell[35338, 1263, 239, 4, 54, "Input"],
Cell[35580, 1269, 135, 3, 31, "Input"],
Cell[35718, 1274, 69, 2, 31, "Input"],
Cell[35790, 1278, 103, 3, 28, "Text"],
Cell[35896, 1283, 266, 6, 77, "Input",
  Evaluatable->False],
Cell[36165, 1291, 76, 2, 31, "Input"],
Cell[36244, 1295, 76, 2, 31, "Input"],
Cell[36323, 1299, 55, 0, 28, "Text"],
Cell[36381, 1301, 77, 2, 31, "Input"],
Cell[36461, 1305, 92, 2, 31, "Input"],
Cell[36556, 1309, 40, 0, 28, "Text"],
Cell[36599, 1311, 84, 2, 31, "Input"],
Cell[36686, 1315, 109, 2, 31, "Input"],
Cell[36798, 1319, 458, 7, 88, "Text"],
Cell[37259, 1328, 624, 13, 192, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[37920, 1346, 33, 0, 37, "Subsubsection"],
Cell[37956, 1348, 95, 2, 31, "Input"],
Cell[38054, 1352, 70, 2, 31, "Input"],
Cell[38127, 1356, 77, 2, 31, "Input"],
Cell[38207, 1360, 119, 3, 31, "Input"],
Cell[38329, 1365, 83, 2, 31, "Input"],
Cell[38415, 1369, 76, 2, 31, "Input"],
Cell[38494, 1373, 119, 3, 31, "Input"],
Cell[38616, 1378, 67, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[38720, 1385, 55, 0, 37, "Subsubsection"],
Cell[38778, 1387, 201, 4, 48, "Text"],
Cell[38982, 1393, 755, 13, 238, "Input"],
Cell[39740, 1408, 109, 2, 31, "Input"],
Cell[39852, 1412, 104, 2, 31, "Input"],
Cell[39959, 1416, 76, 2, 31, "Input"],
Cell[40038, 1420, 119, 3, 31, "Input"],
Cell[40160, 1425, 107, 2, 31, "Input"],
Cell[40270, 1429, 76, 2, 31, "Input"],
Cell[40349, 1433, 119, 3, 31, "Input"],
Cell[40471, 1438, 89, 2, 28, "Text"],
Cell[40563, 1442, 68, 2, 31, "Input"],
Cell[40634, 1446, 72, 0, 28, "Text"],
Cell[40709, 1448, 106, 2, 31, "Input"],
Cell[40818, 1452, 76, 2, 31, "Input"],
Cell[40897, 1456, 119, 3, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[41065, 1465, 83, 1, 38, "Subsection"],
Cell[41151, 1468, 255, 5, 48, "Text"],

Cell[CellGroupData[{
Cell[41431, 1477, 86, 1, 37, "Subsubsection"],
Cell[41520, 1480, 856, 16, 261, "Input",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[42413, 1501, 33, 0, 37, "Subsubsection"],
Cell[42449, 1503, 88, 2, 31, "Input"],
Cell[42540, 1507, 88, 2, 31, "Input"],
Cell[42631, 1511, 88, 2, 31, "Input"],
Cell[42722, 1515, 88, 2, 31, "Input"],
Cell[42813, 1519, 88, 2, 31, "Input"],
Cell[42904, 1523, 115, 3, 28, "Text"],
Cell[43022, 1528, 176, 4, 54, "Input"],
Cell[43201, 1534, 426, 7, 68, "Text"],
Cell[43630, 1543, 209, 5, 54, "Input"],
Cell[43842, 1550, 62, 2, 31, "Input"],
Cell[43907, 1554, 75, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[44019, 1561, 58, 0, 31, "Subsubsection"],
Cell[44080, 1563, 74, 0, 28, "Text"],
Cell[44157, 1565, 282, 7, 77, "Input",
  Evaluatable->False],
Cell[44442, 1574, 85, 2, 28, "Text"],
Cell[44530, 1578, 73, 2, 31, "Input"],
Cell[44606, 1582, 87, 2, 28, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[44742, 1590, 47, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[44814, 1594, 58, 0, 37, "Subsubsection"],
Cell[44875, 1596, 146, 4, 31, "Input"],
Cell[45024, 1602, 74, 2, 31, "Input"],
Cell[45101, 1606, 154, 4, 31, "Input"],
Cell[45258, 1612, 74, 2, 31, "Input"],
Cell[45335, 1616, 69, 2, 31, "Input"],
Cell[45407, 1620, 119, 3, 31, "Input"],
Cell[45529, 1625, 540, 8, 88, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[46106, 1638, 61, 0, 31, "Subsubsection"],
Cell[46170, 1640, 101, 2, 31, "Input"],
Cell[46274, 1644, 74, 2, 31, "Input"],
Cell[46351, 1648, 109, 2, 31, "Input"],
Cell[46463, 1652, 74, 2, 31, "Input"],
Cell[46540, 1656, 72, 2, 31, "Input"],
Cell[46615, 1660, 149, 4, 31, "Input"],
Cell[46767, 1666, 98, 2, 31, "Input"],
Cell[46868, 1670, 68, 2, 31, "Input"],
Cell[46939, 1674, 119, 3, 31, "Input"],
Cell[47061, 1679, 157, 4, 54, "Input"],
Cell[47221, 1685, 74, 2, 31, "Input"],
Cell[47298, 1689, 134, 3, 28, "Text"],
Cell[47435, 1694, 175, 4, 54, "Input"],
Cell[47613, 1700, 68, 0, 28, "Text"],
Cell[47684, 1702, 76, 2, 31, "Input"],
Cell[47763, 1706, 247, 4, 48, "Text"],
Cell[48013, 1712, 64, 2, 31, "Input"],
Cell[48080, 1716, 159, 4, 31, "Input"],
Cell[48242, 1722, 74, 2, 31, "Input"],
Cell[48319, 1726, 178, 5, 54, "Input"],
Cell[48500, 1733, 74, 2, 31, "Input"],
Cell[48577, 1737, 119, 3, 31, "Input"],
Cell[48699, 1742, 134, 4, 31, "Input"],
Cell[48836, 1748, 176, 5, 54, "Input"],
Cell[49015, 1755, 74, 2, 31, "Input"],
Cell[49092, 1759, 184, 5, 100, "Input"],
Cell[49279, 1766, 74, 2, 31, "Input"],
Cell[49356, 1770, 102, 3, 28, "Text"],
Cell[49461, 1775, 176, 5, 54, "Input"],
Cell[49640, 1782, 74, 2, 31, "Input"],
Cell[49717, 1786, 184, 5, 100, "Input"],
Cell[49904, 1793, 74, 2, 31, "Input"],
Cell[49981, 1797, 109, 3, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[50127, 1805, 68, 0, 37, "Subsubsection"],
Cell[50198, 1807, 143, 4, 60, "Text"],
Cell[50344, 1813, 105, 2, 31, "Input"],
Cell[50452, 1817, 87, 2, 31, "Input"],
Cell[50542, 1821, 145, 4, 31, "Input"],
Cell[50690, 1827, 212, 6, 28, "Text"],
Cell[50905, 1835, 120, 3, 31, "Input"],
Cell[51028, 1840, 98, 2, 31, "Input"],
Cell[51129, 1844, 120, 3, 31, "Input"],
Cell[51252, 1849, 384, 8, 68, "Text"],
Cell[51639, 1859, 145, 4, 31, "Input"],
Cell[51787, 1865, 155, 5, 28, "Text"],
Cell[51945, 1872, 69, 1, 35, "Input"],
Cell[52017, 1875, 62, 2, 31, "Input"],
Cell[52082, 1879, 81, 1, 35, "Input"],
Cell[52166, 1882, 99, 2, 32, "Input"],
Cell[52268, 1886, 99, 2, 32, "Input"],
Cell[52370, 1890, 57, 0, 28, "Text"],
Cell[52430, 1892, 109, 2, 32, "Input"],
Cell[52542, 1896, 109, 2, 32, "Input"],
Cell[52654, 1900, 166, 3, 60, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[52869, 1909, 70, 0, 38, "Subsection"],

Cell[CellGroupData[{
Cell[52964, 1913, 44, 0, 37, "Subsubsection"],
Cell[53011, 1915, 350, 6, 68, "Text"],
Cell[53364, 1923, 160, 6, 28, "Text"],
Cell[53527, 1931, 184, 4, 48, "Text"],
Cell[53714, 1937, 133, 3, 31, "Input"],
Cell[53850, 1942, 67, 2, 31, "Input"],
Cell[53920, 1946, 71, 2, 31, "Input"],
Cell[53994, 1950, 69, 2, 31, "Input"],
Cell[54066, 1954, 92, 2, 31, "Input"],
Cell[54161, 1958, 269, 5, 48, "Text"],
Cell[54433, 1965, 265, 5, 48, "Text"],
Cell[54701, 1972, 70, 2, 31, "Input"],
Cell[54774, 1976, 141, 4, 31, "Input",
  Evaluatable->False],
Cell[54918, 1982, 67, 0, 28, "Text"],
Cell[54988, 1984, 275, 5, 48, "Text"],
Cell[55266, 1991, 320, 6, 54, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[55623, 2002, 41, 0, 31, "Subsubsection"],
Cell[55667, 2004, 112, 3, 28, "Text"],
Cell[55782, 2009, 434, 7, 179, "Input"],
Cell[56219, 2018, 68, 0, 28, "Text"],
Cell[56290, 2020, 95, 2, 31, "Input"],
Cell[56388, 2024, 60, 0, 28, "Text"],
Cell[56451, 2026, 100, 2, 31, "Input"],
Cell[56554, 2030, 40, 0, 28, "Text"],
Cell[56597, 2032, 100, 2, 31, "Input"],
Cell[56700, 2036, 74, 2, 31, "Input"],
Cell[56777, 2040, 74, 2, 31, "Input"],
Cell[56854, 2044, 219, 5, 60, "Text"],
Cell[57076, 2051, 76, 2, 31, "Input"],
Cell[57155, 2055, 65, 0, 28, "Text"],
Cell[57223, 2057, 90, 2, 31, "Input"],
Cell[57316, 2061, 37, 0, 28, "Text"],
Cell[57356, 2063, 76, 2, 31, "Input"],
Cell[57435, 2067, 133, 3, 28, "Text"],
Cell[57571, 2072, 376, 12, 31, "Input"],
Cell[57950, 2086, 223, 4, 48, "Text"],
Cell[58176, 2092, 95, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[58308, 2099, 44, 0, 31, "Subsubsection"],
Cell[58355, 2101, 71, 0, 28, "Text"],
Cell[58429, 2103, 80, 2, 31, "Input"],
Cell[58512, 2107, 158, 3, 54, "Input"],
Cell[58673, 2112, 46, 0, 28, "Text"],
Cell[58722, 2114, 95, 2, 31, "Input"],
Cell[58820, 2118, 100, 2, 31, "Input"],
Cell[58923, 2122, 103, 2, 31, "Input"],
Cell[59029, 2126, 85, 2, 31, "Input"],
Cell[59117, 2130, 105, 2, 31, "Input"],
Cell[59225, 2134, 104, 2, 31, "Input"],
Cell[59332, 2138, 21, 0, 28, "Text"],
Cell[59356, 2140, 149, 3, 54, "Input"],
Cell[59508, 2145, 103, 3, 28, "Text"],
Cell[59614, 2150, 153, 4, 54, "Input"],
Cell[59770, 2156, 65, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[59872, 2163, 55, 0, 31, "Subsubsection"],
Cell[59930, 2165, 444, 8, 146, "Input"],
Cell[60377, 2175, 228, 4, 48, "Text"],
Cell[60608, 2181, 77, 2, 31, "Input"],
Cell[60688, 2185, 72, 2, 31, "Input"],
Cell[60763, 2189, 95, 2, 31, "Input"],
Cell[60861, 2193, 89, 2, 31, "Input"],
Cell[60953, 2197, 93, 2, 31, "Input"],
Cell[61049, 2201, 104, 2, 31, "Input"],
Cell[61156, 2205, 127, 3, 31, "Input"],
Cell[61286, 2210, 92, 2, 31, "Input"],
Cell[61381, 2214, 142, 4, 31, "Input"],
Cell[61526, 2220, 94, 2, 31, "Input"],
Cell[61623, 2224, 130, 3, 28, "Text"],
Cell[61756, 2229, 130, 3, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[61923, 2237, 41, 0, 31, "Subsubsection"],
Cell[61967, 2239, 676, 10, 108, "Text"],
Cell[62646, 2251, 77, 2, 31, "Input"],
Cell[62726, 2255, 72, 2, 31, "Input"],
Cell[62801, 2259, 95, 2, 31, "Input"],
Cell[62899, 2263, 90, 2, 31, "Input"],
Cell[62992, 2267, 101, 2, 31, "Input"],
Cell[63096, 2271, 95, 2, 31, "Input"],
Cell[63194, 2275, 62, 2, 31, "Input"],
Cell[63259, 2279, 103, 2, 31, "Input"],
Cell[63365, 2283, 102, 2, 31, "Input"],
Cell[63470, 2287, 65, 2, 31, "Input"],
Cell[63538, 2291, 136, 3, 31, "Input"],
Cell[63677, 2296, 80, 2, 31, "Input"],
Cell[63760, 2300, 221, 4, 48, "Text"]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[64042, 2311, 42, 0, 92, "Section"],

Cell[CellGroupData[{
Cell[64109, 2315, 43, 0, 44, "Subsection"],
Cell[64155, 2317, 430, 7, 68, "Text"],
Cell[64588, 2326, 68, 2, 31, "Input"],
Cell[64659, 2330, 170, 4, 54, "Input",
  Evaluatable->False],
Cell[64832, 2336, 263, 5, 48, "Text"],
Cell[65098, 2343, 70, 2, 31, "Input"],
Cell[65171, 2347, 78, 2, 31, "Input"],
Cell[65252, 2351, 106, 2, 31, "Input"],
Cell[65361, 2355, 69, 2, 31, "Input"],
Cell[65433, 2359, 107, 3, 28, "Text"],
Cell[65543, 2364, 113, 2, 31, "Input"],
Cell[65659, 2368, 121, 3, 28, "Text"],
Cell[65783, 2373, 70, 2, 31, "Input"],
Cell[65856, 2377, 74, 2, 31, "Input"],
Cell[65933, 2381, 146, 3, 28, "Text"],
Cell[66082, 2386, 83, 2, 31, "Input"],
Cell[66168, 2390, 188, 5, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[66393, 2400, 46, 0, 38, "Subsection"],
Cell[66442, 2402, 251, 4, 48, "Text"],
Cell[66696, 2408, 113, 2, 31, "Input"],
Cell[66812, 2412, 165, 3, 48, "Text"],

Cell[CellGroupData[{
Cell[67002, 2419, 32, 0, 37, "Subsubsection"],
Cell[67037, 2421, 84, 2, 31, "Input"],
Cell[67124, 2425, 80, 2, 31, "Input"],
Cell[67207, 2429, 70, 2, 31, "Input"],
Cell[67280, 2433, 99, 2, 31, "Input"],
Cell[67382, 2437, 70, 2, 31, "Input"],
Cell[67455, 2441, 76, 2, 31, "Input"],
Cell[67534, 2445, 93, 2, 31, "Input"],
Cell[67630, 2449, 82, 2, 31, "Input"],
Cell[67715, 2453, 70, 2, 31, "Input"],
Cell[67788, 2457, 69, 2, 31, "Input"],
Cell[67860, 2461, 74, 2, 31, "Input"],
Cell[67937, 2465, 216, 4, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[68190, 2474, 44, 0, 31, "Subsubsection"],
Cell[68237, 2476, 98, 2, 28, "Text"],
Cell[68338, 2480, 83, 2, 31, "Input"],
Cell[68424, 2484, 70, 2, 31, "Input"],
Cell[68497, 2488, 83, 2, 31, "Input"],
Cell[68583, 2492, 133, 3, 31, "Input"],
Cell[68719, 2497, 165, 3, 28, "Text"],
Cell[68887, 2502, 87, 2, 31, "Input"],
Cell[68977, 2506, 133, 3, 31, "Input"],
Cell[69113, 2511, 71, 0, 28, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[69233, 2517, 117, 5, 39, "Subsection"],
Cell[69353, 2524, 236, 4, 48, "Text"],
Cell[69592, 2530, 150, 3, 31, "Input"],
Cell[69745, 2535, 77, 2, 31, "Input"],
Cell[69825, 2539, 208, 4, 48, "Text"],
Cell[70036, 2545, 70, 2, 31, "Input"],
Cell[70109, 2549, 165, 4, 31, "Input",
  Evaluatable->False],
Cell[70277, 2555, 73, 0, 28, "Text"],
Cell[70353, 2557, 105, 3, 28, "Text"],
Cell[70461, 2562, 80, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[70578, 2569, 44, 0, 38, "Subsection"],
Cell[70625, 2571, 213, 5, 80, "Text"],

Cell[CellGroupData[{
Cell[70863, 2580, 42, 0, 37, "Subsubsection"],
Cell[70908, 2582, 90, 2, 31, "Input"],
Cell[71001, 2586, 69, 2, 31, "Input"],
Cell[71073, 2590, 79, 2, 31, "Input"],
Cell[71155, 2594, 75, 2, 31, "Input"],
Cell[71233, 2598, 148, 4, 31, "Input"],
Cell[71384, 2604, 73, 2, 31, "Input"],
Cell[71460, 2608, 65, 2, 31, "Input"],
Cell[71528, 2612, 84, 2, 31, "Input"],
Cell[71615, 2616, 66, 2, 31, "Input"],
Cell[71684, 2620, 79, 2, 31, "Input"],
Cell[71766, 2624, 71, 2, 31, "Input"],
Cell[71840, 2628, 69, 2, 31, "Input"],
Cell[71912, 2632, 70, 2, 31, "Input"],
Cell[71985, 2636, 74, 2, 31, "Input"],
Cell[72062, 2640, 96, 2, 31, "Input"],
Cell[72161, 2644, 56, 0, 28, "Text"],
Cell[72220, 2646, 81, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[72338, 2653, 43, 0, 37, "Subsubsection"],
Cell[72384, 2655, 90, 2, 31, "Input"],
Cell[72477, 2659, 69, 2, 31, "Input"],
Cell[72549, 2663, 79, 2, 31, "Input"],
Cell[72631, 2667, 75, 2, 31, "Input"],
Cell[72709, 2671, 148, 4, 31, "Input"],
Cell[72860, 2677, 74, 2, 31, "Input"],
Cell[72937, 2681, 72, 2, 31, "Input"],
Cell[73012, 2685, 71, 2, 31, "Input"],
Cell[73086, 2689, 75, 2, 31, "Input"],
Cell[73164, 2693, 97, 2, 31, "Input"],
Cell[73264, 2697, 63, 0, 28, "Text"],
Cell[73330, 2699, 83, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[73462, 2707, 62, 0, 38, "Subsection"],
Cell[73527, 2709, 149, 3, 28, "Text"],
Cell[73679, 2714, 107, 2, 31, "Input"],
Cell[73789, 2718, 70, 2, 31, "Input"],
Cell[73862, 2722, 187, 3, 54, "Input"],
Cell[74052, 2727, 70, 2, 31, "Input"],
Cell[74125, 2731, 274, 5, 48, "Text"],

Cell[CellGroupData[{
Cell[74424, 2740, 32, 0, 37, "Subsubsection"],
Cell[74459, 2742, 130, 3, 28, "Text"],
Cell[74592, 2747, 91, 2, 31, "Input"],
Cell[74686, 2751, 65, 2, 31, "Input"],
Cell[74754, 2755, 86, 2, 31, "Input"],
Cell[74843, 2759, 79, 2, 31, "Input"],
Cell[74925, 2763, 81, 2, 31, "Input"],
Cell[75009, 2767, 70, 2, 31, "Input"],
Cell[75082, 2771, 140, 3, 31, "Input"],
Cell[75225, 2776, 66, 2, 31, "Input"],
Cell[75294, 2780, 70, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[75425, 2789, 50, 0, 92, "Section"],

Cell[CellGroupData[{
Cell[75500, 2793, 74, 1, 45, "Subsection"],
Cell[75577, 2796, 1104, 17, 200, "Text",
  Evaluatable->False],
Cell[76684, 2815, 89, 2, 31, "Input"],
Cell[76776, 2819, 91, 2, 31, "Input"],
Cell[76870, 2823, 104, 2, 31, "Input"],
Cell[76977, 2827, 402, 6, 68, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[77416, 2838, 102, 2, 39, "Subsection",
  Evaluatable->False],
Cell[77521, 2842, 735, 11, 108, "Text",
  Evaluatable->False],
Cell[78259, 2855, 140, 4, 28, "Text",
  Evaluatable->False],
Cell[78402, 2861, 153, 5, 59, "Input"],
Cell[78558, 2868, 67, 2, 31, "Input"],
Cell[78628, 2872, 67, 2, 31, "Input"],
Cell[78698, 2876, 68, 2, 31, "Input"],
Cell[78769, 2880, 72, 2, 31, "Input"],
Cell[78844, 2884, 118, 3, 31, "Input"],
Cell[78965, 2889, 351, 6, 68, "Text"],
Cell[79319, 2897, 166, 4, 54, "Input"],
Cell[79488, 2903, 94, 2, 31, "Input"],
Cell[79585, 2907, 69, 2, 31, "Input"],
Cell[79657, 2911, 265, 5, 48, "Text"],
Cell[79925, 2918, 112, 2, 31, "Input"],
Cell[80040, 2922, 465, 7, 88, "Text"],
Cell[80508, 2931, 339, 6, 123, "Input"],
Cell[80850, 2939, 339, 6, 123, "Input"],
Cell[81192, 2947, 344, 6, 123, "Input"],
Cell[81539, 2955, 722, 11, 108, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[82298, 2971, 83, 2, 39, "Subsection",
  Evaluatable->False],
Cell[82384, 2975, 259, 6, 92, "Text",
  Evaluatable->False],
Cell[82646, 2983, 351, 10, 93, "Input"],
Cell[83000, 2995, 88, 3, 31, "Input"],
Cell[83091, 3000, 73, 2, 31, "Input"],
Cell[83167, 3004, 75, 2, 31, "Input"],
Cell[83245, 3008, 227, 7, 47, "Input"],
Cell[83475, 3017, 121, 3, 28, "Text",
  Evaluatable->False],
Cell[83599, 3022, 138, 3, 31, "Input"],
Cell[83740, 3027, 69, 2, 31, "Input"],
Cell[83812, 3031, 82, 2, 31, "Input"],
Cell[83897, 3035, 72, 2, 31, "Input"],
Cell[83972, 3039, 109, 2, 31, "Input"],
Cell[84084, 3043, 866, 20, 148, "Text",
  Evaluatable->False],
Cell[84953, 3065, 344, 10, 93, "Input"],
Cell[85300, 3077, 65, 2, 31, "Input"],
Cell[85368, 3081, 75, 2, 31, "Input"],
Cell[85446, 3085, 77, 2, 31, "Input"],
Cell[85526, 3089, 84, 2, 31, "Input"],
Cell[85613, 3093, 75, 2, 31, "Input"],
Cell[85691, 3097, 110, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[85838, 3104, 83, 2, 39, "Subsection",
  Evaluatable->False],
Cell[85924, 3108, 518, 11, 144, "Text",
  Evaluatable->False],
Cell[86445, 3121, 116, 3, 54, "Input"],
Cell[86564, 3126, 445, 8, 68, "Text",
  Evaluatable->False],
Cell[87012, 3136, 76, 2, 31, "Input"],
Cell[87091, 3140, 225, 4, 48, "Text"],
Cell[87319, 3146, 219, 5, 77, "Input"],
Cell[87541, 3153, 39, 0, 28, "Text"],
Cell[87583, 3155, 80, 2, 31, "Input"],
Cell[87666, 3159, 70, 2, 31, "Input"],
Cell[87739, 3163, 192, 4, 48, "Text"],
Cell[87934, 3169, 84, 1, 28, "Text",
  Evaluatable->False],
Cell[88021, 3172, 321, 10, 70, "Input"],
Cell[88345, 3184, 88, 3, 31, "Input"],
Cell[88436, 3189, 70, 2, 31, "Input"],
Cell[88509, 3193, 76, 2, 31, "Input"],
Cell[88588, 3197, 225, 7, 47, "Input"],
Cell[88816, 3206, 149, 3, 28, "Text"],
Cell[88968, 3211, 78, 2, 31, "Input"],
Cell[89049, 3215, 331, 10, 70, "Input"],
Cell[89383, 3227, 65, 2, 31, "Input"],
Cell[89451, 3231, 88, 2, 28, "Text"],
Cell[89542, 3235, 92, 2, 31, "Input"],
Cell[89637, 3239, 509, 8, 88, "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[90195, 3253, 42, 0, 92, "Section"],
Cell[90240, 3255, 259, 6, 48, "Text"],

Cell[CellGroupData[{
Cell[90524, 3265, 38, 0, 44, "Subsection"],
Cell[90565, 3267, 253, 5, 46, "Text",
  Evaluatable->False],
Cell[90821, 3274, 170, 4, 46, "Text",
  Evaluatable->False],
Cell[90994, 3280, 92, 2, 32, "Input"],
Cell[91089, 3284, 74, 2, 32, "Input"],
Cell[91166, 3288, 138, 4, 27, "Text",
  Evaluatable->False],
Cell[91307, 3294, 95, 2, 32, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[91439, 3301, 34, 0, 38, "Subsection"],
Cell[91476, 3303, 771, 17, 120, "Text"],
Cell[92250, 3322, 91, 2, 31, "Input"],
Cell[92344, 3326, 52, 0, 28, "Text"],
Cell[92399, 3328, 50, 1, 28, "Text"],
Cell[92452, 3331, 141, 3, 28, "Text"],
Cell[92596, 3336, 482, 9, 146, "Input"],
Cell[93081, 3347, 159, 3, 54, "Input"],
Cell[93243, 3352, 109, 2, 31, "Input"],
Cell[93355, 3356, 104, 2, 31, "Input"],
Cell[93462, 3360, 76, 2, 31, "Input"],
Cell[93541, 3364, 75, 2, 31, "Input"],
Cell[93619, 3368, 67, 2, 31, "Input"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[93735, 3376, 27, 0, 92, "Section"],

Cell[CellGroupData[{
Cell[93787, 3380, 42, 0, 44, "Subsection"],
Cell[93832, 3382, 145, 3, 28, "Text"],
Cell[93980, 3387, 73, 2, 31, "Input"],
Cell[94056, 3391, 977, 19, 330, "Input",
  Evaluatable->False],
Cell[95036, 3412, 73, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[95146, 3419, 68, 0, 38, "Subsection"],
Cell[95217, 3421, 240, 4, 48, "Text"],
Cell[95460, 3427, 145, 3, 31, "Input"],
Cell[95608, 3432, 89, 2, 31, "Input"],
Cell[95700, 3436, 145, 3, 31, "Input"],
Cell[95848, 3441, 89, 2, 31, "Input"],
Cell[95940, 3445, 145, 3, 31, "Input"],
Cell[96088, 3450, 89, 2, 31, "Input"],
Cell[96180, 3454, 145, 3, 31, "Input"],
Cell[96328, 3459, 89, 2, 31, "Input"],
Cell[96420, 3463, 76, 2, 31, "Input"],
Cell[96499, 3467, 214, 4, 48, "Text"],
Cell[96716, 3473, 73, 2, 31, "Input"],
Cell[96792, 3477, 677, 12, 215, "Input",
  Evaluatable->False],
Cell[97472, 3491, 100, 2, 31, "Input"],
Cell[97575, 3495, 145, 4, 31, "Input"],
Cell[97723, 3501, 77, 2, 31, "Input"],
Cell[97803, 3505, 128, 3, 31, "Input"],
Cell[97934, 3510, 212, 4, 48, "Text"],
Cell[98149, 3516, 118, 3, 28, "Text"],
Cell[98270, 3521, 417, 8, 146, "Input"],
Cell[98690, 3531, 105, 2, 31, "Input"],
Cell[98798, 3535, 82, 2, 31, "Input"]
}, Closed]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

