(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     62509,       2327]*)
(*NotebookOutlinePosition[     63837,       2366]*)
(*  CellTagsIndexPosition[     63793,       2362]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Pseudo-Euclidean Vector Spaces", "Title",
  TextAlignment->Center,
  FontSize->35],

Cell["by Rolf Sulanke", "Subtitle",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontFamily->"Times New Roman"],

Cell["\<\
January 27, 1999; revised May 26, 1999;
revised November 1, 2000\
\>", "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontSize->13,
  Background->None],

Cell[CellGroupData[{

Cell["Summary", "Subsubsection",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Left,
  FontFamily->"Times New Roman"],

Cell["\<\
This notebook contains the basic definition of vector operations in the \
n-dimensional pseudo-euclidean space, including the euclidean case. The \
dimension dim and the index ind - the number of diagonal elements equal -1 in \
an orthogonalized basis-  are the characterising constants for the \
pseudo-euclidean vector spaces, which are vector spaces over the real \
numbers.  In special relativity theory the \"world\" = \"space-time\"  is \
based on the pseudo-euclidean vector space of dimension dim = 4 and index ind \
= 1, corresponding to the three-dimensionality of the physical space and the \
one-dimensionality of time. For the N-dimensional M\[ODoubleDot]bius space  \
we have to set dim = N +2 and ind = 1 . In the n-dimensional euclidean case \
we have, of course, dim = n and ind = 0. In the Lie geometry of spheres one \
has dim = 6 and ind = 2. 
Changes: The most interesting subject treated in this notebook is the \
orthogonalization of vector sequences in the pseudo-euclidean case, which has \
been corrected and refined in the last revision. Furthermore, the procedure \
psfilter has been simplified: now a repeated application of psfilter  is not \
necessary in any cases.\
\>", "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None]
}, Closed]],

Cell[CellGroupData[{

Cell["Keywords", "Subsubsection"],

Cell["\<\
Dimension dim, index ind, pseudo-euclidean scalar product, generalized cross \
product, spacelike, timelike, isotropic vectors, norming, orthogonalization, \
renorming, psfilter, orthopairs.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Address", "Subsubsection"],

Cell["\<\
sulanke@mathematik.hu-berlin.de
Homepage:
http://www-irm.mathematik.hu-berlin.de/~sulanke/\
\>", "Text"],

Cell["\<\
The author will be very thankful for hints, corrections, comments.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[CellGroupData[{

Cell["The needed packages", "Subsection"],

Cell["\<\
Before initializing the present notebook ensure that the packages
euvec.m
pseuvec.m
and the master package init.m are in your working directory. These packages \
are contained in the item \"Spheres\".
 For the present notebook the packages mspher.m, mcirc.m, spiralsf.m are not \
needed.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["For Windows", "Subsection"],

Cell[TextData[{
  "For the commands in this subsection:\n",
  StyleBox["DO NOT CONFIRM THE EVALUATION OF THE INITIALIZATION CELLS !!!",
    FontWeight->"Bold",
    FontColor->RGBColor[1, 0, 0]]
}], "Text"],

Cell[" If the result of the following command ", "Text"],

Cell[BoxData[
    \(Directory[]\)], "Input",
  CellLabel->"In[4]:="],

Cell["\<\
does not coincide with the working directory, e.g. \"C:\\Eigene \
Dateien\\mymath\\spheres.v4\", containing the needed packages, insert the \
Path of your working directory in the next command line and activate it \
first, without calling the initialization:\
\>", "Text"],

Cell[BoxData[
    \(SetDirectory["\<C:\\Eigene Dateien\\mymath\\spheres4\>"]\)], "Input",
  CellLabel->"In[38]:="],

Cell["\<\
If you want to use the packages euvec.m, pseuvec.m only,  carry out the next \
command.\
\>", "Text"],

Cell[BoxData[
    \(Needs["\<pseuvec`\>"]\)], "Input",
  CellLabel->"In[1]:="],

Cell["\<\
If you want  that all commands contained in the packages of the item \
\"Spheres\" are available, activate\
\>", "Text"],

Cell[BoxData[
    \(Get["\<init.m\>"]\)], "Input",
  CellLabel->"In[2]:="],

Cell[TextData[StyleBox["Now: Activite Kernel/Evaluation/Evaluate \
Initialization from the menu!",
  FontSize->18,
  FontColor->RGBColor[1, 0, 0]]], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["For Linux (and other  systems: not tested!)", "Subsection"],

Cell[TextData[{
  "For the next command:\n",
  StyleBox["DO NOT CONFIRM THE EVALUATION OF THE INITIALIZATION CELLS !!!",
    FontWeight->"Bold",
    FontColor->RGBColor[1, 0, 0]]
}], "Text"],

Cell["\<\
If you want to use the package euvec.m, pseuvec.m only, remove the file \
init.m  from the working directory and carry out the next command.\
\>", "Text"],

Cell[BoxData[
    \(Needs["\<pseuvec`\>"]\)], "Input",
  CellLabel->"In[1]:="],

Cell[TextData[StyleBox["Now: Activite Kernel/Evaluation/Evaluate \
Initialization from the menu!",
  FontSize->18,
  FontColor->RGBColor[1, 0, 0]]], "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1. Introduction. Usages", "Section"],

Cell[CellGroupData[{

Cell["1.1 Introduction", "Subsection"],

Cell["\<\
As we wrote this notebook, we have been interested in the Moebius geometry of \
the 3-sphere. Thus, the default values of dimension and index are \
\>", "Text",
  FontSize->13],

Cell[BoxData[
    \(dim\  = \ 5; ind = 1;\)], "Input",
  CellLabel->"In[3]:=",
  InitializationCell->True,
  FontSize->15],

Cell["\<\
Always have in mind that almost all the defined concepts  depend on dim and \
ind; if you want to change the default values, simply enter the commands
dim = n; ind = m;
 where n and m, 0 \[LessEqual] m \[LessEqual] n, are your preferred values. \
But do this before calling the Initialization, or importing the package \
pseuvec.m , since in the definitions of  this package dim and ind enter as \
global constants. \
\>", "Text",
  FontSize->13],

Cell["\<\
The basic concepts used in the pseudo-euclidean vector algebra are contained  \
in the package pseuvec.m, which automatically imports the package euvec.m \
containing general definitions of  n-dimensional linear algebra. \
\>", "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None],

Cell["\<\
If you want information about these  loaded concepts, enter e.g.\
\>", "Text",
  FontSize->13],

Cell[BoxData[
    \(\(?ide\)\)], "Input",
  CellLabel->"In[4]:=",
  InitializationCell->True,
  FontSize->15],

Cell["\<\
 Often you can inform yourself about the definitions by evoking or applying \
them:\
\>", "Text"],

Cell[BoxData[
    \(ide[]\)], "Input",
  CellLabel->"In[5]:=",
  InitializationCell->True],

Cell["The standard basis is", "Text"],

Cell[BoxData[
    \(Table[stb[i], {i, \ dim}]\)], "Input",
  CellLabel->"In[6]:=",
  InitializationCell->True],

Cell["\<\
and the matrix of the scalar products of the basis vectors is\
\>", "Text"],

Cell[BoxData[
    \(Table[pssp[stb[i], \ stb[j]], \ {i, dim}, \ {j, dim}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[7]:=",
  InitializationCell->True],

Cell["Here pssp denotes the pseudo-euclidean scalar product.", "Text"],

Cell[BoxData[
    \(?? pssp\)], "Input",
  CellLabel->"In[8]:="],

Cell["\<\
Since the names are prepended by the ContextNames, sometimes the definitions \
obtained in this way are hard to read. Therefore you will find some of them \
in the original form in unevaluable cells.  It is recommended to look into \
the packages.\
\>", "Text",
  Evaluatable->False],

Cell["\<\
All the definitions contained in the packages euvec.m and pseuvec.m are \
listed in the next subsection 1.2.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["1.2 The usages", "Subsection"],

Cell[CellGroupData[{

Cell["From the package euvec.m:", "Subsubsection"],

Cell["Valid for arbitrary values of the dimension dim are", "Text"],

Cell[BoxData[
    \(\(?nul\)\)], "Input",
  CellLabel->"In[11]:="],

Cell[BoxData[
    \(\(?stb\)\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(\(?vec\)\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(\(?randomv\)\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[
    \(\(?cross\)\)], "Input",
  CellLabel->"In[15]:="],

Cell[BoxData[
    \(\(?dotnorme\)\)], "Input",
  CellLabel->"In[16]:="],

Cell[BoxData[
    \(\(?esorthonorm\)\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(Options[esorthonorm]\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(\(?normed\)\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(\(?print\)\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(\(?neglect\)\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(\(?orthobase\)\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(\(?outzero\)\)], "Input",
  CellLabel->"In[23]:="],

Cell[BoxData[
    \(\(?sterproj\)\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(\(?invsterproj\)\)], "Input",
  CellLabel->"In[25]:="],

Cell[BoxData[
    \(\(?hyperplane\)\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(\(?unit\)\)], "Input",
  CellLabel->"In[27]:="],

Cell[BoxData[
    \(\(?rank\)\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(\(?renorm\)\)], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(\(?smoothing\)\)], "Input",
  CellLabel->"In[30]:="],

Cell[BoxData[
    \(\(?sphericalreflection\)\)], "Input",
  CellLabel->"In[31]:="],

Cell["The following commands work for dimension dim = 4 only:", "Text"],

Cell[BoxData[
    \(\(?ortho1\)\)], "Input",
  CellLabel->"In[32]:="],

Cell[BoxData[
    \(\(?orthoerg\)\)], "Input",
  CellLabel->"In[33]:="],

Cell[BoxData[
    \(\(?orthoframe\)\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(\(?tg4frame\)\)], "Input",
  CellLabel->"In[35]:="],

Cell[BoxData[
    \(\(?subsphere\)\)], "Input",
  CellLabel->"In[36]:="],

Cell[BoxData[
    \(\(?subspheremf\)\)], "Input",
  CellLabel->"In[37]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["From the package pseuvec.m:", "Subsubsection"],

Cell[BoxData[
    \(\(?ide\)\)], "Input",
  CellLabel->"In[38]:="],

Cell[BoxData[
    \(\(?pssp\)\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(\(?dual\)\)], "Input",
  CellLabel->"In[40]:="],

Cell[BoxData[
    \(\(?psCross\)\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(\(?pscross\)\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(\(?ch\)\)], "Input",
  CellLabel->"In[43]:="],

Cell[BoxData[
    \(\(?pr\)\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(\(?orthonorm\)\)], "Input",
  CellLabel->"In[45]:="],

Cell[BoxData[
    \(Options[orthonorm]\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(\(?normed\)\)], "Input",
  CellLabel->"In[47]:="],

Cell[BoxData[
    \(\(?print\)\)], "Input",
  CellLabel->"In[48]:="],

Cell[BoxData[
    \(\(?neglect\)\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(\(?psfilter\)\)], "Input",
  CellLabel->"In[50]:="],

Cell[BoxData[
    \(\(?normalize\)\)], "Input",
  CellLabel->"In[51]:="],

Cell[BoxData[
    \(\(?orthopair\)\)], "Input",
  CellLabel->"In[52]:="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
2. Vector Operations in Euclidean and Pseudo-Euclidean Spaces\
\>", "Section",
  Evaluatable->False,
  Background->None],

Cell[CellGroupData[{

Cell["2.1 Basic definitions", "Subsection",
  Evaluatable->False,
  FontSize->14,
  Background->None],

Cell["\<\
Of course, we can apply all the operations of real linear vector algebra. We \
have the null vector:\
\>", "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(\(\(\n\)\(nul[]\)\)\)], "Input",
  CellLabel->"In[8]:=",
  FontSize->15],

Cell["The standard basis is:", "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(Table[stb[i], {i, \ dim}]\)], "Input",
  CellLabel->"In[9]:=",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontSize->15,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->None],

Cell["\<\
The basic concept of pseudo-euclidean geometry is the indefinite scalar \
product pssp. 
The standard basis {stb[i]} is orthonormal with respect to this scalar \
product:\
\>", "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(MatrixForm[
      Table[pssp[stb[i], \ stb[j]], \ {i, \ dim}, \ {j, \ dim}]]\)], "Input",
  CellLabel->"In[10]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell["The diagonal of this matrix is", "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(ide[]\)], "Input",
  CellLabel->"In[11]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell["The dualization of a vector ", "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(vec[x]\)], "Input",
  CellLabel->"In[12]:=",
  FontSize->15],

Cell[" is given by", "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(dual[vec[x]]\)], "Input",
  CellLabel->"In[13]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell[TextData[{
  "This corresponds to the dual coordinates of a vector in Riemannian \
geometry. The scalar product pssp is related to the usual positive definite \
scalar product, denoted in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " by  Dot ., as follows"
}], "Text",
  FontSize->13],

Cell[BoxData[
    \(pssp[vec[x], \ vec[y]]\  == \ vec[x] . dual[vec[y]]\)], "Input",
  CellLabel->"In[54]:=",
  FontSize->15],

Cell["\<\
The definition of the pseudo-orthogonal scalar product of type (dim - ind, \
ind) implies the following rules:\
\>", "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(pssp[x, y] == pssp[y, x]\)], "Input",
  CellLabel->"In[14]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(pssp[x + y, u] == pssp[x, u] + pssp[y, u]\)], "Input",
  CellLabel->"In[15]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(pssp[a\ k, l] == a\ pssp[k, l]\)], "Input",
  CellLabel->"In[16]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell["\<\
The Dot product is more general. Therefore commutativity is not valid, in \
general:\
\>", "Text",
  FontSize->13],

Cell[BoxData[
    \(x . y\  == \ y . x\)], "Input",
  CellLabel->"In[20]:=",
  FontSize->15],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(\((vec[x] + vec[a])\) . vec[y]\  == \ 
      vec[y] . vec[x]\  + \ vec[y] . vec[a]\)], "Input",
  CellLabel->"In[18]:=",
  FontSize->15],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[19]:=",
  FontSize->15]
}, Closed]],

Cell[CellGroupData[{

Cell["2.2 Type of a vector", "Subsection",
  FontSize->14],

Cell["\<\
A non zero vector v is called  space-, time-, or light-like (in relativity \
theory),  if  its scalar square pssp[x,x]  is positive, negative, or zero, \
respectively. In geometric context one mostly uses \"isotropic\" instead of \
\"lightlike\". This property of the vector can be indicated by the function \
ch (== character), which gives the sign of pssp[v, v].  \
\>", "Text",
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(\(?ch\)\)], "Input",
  CellLabel->"In[22]:=",
  FontSize->15],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  FontSize->11],

Cell["Error messages: The nullvector", "Text",
  FontSize->13],

Cell[BoxData[
    \(ch[nul[]]\)], "Input",
  CellLabel->"In[23]:=",
  FontSize->15],

Cell["or a vector with incorrect dimension:", "Text",
  FontSize->13],

Cell[BoxData[
    \(ch[stb[1, 3]]\)], "Input",
  CellLabel->"In[24]:=",
  FontSize->15],

Cell["Timelike vectors", "Text",
  FontSize->13],

Cell[BoxData[
    StyleBox[\(ch[stb[dim]]\),
      FontSize->12]], "Input",
  CellLabel->"In[25]:=",
  FontSize->23,
  Background->None],

Cell[BoxData[
    \(ch[stb[5]*Cosh[t]\  + \ stb[1]*Sinh[t]]\)], "Input",
  CellLabel->"In[26]:=",
  FontSize->15],

Cell["Isotropic vectors:", "Text",
  FontSize->13],

Cell[BoxData[
    \(ch[stb[1]\ Cos[Pi/4]\  + \ stb[dim]\ Sin[Pi/4]]\)], "Input",
  CellLabel->"In[27]:=",
  FontSize->15],

Cell[BoxData[
    \(ch[stb[1]*a + \ stb[dim]*a\ ]\)], "Input",
  CellLabel->"In[28]:=",
  FontSize->15],

Cell["Spacelike vectors:", "Text",
  FontSize->13],

Cell[BoxData[
    \(ch[stb[1]]\)], "Input",
  CellLabel->"In[29]:=",
  FontSize->15],

Cell[BoxData[
    \(ch[stb[1]*Cosh[t]\  + \ stb[5]*Sinh[t]]\)], "Input",
  CellLabel->"In[30]:=",
  FontSize->15],

Cell["ch[stb[1]*Cos[t] + stb[2]*Sin[t]]", "Input",
  CellLabel->"In[31]:=",
  FontSize->14,
  Background->None],

Cell[BoxData[
    \(vec[x]\  - \ stb[dim]*x[dim]\)], "Input",
  CellLabel->"In[32]:="],

Cell["ch[vec[x] - stb[dim]*x[dim]]", "Input",
  CellLabel->"In[33]:=",
  FontSize->14,
  Background->None],

Cell["Type of a random vector", "Text"],

Cell[BoxData[
    \(ch[randomv[]]\)], "Input",
  CellLabel->"In[34]:="],

Cell[BoxData[
    \(\(\(v1\)\(\ \)\(=\)\(\ \)\(randomv[]\)\(\ \ \)\)\)], "Input",
  CellLabel->"In[46]:="],

Cell[BoxData[
    \(ch[v1]\)], "Input",
  CellLabel->"In[47]:="],

Cell["\<\
The empirical probability that such a random vector is timelike equals \
approximately 0.06:\
\>", "Text"],

Cell[BoxData[
    \(zj\  = \ 0; 
    For[i = 1, 
      i < 10000, \ \(i++\), \n\t\((v1\  = \ randomv[]; 
        If[ch[v1] == \ \(-1\), \ zj\  = \ zj + 1])\)]; N[zj/10000]\)], "Input",\

  CellLabel->"In[37]:="],

Cell[BoxData[
    \(Remove[zj, \ v1, i]\)], "Input",
  CellLabel->"In[38]:="],

Cell["\<\
Sometimes, clearly, it is impossible to decide which character  the vector \
has, since it depends of the value of parameters:\
\>", "Text",
  FontSize->13],

Cell["ch[vec[x]]", "Input",
  CellLabel->"In[39]:=",
  FontSize->14,
  Background->None],

Cell[BoxData[
    \(ch[stb[1]\ Cos[t]\  + \ stb[dim]\ Sin[t]]\)], "Input",
  CellLabel->"In[40]:=",
  FontSize->15],

Cell["Plotting shows the character in dependence of the parameter:", "Text",
  FontSize->13],

Cell[BoxData[
    \(Plot[
      ch[stb[1]\ Cos[t]\  + \ stb[dim]\ Sin[t]], \ {t, 0, \ 
        2\ Pi}]\)], "Input",
  CellLabel->"In[41]:=",
  FontSize->15],

Cell["\<\
In the next example the character does not depend on the parameter; therefore \
it will be evaluated:\
\>", "Text"],

Cell[BoxData[
    \(ch[stb[1]\ Cos[t]\  + \ stb[dim - ind]\ Sin[t]]\)], "Input",
  CellLabel->"In[42]:=",
  FontSize->15]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.3 The norming", "Subsection",
  FontSize->14],

Cell["Now we consider the norming of the vector", "Text",
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(normalize[vec[x]]\)], "Input",
  CellLabel->"In[48]:=",
  FontSize->15,
  Background->None],

Cell[TextData[{
  "Since ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " often has difficulties with evaluations and simplifications of  Sqrt or \
Abs it is recommended to use ",
  "normalize",
  " only in unavoidable cases, and, if possible, only  at the end of a module \
or calculation."
}], "Text",
  FontSize->13],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  Evaluatable->False,
  FontSize->11,
  Background->None],

Cell[BoxData[
    \(\(?normalize\)\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(normalize[stb[1] + 2\ stb[2]]\)], "Input",
  CellLabel->"In[50]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell["\<\
Here you can see the problems if you want to norm general vectors:\
\>", "Text",
  FontSize->13],

Cell[BoxData[
    \(normalize[stb[1]*Cosh[t] + stb[2]]\)], "Input",
  CellLabel->"In[65]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(pssp[%, %]\)], "Input",
  CellLabel->"In[66]:=",
  FontSize->15],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[67]:=",
  FontSize->15],

Cell[BoxData[
    \(Simplify[%, \ t\  \[Element] \ Reals]\)], "Input",
  CellLabel->"In[68]:=",
  FontSize->15],

Cell[TextData[{
  "In newer versions, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " simplifies expressions with additional conditions, see Simplify in the \
help system. Sometimes the result depends on the place in the sequence of \
evaluations:"
}], "Text"],

Cell[BoxData[
    \(TrueQ[1 + \ Abs[\ Cosh[t]]\  >= \ 0]\)], "Input",
  CellLabel->"In[83]:=",
  FontSize->15],

Cell[BoxData[
    \(Simplify[1 + \ Abs[\ Cosh[t]]\  >= \ 0, \ 
      t\  \[Element] \ Reals]\)], "Input",
  CellLabel->"In[71]:=",
  FontSize->15],

Cell[BoxData[
    \(Simplify[Abs[\ Cosh[t]]\  >= \ 0, \ 
      t\  \[Element] \ Complexes]\)], "Input",
  CellLabel->"In[72]:=",
  FontSize->15],

Cell[BoxData[
    \(Simplify[Abs[t] >= \ 0, \ t\  \[Element] \ Complexes]\)], "Input",
  CellLabel->"In[82]:=",
  FontSize->15],

Cell[BoxData[
    \(TrueQ[Simplify[1 + Abs[Cosh[abcd]] >= \ 0]]\)], "Input",
  CellLabel->"In[84]:=",
  FontSize->15],

Cell["\<\
Mathematica does not return True, since generally t could be a complex \
parameter. But\
\>", "Text",
  FontSize->13],

Cell[BoxData[
    \(TrueQ[Abs[1 + \ \ Cosh[t]^2\ ] >= \ 0]\)], "Input",
  CellLabel->"In[85]:=",
  FontSize->15],

Cell[BoxData[
    \(TrueQ[Simplify[Abs[1 + \ \ Cosh[t]^2\ ]] >= \ 0]\)], "Input",
  CellLabel->"In[56]:=",
  FontSize->15],

Cell["\<\
is not recognized, too.  In the next example the norming works ignoring the \
isotropic case t=0:\
\>", "Text",
  FontSize->13],

Cell[BoxData[
    \(normalize[stb[1]*Cosh[t] + stb[5]]\)], "Input",
  CellLabel->"In[57]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(% /. \ t\  -> 0\)], "Input",
  CellLabel->"In[58]:="],

Cell["\<\
Norming an isotropic vector, or the nullvector, returns the vector itself\
\>", "Text",
  FontSize->13],

Cell[BoxData[
    \(normalize[stb[1] + \ stb[5]]\)], "Input",
  CellLabel->"In[59]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(normalize[stb[1]*a + \ stb[5]*a]\)], "Input",
  CellLabel->"In[60]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(normalize[stb[5]*a + \ stb[2]*a\  + stb[3\ ]*b]\)], "Input",
  CellLabel->"In[61]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(pssp[%, %]\)], "Input",
  CellLabel->"In[62]:=",
  FontSize->15],

Cell[BoxData[
    \(normalize[nul[]]\)], "Input",
  CellLabel->"In[63]:=",
  FontSize->15,
  Background->None]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["2.4 The general cross product", "Subsection",
  Evaluatable->False,
  FontSize->14,
  Background->None],

Cell[TextData[{
  "As in euclidean geometry, the cross product of dim -1 vectors  ",
  Cell[BoxData[
      \(TraditionalForm\`a\_i\)]],
  "  is the vector v uniquely defined by\npssp[v, x] = Det[",
  Cell[BoxData[
      \(TraditionalForm\`a\_1\)]],
  ",..., ",
  Cell[BoxData[
      \(TraditionalForm\`a\_\(n - 1\)\)]],
  ", x]\t   (n = dim).\n First we remember our definition of  the euclidean \
cross product for arbitrary dimension, contained in euvec.m:"
}], "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(\(?cross\)\)], "Input",
  CellLabel->"In[86]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(cross[b_List]\  := \ 
      Module[{d\  = \ Length[b]\  + \ 1, v}, \n\t\tv\  = \ 
          nul[d]; \n\t\t\tFor[i = 1, i < d, \(i++\), 
          If[Length[b[\([i]\)]\ ] != \ d, \ \n\t\t\t\t\t\t\t\tReturn[
              Print[b[\([i]\)], \ "\< is not a correct argument!\>"]], \n\t\t\
\t\t\t\t\t\tv[\([i]\)]\  = \ 
              Det[Append[b, \ stb[i, \ d]]]]]; \n\t\t\tv[\([d]\)]\  = \ 
          Det[Append[b, \ stb[d, \ d]]]; \n\t\t\tReturn[v]]\)], "Input",
  CellLabel->"In[17]:=",
  Evaluatable->False,
  FontSize->15],

Cell[TextData[{
  "The difference to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s function Cross is that cross' first argument is a list of lists, while \
Cross depends of dim -1 vectors."
}], "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(?? Cross\)], "Input",
  CellLabel->"In[87]:=",
  FontSize->15],

Cell["Here are the generalizations to the pseudo-Euclidean case:", "Text",
  Evaluatable->False,
  FontSize->13,
  Background->None],

Cell[BoxData[
    \(?? pscross\)], "Input",
  CellLabel->"In[88]:=",
  PageWidth->Infinity,
  FontSize->15,
  Background->None],

Cell["\<\
If  you want that the cross product depends of dim - 1 arguments, and not of \
the list of these arguments, you may use psCross:\
\>", "Text",
  FontSize->13],

Cell[BoxData[
    \(psCross := Composition[dual, \ Cross]\)], "Input",
  CellLabel->"In[13]:=",
  PageWidth->Infinity,
  Evaluatable->False,
  FontSize->15,
  Background->None],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  Evaluatable->False,
  FontSize->11,
  Background->None],

Cell[BoxData[
    \(cross[{stb[1], stb[2], \ stb[3], stb[4]}]\)], "Input",
  CellLabel->"In[89]:=",
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(Cross[stb[1], stb[2], \ stb[3], stb[4]]\)], "Input",
  CellLabel->"In[90]:=",
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(pscross[{stb[1], stb[2], \ stb[3], stb[4]}]\)], "Input",
  CellLabel->"In[91]:=",
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(vt\  = \ %\)], "Input",
  CellLabel->"In[92]:=",
  FontSize->15],

Cell[BoxData[
    \(Det[{stb[1], stb[2], \ stb[3], stb[4], vec[x]}]\)], "Input",
  CellLabel->"In[93]:=",
  FontSize->15],

Cell[BoxData[
    \(pssp[vt, \ vec[x]]\)], "Input",
  CellLabel->"In[94]:=",
  FontSize->15],

Cell[BoxData[
    \(pscross[{stb[5], stb[2], \ stb[3], stb[4]}]\)], "Input",
  CellLabel->"In[95]:=",
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(psCross[stb[1], \ stb[2], \ stb[3], \ 
      stb[3]*Cos[t] + stb[4]*Sin[t]]\)], "Input",
  CellLabel->"In[96]:="],

Cell[BoxData[
    \(pscross[{stb[1], \ stb[2], \ stb[3], \ 
        stb[3]*Cos[t] + stb[4]*Sin[t]}]\)], "Input",
  CellLabel->"In[97]:="],

Cell[BoxData[
    \(cross[{stb[1], \ stb[2], \ stb[3], \ 
        stb[3]*Cos[t] + stb[4]*Sin[t]}]\)], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \(psCross[stb[5], stb[2], \ stb[3], stb[4]]\)], "Input",
  CellLabel->"In[99]:=",
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(cross[{stb[5], stb[2], \ stb[3], stb[4]}]\)], "Input",
  CellLabel->"In[100]:=",
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(pscross[{stb[2], \ stb[3]*Cos[t] + \ stb[4]*Sin[t], \ 
        stb[4]*Cos[t] - \ stb[3]*Sin[t], stb[5]}]\)], "Input",
  CellLabel->"In[101]:=",
  FontSize->15,
  Background->None],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[102]:=",
  FontSize->15],

Cell[BoxData[
    \(cross[{vec[a], vec[b], vec[c], 
          vec[d]}]\  == \ \(-cross[{vec[b], vec[a], vec[c], 
            vec[d]}]\)\)], "Input",
  CellLabel->"In[103]:="],

Cell[BoxData[
    \(Cross[vec[a], vec[b], vec[c], 
        vec[d]]\  == \ \(-Cross[vec[b], vec[a], vec[c], vec[d]]\)\)], "Input",\

  CellLabel->"In[104]:="],

Cell[BoxData[
    \(pscross[{vec[a], vec[b], vec[c], 
          vec[d]}]\  == \ \(-pscross[{vec[b], vec[a], vec[c], 
            vec[d]}]\)\)], "Input",
  CellLabel->"In[105]:="],

Cell[BoxData[
    \(pscross[{vec[a], vec[b], vec[c], vec[d]}]\  == \ \(-psCross[vec[b], 
          vec[a], vec[c], vec[d]]\)\)], "Input",
  CellLabel->"In[106]:="]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3. Orthogonality", "Section"],

Cell[CellGroupData[{

Cell["3.1 Definition", "Subsection",
  FontSize->14],

Cell[TextData[{
  "As in euclidean geometry a list {",
  Cell[BoxData[
      \(TraditionalForm\`v\_i\)]],
  "} of non zero vectors is called orthogonal, if pssp[",
  Cell[BoxData[
      \(TraditionalForm\`v\_i\)]],
  ",",
  Cell[BoxData[
      \(TraditionalForm\`v\_j\)]],
  " ] = 0 for i \[NotEqual] j; it is called orthonormal if it is orthogonal \
and for each i one has pssp[",
  Cell[BoxData[
      \(TraditionalForm\`v\_i\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`v\_i\)]],
  "] = 1, pssp[",
  Cell[BoxData[
      \(TraditionalForm\`v\_i\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`v\_i\)]],
  "] = -1, or pssp[",
  Cell[BoxData[
      \(TraditionalForm\`v\_i\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`v\_i\)]],
  "] = 0 with ",
  Cell[BoxData[
      \(TraditionalForm\`v\_i\)]],
  "\[NotEqual] o. An orthonormal sequence of vectors which does not contain \
isotropic vectors is linearly independent, therefore its length is less or  \
equal dim. By the theorem of inertia the number of timelike vectors in an \
orthonormal sequence is less or equal the index ind. An orthonormal sequence \
of maximal length dim which does not contain isotropic vectors is called an \
orthonormal frame; in any orthonormal frame the number of timelike vectors \
equals ind.\nIn this section we collect definitions and modules for \
orthogonal vectors in the pseudo-euclidean space, which we need for certain \
applications, especially in M\[ODoubleDot]bius geometry.  The basic \
orthonormalization procedure orthonorm is contained in the package pseuvec.m. \
The Erhard Schmidt orthogonalization (for positive semidefinite scalar \
products) esorthonorm (defined in euvec.m) is tested in the notebook \
eusphere.nb, and the general orthogonalization orthonorm  is treated below.  \
A generalization of orthonorm for scalar products in matrix spaces and \
applications to Killing forms can be found in the item Lie algebras, in my \
homepage, or in :"
}], "Text",
  FontSize->13],

Cell["http://www.mathsource.com/cgi-bin/msitem?0210-845", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["3.2 Pseudo-euclidean orthogonalization", "Subsection",
  FontSize->14],

Cell[CellGroupData[{

Cell["3.2.1 General remarks", "Subsubsection"],

Cell[TextData[{
  "For definite scalar products J. M.Novak's GramSchmidt orthogonalization \
works well. Clearly, Novak's procedure GramSchmidt (see \
.../mathematica/AddOns/StandardPackages/LinearAlgebra/Orthogonalization.m) \
was not intended for applications to indefinite scalar products, and it did \
not work correctly for these. For pseudo-euclidean vector spaces we \
programmed a procedure orthonorm with the following properties:\nGiven a \
sequence of vectors ",
  Cell[BoxData[
      \(TraditionalForm\`v\_1\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`v\_2\)]],
  ",..., ",
  Cell[BoxData[
      \(TraditionalForm\`v\_k\)]],
  ". Construct an orthogonal sequence  ",
  Cell[BoxData[
      \(TraditionalForm\`w\_1\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`w\_2\)]],
  ",...,",
  Cell[BoxData[
      \(TraditionalForm\`w\_m\)]],
  ", which  has the same span \n                                              \
                 span[ ",
  Cell[BoxData[
      \(TraditionalForm\`v\_1\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`v\_2\)]],
  ",..., ",
  Cell[BoxData[
      \(TraditionalForm\`v\_k\)]],
  "] = span[ ",
  Cell[BoxData[
      \(TraditionalForm\`w\_1\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`w\_2\)]],
  ",...,",
  Cell[BoxData[
      \(TraditionalForm\`w\_m\)]],
  "]. \nTo this aim we use a function defined in euvec.m which can be useful \
also for other applications:"
}], "Text",
  TextAlignment->Left,
  FontSize->13],

Cell[BoxData[
    \(\(?outzero\)\)], "Input",
  CellLabel->"In[107]:=",
  FontSize->15],

Cell[BoxData[
    \(outzero[{nul[], \ stb[1]}]\)], "Input",
  CellLabel->"In[108]:=",
  FontSize->15]
}, Closed]],

Cell[CellGroupData[{

Cell["3.2.2 Orthonormalization", "Subsubsection"],

Cell["\<\
The orthogonalization is more complicated as in the euclidean case; the \
following procedure is contained in pseuvec.m:\
\>", "Text"],

Cell[BoxData[
    \(\(?orthonorm\)\)], "Input",
  CellLabel->"In[109]:="],

Cell[BoxData[
    \(Options[orthonorm]\)], "Input",
  CellLabel->"In[110]:="],

Cell[BoxData[
    \(\(?neglect\)\)], "Input",
  CellLabel->"In[111]:="],

Cell[BoxData[
    \(orthonorm[b_List, opts___] := 
      Module[{v, w, va, vb, l, ort = {}, i, j, f, h, sm, aus, prod, nor, pr, 
          bn, follow, iso, 
          control}, {prod, nor, pr, 
            bn} = \({innerprod, normed, print, neglect} /. {opts}\) /. 
            Options[orthonorm]; \[IndentingNewLine]f[h_] := 
          smoothing[h, 10^bn]; \[IndentingNewLine]sm[aus_] := 
          MapAll[f, aus]; \[IndentingNewLine]w = 
          outzero[sm[b]]; \[IndentingNewLine]Label[
          follow]; \[IndentingNewLine]l = Length[w]; \[IndentingNewLine]If[
          l \[Equal] 0, Goto[control]]; \[IndentingNewLine]For[i = 1, 
          i \[LessEqual] l, \(i++\), 
          If[Not[Simplify[sm[prod[w[\([i]\)], w[\([i]\)]]]] \[Equal] 0], 
            AppendTo[ort, w[\([i]\)]]; \[IndentingNewLine]If[
              pr \[Equal] True, Print[w[\([i]\)]]]; \[IndentingNewLine]v = 
              Simplify[
                sm[Table[
                    w[\([j]\)] - 
                      w[\([i]\)]*
                        prod[w[\([i]\)], w[\([j]\)]]/
                          prod[w[\([i]\)], w[\([i]\)]], {j, 
                      l}]]]; \[IndentingNewLine]w = outzero[v]; \ 
            Goto[follow], Null, 
            AppendTo[ort, w[\([i]\)]]; \[IndentingNewLine]If[
              pr \[Equal] True, Print[w[\([i]\)]]]; \[IndentingNewLine]v = 
              Simplify[
                sm[Table[
                    w[\([j]\)] - 
                      w[\([i]\)]*
                        prod[w[\([i]\)], w[\([j]\)]]/
                          prod[w[\([i]\)], w[\([i]\)]], {j, 
                      l}]]]; \[IndentingNewLine]w = outzero[v]; \ 
            Goto[follow]]]; \[IndentingNewLine]Label[iso]; 
        l = Length[w]; \[IndentingNewLine]If[l \[Equal] 0, 
          Goto[control]]; \[IndentingNewLine]v = w; \[IndentingNewLine]If[
          l \[Equal] 1, AppendTo[ort, v[\([1]\)]]; \ 
          If[pr \[Equal] True, Print[v[\([1]\)]]]; \[IndentingNewLine]Goto[
            control]]; \[IndentingNewLine]For[i = 1, 
          i \[LessEqual] l, \(i++\), 
          For[j = i + 1, j \[LessEqual] l, \(j++\), 
            If[Not[Simplify[sm[prod[v[\([i]\)], v[\([j]\)]]]] \[Equal] 0], 
              va = v[\([j]\)] - v[\([i]\)]; \[IndentingNewLine]vb = 
                v[\([j]\)] + v[\([i]\)]; \ \t\t\t\tv[\([i]\)] = va; 
              v[\([j]\)] = vb; \[IndentingNewLine]w = 
                outzero[Simplify[sm[v]]]; Goto[follow]]]]; \ \t\t\tort = 
          Join[ort, v]; \[IndentingNewLine]Label[
          control]; \[IndentingNewLine]l = Length[ort]; \[IndentingNewLine]If[
          l \[Equal] 0, Return[ort]]; \[IndentingNewLine]If[
          nor \[Equal] True, 
          ort = sm[
              Table[Simplify[normalize[ort[\([i]\)], prod]], {i, 
                  Length[ort]}]]]; \[IndentingNewLine]For[i = 1, 
          i \[LessEqual] l - 1, \(i++\), 
          For[j = i + 1, j \[LessEqual] l, \(j++\), 
            If[Not[Simplify[sm[prod[ort[\([i]\)], ort[\([j]\)]]]] \[Equal] 
                  0], Return[
                Print["\<I cannot orthogonalize the given sequence!\>"]], 
              Null, Return[
                Print["\<I cannot orthogonalize the given sequence!\>"]]]]]; \
\ \tReturn[ort]]\)], "Input",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["3.2.3  WARNINGS:", "Subsubsection"],

Cell["\<\
1. In certain cases the pseudo-euclidean orthonormalization procedure \
orthonorm cannot  be carried out. Then instead of the orthonormed matrix \
appears the message  \"I cannot orthogonalize the given sequence!\"  \
Sometimes the norming is the reason; then try the orthogonalization with the \
option normed -> False. Besides of the parameter dependence the reason can be \
a numerical badly conditioned matrix b (e.g. contains vectors with very \
small, or very different in magnitude, components). In this case one can try \
applying orthonorm  on the renormed matrix renorm[b]. \
\>", "Text"],

Cell["\<\
2. In the pseudo-euclidean case the isotropic vectors are orthogonal to \
itself. By our definition, they are considered as being normed. Therefore in \
the resulting orthogonal or orthonormal sequence there may appear linearly \
dependent isotropic vectors.
3. In the pseudo-euclidean vector spaces exist vector sequences which may not \
be orthogonalized preserving its flag structure. In difference to Erhard \
Schmidt's orthogonalization the procedure orthonorm does not preserve the \
flag structure of the given sequence in general.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.3 Examples, tests", "Subsection",
  FontSize->14],

Cell[CellGroupData[{

Cell["3.3.1 Simple, parameterfree vectors", "Subsubsection"],

Cell[BoxData[
    \(orthonorm[{nul[]}]\)], "Input",
  CellLabel->"In[112]:="],

Cell["\<\
Isotropic vectors are treated correctly; here the orthonormal basis of the \
2-space is non isotropic:\
\>", "Text"],

Cell[BoxData[
    \(orthonorm[{stb[1] + stb[5], stb[5]}]\)], "Input",
  CellLabel->"In[113]:="],

Cell[BoxData[
    \(orthonorm[{stb[1] + stb[5], stb[2] + stb[5]}]\)], "Input",
  CellLabel->"In[114]:="],

Cell[BoxData[
    \(orthonorm[{stb[1] + stb[5], stb[1] - stb[5]}]\)], "Input",
  CellLabel->"In[115]:="],

Cell["\<\
Sometimes appear orthonormal linearly dependent isotropic vectors:\
\>", "Text"],

Cell[BoxData[
    \(\(\(\ \)\(orthonorm[{stb[2], 
        stb[1] + stb[5], \((stb[1]\  + \ stb[5])\)*3}]\)\)\)], "Input",
  CellLabel->"In[116]:="],

Cell[BoxData[
    \(\(\(\ \)\(orthonorm[{stb[1], stb[5]\  - stb[2], nul[], \ 
        stb[2]*5 + \ stb[5]}]\)\)\)], "Input",
  CellLabel->"In[117]:="],

Cell[BoxData[
    \(Table[pssp[%[\([i]\)], %[\([j]\)]], {i, 3}, {j, 3}]\)], "Input",
  CellLabel->"In[118]:="],

Cell[BoxData[
    \(\(\(\ \)\(orthonorm[{stb[1], \((stb[5]\  - stb[2])\)*25, \ 
        stb[1]*5 + \ stb[4]}]\)\)\)], "Input",
  CellLabel->"In[119]:="],

Cell[BoxData[
    \(orthonorm[{stb[1] + stb[4], stb[1]\  - \ stb[5], 
        stb[1]*3\  - stb[5]}]\)], "Input",
  CellLabel->"In[120]:="],

Cell[BoxData[
    \(Table[pssp[%[\([i]\)], %[\([j]\)]], {i, 3}, {j, 3}]\)], "Input",
  CellLabel->"In[121]:="],

Cell["\<\
Clearly,  by numerical reason,  one may expect an approximative result only, \
if during the calculation appear floating point numbers.\
\>", "Text"],

Cell[BoxData[
    \(orthonorm[{stb[1] + stb[4], stb[1]\  - \ stb[5], 
        stb[1]*3.1\  - stb[5]}]\)], "Input",
  CellLabel->"In[122]:="],

Cell[BoxData[
    \(Table[pssp[%[\([i]\)], %[\([j]\)]], {i, 3}, {j, 3}]\)], "Input",
  CellLabel->"In[123]:="],

Cell["Here is a test with a random floating point number:", "Text"],

Cell[BoxData[{
    \(\(t1\  = \ 10*Random[];\)\), "\n", 
    \(\(avt\  = 
        orthonorm[{\ stb[3], stb[5]\  + \ stb[2], \ nul[], \ 
            stb[5] Cosh[t1]\  + \ 5*stb[1] Sinh[t1], 
            stb[5]\  + \ stb[2]}];\)\), "\n", 
    \(\tsmoothing[
      Table[pssp[avt[\([i]\)], avt[\([j]\)]], {i, Length[avt]}, {j, 
          Length[avt]}]]\)}], "Input",
  CellLabel->"In[124]:=",
  FontSize->15],

Cell["Repeat the evaluation of the next cell several times!", "Text"],

Cell[BoxData[{
    \(\(t1\  = \ 10*Random[];\)\), "\n", 
    \(\(avt\  = 
        orthonorm[{\ stb[3], stb[5]\  + \ stb[2], \ nul[], \ 
            stb[5] Cosh[t1]\  + \ 5*stb[1] Sinh[t1], 
            stb[5]\  + \ stb[2]}];\)\), "\n", 
    \(\tTable[pssp[avt[\([i]\)], avt[\([j]\)]], {i, 3}, {j, 3}]\)}], "Input",
  CellLabel->"In[146]:=",
  FontSize->15],

Cell[BoxData[
    \(Remove[avt]\)], "Input",
  CellLabel->"In[149]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["3.3.2 Vectors depending on parameters", "Subsubsection"],

Cell["\<\
The following examples show that orthonorm worked correctly also on sequences \
depending on parameters.\
\>", "Text"],

Cell[BoxData[
    \(\(\(\ \)\(orthonorm[{stb[
          1], \((stb[5]\  - stb[2])\)*25, \ \((stb[1]*5 + \ stb[4])\)*
          a}, \[IndentingNewLine]print \[Rule] True]\)\)\)], "Input",
  CellLabel->"In[150]:="],

Cell[BoxData[
    \(\(\(\ \)\(orthonorm[{stb[
          1], \((stb[5]\  - stb[2])\)*25, \ \((stb[1]*5 + \ stb[4])\)*
          a}, \[IndentingNewLine]print \[Rule] True, \ 
      normed\  \[Rule] \ False]\)\)\)], "Input",
  CellLabel->"In[151]:="],

Cell[BoxData[
    \(orthonorm[{stb[1]\ *c + \ stb[5]*3, stb[1]\  + \ stb[5]}, 
      print \[Rule] True, \ normed\  \[Rule] \ False]\)], "Input",
  CellLabel->"In[153]:="],

Cell[BoxData[
    \(orthonorm[{stb[1]\ *c + \ stb[5]*3, stb[1]\  + \ stb[5]}, 
      print \[Rule] True]\)], "Input",
  CellLabel->"In[177]:="],

Cell[BoxData[
    \(Table[pssp[%[\([i]\)], %[\([j]\)]], {i, 2}, {j, 2}]\)], "Input",
  CellLabel->"In[178]:="],

Cell[BoxData[
    \(Simplify[Together[%]]\)], "Input",
  CellLabel->"In[179]:="],

Cell[BoxData[
    \(te[c_] = \ %\)], "Input",
  CellLabel->"In[180]:="],

Cell[BoxData[
    \(te[c]\)], "Input",
  CellLabel->"In[158]:="],

Cell[BoxData[
    \(te[3]\)], "Input",
  CellLabel->"In[159]:="],

Cell[BoxData[
    \(Limit[te[c], \ c\  \[Rule] \ 3]\)], "Input",
  CellLabel->"In[160]:="],

Cell[BoxData[
    \(Limit[te[c], \ c\  \[Rule] 3, \ 
      Direction\  \[Rule] \ \(-1\)]\)], "Input",
  CellLabel->"In[161]:="],

Cell[BoxData[
    \(Limit[te[c], \ c\  \[Rule] \ 3, \ Direction\  \[Rule] \ 1]\)], "Input",
  CellLabel->"In[162]:="],

Cell["\<\
The evaluation worked for general c, in spite of the fact that at c = 3  the \
character  of the constructed vectors changes. Moreover, for c= 3 the 2-space \
degenerates to an isotropic line. \
\>", "Text"],

Cell[BoxData[
    \(orthonorm[{nul[], \ 3*stb[2], \ nul[], \ 
        stb[1] Cosh[t]\  + \ 2*stb[5] Sinh[t], \n\ \ \ \ stb[1]\  + \ 
          stb[2]}, \ print\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[163]:="],

Cell[BoxData[
    \(Table[
      pssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, 
        Length[%]}]\)], "Input",
  CellLabel->"In[164]:="],

Cell[BoxData[
    \(ftest[t_]\  = \ Simplify[Together[%]]\)], "Input",
  CellLabel->"In[165]:="],

Cell[BoxData[
    \(ftest[0]\)], "Input",
  CellLabel->"In[166]:="],

Cell[BoxData[
    \(Limit[ftest[t], \ t\  \[Rule] \ 0, \ Direction\  \[Rule] 1]\)], "Input",\

  CellLabel->"In[167]:="],

Cell[BoxData[
    \(Limit[ftest[t], \ t\  \[Rule] \ 0, \ 
      Direction\  \[Rule] \(-1\)]\)], "Input",
  CellLabel->"In[168]:="],

Cell[BoxData[
    \(ftest[1. ]\)], "Input",
  CellLabel->"In[169]:="],

Cell[BoxData[
    \(ftest[0.1]\)], "Input",
  CellLabel->"In[170]:="],

Cell[BoxData[
    \(orthonorm[{nul[], \ 3*stb[2], \ nul[], \ 
        stb[2] Cosh[t]\  + \ stb[5] Sinh[t], \n\ \ \ \ stb[1]\  + \ 
          stb[2]}]\)], "Input",
  CellLabel->"In[171]:="],

Cell[BoxData[
    \(orthonorm[{\ stb[1] Cosh[t]\  + \ stb[5] Sinh[t], 
        stb[1]\  + \ \ stb[2], stb[2], \ nul[], 
        stb[1]\  - \ stb[2]}]\)], "Input",
  CellLabel->"In[172]:="],

Cell[BoxData[
    \(Table[
      pssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, 
        Length[%]}]\)], "Input",
  CellLabel->"In[173]:="],

Cell[BoxData[
    \(Simplify[Together[%]]\)], "Input",
  CellLabel->"In[174]:="],

Cell[BoxData[
    \(orthonorm[{nul[], \ 3*stb[2], \ nul[], \ 
        stb[1] Cosh[t]\  + \ stb[5] Sinh[t], \n\ \ \ \ stb[3]\  + \ 
          stb[2]}]\)], "Input",
  CellLabel->"In[175]:="],

Cell[BoxData[
    \(Remove[te, \ ftest]\)], "Input",
  CellLabel->"In[181]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["3.3.3 Index ind = 2", "Subsubsection"],

Cell["\<\
If the index is greater than 1, you can meet also mutual orthogonal, \
isotropic,  linearly independent vectors. If you wish to consider such \
examples, first quit the kernel! Change the first command line in subsection \
1.1 according to yor wishes, and repeat the  initialization! If you have \
chosen ind \[NotEqual] 2,  ignore the comments in this subsection.\
\>", "Text",
  FontSize->13],

Cell[BoxData[
    \(ch[stb[4] + stb[2]]\)], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \(pssp[stb[4], stb[4]]\)], "Input",
  CellLabel->"In[9]:="],

Cell["\<\
Here we have two isotropic orthogonal vectors spanning a totally isotropic \
2-dimensional subspace:\
\>", "Text"],

Cell[BoxData[
    \(orthonorm[{\ stb[1]\  + \ stb[5], stb[4]\  + stb[2]}]\)], "Input",
  CellLabel->"In[10]:=",
  FontSize->15],

Cell["\<\
These vectors are orthogonal and have norm 0, therefore they have been \
returned correctly. Moreover, they are linearly independent.\
\>", "Text",
  FontSize->13],

Cell[BoxData[
    \(orthonorm[{\ stb[1]\  + \ stb[5], stb[1]*2\  + \ stb[3], 
        stb[4]\  + stb[2]}]\)], "Input",
  CellLabel->"In[11]:=",
  FontSize->15],

Cell[BoxData[
    \(Table[pssp[%[\([i]\)], %[\([j]\)]], {i, 3}, {j, 3}]\)], "Input",
  CellLabel->"In[12]:="],

Cell["\<\
The next two isotropic vectors span a 2-dimensional pseudoeuclidean subspace:\
\
\>", "Text",
  FontSize->13],

Cell[BoxData[
    \(orthonorm[{\ stb[1]\  + \ stb[5], stb[5]\  + stb[2]}, 
      print\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[13]:=",
  FontSize->15]
}, Closed]],

Cell[CellGroupData[{

Cell["3.3.4 Tests with random vectors (case ind = 2)", "Subsubsection"],

Cell[BoxData[
    \(ind\)], "Input",
  CellLabel->"In[14]:="],

Cell[BoxData[{
    \(\(atest\  = Table[randomv[5], {i, 5}];\)\), "\n", 
    \(ot\  = orthonorm[atest]; Print[ot // MatrixForm];\), "\n", 
    \(Table[
      smoothing[pssp[ot[\([i]\)], ot[\([j]\)]]], {i, 5}, {j, 5}]\)}], "Input",\

  CellLabel->"In[15]:="],

Cell[BoxData[{
    \(\(atest\  = 
        PrependTo[Table[randomv[5], {i, 7}], 
          stb[1]\  + \ stb[5]];\)\), "\n", 
    \(ot\  = orthonorm[atest]; Print[ot // MatrixForm];\), "\n", 
    \(Table[
      smoothing[pssp[ot[\([i]\)], ot[\([j]\)]]], {i, Length[ot]}, {j, 
        Length[ot]}]\)}], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[{
    \(\(atest\  = Table[randomv[5], {i, 3}];\)\), "\n", 
    \(\(atest\  = \ 
        Join[atest, {stb[1]\  + \ stb[5], stb[2]\  + \ stb[4]}];\)\), "\n", 
    \(ot\  = orthonorm[atest]; Print[ot // MatrixForm];\), "\n", 
    \(Table[
      smoothing[pssp[ot[\([i]\)], ot[\([j]\)]]], {i, 5}, {j, 5}]\)}], "Input",\

  CellLabel->"In[21]:="],

Cell[BoxData[{
    \(\(atest\  = Table[randomv[5], {i, 7}];\)\), "\n", 
    \(\(atest\  = \ 
        Join[{stb[1]\  + \ stb[5], stb[2]\  + \ stb[4]}, atest];\)\), "\n", 
    \(ot\  = orthonorm[atest]; Print[ot // MatrixForm];\), "\n", 
    \(Table[
      smoothing[pssp[ot[\([i]\)], ot[\([j]\)]]], {i, 5}, {j, 5}]\)}], "Input",\

  CellLabel->"In[25]:="],

Cell[BoxData[{
    \(\(atest\  = Table[randomv[5], {i, 7}];\)\), "\n", 
    \(ot\  = orthonorm[atest, print\  \[Rule] \ True]; 
    Print[ot // MatrixForm];\), "\n", 
    \(Table[
      smoothing[pssp[ot[\([i]\)], ot[\([j]\)]]], {i, Length[ot]}, {j, 
        Length[ot]}]\)}], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(Remove[atest, \ ot]\)], "Input",
  CellLabel->"In[32]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["3.3.5 Renorming of badly conditioned matrices", "Subsubsection"],

Cell["\<\
PRESERVE ind = 2, dim = 5 !
The comments are valid in this case only. For ind =1, dim = 5 the sequence ot \
will be orthonormed with approximative correctness, test it later!\
\>", "Text"],

Cell["\<\
The next matrix ot has vectors with coordinates of very different absolute \
values. Applying orthonorm ,  we  do not get orthogonal vectors:\
\>", "Text"],

Cell[BoxData[
    \(ot\  = {{0.675496095887281988`, 0.838111706002798051`, 
          0.0599127084283752431`, 0.0726205437662291686`, 
          0.396285246447809988`}, {2.90602662075336581`, \
\(-0.838111706002798761`\), \(-0.0599127084283752875`\), \
\(-0.0726205437662292307`\), 
          3.18523747019283787`}, {\(-9.19936356392773468`*^7\), 
          3.3554431999999994`*^7, \(-6.36335108454432329`*^-10\), 
          3.35544319999999984`*^7, \(-9.19936356392773468`*^7\)}}\)], "Input",\

  CellLabel->"In[53]:="],

Cell["The three vectors are \"almost\" orthogonal:", "Text"],

Cell[BoxData[
    \(Table[
        smoothing[pssp[ot[\([i]\)], ot[\([j]\)]]], {i, Length[ot]}, {j, 
          Length[ot]}] // MatrixForm\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \(at\  = orthonorm[ot]\)], "Input",
  CellLabel->"In[55]:="],

Cell["We try it without norming:", "Text"],

Cell[BoxData[
    \(at\  = orthonorm[ot, \ normed\  \[Rule] \ False]\)], "Input",
  CellLabel->"In[56]:="],

Cell["\<\
If one renorms the vectors of ot, one preserves their span, and hopefully, \
can find  a nearly orthonormal basis:\
\>", "Text"],

Cell[BoxData[
    \(\(?renorm\)\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(renorm[b_?MatrixQ] := \ 
      Module[{bo\  = \ outzero[b], i\ }, \n\t
        Table[bo[\([i]\)]/Sqrt[bo[\([i]\)] . bo[\([i]\)]], 
          \ {i, \ Length[bo]}]]\)], "Input",
  CellLabel->"In[13]:=",
  Evaluatable->False],

Cell[BoxData[
    \(otre\  = \ renorm[ot]\)], "Input",
  CellLabel->"In[57]:="],

Cell[BoxData[
    \(orthonorm[otre, \ \ print\  \[Rule] \ True, \ 
      normed\  \[Rule] \ False]\)], "Input",
  CellLabel->"In[58]:="],

Cell[BoxData[
    \(\(at\  = \ %;\)\)], "Input",
  CellLabel->"In[59]:="],

Cell[BoxData[
    \(Table[
        smoothing[pssp[at[\([i]\)], at[\([j]\)]]], {i, Length[at]}, {j, 
          Length[at]}] // \n\ \ MatrixForm\)], "Input",
  CellLabel->"In[60]:="],

Cell["\<\
Here  we succeeded,  but if we want to get normed vectors, the procedure does \
not work:. \
\>", "Text"],

Cell[BoxData[
    \(orthonorm[otre, \ \ print\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[61]:="],

Cell["\<\
The reason is that the last vector may not be normalized correctly:\
\>", "Text"],

Cell[BoxData[
    \(atn = \ Table[normalize[at[\([i]\)]], {i, 3}]\)], "Input",
  CellLabel->"In[62]:="],

Cell["\<\
The scalar products after normalization are incorrect; therefore the \
procedure did not return the found vectors:\
\>", "Text"],

Cell[BoxData[
    \(Table[
        smoothing[pssp[atn[\([i]\)], atn[\([j]\)]]], {i, Length[atn]}, {j, 
          Length[atn]}] // \n\ \ MatrixForm\)], "Input",
  CellLabel->"In[63]:="],

Cell["\<\
The third vector of at seems to be isotropic; but copying it we get its \
'inner hidden' value:\
\>", "Text"],

Cell[BoxData[
    \(badv\  = {\(-0.6642969596426436`\), 0.24230053530593257`, 0, 
        0.24230053530593232`, \(-0.6642969596426437`\)}\)], "Input",
  CellLabel->"In[64]:="],

Cell[BoxData[
    \(pssp[badv, badv]\)], "Input",
  CellLabel->"In[65]:="],

Cell[BoxData[
    \(normalize[badv]\)], "Input",
  CellLabel->"In[66]:="],

Cell[BoxData[
    \(badnormv\  = \ %\)], "Input",
  CellLabel->"In[67]:="],

Cell[BoxData[
    \(pssp[badnormv, badnormv]\)], "Input",
  CellLabel->"In[68]:="],

Cell["The hidden value of badnormv is", "Text"],

Cell[BoxData[
    \({\(-6.304594370522744`*^7\), 2.2995838964643177`*^7, 0, 
      2.299583896464315`*^7, \(-6.304594370522745`*^7\)}\)], "Input"],

Cell["\<\
Here  we succeeded,  but in the next example we have certain difficulties. \
\>", "Text"],

Cell[BoxData[
    \(\(ot\  = {{0.675496095887281988`, 0.838111706002798051`, 
            0.0599127084283752431`, 0.0726205437662291686`, 
            0.396285246447809988`}, {2.90602662075336581`, \
\(-0.838111706002798761`\), \(-0.0599127084283752875`\), \
\(-0.0726205437662292307`\), 
            3.18523747019283787`}, {\(-9.19936356392773468`*^7\), 
            3.3554431999999994`*^7, \(-6.36335108454432329`*^-10\), 
            3.35544319999999984`*^7, \(-9.19936356392773468`*^7\)}, \
{6.82635841776287133`*^7, \(-2.48989593404715137`*^7\), 
            9.9150403743008848`*^-9, \(-2.48989593404715048`*^7\), 
            6.82635841776287133`*^7}, {\(-1.82640220915799815`*^7\), 
            6.66175309911068413`*^6, 2.6549032889775681`*^-7, 
            6.66175309911060686`*^6, \(-1.82640220915799815`*^7\)}};\)\)], \
"Input",
  CellLabel->"In[69]:="],

Cell["\<\
This matrix is bad conditioned too, but not nearly orthogonal:\
\>", "Text"],

Cell[BoxData[
    \(Table[
        smoothing[pssp[ot[\([i]\)], ot[\([j]\)]]], {i, Length[ot]}, {j, 
          Length[ot]}] // MatrixForm\)], "Input",
  CellLabel->"In[70]:="],

Cell[BoxData[
    \(orthonorm[ot]\)], "Input",
  CellLabel->"In[71]:="],

Cell[BoxData[
    \(otre\  = \ renorm[ot]\)], "Input",
  CellLabel->"In[72]:="],

Cell["\<\
The matrix  otre is badly conditioned again: The last three vectors are \
pairwise almost linearly dependent, and we have\
\>", "Text"],

Cell[BoxData[
    \(Det[otre]\)], "Input",
  CellLabel->"In[73]:="],

Cell[BoxData[
    \(Table[pssp[otre[\([i]\)], otre[\([j]\)]], {i, 5}, {j, 5}]\)], "Input",
  CellLabel->"In[74]:="],

Cell["With the default norming we do not succeed:", "Text"],

Cell[BoxData[
    \(at1\  = \ orthonorm[otre, \ print\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[75]:="],

Cell["but without norming we get a reasonable result:", "Text"],

Cell[BoxData[
    \(at1\  = \ 
      orthonorm[otre, \ print\  \[Rule] \ True, 
        normed\  \[Rule] \ False]\)], "Input",
  CellLabel->"In[76]:="],

Cell[BoxData[
    \(Table[
        smoothing[pssp[at1[\([i]\)], at1[\([j]\)]]], {i, Length[at1]}, {j, 
          Length[at1]}] // MatrixForm\)], "Input",
  CellLabel->"In[77]:="],

Cell["\<\
That looks wrong: in the pseudo-euclidean vector space of index 2 there may \
not exist three mutual orthogonal linearly independent isotropic vectors! The \
numerical deviation from orthonormality one can see evaluating the scalar \
products not using smoothing:\
\>", "Text"],

Cell[BoxData[
    \(Table[
      pssp[at1[\([i]\)], at1[\([j]\)]], {i, Length[at1]}, {j, 
        Length[at1]}]\)], "Input",
  CellLabel->"In[78]:="],

Cell["\<\
Thus, orthonorm yields very small, almost orthogonal vectors, and the last \
three vectors are almost isotropic; the five vectors are \"almost linearly \
dependent\":\
\>", "Text"],

Cell[BoxData[
    \(Det[at1]\)], "Input",
  CellLabel->"In[79]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.4  psfilter", "Subsection",
  FontSize->14],

Cell["\<\
We now go back to index 1: Quit kernel, set dim = 5, ind = 1 in subsection \
1.1,  and repeat the Initialization!\
\>", "Text"],

Cell[BoxData[
    \({dim\ , \ ind}\)], "Input",
  CellLabel->"In[8]:="],

Cell["\<\
Remember that in a space of index ind  any orthogonal sequence of linearly \
independent vectors may contain not more than ind non spacelike vectors.\
\>", "Text"],

Cell["\<\
Sometimes it is necessary to place the spacelike vectors of a sequence b at \
the first places. The following procedure  realizes such a permutation: \
\>", "Text"],

Cell[BoxData[
    \(\(?psfilter\)\)], "Input",
  CellLabel->"In[9]:="],

Cell[BoxData[
    \(psfilter[b_List]\  := \ 
      Module[{i, \ lb\  = \ Length[b], \ bn = \ {}, qn\  = \ {}}, \n\t\t
        For[i = 1, i <= lb, \(i++\), \n\t\t\t
          If[Length[b[\([i]\)]\ ]\  != \ dim, \ 
            Return[\ Print["\<Incorrect argument \>", b[\([i]\)], 
                "\<!!!\>"]]]]; \n\t\t
        For[i = 1, \ i <= lb, \ \(i++\), \n\t\t\t
          If[ch[b[\([i]\)]\ ] == \ 1, \ AppendTo[bn, \ b[\([i]\)]], 
            AppendTo[qn, b[\([i]\)]], AppendTo[qn, b[\([i]\)]]]]; \n\t\t\t\t
        Return[Join[bn, qn]]]\)], "Input",
  CellLabel->"In[16]:=",
  Evaluatable->False],

Cell[BoxData[
    \(\(\(\t\t\)\(psfilter[{a}]\)\)\)], "Input",
  CellLabel->"In[10]:="],

Cell[BoxData[
    \(\(\(\t\t\)\(psfilter[{nul[], \ stb[5], \ stb[1]}]\)\)\)], "Input",
  CellLabel->"In[11]:="],

Cell["\<\
psfilter  yields the expected result also for non orthogonal sequences:\
\>", "Text"],

Cell[BoxData[
    \(\(\(\t\t\)\(psfilter[{stb[2], stb[5]*Cos[t], \ stb[5], \ 
        stb[1]}]\)\)\)], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(psfilter[%]\)], "Input",
  CellLabel->"In[13]:="],

Cell[BoxData[
    \(psfilter[{\ stb[1]\  + \ stb[5], 
        stb[5]\  + stb[2], \ {Cos[t], \ Sin[t], \ 1, 0, 1}}]\)], "Input",
  CellLabel->"In[15]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["3.5 Orthogonal pairs", "Subsection",
  FontSize->14],

Cell["\<\
Now we generate a pair of orthonormal spacelike vectors starting from a given \
spacelike one; that we need for the geometry of circles on the 3-sphere.The \
definition is contained in the  package pseuvec.m.\
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(\(?orthopair\)\)], "Input",
  CellLabel->"In[16]:=",
  FontSize->14],

Cell[BoxData[
    \(orthopair[l_List] := 
      Module[{a, l1 = l, m1 = nul[], b = dim - ind, j, k}, 
          a = Simplify[pssp[l, l]]; \[IndentingNewLine]If[a \[LessEqual] 0, 
            Print["\<Vector \>", l, "\< is not spacelike!\>"]; Return[], 
            Null, Print["\<Maybe, Vector \>", l, "\< is not spacelike?\>"]; 
            Return[]]; \ 
          If[b < 2, Print["\<Orthogonal spacelike pairs don't exist!\>"]; 
            Return[]]; \ \t\ l1 = l1/Sqrt[a]; \ \tFor[k = 1, 
            k \[LessEqual] b, \(k++\), 
            If[l1[\([k]\)] \[Equal] 0, m1 = stb[k]; \ 
              Return[{l1, m1}]]]; \ \t\ For[j = 1, j \[LessEqual] b, \(j++\), 
            If[OddQ[j] && j < b, 
              m1[\([j]\)] = \(-l1[\([j + 1]\)]\)]; \ \ \t\tIf[EvenQ[j], 
              m1[\([j]\)] = l1[\([j - 1]\)]]]; \ \t\tm1 = 
            m1/Sqrt[pssp[m1, m1]]; \[IndentingNewLine]\ {l1, m1}] /; 
        Length[l] \[Equal] dim\)], "Input",
  Evaluatable->False],

Cell[" The following function controls orthonormality:", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman",
  FontWeight->"Plain",
  FontSlant->"Plain",
  FontTracking->"Plain",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1],
  FontVariations->{"Underline"->False,
  "Outline"->False,
  "Shadow"->False}],

Cell[BoxData[
    \(control[v_, w_] := 
      Simplify[\((pssp[v, v] - 1)\)\^2 + pssp[v, w]\^2 + 
          \((pssp[w, w] - 1)\)\^2]\)], "Input",
  CellLabel->"In[4]:=",
  Evaluatable->False,
  FontSize->14],

Cell[BoxData[
    \(\(?control\)\)], "Input",
  CellLabel->"In[17]:="],

Cell["\<\
If all is correct the result mus be 0, or numerically very small. If the \
result depends on a parameter, integrate!\
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman",
  FontWeight->"Plain",
  FontSlant->"Plain",
  FontTracking->"Plain",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1],
  FontVariations->{"Underline"->False,
  "Outline"->False,
  "Shadow"->False}],

Cell[CellGroupData[{

Cell["Tests", "Subsubsection",
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(orthopair[stb[1]]\)], "Input",
  CellLabel->"In[18]:=",
  PageWidth->Infinity,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(orthopair[stb[2]]\)], "Input",
  CellLabel->"In[19]:=",
  PageWidth->Infinity,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(orthopair[stb[3]]\)], "Input",
  CellLabel->"In[20]:=",
  PageWidth->Infinity,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(orthopair[stb[4]]\)], "Input",
  CellLabel->"In[21]:=",
  PageWidth->Infinity,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(orthopair[stb[5]]\)], "Input",
  CellLabel->"In[22]:=",
  PageWidth->Infinity,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(orthopair[{Cos[t], Sin[t], Cos[s], Sin[s], 0}\/\@2]\)], "Input",
  CellLabel->"In[23]:=",
  PageWidth->Infinity,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[24]:="],

Cell["\<\
The argument of orthopair must be a spacelike vector, but it needs not to be \
a unit vector:\
\>", "Text",
  Evaluatable->False,
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(orthopair[{Cos[t], Sin[t], Cos[s], Sin[s], 0}]\)], "Input",
  CellLabel->"In[25]:=",
  PageWidth->Infinity,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(Simplify[orthopair[{1, Cos[t], Sin[t], Cos[s], Sin[s]}]]\)], "Input",
  CellLabel->"In[27]:=",
  PageWidth->Infinity,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell["\<\
In spite of the fact that the argument is spacelike for all values of s and \
t!\
\>", "Text"],

Cell[BoxData[
    \(pssp[{1, Cos[t], Sin[t], Cos[s], Sin[s]}, {1, Cos[t], Sin[t], Cos[s], 
        Sin[s]}]\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(orthopair[\ 3*randomv[\ ]]\)], "Input",
  CellLabel->"In[33]:=",
  PageWidth->Infinity,
  FontFamily->"Times New Roman",
  FontSize->14],

Cell[BoxData[
    \(Table[
      Simplify[pssp[%[\([i]\)], \ %[\([j]\)]]], {i, 2}, {j, 2}]\)], "Input",
  CellLabel->"In[34]:=",
  FontSize->14],

Cell[BoxData[
    \(control[%%[\([1]\)], \ %%[\([2]\)]]\)], "Input",
  CellLabel->"In[35]:="]
}, Closed]]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 695}},
AutoGeneratedPackage->Automatic,
WindowToolbars->"RulerBar",
WindowSize->{852, 668},
WindowMargins->{{39, Automatic}, {Automatic, -6}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{597.563, 842.375},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{"home", "rolf", "mymath", \
"spheres.v3"}, "pseuklid.nb.ps", CharacterEncoding -> "WindowsANSI"],
"Magnification"->1},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
SpellingDictionaries->{"CorrectWords"->{
  "orthonormalization", "orthonorm", "Erhard", "parameterfree", "nul"}},
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
StyleDefinitions -> "TutorialBook.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 88, 2, 72, "Title"],
Cell[1830, 55, 149, 4, 33, "Subtitle",
  Evaluatable->False],
Cell[1982, 61, 205, 8, 58, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[2212, 73, 144, 4, 37, "Subsubsection",
  Evaluatable->False],
Cell[2359, 79, 1278, 21, 191, "Text",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[3674, 105, 33, 0, 31, "Subsubsection"],
Cell[3710, 107, 214, 4, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[3961, 116, 32, 0, 31, "Subsubsection"],
Cell[3996, 118, 114, 4, 89, "Text"],
Cell[4113, 124, 90, 2, 27, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[4240, 131, 33, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[4298, 135, 41, 0, 44, "Subsection"],
Cell[4342, 137, 311, 8, 156, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[4690, 150, 33, 0, 38, "Subsection"],
Cell[4726, 152, 205, 5, 61, "Text"],
Cell[4934, 159, 56, 0, 28, "Text"],
Cell[4993, 161, 68, 2, 31, "Input"],
Cell[5064, 165, 282, 5, 48, "Text"],
Cell[5349, 172, 114, 2, 31, "Input"],
Cell[5466, 176, 111, 3, 28, "Text"],
Cell[5580, 181, 78, 2, 31, "Input"],
Cell[5661, 185, 130, 3, 28, "Text"],
Cell[5794, 190, 74, 2, 31, "Input"],
Cell[5871, 194, 156, 3, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6064, 202, 65, 0, 44, "Subsection"],
Cell[6132, 204, 190, 5, 61, "Text"],
Cell[6325, 211, 164, 3, 48, "Text"],
Cell[6492, 216, 78, 2, 31, "Input"],
Cell[6573, 220, 156, 3, 32, "Text"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6778, 229, 42, 0, 92, "Section"],

Cell[CellGroupData[{
Cell[6845, 233, 38, 0, 44, "Subsection"],
Cell[6886, 235, 186, 4, 27, "Text"],
Cell[7075, 241, 122, 4, 30, "Input",
  InitializationCell->True],
Cell[7200, 247, 456, 9, 127, "Text"],
Cell[7659, 258, 303, 7, 46, "Text",
  Evaluatable->False],
Cell[7965, 267, 104, 3, 27, "Text"],
Cell[8072, 272, 109, 4, 30, "Input",
  InitializationCell->True],
Cell[8184, 278, 107, 3, 28, "Text"],
Cell[8294, 283, 90, 3, 31, "Input",
  InitializationCell->True],
Cell[8387, 288, 37, 0, 28, "Text"],
Cell[8427, 290, 110, 3, 31, "Input",
  InitializationCell->True],
Cell[8540, 295, 85, 2, 28, "Text"],
Cell[8628, 299, 159, 4, 31, "Input",
  InitializationCell->True],
Cell[8790, 305, 70, 0, 28, "Text"],
Cell[8863, 307, 64, 2, 31, "Input"],
Cell[8930, 311, 293, 6, 48, "Text",
  Evaluatable->False],
Cell[9226, 319, 132, 3, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[9395, 327, 36, 0, 44, "Subsection"],

Cell[CellGroupData[{
Cell[9456, 331, 50, 0, 37, "Subsubsection"],
Cell[9509, 333, 67, 0, 28, "Text"],
Cell[9579, 335, 66, 2, 31, "Input"],
Cell[9648, 339, 66, 2, 31, "Input"],
Cell[9717, 343, 66, 2, 31, "Input"],
Cell[9786, 347, 70, 2, 31, "Input"],
Cell[9859, 351, 68, 2, 31, "Input"],
Cell[9930, 355, 71, 2, 31, "Input"],
Cell[10004, 359, 74, 2, 31, "Input"],
Cell[10081, 363, 78, 2, 31, "Input"],
Cell[10162, 367, 69, 2, 31, "Input"],
Cell[10234, 371, 68, 2, 31, "Input"],
Cell[10305, 375, 70, 2, 31, "Input"],
Cell[10378, 379, 72, 2, 31, "Input"],
Cell[10453, 383, 70, 2, 31, "Input"],
Cell[10526, 387, 71, 2, 31, "Input"],
Cell[10600, 391, 74, 2, 31, "Input"],
Cell[10677, 395, 73, 2, 31, "Input"],
Cell[10753, 399, 67, 2, 31, "Input"],
Cell[10823, 403, 67, 2, 31, "Input"],
Cell[10893, 407, 69, 2, 31, "Input"],
Cell[10965, 411, 72, 2, 31, "Input"],
Cell[11040, 415, 82, 2, 31, "Input"],
Cell[11125, 419, 71, 0, 28, "Text"],
Cell[11199, 421, 69, 2, 31, "Input"],
Cell[11271, 425, 71, 2, 31, "Input"],
Cell[11345, 429, 73, 2, 31, "Input"],
Cell[11421, 433, 71, 2, 31, "Input"],
Cell[11495, 437, 72, 2, 31, "Input"],
Cell[11570, 441, 74, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[11681, 448, 52, 0, 37, "Subsubsection"],
Cell[11736, 450, 66, 2, 31, "Input"],
Cell[11805, 454, 67, 2, 31, "Input"],
Cell[11875, 458, 67, 2, 31, "Input"],
Cell[11945, 462, 70, 2, 31, "Input"],
Cell[12018, 466, 70, 2, 31, "Input"],
Cell[12091, 470, 65, 2, 31, "Input"],
Cell[12159, 474, 65, 2, 31, "Input"],
Cell[12227, 478, 72, 2, 31, "Input"],
Cell[12302, 482, 76, 2, 31, "Input"],
Cell[12381, 486, 69, 2, 31, "Input"],
Cell[12453, 490, 68, 2, 31, "Input"],
Cell[12524, 494, 70, 2, 31, "Input"],
Cell[12597, 498, 71, 2, 31, "Input"],
Cell[12671, 502, 72, 2, 31, "Input"],
Cell[12746, 506, 72, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[12879, 515, 130, 4, 60, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[13034, 523, 101, 3, 42, "Subsection",
  Evaluatable->False],
Cell[13138, 528, 182, 6, 27, "Text",
  Evaluatable->False],
Cell[13323, 536, 92, 3, 51, "Input"],
Cell[13418, 541, 96, 3, 27, "Text",
  Evaluatable->False],
Cell[13517, 546, 219, 8, 30, "Input"],
Cell[13739, 556, 252, 8, 58, "Text",
  Evaluatable->False],
Cell[13994, 566, 193, 6, 30, "Input"],
Cell[14190, 574, 104, 3, 27, "Text",
  Evaluatable->False],
Cell[14297, 579, 122, 5, 30, "Input"],
Cell[14422, 586, 102, 3, 27, "Text",
  Evaluatable->False],
Cell[14527, 591, 80, 3, 30, "Input"],
Cell[14610, 596, 86, 3, 27, "Text",
  Evaluatable->False],
Cell[14699, 601, 129, 5, 30, "Input"],
Cell[14831, 608, 302, 8, 46, "Text"],
Cell[15136, 618, 125, 3, 30, "Input"],
Cell[15264, 623, 192, 6, 27, "Text",
  Evaluatable->False],
Cell[15459, 631, 141, 5, 30, "Input"],
Cell[15603, 638, 158, 5, 30, "Input"],
Cell[15764, 645, 147, 5, 30, "Input"],
Cell[15914, 652, 124, 4, 27, "Text"],
Cell[16041, 658, 92, 3, 30, "Input"],
Cell[16136, 663, 69, 2, 31, "Input"],
Cell[16208, 667, 156, 4, 30, "Input"],
Cell[16367, 673, 85, 3, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[16489, 681, 58, 1, 36, "Subsection"],
Cell[16550, 684, 426, 8, 65, "Text"],
Cell[16979, 694, 81, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[17085, 701, 49, 1, 36, "Subsubsection"],
Cell[17137, 704, 62, 1, 27, "Text"],
Cell[17202, 707, 83, 3, 30, "Input"],
Cell[17288, 712, 69, 1, 27, "Text"],
Cell[17360, 715, 87, 3, 30, "Input"],
Cell[17450, 720, 48, 1, 27, "Text"],
Cell[17501, 723, 136, 5, 29, "Input"],
Cell[17640, 730, 113, 3, 30, "Input"],
Cell[17756, 735, 50, 1, 27, "Text"],
Cell[17809, 738, 121, 3, 30, "Input"],
Cell[17933, 743, 103, 3, 30, "Input"],
Cell[18039, 748, 50, 1, 27, "Text"],
Cell[18092, 751, 84, 3, 30, "Input"],
Cell[18179, 756, 113, 3, 30, "Input"],
Cell[18295, 761, 111, 3, 31, "Input"],
Cell[18409, 766, 86, 2, 31, "Input"],
Cell[18498, 770, 106, 3, 31, "Input"],
Cell[18607, 775, 39, 0, 28, "Text"],
Cell[18649, 777, 71, 2, 31, "Input"],
Cell[18723, 781, 106, 2, 31, "Input"],
Cell[18832, 785, 64, 2, 31, "Input"],
Cell[18899, 789, 116, 3, 28, "Text"],
Cell[19018, 794, 211, 6, 77, "Input"],
Cell[19232, 802, 77, 2, 31, "Input"],
Cell[19312, 806, 166, 4, 27, "Text"],
Cell[19481, 812, 88, 3, 31, "Input"],
Cell[19572, 817, 115, 3, 30, "Input"],
Cell[19690, 822, 92, 1, 27, "Text"],
Cell[19785, 825, 156, 5, 30, "Input"],
Cell[19944, 832, 125, 3, 28, "Text"],
Cell[20072, 837, 121, 3, 30, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[20242, 846, 53, 1, 36, "Subsection"],
Cell[20298, 849, 93, 2, 27, "Text"],
Cell[20394, 853, 111, 4, 30, "Input"],
Cell[20508, 859, 329, 10, 46, "Text"],

Cell[CellGroupData[{
Cell[20862, 873, 91, 3, 36, "Subsubsection",
  Evaluatable->False],
Cell[20956, 878, 72, 2, 31, "Input"],
Cell[21031, 882, 146, 5, 30, "Input"],
Cell[21180, 889, 106, 3, 27, "Text"],
Cell[21289, 894, 151, 5, 30, "Input"],
Cell[21443, 901, 84, 3, 30, "Input"],
Cell[21530, 906, 85, 3, 30, "Input"],
Cell[21618, 911, 111, 3, 30, "Input"],
Cell[21732, 916, 271, 7, 48, "Text"],
Cell[22006, 925, 110, 3, 30, "Input"],
Cell[22119, 930, 146, 4, 30, "Input"],
Cell[22268, 936, 144, 4, 30, "Input"],
Cell[22415, 942, 127, 3, 30, "Input"],
Cell[22545, 947, 117, 3, 30, "Input"],
Cell[22665, 952, 127, 4, 27, "Text"],
Cell[22795, 958, 112, 3, 30, "Input"],
Cell[22910, 963, 122, 3, 30, "Input"],
Cell[23035, 968, 137, 4, 27, "Text"],
Cell[23175, 974, 151, 5, 30, "Input"],
Cell[23329, 981, 73, 2, 31, "Input"],
Cell[23405, 985, 113, 3, 27, "Text"],
Cell[23521, 990, 145, 5, 30, "Input"],
Cell[23669, 997, 149, 5, 30, "Input"],
Cell[23821, 1004, 164, 5, 30, "Input"],
Cell[23988, 1011, 84, 3, 30, "Input"],
Cell[24075, 1016, 110, 4, 30, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[24234, 1026, 109, 3, 36, "Subsection",
  Evaluatable->False],
Cell[24346, 1031, 529, 15, 89, "Text",
  Evaluatable->False],
Cell[24878, 1048, 127, 5, 30, "Input"],
Cell[25008, 1055, 543, 11, 177, "Input",
  Evaluatable->False],
Cell[25554, 1068, 274, 9, 27, "Text",
  Evaluatable->False],
Cell[25831, 1079, 82, 3, 30, "Input"],
Cell[25916, 1084, 132, 3, 27, "Text",
  Evaluatable->False],
Cell[26051, 1089, 127, 5, 30, "Input"],
Cell[26181, 1096, 168, 4, 27, "Text"],
Cell[26352, 1102, 176, 6, 30, "Input",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[26553, 1112, 91, 3, 36, "Subsubsection",
  Evaluatable->False],
Cell[26647, 1117, 135, 4, 30, "Input"],
Cell[26785, 1123, 133, 4, 30, "Input"],
Cell[26921, 1129, 137, 4, 30, "Input"],
Cell[27061, 1135, 84, 3, 30, "Input"],
Cell[27148, 1140, 121, 3, 30, "Input"],
Cell[27272, 1145, 92, 3, 30, "Input"],
Cell[27367, 1150, 137, 4, 30, "Input"],
Cell[27507, 1156, 133, 3, 31, "Input"],
Cell[27643, 1161, 137, 3, 31, "Input"],
Cell[27783, 1166, 135, 3, 31, "Input"],
Cell[27921, 1171, 135, 4, 30, "Input"],
Cell[28059, 1177, 136, 4, 30, "Input"],
Cell[28198, 1183, 199, 5, 51, "Input"],
Cell[28400, 1190, 86, 3, 30, "Input"],
Cell[28489, 1195, 174, 4, 54, "Input"],
Cell[28666, 1201, 157, 4, 54, "Input"],
Cell[28826, 1207, 178, 4, 54, "Input"],
Cell[29007, 1213, 163, 3, 54, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[29231, 1223, 35, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[29291, 1227, 52, 1, 42, "Subsection"],
Cell[29346, 1230, 2003, 49, 210, "Text"],
Cell[31352, 1281, 65, 0, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[31454, 1286, 76, 1, 36, "Subsection"],

Cell[CellGroupData[{
Cell[31555, 1291, 46, 0, 37, "Subsubsection"],
Cell[31604, 1293, 1487, 48, 158, "Text"],
Cell[33094, 1343, 87, 3, 30, "Input"],
Cell[33184, 1348, 101, 3, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[33322, 1356, 49, 0, 37, "Subsubsection"],
Cell[33374, 1358, 144, 3, 28, "Text"],
Cell[33521, 1363, 73, 2, 31, "Input"],
Cell[33597, 1367, 77, 2, 31, "Input"],
Cell[33677, 1371, 71, 2, 31, "Input"],
Cell[33751, 1375, 3279, 61, 1204, "Input",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[37067, 1441, 41, 0, 37, "Subsubsection"],
Cell[37111, 1443, 608, 9, 108, "Text"],
Cell[37722, 1454, 562, 9, 100, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[38333, 1469, 57, 1, 36, "Subsection"],

Cell[CellGroupData[{
Cell[38415, 1474, 60, 0, 37, "Subsubsection"],
Cell[38478, 1476, 77, 2, 31, "Input"],
Cell[38558, 1480, 126, 3, 28, "Text"],
Cell[38687, 1485, 95, 2, 31, "Input"],
Cell[38785, 1489, 104, 2, 31, "Input"],
Cell[38892, 1493, 104, 2, 31, "Input"],
Cell[38999, 1497, 90, 2, 28, "Text"],
Cell[39092, 1501, 147, 3, 31, "Input"],
Cell[39242, 1506, 150, 3, 31, "Input"],
Cell[39395, 1511, 110, 2, 31, "Input"],
Cell[39508, 1515, 152, 3, 31, "Input"],
Cell[39663, 1520, 138, 3, 31, "Input"],
Cell[39804, 1525, 110, 2, 31, "Input"],
Cell[39917, 1529, 159, 3, 28, "Text"],
Cell[40079, 1534, 140, 3, 31, "Input"],
Cell[40222, 1539, 110, 2, 31, "Input"],
Cell[40335, 1543, 67, 0, 28, "Text"],
Cell[40405, 1545, 405, 10, 114, "Input"],
Cell[40813, 1557, 69, 0, 28, "Text"],
Cell[40885, 1559, 356, 8, 93, "Input"],
Cell[41244, 1569, 70, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[41351, 1576, 62, 0, 31, "Subsubsection"],
Cell[41416, 1578, 128, 3, 28, "Text"],
Cell[41547, 1583, 212, 4, 54, "Input"],
Cell[41762, 1589, 247, 5, 54, "Input"],
Cell[42012, 1596, 171, 3, 54, "Input"],
Cell[42186, 1601, 143, 3, 31, "Input"],
Cell[42332, 1606, 110, 2, 31, "Input"],
Cell[42445, 1610, 80, 2, 31, "Input"],
Cell[42528, 1614, 71, 2, 31, "Input"],
Cell[42602, 1618, 64, 2, 31, "Input"],
Cell[42669, 1622, 64, 2, 31, "Input"],
Cell[42736, 1626, 90, 2, 31, "Input"],
Cell[42829, 1630, 127, 3, 31, "Input"],
Cell[42959, 1635, 117, 2, 31, "Input"],
Cell[43079, 1639, 217, 4, 48, "Text"],
Cell[43299, 1645, 216, 4, 54, "Input"],
Cell[43518, 1651, 142, 4, 31, "Input"],
Cell[43663, 1657, 96, 2, 31, "Input"],
Cell[43762, 1661, 67, 2, 31, "Input"],
Cell[43832, 1665, 120, 3, 31, "Input"],
Cell[43955, 1670, 130, 3, 31, "Input"],
Cell[44088, 1675, 69, 2, 31, "Input"],
Cell[44160, 1679, 69, 2, 31, "Input"],
Cell[44232, 1683, 188, 4, 54, "Input"],
Cell[44423, 1689, 188, 4, 54, "Input"],
Cell[44614, 1695, 142, 4, 31, "Input"],
Cell[44759, 1701, 80, 2, 31, "Input"],
Cell[44842, 1705, 188, 4, 54, "Input"],
Cell[45033, 1711, 78, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[45148, 1718, 44, 0, 31, "Subsubsection"],
Cell[45195, 1720, 404, 7, 65, "Text"],
Cell[45602, 1729, 76, 2, 31, "Input"],
Cell[45681, 1733, 77, 2, 31, "Input"],
Cell[45761, 1737, 124, 3, 28, "Text"],
Cell[45888, 1742, 127, 3, 30, "Input"],
Cell[46018, 1747, 173, 4, 27, "Text"],
Cell[46194, 1753, 159, 4, 30, "Input"],
Cell[46356, 1759, 109, 2, 31, "Input"],
Cell[46468, 1763, 119, 4, 27, "Text"],
Cell[46590, 1769, 158, 4, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[46785, 1778, 71, 0, 31, "Subsubsection"],
Cell[46859, 1780, 61, 2, 31, "Input"],
Cell[46923, 1784, 256, 6, 77, "Input"],
Cell[47182, 1792, 333, 8, 100, "Input"],
Cell[47518, 1802, 354, 8, 100, "Input"],
Cell[47875, 1812, 354, 8, 100, "Input"],
Cell[48232, 1822, 310, 7, 100, "Input"],
Cell[48545, 1831, 77, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[48659, 1838, 70, 0, 31, "Subsubsection"],
Cell[48732, 1840, 198, 4, 80, "Text"],
Cell[48933, 1846, 165, 3, 48, "Text"],
Cell[49101, 1851, 520, 10, 215, "Input"],
Cell[49624, 1863, 60, 0, 28, "Text"],
Cell[49687, 1865, 174, 4, 54, "Input"],
Cell[49864, 1871, 78, 2, 31, "Input"],
Cell[49945, 1875, 42, 0, 28, "Text"],
Cell[49990, 1877, 106, 2, 31, "Input"],
Cell[50099, 1881, 138, 3, 28, "Text"],
Cell[50240, 1886, 69, 2, 31, "Input"],
Cell[50312, 1890, 242, 6, 54, "Input",
  Evaluatable->False],
Cell[50557, 1898, 79, 2, 31, "Input"],
Cell[50639, 1902, 136, 3, 31, "Input"],
Cell[50778, 1907, 73, 2, 31, "Input"],
Cell[50854, 1911, 180, 4, 77, "Input"],
Cell[51037, 1917, 115, 3, 28, "Text"],
Cell[51155, 1922, 101, 2, 31, "Input"],
Cell[51259, 1926, 91, 2, 28, "Text"],
Cell[51353, 1930, 103, 2, 31, "Input"],
Cell[51459, 1934, 138, 3, 28, "Text"],
Cell[51600, 1939, 184, 4, 77, "Input"],
Cell[51787, 1945, 119, 3, 28, "Text"],
Cell[51909, 1950, 175, 3, 54, "Input"],
Cell[52087, 1955, 74, 2, 31, "Input"],
Cell[52164, 1959, 73, 2, 31, "Input"],
Cell[52240, 1963, 74, 2, 31, "Input"],
Cell[52317, 1967, 82, 2, 31, "Input"],
Cell[52402, 1971, 47, 0, 28, "Text"],
Cell[52452, 1973, 146, 2, 54, "Input"],
Cell[52601, 1977, 99, 2, 28, "Text"],
Cell[52703, 1981, 863, 15, 353, "Input"],
Cell[53569, 1998, 86, 2, 28, "Text"],
Cell[53658, 2002, 174, 4, 54, "Input"],
Cell[53835, 2008, 71, 2, 31, "Input"],
Cell[53909, 2012, 79, 2, 31, "Input"],
Cell[53991, 2016, 145, 3, 28, "Text"],
Cell[54139, 2021, 67, 2, 31, "Input"],
Cell[54209, 2025, 115, 2, 31, "Input"],
Cell[54327, 2029, 59, 0, 28, "Text"],
Cell[54389, 2031, 109, 2, 31, "Input"],
Cell[54501, 2035, 63, 0, 28, "Text"],
Cell[54567, 2037, 151, 4, 31, "Input"],
Cell[54721, 2043, 178, 4, 54, "Input"],
Cell[54902, 2049, 287, 5, 48, "Text"],
Cell[55192, 2056, 149, 4, 31, "Input"],
Cell[55344, 2062, 190, 4, 48, "Text"],
Cell[55537, 2068, 66, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[55652, 2076, 51, 1, 36, "Subsection"],
Cell[55706, 2079, 137, 3, 28, "Text"],
Cell[55846, 2084, 71, 2, 31, "Input"],
Cell[55920, 2088, 173, 3, 48, "Text"],
Cell[56096, 2093, 174, 3, 48, "Text"],
Cell[56273, 2098, 70, 2, 31, "Input"],
Cell[56346, 2102, 603, 12, 192, "Input",
  Evaluatable->False],
Cell[56952, 2116, 87, 2, 31, "Input"],
Cell[57042, 2120, 111, 2, 31, "Input"],
Cell[57156, 2124, 95, 2, 28, "Text"],
Cell[57254, 2128, 136, 3, 31, "Input"],
Cell[57393, 2133, 69, 2, 31, "Input"],
Cell[57465, 2137, 152, 3, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[57654, 2145, 58, 1, 36, "Subsection"],
Cell[57715, 2148, 287, 6, 48, "Text",
  Evaluatable->False],
Cell[58005, 2156, 88, 3, 29, "Input"],
Cell[58096, 2161, 968, 17, 284, "Input",
  Evaluatable->False],
Cell[59067, 2180, 323, 10, 44, "Text",
  Evaluatable->False],
Cell[59393, 2192, 207, 6, 29, "Input",
  Evaluatable->False],
Cell[59603, 2200, 70, 2, 31, "Input"],
Cell[59676, 2204, 399, 13, 44, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[60100, 2221, 79, 2, 39, "Subsubsection"],
Cell[60182, 2225, 147, 5, 30, "Input"],
Cell[60332, 2232, 147, 5, 30, "Input"],
Cell[60482, 2239, 147, 5, 30, "Input"],
Cell[60632, 2246, 147, 5, 30, "Input"],
Cell[60782, 2253, 147, 5, 30, "Input"],
Cell[60932, 2260, 181, 5, 51, "Input"],
Cell[61116, 2267, 69, 2, 31, "Input"],
Cell[61188, 2271, 172, 5, 28, "Text",
  Evaluatable->False],
Cell[61363, 2278, 176, 5, 30, "Input"],
Cell[61542, 2285, 69, 2, 31, "Input"],
Cell[61614, 2289, 186, 5, 30, "Input"],
Cell[61803, 2296, 104, 3, 28, "Text"],
Cell[61910, 2301, 145, 3, 54, "Input"],
Cell[62058, 2306, 156, 5, 30, "Input"],
Cell[62217, 2313, 144, 4, 29, "Input"],
Cell[62364, 2319, 93, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

